/*
 * arch/arm/mach-cxd4105/arch.c
 *
 * machine_desc and initialize function
 *
 * Copyright 2005,2006 Sony Corporation
 *
 * This code is based on arch/arm/mach-integrator/arch.c.
 */
/*
 *  linux/arch/arm/mach-integrator/arch.c
 *
 *  Copyright (C) 2000 Deep Blue Solutions Ltd
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <linux/config.h>
#include <linux/types.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/init.h>
#include <linux/pm.h>
#include <linux/device.h>

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/setup.h>
#include <asm/mach-types.h>
#include <asm/hardware/amba.h>

#include <asm/mach/arch.h>

#include <asm/arch/system.h>
#include <asm/arch/regs-timer.h>
#include <asm/arch/regs-clk.h>

#include <asm/mach/time.h>
#include <asm/arch/time.h>

#include <asm/hardware/serial_amba2.h>
#include <asm/arch/uart.h>
#include <asm/arch/regs-uart.h>
#include <asm/arch/regs-dmac.h>
#include <asm/arch/regs-pci.h>

#include "mm.h"
#include "irq.h"

static void cxd4105_power_off(void)
{
#if defined(CONFIG_PM)	
	extern void cxd4105_mk_pwr_cmd(u8, u16);
	cxd4105_mk_pwr_cmd(0, 0);/*cmd=COLD, reboot=0*/
#endif	
}

//static struct platform_device *devices[] __initdata = {
//};

/* UART */
static struct uart_platform_data uart0_data = {
	.chan		= 0,
#if defined(CONFIG_ARCH_CXD4108)
	.uartclk	= 14666667,
#else  /* CONFIG_ARCH_CXD4105 */
	.uartclk	= 14769231,
#endif 

	.fifosize	= 32,
#if defined(CONFIG_SERIAL_AMBA2_TXDMA) &&  defined(UART0TX_DMA_CH)
	.tx_use_dma	= 1,
	.tx_dma_ch	= UART0TX_DMA_CH,
#endif
	.tx_dma_dev	= DMA_REQID_UART0TX << DMACCCONFIG_DESTDEV_SHIFT,
#if defined(CONFIG_SERIAL_AMBA2_RXDMA) &&  defined(UART0RX_DMA_CH)
	.rx_use_dma	= 1,
	.rx_dma_ch	= UART0RX_DMA_CH,
#endif
	.rx_dma_dev	= DMA_REQID_UART0RX << DMACCCONFIG_SRCDEV_SHIFT,
	.rx_timeout	= UART_RTO_DEFAULT,
};

static struct uart_platform_data uart1_data = {
	.chan		= 1,
#if defined(CONFIG_ARCH_CXD4108)
	.uartclk	= 14666667,
#else /* CONFIG_ARCH_CXD4105 */
	.uartclk	= 14769231,
#endif
	.fifosize	= 32,
};

#if defined(CONFIG_ARCH_CXD4105)
static struct uart_platform_data uart2_data = {
	.chan		= 2,
	.uartclk	= 14769231,
	.fifosize	= 32,
};

#ifdef CONFIG_CXD4105_UART3
static struct uart_platform_data uart3_data = {
	.chan		= 3,
	.uartclk	= 14769231,
	.fifosize	= 32,
};
#endif /* CONFIG_CXD4105_UART3 */
#endif /* CONFIG_ARCH_CXD4105 */

static struct amba_device uart0_device = {
	.dev = {
		.bus_id = "apb:42",
		.platform_data	= &uart0_data,
	},
	.res = {
		 .start = CXD4105_UART0_BASE,
		 .end   = CXD4105_UART0_BASE+SZ_4K-1,
		 .flags = IORESOURCE_MEM,
	 },
	.irq = { IRQ_UART0, },
};

static struct amba_device uart1_device = {
	.dev = {
		.bus_id = "apb:43",
		.platform_data	= &uart1_data,
	},
	.res = {
		 .start = CXD4105_UART1_BASE,
		 .end   = CXD4105_UART1_BASE+SZ_4K-1,
		 .flags = IORESOURCE_MEM,
	 },
	.irq = { IRQ_UART1, },
};

#if defined(CONFIG_ARCH_CXD4105)
static struct amba_device uart2_device = {
	.dev = {
		.bus_id = "apb:44",
		.platform_data	= &uart2_data,
	},
	.res = {
		 .start = CXD4105_UART2_BASE,
		 .end   = CXD4105_UART2_BASE+SZ_4K-1,
		 .flags = IORESOURCE_MEM,
	 },
	.irq = { IRQ_UART2, },
};

#ifdef CONFIG_CXD4105_UART3
static struct amba_device uart3_device = {
	.dev = {
		.bus_id = "apb:45",
		.platform_data	= &uart3_data,
	},
	.res = {
		 .start = CXD4105_UART3_BASE,
		 .end   = CXD4105_UART3_BASE+SZ_4K-1,
		 .flags = IORESOURCE_MEM,
	 },
	.irq = { IRQ_UART3, },
};
#endif /* CONFIG_CXD4105_UART3 */
#endif /* CONFIG_ARCH_CXD4105 */

#ifdef CONFIG_CXD4105_PCIBRIDGE_ENABLE
static struct amba_device pcibridge_device = {
	.dev = {
		.bus_id = "ahb:00",
	},
	.res = {
		 .start = CXD4105_PCI_BASE,
		 .end   = CXD4105_PCI_BASE+SZ_1K-1,
		 .flags = IORESOURCE_MEM,
	 },
	.irq = { IRQ_PCI_BRIDGE, },
	.periphid = CXD4105_PCI_BASE,
};
#endif /* CONFIG_CXD4105_PCIBRIDGE_ENABLE */

static struct amba_device *amba_devs[] __initdata = {
#ifdef CONFIG_CXD4105_UART3
	&uart3_device,
#else
	&uart0_device,
	&uart1_device,
#if defined(CONFIG_ARCH_CXD4105) 
	&uart2_device,
#endif /* CONFIG_ARCH_CXD4105 */
#endif /* CONFIG_CXD4105_UART3 */

#ifdef CONFIG_CXD4105_PCIBRIDGE_ENABLE
	&pcibridge_device,
#endif
};

static void __init cxd4105_init(void)
{
	int i;
	struct amba_device **dev;

	pm_power_off = cxd4105_power_off;

	//platform_add_devices(devices, ARRAY_SIZE(devices));

	for (i = 0, dev = amba_devs; i < ARRAY_SIZE(amba_devs); i++, dev++){
		amba_device_register(*dev, &iomem_resource);
	}
}

static void __init
cxd4105_fixup(struct machine_desc *desc, struct tag *tags,
		 char **cmdline, struct meminfo *mi)
{
}


static int timer_initialized = 0;

/* sched clock */
unsigned long long sched_clock(void)
{
	unsigned long long v;

	if (unlikely(!timer_initialized)) 
		return 0;
	v = (unsigned long long)__raw_readl(TIMER_VA_BASE(TIMER_FOR_SCHED)+CXD4105_TIMERREAD);
#if defined(CONFIG_ARCH_CXD4108)
	v *= 500;	/* 1/500 = 0.002 */

#else /* (CONFIG_ARCH_CXD4105) */
	v *= 3125;	/* 1/781.25 = 0.00128 */
	v >>= 2;

#endif

	return v;
}


/*
 * Returns number of ms since last clock interrupt.  Note that interrupts
 * will have been disabled by do_gettimeoffset()
 */
static unsigned long cxd4105_gettimeoffset(void)
{
	volatile TimerStruct_t *timer0 = (volatile TimerStruct_t *)TIMER_VA_BASE(TIMER_FOR_TICK);
	unsigned long ticks1;

	/* If TimerRead counter reaches TimerCompare value
	 * the counter will be cleared, so that just read TimerRead value.
	 */

	ticks1 = timer0->TimerRead;
	if (readl(INT_CTRL(TIMER_STATUS)) & (1 << TIMER_FOR_TICK)){
		if (ticks1 < TIMER_RELOAD-1)
			ticks1 += TIMER_RELOAD;
	}
	return TICKS2USECS(ticks1);
}

/*
 * IRQ handler for the timer
 */
static irqreturn_t
cxd4105_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	volatile TimerStruct_t *timer0 = (volatile TimerStruct_t *)TIMER_VA_BASE(TIMER_FOR_TICK);

	write_seqlock(&xtime_lock);

	// ...clear the interrupt
	// Counter automatically cleared in periodic mode.
	timer0->TimerClear = 1;

	timer_tick(regs);

	write_sequnlock(&xtime_lock);

	return IRQ_HANDLED;
}

static struct irqaction cxd4105_timer_irq = {
	.name		= "CXD4105 timer",
	.flags		= SA_INTERRUPT,
	.handler	= cxd4105_timer_interrupt
};

/*
 * Set up timer interrupt, and return the current time in seconds.
 */
void profile_timer_init(void)
{
	volatile TimerStruct_t *timer1 = (volatile TimerStruct_t *)TIMER_VA_BASE(TIMER_FOR_SCHED); /* sched clock */

	if(timer_initialized) return;

	timer1->TimerControl = 0;
	timer1->TimerClear = 0x11;	/* clear counter and irq */
	timer1->TimerControl = TM1CKDIV1 | TM1CSFREERUN | TM1ST;
	
	timer_initialized = 1;
}

static void __init cxd4105_time_init(void)
{
	volatile TimerStruct_t *timer0 = (volatile TimerStruct_t *)TIMER_VA_BASE(TIMER_FOR_TICK); /* tick clock */
	volatile TimerStruct_t *timer2 = (volatile TimerStruct_t *)TIMER_VA_BASE(TIMER_FOR_APP);

	writeb(1, VA_CLKBLK_BASE + CLKBLK_PSCKTIM);
	/*
	 * Initialise to a known state (all timers off)
	 */
	timer0->TimerControl = 0;
	timer2->TimerControl = 0;

	timer0->TimerClear = 0x11;	/* clear counter and irq */
	timer0->TimerControl = TM1CKDIV1 | TM1CSPERIODIC | TM1INT;
	timer0->TimerCompare = TIMER_RELOAD;

	/* 
	 * Make irqs happen for the system timer
	 */
	setup_irq(IRQ_TIMERx(TIMER_FOR_TICK), &cxd4105_timer_irq);

	/* start tick clock */
	timer0->TimerControl = TM1CKDIV1 | TM1CSPERIODIC | TM1INT | TM1ST;

	profile_timer_init();
}

static void __init cxd4105_init_timer(void)
{
	cxd4105_time_init();
}

static struct sys_timer cxd4105_timer = {
	.init		= cxd4105_init_timer,
	.offset		= cxd4105_gettimeoffset,
};

#if defined(CONFIG_ARCH_CXD4108)
MACHINE_START(CXD4105, "ARM-CXD4108")
#else /* (CONFIG_ARCH_CXD4105) */
MACHINE_START(CXD4105, "ARM-CXD4105")
#endif
     MAINTAINER("Sony Corporation")
     BOOT_MEM(CONFIG_CXD4105_PHYS_OFFSET, 0x70000000, 0xe0000000)
     //BOOT_PARAMS(0x00000100)
     FIXUP(cxd4105_fixup)
     MAPIO(cxd4105_map_io)
     INITIRQ(cxd4105_init_irq)
     .timer = &cxd4105_timer,
     INIT_MACHINE(cxd4105_init)
MACHINE_END
