/*
 * arch/arm/mach-cxd4105/irq.c
 *
 * interrupt handler
 *
 * Copyright 2005,2006 Sony Corporation
 *
 * This code is based on arch/arm/mach-integrator/irq.c.
 */
/*
 *  linux/arch/arm/mach-integrator/irq.c
 *
 *  Copyright (C) 1999 ARM Limited
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <linux/config.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/ptrace.h>

#include <asm/hardware.h>
#include <asm/arch/regs-intctrl.h>
#include <asm/arch/regs-dmac.h>
#include <asm/arch/regs-gpio.h>
#include <asm/arch/regs-cf.h>
#include <asm/arch/regs-pci.h>
#include <asm/irq.h>
#include <asm/io.h>
#include <asm/mach/irq.h>


static void cxd4105_sysv_demux(int irq_unused, struct irqdesc *desc,
			       struct pt_regs *regs)
{
	unsigned long sts;
	unsigned int irq;

	sts = __raw_readl(INT_CTRL(SYSV_STATUS));
	sts &= SYSV_VALIDBITMASK;
	irq = IRQ_SYSV0;
	desc = irq_desc + irq;
	while (sts){
		if (sts & 1){
			desc->handle(irq, desc, regs);
		}
		irq++;
		desc++;
		sts >>= 1;
	}
}

static void cxd4105_pci_demux(int irq_unused, struct irqdesc *desc, 
			      struct pt_regs *regs)
{
	unsigned long sts;
	unsigned int irq;

	sts = __raw_readl(INT_CTRL(PCIBRIDGE_STATUS));
	sts &= PCIBRIDGE_VALIDBITMASK;
	irq = IRQ_PCI_BRIDGE;
	desc = irq_desc + irq;
	while (sts){
		if (sts & 1){
			desc->handle(irq, desc, regs);
		}
		irq++;
		desc++;
		sts >>= 1;
	}
}

static void cxd4105_usb_demux(int irq_unused, struct irqdesc *desc, 
			      struct pt_regs *regs)
{
	unsigned long sts;
	unsigned int irq;

	sts = __raw_readl(INT_CTRL(USB_STATUS));
	sts &= USB_VALIDBITMASK;
	irq = IRQ_USB_HS;
	desc = irq_desc + irq;
	while (sts){
		if (sts & 1){
			desc->handle(irq, desc, regs);
		}
		irq++;
		desc++;
		sts >>= 1;
	}
}

static void cxd4105_gpio_demux(int irq_unused, struct irqdesc *desc, 
			       struct pt_regs *regs)
{
	unsigned long sts;
	unsigned int irq;

	sts = __raw_readl(INT_CTRL(GPIO_STATUS));
	sts &= GPIO_VALIDBITMASK;
	irq = IRQ_GPIO0;
	desc = irq_desc + irq;
	while (sts){
		if (sts & 1){
			desc->handle(irq, desc, regs);
		}
		irq++;
		desc++;
		sts >>= 1;
	}
}

static void cxd4105_cfif_demux(int irq_unused, struct irqdesc *desc, 
			       struct pt_regs *regs)
{
	unsigned long sts;

	sts = __raw_readl(VA_CF_BASE + CF_IRQ_STAT);
	sts &= __raw_readl(VA_CF_BASE + CF_IRQ_EN);
	if (sts & CF_IRQ_STAT_INT_CF0_IRQ){
		desc = irq_desc + IRQ_CF0;
		desc->handle(IRQ_CF0, desc, regs);
	}
	if (sts & CF_IRQ_STAT_INT_EXIO_IRQ){
		desc = irq_desc + IRQ_EXIO;
		desc->handle(IRQ_EXIO, desc, regs);
	}
}

static void cxd4105_adc_demux(int irq_unused, struct irqdesc *desc,
			       struct pt_regs *regs)
{
	unsigned long sts;
	unsigned int irq;

	sts = __raw_readl(INT_CTRL(ADC_STATUS));
	sts &= ADC_VALIDBITMASK;
	irq = IRQ_ADC0;
	desc = irq_desc + irq;
	while (sts){
		if (sts & 1){
			desc->handle(irq, desc, regs);
		}
		irq++;
		desc++;
		sts >>= 1;
	}
}

static void cxd4105_sio_demux(int irq_unused, struct irqdesc *desc,
			       struct pt_regs *regs)
{
	unsigned long sts;
	unsigned int irq;

	sts = __raw_readl(INT_CTRL(SIO_STATUS));
	sts &= SIO_VALIDBITMASK;
	irq = IRQ_SIO0;
	desc = irq_desc + irq;
	while (sts){
		if (sts & 1){
			desc->handle(irq, desc, regs);
		}
		irq++;
		desc++;
		sts >>= 1;
	}
}

static void cxd4105_jog_demux(int irq_unused, struct irqdesc *desc,
			       struct pt_regs *regs)
{
	unsigned long sts;
	unsigned int irq;

	sts = __raw_readl(INT_CTRL(JOG_STATUS));
	sts &= JOG_VALIDBITMASK;
	irq = IRQ_JOG0;
	desc = irq_desc + irq;
	while (sts){
		if (sts & 1){
			desc->handle(irq, desc, regs);
		}
		irq++;
		desc++;
		sts >>= 1;
	}
}

static void cxd4105_uart_demux(int irq_unused, struct irqdesc *desc, 
			       struct pt_regs *regs)
{
	unsigned long sts;
	unsigned int irq;

	sts = __raw_readl(INT_CTRL(UART_STATUS));
	sts &= UART_VALIDBITMASK;
	irq = IRQ_UART0;
	desc = irq_desc + irq;
	while (sts){
		if (sts & 1){
			desc->handle(irq, desc, regs);
		}
		irq++;
		desc++;
		sts >>= 1;
	}
}

static void cxd4105_dma_demux(int irq_unused, struct irqdesc *desc, 
			      struct pt_regs *regs)
{
	unsigned long sts;
	unsigned int irq;

	sts = __raw_readl(DMACINTSTATUS);
	sts &= DMAC_VALIDBITMASK;
	irq = IRQ_DMA0_0;
	desc = irq_desc + irq;
	while (sts){
		if (sts & 1){
			desc->handle(irq, desc, regs);
		}
		irq++;
		desc++;
		sts >>= 1;
	}
}

static void cxd4105_timer_demux(int irq_unused, struct irqdesc *desc, 
				struct pt_regs *regs)
{
	unsigned long sts;
	unsigned int irq;

	sts = __raw_readl(INT_CTRL(TIMER_STATUS));
	sts &= TIMER_VALIDBITMASK;
	irq = IRQ_TIMER0;
	desc = irq_desc + irq;
	while (sts){
		if (sts & 1){
			desc->handle(irq, desc, regs);
		}
		irq++;
		desc++;
		sts >>= 1;
	}
}

static void cxd4105_mask_ack_sysv_irq(unsigned int irq)
{
	irq -= IRQ_SYSV0;
	__raw_writel(1 << irq, INT_CTRL(SYSV_ENABLE_CLEAR));
	/* clear interrupt status */
	__raw_writel(1 << (irq + 3), VA_AGPIOSYS + GPIO_INTCL);
}

static void cxd4105_mask_sysv_irq(unsigned int irq)
{
	__raw_writel(1 << (irq - IRQ_SYSV0), INT_CTRL(SYSV_ENABLE_CLEAR));
}

static void cxd4105_unmask_sysv_irq(unsigned int irq)
{
	__raw_writel(1 << (irq - IRQ_SYSV0), INT_CTRL(SYSV_ENABLE));
}

static struct irqchip sysv_chip = {
	.ack    = cxd4105_mask_ack_sysv_irq,
	.mask   = cxd4105_mask_sysv_irq,
	.unmask = cxd4105_unmask_sysv_irq,
};

#if 0
#define reg_PCI_INT_ENABLE \
           (PCI_INT_INTA | PCI_INT_INTB | PCI_INT_INTC | PCI_INT_INTD | \
            PCI_INT_AHBPCIWIN | PCI_INT_PCIAHBWIN2 | PCI_INT_PCIAHBWIN1 | \
            PCI_INT_DMAAHBPCI | PCI_INT_DMAPCIAHB | PCI_INT_CLKRUN | \
	    PCI_INT_PM | PCI_INT_SERR | PCI_INT_PERR | PCI_INT_MABORT | \
            PCI_INT_TABORT | PCI_INT_SIGTABORT)

static unsigned long cxd4105_pci_irq_masks[] =
{
	reg_PCI_INT_ENABLE, //PCI_INT_BRIDGEMASK,
	PCI_INT_INTA,
	PCI_INT_INTB,
	PCI_INT_INTC,
	PCI_INT_INTD,
};

static unsigned long cxd4105_pci_status_masks[] =
{
	reg_PCI_INT_ENABLE, //PCI_INT_BRIDGEMASK,
	0,
	0,
	0,
	0,
};
#endif

static void cxd4105_mask_pci_irq(unsigned int irq)
{
#if 0
	unsigned long temp;
#endif
	__raw_writel(1<<(irq-IRQ_PCI_BRIDGE), INT_CTRL(PCIBRIDGE_ENABLE_CLEAR));
#if 0
	temp = __raw_readl(PCI_CTRL(PCI_INT_ENABLE));
	temp &= ~cxd4105_pci_irq_masks[irq-IRQ_PCI_BRIDGE];
	__raw_writel(temp, PCI_CTRL(PCI_INT_ENABLE));

	temp = cxd4105_pci_status_masks[irq-IRQ_PCI_BRIDGE];
	__raw_writel(temp, PCI_CTRL(PCI_INT_STATUS));
#endif
}

static void cxd4105_unmask_pci_irq(unsigned int irq)
{
#if 0
	unsigned long temp;
#endif

	__raw_writel(1<<(irq-IRQ_PCI_BRIDGE), INT_CTRL(PCIBRIDGE_ENABLE));
#if 0
	temp = __raw_readl(PCI_CTRL(PCI_INT_ENABLE));
	temp |= cxd4105_pci_irq_masks[irq-IRQ_PCI_BRIDGE];
	__raw_writel(temp, PCI_CTRL(PCI_INT_ENABLE));
#endif
}

static struct irqchip pci_chip = {
	.ack	= cxd4105_mask_pci_irq,
	.mask	= cxd4105_mask_pci_irq,
	.unmask = cxd4105_unmask_pci_irq,
};


static void cxd4105_mask_usb_irq(unsigned int irq)
{
	__raw_writel(1 << (irq - IRQ_USB_HS), INT_CTRL(USB_ENABLE_CLEAR));
}

static void cxd4105_unmask_usb_irq(unsigned int irq)
{
	__raw_writel(1 << (irq - IRQ_USB_HS), INT_CTRL(USB_ENABLE));
}

static struct irqchip usb_chip = {
	.ack	= cxd4105_mask_usb_irq,
	.mask	= cxd4105_mask_usb_irq,
	.unmask = cxd4105_unmask_usb_irq,
};


static void cxd4105_mask_ack_gpio_irq(unsigned int irq)
{
	irq -= IRQ_GPIO0;
	__raw_writel(1 << irq, INT_CTRL(GPIO_ENABLE_CLEAR));
	/* clear interrupt status */
	__raw_writel(1 << irq, VA_GPIOSYS + GPIO_INTCL);
}

static void cxd4105_mask_gpio_irq(unsigned int irq)
{
	__raw_writel(1 << (irq - IRQ_GPIO0), INT_CTRL(GPIO_ENABLE_CLEAR));
}

static void cxd4105_unmask_gpio_irq(unsigned int irq)
{
	__raw_writel(1 << (irq - IRQ_GPIO0), INT_CTRL(GPIO_ENABLE));
}

static struct irqchip gpio_chip = {
	.ack	= cxd4105_mask_ack_gpio_irq,
	.mask	= cxd4105_mask_gpio_irq,
	.unmask = cxd4105_unmask_gpio_irq,
};


/* CF IRQ */
static unsigned long cxd4105_get_cfirq_mask(unsigned int irq)
{
	unsigned long mask = 0;

	switch (irq) {
	case IRQ_CF0:
		mask = CF_IRQ_EN_INT_CF0_IRQ_EN;
		break;
	case IRQ_EXIO:
		mask = CF_IRQ_EN_INT_EXIO_IRQ_EN;
		break;
	}
	return mask;
}

static void cxd4105_mask_ack_cfif_irq(unsigned int irq)
{
	unsigned long tmp, mask;

	mask = cxd4105_get_cfirq_mask(irq);
	tmp = __raw_readl(VA_CF_BASE + CF_IRQ_EN);
	tmp &= ~mask;
	__raw_writel(tmp, VA_CF_BASE + CF_IRQ_EN);
	__raw_writel(mask & 0x07, VA_CF_BASE + CF_IRQ_CLR);
}

static void cxd4105_mask_cfif_irq(unsigned int irq)
{
	unsigned long tmp, mask;

	mask = cxd4105_get_cfirq_mask(irq);
	tmp = __raw_readl(VA_CF_BASE + CF_IRQ_EN);
	tmp &= ~mask;
	__raw_writel(tmp, VA_CF_BASE + CF_IRQ_EN);
}

static void cxd4105_unmask_cfif_irq(unsigned int irq)
{
	unsigned long tmp, mask;

	mask = cxd4105_get_cfirq_mask(irq);
	tmp = __raw_readl(VA_CF_BASE + CF_IRQ_EN);
	tmp |= mask;
	__raw_writel(tmp, VA_CF_BASE + CF_IRQ_EN);
}

static struct irqchip cfif_chip = {
	.ack	= cxd4105_mask_ack_cfif_irq,
	.mask	= cxd4105_mask_cfif_irq,
	.unmask = cxd4105_unmask_cfif_irq,
};


static void cxd4105_mask_adc_irq(unsigned int irq)
{
	__raw_writel(1 << (irq - IRQ_ADC0), INT_CTRL(ADC_ENABLE_CLEAR));
}

static void cxd4105_unmask_adc_irq(unsigned int irq)
{
	__raw_writel(1 << (irq - IRQ_ADC0), INT_CTRL(ADC_ENABLE));
}

static struct irqchip adc_chip = {
	.ack	= cxd4105_mask_adc_irq,
	.mask	= cxd4105_mask_adc_irq,
	.unmask = cxd4105_unmask_adc_irq,
};


static void cxd4105_mask_sio_irq(unsigned int irq)
{
	__raw_writel(1 << (irq - IRQ_SIO0), INT_CTRL(SIO_ENABLE_CLEAR));
}

static void cxd4105_unmask_sio_irq(unsigned int irq)
{
	__raw_writel(1 << (irq - IRQ_SIO0), INT_CTRL(SIO_ENABLE));
}

static struct irqchip sio_chip = {
	.ack	= cxd4105_mask_sio_irq,
	.mask	= cxd4105_mask_sio_irq,
	.unmask = cxd4105_unmask_sio_irq,
};


static void cxd4105_mask_jog_irq(unsigned int irq)
{
	__raw_writel(1 << (irq - IRQ_JOG0), INT_CTRL(JOG_ENABLE_CLEAR));
}

static void cxd4105_unmask_jog_irq(unsigned int irq)
{
	__raw_writel(1 << (irq - IRQ_JOG0), INT_CTRL(JOG_ENABLE));
}

static struct irqchip jog_chip = {
	.ack	= cxd4105_mask_jog_irq,
	.mask	= cxd4105_mask_jog_irq,
	.unmask = cxd4105_unmask_jog_irq,
};


static void cxd4105_mask_uart_irq(unsigned int irq)
{
	__raw_writel(1 << (irq - IRQ_UART0), INT_CTRL(UART_ENABLE_CLEAR));
}

static void cxd4105_unmask_uart_irq(unsigned int irq)
{
	__raw_writel(1 << (irq - IRQ_UART0), INT_CTRL(UART_ENABLE));
}

static struct irqchip uart_chip = {
	.ack	= cxd4105_mask_uart_irq,
	.mask	= cxd4105_mask_uart_irq,
	.unmask = cxd4105_unmask_uart_irq,
};


static void cxd4105_mask_dma_irq(unsigned int irq)
{
	__raw_writel(DMAC_HOST, INT_CTRL(DMAC_ENABLE_CLEAR));
}

static void cxd4105_unmask_dma_irq(unsigned int irq)
{
	__raw_writel(DMAC_HOST, INT_CTRL(DMAC_ENABLE));
}

static struct irqchip dma_chip = {
	.ack	= cxd4105_mask_dma_irq,
	.mask	= cxd4105_mask_dma_irq,
	.unmask	= cxd4105_unmask_dma_irq,
};


static void cxd4105_mask_timer_irq(unsigned int irq)
{
	__raw_writel(1 << (irq - IRQ_TIMER0), INT_CTRL(TIMER_ENABLE_CLEAR));
}

static void cxd4105_unmask_timer_irq(unsigned int irq)
{
	__raw_writel(1 << (irq - IRQ_TIMER0), INT_CTRL(TIMER_ENABLE));
}

static struct irqchip timer_chip = {
	.ack	= cxd4105_mask_timer_irq,
	.mask	= cxd4105_mask_timer_irq,
	.unmask = cxd4105_unmask_timer_irq,
};


static void sc_mask_irq(unsigned int irq)
{
        __raw_writel(1 << irq, INT_CTRL(IRQ_ENABLE_CLEAR));
}

static void sc_unmask_irq(unsigned int irq)
{
        __raw_writel(1 << irq, INT_CTRL(IRQ_ENABLE));
}
 
static struct irqchip sc_chip = {
        .ack	= sc_mask_irq,
	.mask	= sc_mask_irq,
	.unmask = sc_unmask_irq,
};

void __init cxd4105_init_irq(void)
{
	unsigned int i;

	/* select all interrupt as IRQ */
	__raw_writel(0x00000000, INT_CTRL(IRQ_SELECT));
	/* Disable all interrupts initially. */
	__raw_writel(~0, INT_CTRL(IRQ_ENABLE_CLEAR));

	/* Disable interrupts */
	__raw_writel(~0, INT_CTRL(TIMER_ENABLE_CLEAR));
	__raw_writel(~0, INT_CTRL(UART_ENABLE_CLEAR));
	__raw_writel(~0, INT_CTRL(DMAC_ENABLE_CLEAR));
	__raw_writel(~0, INT_CTRL(JOG_ENABLE_CLEAR));
	__raw_writel(~0, INT_CTRL(SIO_ENABLE_CLEAR));
	__raw_writel(~0, INT_CTRL(USB_ENABLE_CLEAR));
	__raw_writel(~0, INT_CTRL(ADC_ENABLE_CLEAR));
	__raw_writel(~0, INT_CTRL(GPIO_ENABLE_CLEAR));
	__raw_writel(~0, INT_CTRL(PCIBRIDGE_ENABLE_CLEAR));
	__raw_writel(~0, INT_CTRL(SYSV_ENABLE_CLEAR));
	__raw_writel(0, VA_CF_BASE+CF_IRQ_EN);
	
	for (i = 0; i < MAXIRQNUM; i++) {
	        if (((1 << i) && CXD4105_SC_VALID_INT) != 0) {
			set_irq_chip(i, &sc_chip);
			set_irq_handler(i, do_level_IRQ);
			set_irq_flags(i, IRQF_VALID|IRQF_PROBE);
		}
	}

	/* Timer interrupt */
	for (i = IRQ_TIMER0; i <= IRQ_WDT; i++) {
		set_irq_chip(i, &timer_chip);
		set_irq_handler(i, do_level_IRQ);
		set_irq_flags(i, IRQF_VALID|IRQF_PROBE);
	}
	set_irq_chained_handler(IRQ_TIMER, (irq_handler_t)cxd4105_timer_demux);

	/* UART interrupt */
#ifdef CONFIG_CXD4105_UART3
	for (i = IRQ_UART3; i <= IRQ_UART3; i++) {
#else
	for (i = IRQ_UART0; i <= IRQ_UART2; i++) {
#endif
		set_irq_chip(i, &uart_chip);
		set_irq_handler(i, do_level_IRQ);
		set_irq_flags(i, IRQF_VALID|IRQF_PROBE);
	}
	set_irq_chained_handler(IRQ_UART, (irq_handler_t)cxd4105_uart_demux);

	/* CFIF and EXIO interrupt */
	for (i = IRQ_CF0; i <= IRQ_EXIO; i++) {
		set_irq_chip(i, &cfif_chip);
		set_irq_handler(i, do_level_IRQ);
		set_irq_flags(i, IRQF_VALID|IRQF_PROBE);
	}
	set_irq_chained_handler(IRQ_CF, (irq_handler_t)cxd4105_cfif_demux);

	/* GPIO cascade interrupt */
	for (i = IRQ_GPIO0; i <= IRQ_GPIO15; i++) {
		set_irq_chip(i, &gpio_chip);
		set_irq_handler(i, do_level_IRQ);
		set_irq_flags(i, IRQF_VALID|IRQF_PROBE);
	}
	set_irq_chained_handler(IRQ_GPIO, (irq_handler_t)cxd4105_gpio_demux);

	/* DMAC interrupt*/
	for (i = IRQ_DMA0_0; i <= IRQ_DMA0_7; i++) {
		set_irq_chip(i, &dma_chip);
		set_irq_handler(i, do_level_IRQ);
		set_irq_flags(i, IRQF_VALID|IRQF_PROBE);
	}
	set_irq_chained_handler(IRQ_DMAC, (irq_handler_t)cxd4105_dma_demux);

	/* PCI interrupt */
	for (i = IRQ_PCI_BRIDGE; i <= IRQ_PCI_INT3; i++){
		set_irq_chip(i, &pci_chip);
		set_irq_handler(i, do_level_IRQ);
		set_irq_flags(i, IRQF_VALID|IRQF_PROBE);
	}
	set_irq_chained_handler(IRQ_PCI, (irq_handler_t)cxd4105_pci_demux);

	/* JOG cascade interrupt */
	for (i = IRQ_JOG0; i <= IRQ_JOG1; i++) {
		set_irq_chip(i, &jog_chip);
		set_irq_handler(i, do_level_IRQ);
		set_irq_flags(i, IRQF_VALID|IRQF_PROBE);
	}
	set_irq_chained_handler(IRQ_JOG, (irq_handler_t)cxd4105_jog_demux);

	/* SIO cascade interrupt */
	for (i = IRQ_SIO0; i <= IRQ_SIO3; i++) {
		set_irq_chip(i, &sio_chip);
		set_irq_handler(i, do_level_IRQ);
		set_irq_flags(i, IRQF_VALID|IRQF_PROBE);
	}
	set_irq_chained_handler(IRQ_SIO, (irq_handler_t)cxd4105_sio_demux);

	/* USB cascade interrupt */
	for (i = IRQ_USB_HS; i <= IRQ_USB_DMAC; i++) {
		set_irq_chip(i, &usb_chip);
		set_irq_handler(i, do_level_IRQ);
		set_irq_flags(i, IRQF_VALID|IRQF_PROBE);
	}
	set_irq_chained_handler(IRQ_USB, (irq_handler_t)cxd4105_usb_demux);

	/* ADC cascade interrupt */
	for (i = IRQ_ADC0; i <= IRQ_ADC0; i++) {
		set_irq_chip(i, &adc_chip);
		set_irq_handler(i, do_level_IRQ);
		set_irq_flags(i, IRQF_VALID|IRQF_PROBE);
	}
	set_irq_chained_handler(IRQ_ADC, (irq_handler_t)cxd4105_adc_demux);

	/* SYSV cascade interrupt */
	for (i = IRQ_SYSV0; i <= IRQ_SYSV8; i++) {
		set_irq_chip(i, &sysv_chip);
		set_irq_handler(i, do_level_IRQ);
		set_irq_flags(i, IRQF_VALID|IRQF_PROBE);
	}
	set_irq_chained_handler(IRQ_SYSV, (irq_handler_t)cxd4105_sysv_demux);
}
