/*
 * arch/arm/mach-cxd4105/panic.c
 * 
 * panic dump routine 
 * 
 * Copyright 2005,2006 Sony Corporation
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 *
 */

#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/buffer_head.h>
#include <linux/notifier.h>
#include <linux/syscalls.h>

#define MACHINE_PANICED		1
static unsigned long machine_state;

#ifdef CONFIG_PANIC_LOG_PATH
#define PANIC_FILE_NAME CONFIG_PANIC_LOG_PATH
#else
#define PANIC_FILE_NAME "/var/PANIC.TXT"
#endif

static int do_fsync(struct file *file)
{
#if 0	
	struct address_space *mapping = file->f_mapping;
	int ret, err;

	ret = -EINVAL;
	if (!file->f_op || !file->f_op->fsync) {
		goto out;
	}

	current->flags |= PF_SYNCWRITE;
	ret = filemap_fdatawrite(mapping);

	err = file->f_op->fsync(file, file->f_dentry, 0);
	if (!ret)
		ret = err;

	err = filemap_fdatawait(mapping);
	if (!ret)
		ret = err;
	current->flags &= ~PF_SYNCWRITE;

out:
	return ret;
#else
	return sys_sync();
#endif
}

static void cxd4105_dump_panic(unsigned long event, char *buf)
{
	struct file *log_file = NULL;
	int flags = O_CREAT | O_NOFOLLOW | O_TRUNC | O_WRONLY;
	int irq_disabled = 0;
	int ret = 0;
	
	if (irqs_disabled()) {
		irq_disabled = 1;
		local_irq_enable();
	}

	log_file = filp_open(PANIC_FILE_NAME, flags, 0444);
	if (IS_ERR(log_file))
		goto err;

	if (log_file->f_op && log_file->f_op->write) {
		mm_segment_t fs = get_fs();
		set_fs(KERNEL_DS);
	
		ret = log_file->f_op->write(log_file, buf, strlen(buf)+1,
				      &log_file->f_pos);
		if(ret<0)
			printk(KERN_EMERG "%s write error: %d!\n", __func__, ret);
		
		set_fs(fs);
	}

	ret = do_fsync(log_file);
	if(ret)
		printk(KERN_EMERG "%s: fsync error: %d!\n", __func__, ret);

	filp_close(log_file, NULL);

 err:
	if(irq_disabled)
		local_irq_disable();

	return;
}

static int cxd4105_panic(
	struct notifier_block *this,
	unsigned long event,
	 void *ptr)
{
	if (test_and_set_bit(MACHINE_PANICED, &machine_state))
		return NOTIFY_DONE;

	cxd4105_dump_panic(event, ptr);
	
	return NOTIFY_DONE;
}

static struct notifier_block cxd4105_panic_block = {
	.notifier_call	= cxd4105_panic,
};

static int __init setup_notifier(void)
{
	/* Setup panic notifier */
	notifier_chain_register(&panic_notifier_list, &cxd4105_panic_block);

	return 0;
}

postcore_initcall(setup_notifier);

