/*
 * arch/arm/mach-cxd4105/pm.c
 *
 * pm callback for cxd4105
 *
 * Copyright 2004,2005,2006 Sony Corporation
 *
 * This code is based on arch/arm/mach-sa1100/pm.c.
 */
/*
 * SA1100 Power Management Routines
 *
 * Copyright (c) 2001 Cliff Brake <cbrake@accelent.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License.
 *
 * History:
 *
 * 2001-02-06:	Cliff Brake         Initial code
 *
 * 2001-02-25:	Sukjae Cho <sjcho@east.isi.edu> &
 * 		Chester Kuo <chester@linux.org.tw>
 * 			Save more value for the resume function! Support
 * 			Bitsy/Assabet/Freebird board
 *
 * 2001-08-29:	Nicolas Pitre <nico@cam.org>
 * 			Cleaned up, pushed platform dependent stuff
 * 			in the platform specific files.
 *
 * 2002-05-27:	Nicolas Pitre	Killed sleep.h and the kmalloced save array.
 * 				Storage is local on the stack now.
 */
#include <linux/init.h>
#include <linux/suspend.h>
#include <linux/time.h>
#include <linux/bootmem.h>
#include <linux/debug_profile.h>

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/setup.h>
#include <asm/mach-types.h>
#include <asm/page.h>
#include <asm/mach/arch.h>
#include <asm/arch/system.h>
#include <asm/mach/time.h>
#include <asm/arch/time.h>
#include <asm/uaccess.h>
#include <asm/irq.h>
#include <asm/system.h>
#include <asm/mach/irq.h>
#include <linux/delay.h>
#include <asm/arch/regs-dmac.h>
#include <asm/arch/regs-clk.h>
#include <asm/arch/regs-gpio.h>
#include <asm/arch/regs-timer.h>
#include <asm/arch/hardware.h>
#include <asm/arch/memory.h>

#include <asm/mach/warmboot.h>

extern int cxd4105_cpu_suspend(void);
extern void cxd4105_cpu_resume(void);
/*
 * List of global Cxd4105 peripheral registers to preserve.
 * More ones like CP and general purpose register values are preserved
 * on the stack and then the stack pointer is stored last in sleep.S.
 */
enum {	SLEEP_SAVE_IRQ_ENABLE,

	SLEEP_SAVE_IRQ_TIMER_ENABLE,
	
	SLEEP_SAVE_TIMER1_CONTROL,
	SLEEP_SAVE_TIMER1_COMPARE,
	SLEEP_SAVE_TIMER2_CONTROL,
	SLEEP_SAVE_TIMER2_COMPARE,
	SLEEP_SAVE_TIMER3_CONTROL,
	SLEEP_SAVE_TIMER3_COMPARE,
	SLEEP_SAVE_TIMER4_CONTROL,
	SLEEP_SAVE_TIMER4_COMPARE,

	SLEEP_SAVE_SIZE
};

static unsigned long sleep_save[SLEEP_SAVE_SIZE];

static void cxd4105_enable_sio(void)
{
	u32 temp = __raw_readl(VA_GPIO(3) + GPIOE_RESEN) & ~0x0FUL;
	
	__raw_writel(temp, VA_GPIO(3) + GPIOE_RESEN);

	__raw_writel(1, VA_GPIO(3)+GPIOE_PORTSEL(0));
	__raw_writel(1, VA_GPIO(3)+GPIOE_PORTSEL(1));
	__raw_writel(1, VA_GPIO(3)+GPIOE_PORTSEL(2));
	__raw_writel(1, VA_GPIO(3)+GPIOE_PORTSEL(3));

	__raw_writel(1, VA_CLKBLK_BASE + CLKBLK_PSCKSIO);
}

static void save_irq_enable(int irqtype,
			    unsigned long addr_enable)
{
	sleep_save[irqtype] = __raw_readl(addr_enable);
}

static void restore_irq_enable(int irqtype,
			       unsigned long addr_enable,
			       unsigned long addr_disable)
{
  	__raw_writel(sleep_save[irqtype], addr_enable);
	__raw_writel(~sleep_save[irqtype], addr_disable);
}

#define CLK_REGS (sizeof(clkreg_addr)/4)
static u32 clkreg_addr[] = {
#if 0	
	CLKBLK_PSCK54,
	CLKBLK_PSCK112,
	CLKBLK_PSCK24,
	CLKBLK_PSCK192,
	CLKBLK_PSCK38,
	CLKBLK_PSCK19,
	CLKBLK_PSCK32,
	CLKBLK_PSCK16,
	CLKBLK_PSCK264,
	CLKBLK_PSCK66,
	CLKBLK_PSCK33,
	CLKBLK_PSCK48,

	CLKBLK_PLL1MODE,
	CLKBLK_PLL2MODE,
	CLKBLK_PLL2PAT,
	CLKBLK_PLL3MODE,
	CLKBLK_PLL3PAT,
	CLKBLK_PLL4MODE,
	CLKBLK_PLL4PAT,
	CLKBLK_PLL4OUTDIV,
	
	CLKBLK_SELCKARM,
	CLKBLK_SELCKAHB,
	CLKBLK_SELCKAPB,

	CLKBLK_PSCKAARM,
	CLKBLK_AARMSRST,

	CLKBLK_USBPHYCKEN,

	CLKBLK_PSCKEMC,
	CLKBLK_SELCKEMC,
	CLKBLK_PSCKEMCREF,
	CLKBLK_PSCKPCI,
	CLKBLK_SELCKPCI,

	CLKBLK_PSCKATA,
	CLKBLK_SELCKATA,

	CLKBLK_PSCKCF,

	CLKBLK_PSCKMSI,
	CLKBLK_SELCKMSI,

	CLKBLK_PSCKMIW,
	CLKBLK_SELCKMIW,

	CLKBLK_PSCKGIR,
	CLKBLK_PSCKI2C,

	CLKBLK_PSCKUART0,
	CLKBLK_SELCKUART0,
	CLKBLK_PSCKUART1,
	CLKBLK_SELCKUART1,
	CLKBLK_PSCKUART2,
	CLKBLK_SELCKUART2,
	CLKBLK_PSCKUART3,
	CLKBLK_SELCKUART3,

	CLKBLK_PSCKADC0,
	CLKBLK_PSCKADC1,
	CLKBLK_PSCKADC2,
	CLKBLK_PSCKTIM,
	CLKBLK_PSCKTPU,
	CLKBLK_PSCKSIO,

	CLKBLK_PSCKSSP0,
	CLKBLK_SELCKSSP0,

	CLKBLK_PSCKSSP1,
	CLKBLK_SELCKSSP1,

	CLKBLK_PSCKJOG1,
	CLKBLK_PSCKICON,

	CLKBLK_PSCKDDRHCLK,
	CLKBLK_SELCKDDRHCLK,

	CLKBLK_PSCKDDRPHY,
	CLKBLK_SELCKDDRPHY,

	CLKBLK_PSCKAUDIO,
	CLKBLK_SELCKAUDIO,
#endif	
};
static u32 clkreg_val[CLK_REGS];

static void cxd4105_restore_registers(void)
{
	u32 i;
	//IRQ
	restore_irq_enable(SLEEP_SAVE_IRQ_TIMER_ENABLE, INT_CTRL(TIMER_ENABLE), INT_CTRL(TIMER_ENABLE_CLEAR));
  	restore_irq_enable(SLEEP_SAVE_IRQ_ENABLE, INT_CTRL(IRQ_ENABLE_SET), INT_CTRL(IRQ_ENABLE_CLEAR));
	//CLK
	for(i = 0; i<CLK_REGS; i++){
		__raw_writel(clkreg_val[i], VA_CLKBLK_BASE + clkreg_addr[i]);
	}
}

static void cxd4105_save_registers(void)
{
	u32 i;	
	//IRQ
	save_irq_enable(SLEEP_SAVE_IRQ_ENABLE, INT_CTRL(IRQ_ENABLE));
	save_irq_enable(SLEEP_SAVE_IRQ_TIMER_ENABLE, INT_CTRL(TIMER_ENABLE));
	//CLK
	for(i = 0; i<CLK_REGS; i++){
		clkreg_val[i] = __raw_readl(VA_CLKBLK_BASE + clkreg_addr[i]);
	}
}

static int cxd4105_pm_enter(suspend_state_t state)
{
	volatile TimerStruct_t *timer1 = (volatile TimerStruct_t *)TIMER_VA_BASE(TIMER_FOR_TICK); /* tick clock */
	volatile TimerStruct_t *timer2 = (volatile TimerStruct_t *)TIMER_VA_BASE(TIMER_FOR_SCHED); /* sched clock */
	volatile TimerStruct_t *timer3 = (volatile TimerStruct_t *)TIMER_VA_BASE(TIMER_FOR_APP);
	volatile TimerStruct_t *timer4 = (volatile TimerStruct_t *)TIMER_VA_BASE(TIMER_FOR_ETC);
	int err=0;
	
	//there is not difference betwenn suspend and hibernate in cxd4105
	if(state != PM_SUSPEND_DISK && state != PM_SUSPEND_MEM){
		return -EINVAL;
	}

	// save important REGISTERs
	cxd4105_save_registers();
	
	//Timer
	sleep_save[SLEEP_SAVE_TIMER1_CONTROL] = timer1->TimerControl;
	sleep_save[SLEEP_SAVE_TIMER1_COMPARE] = timer1->TimerCompare;
	sleep_save[SLEEP_SAVE_TIMER2_CONTROL] = timer2->TimerControl;
	sleep_save[SLEEP_SAVE_TIMER2_COMPARE] = timer2->TimerCompare;
	sleep_save[SLEEP_SAVE_TIMER3_CONTROL] = timer3->TimerControl;
	sleep_save[SLEEP_SAVE_TIMER3_COMPARE] = timer3->TimerCompare;
	sleep_save[SLEEP_SAVE_TIMER4_CONTROL] = timer4->TimerControl;
	sleep_save[SLEEP_SAVE_TIMER4_COMPARE] = timer4->TimerCompare;

	//disable interrupts
	__raw_writel((u32)-1, INT_CTRL(IRQ_ENABLE_CLEAR));

	printk("resume vector: %08x\n", CONFIG_RESUME_VECTOR);
	
	cxd4105_enable_sio();
	/* go zzz */
	if(cxd4105_cpu_suspend()){
		err = -1;
		printk("suspend error!\n");
	}

	profile_soft_init();
	
	//Timer
	timer1->TimerClear = ~0UL;
	timer1->TimerControl = sleep_save[SLEEP_SAVE_TIMER1_CONTROL];
	timer1->TimerCompare = sleep_save[SLEEP_SAVE_TIMER1_COMPARE];
	
	timer2->TimerClear = ~0UL;
	timer2->TimerControl = sleep_save[SLEEP_SAVE_TIMER2_CONTROL];
	timer2->TimerCompare = sleep_save[SLEEP_SAVE_TIMER2_COMPARE];
	
	timer3->TimerClear = ~0UL;
	timer3->TimerControl = sleep_save[SLEEP_SAVE_TIMER3_CONTROL];
	timer3->TimerCompare = sleep_save[SLEEP_SAVE_TIMER3_COMPARE];
	
	timer4->TimerClear = ~0UL;
	timer4->TimerControl = sleep_save[SLEEP_SAVE_TIMER4_CONTROL];
	timer4->TimerCompare = sleep_save[SLEEP_SAVE_TIMER4_COMPARE];
	
	//Enable DMAC
	writel(0x1, DMACCONFIGRATION);

	//restore REGISTERs
	cxd4105_restore_registers();
	
	return err;
}

unsigned long sleep_phys_sp(void *sp)
{
	return virt_to_phys(sp);
}

/*
  -------------
  category id
  -------------
  reserved
  -------------
  parity MSB
  ------------
  parity LSB
  ------------
  backup powr
  ------------
  system state
  -----------
 */
#define CATEGORY_ID   (0x90)
#define CATEGORY(buf)	(buf[0])
#define RESERVE(buf)	(buf[1])
#define PAR1(buf)	(buf[2])
#define PAR2(buf)	(buf[3])
#define BK_PWR(buf)	(buf[4])
#define SYS_STATE(buf)	(buf[5])
#define      CMD_HOT    (2)
#define      CMD_WARM   (1)
#define      CMD_COLD   (0)
#define RB_REASON(buf)  (buf[6])
#define RB_TMR_H(buf)	(buf[7])
#define RB_TMR_L(buf)   (buf[8])
#define BK_TMR_H(buf)   (buf[9])
#define BK_TMR_L(buf)   (buf[10])

static void cxd4105_mk_pwr_pkt(u8 cmd, u8 reason, u16 timer, u16 bk_timer)
{
	extern u8 suscmd;
	u8 *suscmd_b = &suscmd;
	u16 *suscmd_w = (u16 *)&suscmd;
	u16 parity;
	int i;

	memset(suscmd_b, 0, 128);
	CATEGORY(suscmd_b) = CATEGORY_ID;
	SYS_STATE(suscmd_b) = cmd;
	BK_PWR(suscmd_b) = (cmd == CMD_HOT);
	/* reboot timer */
	RB_REASON(suscmd_b) = reason;
	RB_TMR_H(suscmd_b) = (u8)((timer>>8)&0xFF);
	RB_TMR_L(suscmd_b) = (u8)(timer&0xFF);
	/* backup timer */
	BK_TMR_H(suscmd_b) = (u8)((bk_timer>>8)&0xFF);
	BK_TMR_L(suscmd_b) = (u8)(bk_timer&0xFF);

	parity = cpu_to_be16(suscmd_w[0]) ^ cpu_to_be16(suscmd_w[2]);
	for(i = 3; i < 128/2; i++){
		parity ^= cpu_to_be16(suscmd_w[i]);
	}
	parity ^= cpu_to_be16(0x0f0f);

	PAR1(suscmd_b) = (u8)((parity >> 8) & 0xFF);
	PAR2(suscmd_b) = (u8)(parity & 0xFF);
	
	printk("command: ");
	for(i=0; i<11; i++){
		printk("%02x ",suscmd_b[i]);
	}
	printk("\n");
}

static u8 reboot_reason;
static u16 reboot_timer;
static u16 state_timer;

void cxd4105_mk_pwr_cmd(u8 cmd, u16 reboot)
{
	if(reboot)
		cxd4105_mk_pwr_pkt(cmd, reboot, 0, 0);
	else
		cxd4105_mk_pwr_pkt(cmd, reboot_reason, reboot_timer, state_timer);
	
	cxd4105_pm_enter(PM_SUSPEND_MEM);/*state has no effect*/
}

static void cxd4105_prepare_cmd(suspend_state_t state)
{
	cxd4105_mk_pwr_pkt((state == PM_SUSPEND_MEM? CMD_HOT:CMD_WARM),
			   reboot_reason,
			   reboot_timer,
			   state_timer);
}

/*
 * Called after processes are frozen, but before we shut down devices.
 */
static int cxd4105_pm_prepare(suspend_state_t state)
{
	cxd4105_prepare_cmd(state);
	return warm_boot_pm_prepare(state);
}

/*
 * Called after devices are re-setup, but before processes are thawed.
 */
static int cxd4105_pm_finish(suspend_state_t state)
{
	reboot_reason = 0;
	reboot_timer = 0;
	state_timer = 0;

	return warm_boot_pm_finish(state);
}

/*
 * Set to PM_DISK_FIRMWARE so we can quickly veto suspend-to-disk.
 */
static struct pm_ops cxd4105_pm_ops = {
	.pm_disk_mode	= PM_DISK_FIRMWARE,
	.prepare	= cxd4105_pm_prepare,
	.enter		= cxd4105_pm_enter,
	.finish		= cxd4105_pm_finish,
};


extern struct subsystem power_subsys;

#define wb_attr(_name) \
static ssize_t _name##_show(struct subsystem * s, char * buf)            \
{                                                                        \
	return sprintf(buf, "%u\n", _name);                              \
}                                                                        \
static ssize_t _name##_store(struct subsystem * s,                       \
			     const char * buf, size_t n)                 \
{                                                                        \
	char *endp;                                                      \
	int value;                                                       \
	                                                                 \
	value = simple_strtoul(buf, &endp, 0);                           \
	if(*endp)                                                        \
		return -EINVAL;                                          \
                                                                         \
	_name = value;                                                   \
	return n;                                                        \
}                                                                        \
static struct subsys_attribute _name##_attr = {	                         \
	.attr	= {				                         \
		.name = __stringify(_name),	                         \
		.mode = 0644,			                         \
	},					                         \
	.show	= _name##_show,			                         \
	.store	= _name##_store,		                         \
}

wb_attr(reboot_reason);
wb_attr(reboot_timer);
wb_attr(state_timer);

static int __init cxd4105_pm_init(void)
{
	subsys_create_file(&power_subsys, &reboot_reason_attr);
	subsys_create_file(&power_subsys, &reboot_timer_attr);
	subsys_create_file(&power_subsys, &state_timer_attr);
	
	pm_set_ops(&cxd4105_pm_ops);
	return 0;
}

late_initcall(cxd4105_pm_init);

#ifdef CONFIG_WARM_BOOT_IMAGE
extern int cxd_create_warmbootimage(unsigned long,unsigned long);
int cxd4105_create_warmbootimage(void)
{
	return cxd_create_warmbootimage((unsigned long)cxd4105_cpu_resume, 0);
}
#else
int cxd4105_create_warmbootimage(void)
{
	return 0;
}
#endif
