/*
 * arch/arm/mach-cxd4105/sleep.S
 *
 * pm callback for cxd4105
 *
 * Copyright 2004,2005,2006 Sony Corporation
 *
 * This code is based on arch/arm/mach-sa1100/sleep.S
 */
/*
 * SA11x0 Assembler Sleep/WakeUp Management Routines
 *
 * Copyright (c) 2001 Cliff Brake <cbrake@accelent.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License.
 *
 * History:
 *
 * 2001-02-06: Cliff Brake         Initial code
 *
 * 2001-08-29:	Nicolas Pitre	Simplified.
 *
 * 2002-05-27:	Nicolas Pitre	Revisited, more cleanup and simplification.
 *				Storage is on the stack now.
 */

#include <linux/linkage.h>
#include <asm/assembler.h>
#include <asm/hardware.h>
#include <asm/arch/regs-uart.h>
#include <asm/arch/regs-emc.h>
#include <asm/arch/regs-sdc.h>
#include <asm/arch/regs-sio.h>
#include <asm/arch/regs-clk.h>	
#include <asm/arch/debug-macro.S>
				
/*
	resume sram base
	we set it to the ETM area
*/	
#define RESUMESRAM_BASE (VA_ETM_BASE)

/* below is used in system MMU off */	
/*
	SDRAM control region base
*/
#define EMC_BASE CXD4105_EMC_BASE
/*
	DDR contorl region base
*/
#define SDC_BASE CXD4105_SDC_BASE
/*
	sio control region base
*/
#define SIO_BASE CXD4105_SIO_BASE
/*
	clk control region base
*/
#define CLK_BASE (VA_CLKBLK_BASE+CLKBLK_PSCKETB)	
#define		PSCKETB (0x0)
#define		RSTETB	(CLKBLK_RSTETB-CLKBLK_PSCKETB)
			
		.data


/*
 * cxd4105_cpu_suspend()
 *
 * Causes cxd4105 to enter sleep state
 *
 */

ENTRY(cxd4105_cpu_suspend)

	stmfd	sp!, {r4 - r12, lr}		@ save registers on stack

	@ get coprocessor registers
	mrc 	p15, 0, r4, c3, c0, 0		@ domain ID
	mrc 	p15, 0, r5, c2, c0, 0		@ translation table base addr
	mrc	p15, 0, r6, c13, c0, 0		@ PID
	mrc 	p15, 0, r7, c1, c0, 0		@ control reg
	
	@ store them plus current virtual stack ptr on stack
	mov	r8, sp
	stmfd	sp!, {r4 - r8}

	@ preserve phys address of stack
	mov	r0, sp
	bl	sleep_phys_sp
	ldr	r1, =sleep_save_sp
	str	r0, [r1]
	
	@ disable interrupt 
	mov	r0, #PSR_F_BIT | PSR_I_BIT | SVC_MODE
	msr	cpsr_c, r0

	@ clean & invalide D&I cache, necessary?
	bl	arm926_flush_kern_cache_all
	
	@ for creating Warm-Boot-Image
	bl	cxd4105_create_warmbootimage

	cmp	r0, #0
	beq	goto_suspend

	/*
		failure handling for warm_boot_image
	*/
	mov	r0, sp
	ldmfd	r0, {r4 - r7, sp}
	mcr 	p15, 0, r4, c3, c0, 0		@ domain ID
	mcr 	p15, 0, r5, c2, c0, 0		@ translation table base addr
	mcr	p15, 0, r6, c13, c0, 0		@ PID
	mcr 	p15, 0, r7, c1, c0, 0		@ page table
	ldmfd	sp!, {r4 - r12, pc}
	
	/*
		sdram refresh - preparation
	*/
goto_suspend:
#ifdef CONFIG_DEBUG_PROFILE	
	adr	r0, __printascii
	bl	profile_dump_record
#endif
	@ enable ETB 
	ldr	r0, =CLK_BASE
	mov	r1, #0x01
	str	r1, [r0, #PSCKETB]		@ enable PSCKETB
	
	@ copy the standby operation code to the resume ram at 0x69700000-0x6970FFFF
	ldr	r0, =RESUMESRAM_BASE
	adr	r1, excute_in_resume_ram
	adr	r2, excute_in_resume_ram_end

code_copy:		
	ldr	r3, [r1], #4
	str	r3, [r0], #4
	teq	r1, r2
	bne	code_copy

	@ r4-r11 are for control region base address
	@ load neccesary data to register to avoid accessing sdr/ddr
	ldr	r4, =SDC_BASE
	ldr	r5, =EMC_BASE
	ldr	r6, =SIO_BASE
	ldr	r7, resume_vector_addr		@ address of the end of DDR	
	@ physical address of cxd4105_cpu_resume
	ldr	r0, =cxd4105_cpu_resume
	bl	sleep_phys_sp
	mov	r8, r0				@ resume vector
	bl	__printhex8
	mov	r0, #':'
	bl	__printch
	@ resume ram base address
	ldr	r9, =RESUMESRAM_BASE
	@ resume_vector_backup phys addr
	ldr	r0, =resume_vector_backup
	bl	sleep_phys_sp
	mov	r10, r0				
	@ loops per jiffy
	ldr	r0, LC0				
	ldr	r11, [r0]
	
	@ clean & invalide D&I cache
	bl	arm926_flush_kern_cache_all
	mrc	p15, 0, r0, c1, c0, 0		@ ctrl register
	bic	r0, r0, #0x1000			@ ...i............
	bic	r0, r0, #0x000e			@ ............wca.
	mcr	p15, 0, r0, c1, c0, 0		@ disable caches
	@ jump to the resume ram
	mov	pc, r9
	
	.align	5
excute_in_resume_ram:
	@ disable MMU, the mapping is flat so
	mrc	p15, 0, r0, c1, c0, 0		@ ctrl register
	bic	r0, r0, #0x0001			@ ...............m
	mcr	p15, 0, r0, c1, c0, 0		@ disable MMU
	nop
	nop
	nop

	@ save resume vector
	ldr	r0, [r7]			@ load original data
	str	r0, [r10]			@ save to resume_vector_backup
	str	r8, [r7]			@ save resume vector
	
	@ set DDR to self-refresh
	@ REFMD
	ldr	r0, [r4, #SD_PARA_4]
	orr	r0, r0, #0x590			
	str	r0, [r4, #SD_PARA_4]		@ 15-4 = 0x59
	@ wait for >39us
	mov	r0, #40
	bl	__udelay			
	@ CLK24
	mov	r0, #0x79000000
	orr	r0, r0, #0x00400000
	orr	r0, r0, #0x10			@ r0 = 0x79400010
	ldr	r1, [r0]
	bic	r1, r1, #1<<6
	str	r1, [r0]
	@ wait for 610ns~39us	
	mov	r0, #20
	bl	__udelay			
ddr_sref:		
	ldr	r0, [r4, #SD_PARA_4]
	bic	r0, r0, #(1 << 25 | 1 << 26)    @ clear SRFEX & SREET
	str	r0, [r4, #SD_PARA_4]
	
	orr	r0, r0, #1 << 26                @ set SREET
	str	r0, [r4, #SD_PARA_4]

	mov	r1, #0x10000
10:	cmp	r1, #0
	beq	ddr_sref
	ldr	r0, [r4, #SD_PARA_4]
	ands	r0, r0, #1 << 30                @ SRF_IN
	subeq	r1, r1, #1
	beq	10b	

#if defined(CONFIG_CXD4105_USE_SDR)	
        @ set SDRAM Self-Refresh
	mov	r0, #0x07
	str	r0, [r5, #EMC_TRANS_CTRL]
	
10:	ldr	r0, [r5, #EMC_TRANS_CTRL]
	cmp	r0, #0
	bne	10b
	
        mov	r0, #0x8000
	@ DPD mode seems not work
#if defined(CONFIG_CXD4105_DPD_MODE_dummy)	
	orr	r0, r0, #0x0C			@ DPD
#else
	orr	r0, r0, #0x03			@ SREF
#endif	
        str	r0, [r5, #EMC_PHY_SDR_CMD]
	@ delay > 1ms for sdr SREF
	mov	r0, #0x400
	bl	__udelay
#endif

	mov	r0, #'E'
	bl	__printch
	mov	r0, #'\r'
	bl	__printch
	mov	r0, #'\n'
	bl	__printch
		
	@ the suscmd will be set properly in pm.c
	@ so we just send it, suspend or hibernate	
send_cmd:	
	adr	r0, suscmd
	mov	r1, #128
	bl	sio_send

	mov	r0, #'.'
	bl	__printch

	b	send_cmd
	
	@ wait for interrupt
	mcr	p15, 0, r0, c7, c0, 4
	
20:	b	20b			@ loop waiting for sleep

	@ for sio tranfer
	@ r0 = buffer address, r1 = count
sio_send:
	sub	r2, r1, #1
	mov	r2, r2, lsl #24
	str	r2, [r6, #SION]		@ store the tranfer count first

	add	r2, r6, #SIOBUFF	@ r2 = sio buffer address
	add	r1, r0, r1		@ r1 = the end of buffer
cpy2siobuf:		
	ldrb	r3, [r0], #1
	strb	r3, [r2], #1
	cmp	r0, r1
	bne	cpy2siobuf

	mov	r0, #0
	str	r0, [r6, #SIOSA]	@ start address
	mov	r0, #0xE0000000
	str	r0, [r6, #SIOM]		@ CSEN | IMOD | TRM1, send only
	mov	r0, #0x01000000
	str	r0, [r6, #SIOBGC0]	@ bgc0
	mov	r0, #0x0
	str	r0, [r6, #SIOBGC1]	@ bgc1
	mov	r0, #0x21000000
	str	r0, [r6, #SIOCS]        @ SIOST | CSEL | LSB
	mov	r0, #0x01000000
	str	r0, [r6, #SIOEN]	@ tranfer start

	@ transfer complete?
	ldr	r1, =0x10000		@ time out value
1:	ldr	r0, [r6, #SIOCS]
	ands	r0, r0, #1 << 29
	subnes	r1, r1, #1
	bne	1b

	mov	r0, #0xC0000000
	str	r0, [r6, #SIOCS]
	mov	r0, #0x00000000
	str	r0, [r6, #SIOEN]
	
	mov	pc, lr
		
LC0:		.word	loops_per_jiffy

/* copy from arch/arm/lib/delay.S
 * 0 <= r0 <= 2000
 */
__udelay:	
		mov	r2,     #0x6800
		orr	r2, r2, #0x00db
		mul	r0, r2, r0

		mov	r2, r11
		mov	r0, r0, lsr #11		@ max = 0x00003fff
		mov	r2, r2, lsr #11		@ max = 0x0003ffff
		mul	r0, r2, r0		@ max = 2^32-1
		movs	r0, r0, lsr #6
		moveq	pc,lr
1:		subs	r0, r0, #1
		bhi	1b
		mov	pc,lr
	
ENTRY(suscmd)
	.long	0x12345678		@dummy
	.space  124
	.long	0x00

#if defined(CONFIG_DEBUG_LL)	
	@ for debug
__printhex8:	
	mov	r1, #8
	b	__printhex

__printhex4:	
	mov	r1, #4
	b	__printhex

__printhex2:	
	mov	r1, #2
__printhex:
	adr	r2, hexbuf
	add	r3, r2, r1
	mov	r1, #0
	strb	r1, [r3]
1:	and	r1, r0, #15
	mov	r0, r0, lsr #4
	cmp	r1, #10
	addlt	r1, r1, #'0'
	addge	r1, r1, #'a' - 10
	strb	r1, [r3, #-1]!
	teq	r3, r2
	bne	1b
	mov	r0, r2
	b	__printascii

	.ltorg

__printascii:	
	addruart r3
	b	2f
1:	waituart r2, r3
	senduart r1, r3
	busyuart r2, r3
	teq	r1, #'\n'
	moveq	r1, #'\r'
	beq	1b
2:	teq	r0, #0
	ldrneb	r1, [r0], #1
	teqne	r1, #0
	bne	1b
	mov	pc, lr

__printch:	
	addruart r3
	mov	r1, r0
	mov	r0, #0
	b	1b

hexbuf:		.space 16
#else
__printhex8:
	mov	pc, lr
__printhex4:
	mov	pc, lr
__printhex2:
	mov	pc, lr
__printhex:
	mov	pc, lr
__printascii:
	mov	pc, lr
__printch:
	mov	pc, lr
#endif
excute_in_resume_ram_end:

/*
 * cxd4105_cpu_resume()
 *
 * entry point from bootloader into kernel during resume
 *   Redboot does not have resume vector.
 *   Please jump to this vector by "go" command of RedBoot.
 */

	.align 5
ENTRY(cxd4105_cpu_resume)
	mov	r0, #'R'
	bl	__printch
	
	mov	r0, #PSR_F_BIT | PSR_I_BIT | MODE_SVC
	msr	cpsr_c, r0			@ set SVC, irqs off

tc_loop:mrc	p15, 0, r15, c7, c10, 3		@ test & clean D cache
	bne	tc_loop
	
	mov	r0, #0
	mcr	p15, 0, r0, c7, c7, 0		@ invalidate I,D caches
	mcr	p15, 0, r0, c7, c10, 4		@ drain WB
	mcr	p15, 0, r0, c8, c7, 0		@ invalidate I & D TLBs
	mrc	p15, 0, r0, c1, c0, 0		@ ctrl register
	bic	r0, r0, #0x000f			@ ............wcam
	bic	r0, r0, #0x1100			@ ...i...s........
	mcr	p15, 0, r0, c1, c0, 0		@ ctrl register
	nop
	nop

	@ restore data of resume vector
	ldr	r0, resume_vector_backup
	ldr	r1, resume_vector_addr
	str	r0, [r1]
 
	ldr	r0, sleep_save_sp		@ stack phys addr
	ldr	r2, =resume_after_mmu		@ its absolute virtual address
	ldmfd	r0, {r4 - r7, sp}		@ CP regs + virt stack ptr

	mcr 	p15, 0, r4, c3, c0, 0		@ domain ID
	mcr 	p15, 0, r5, c2, c0, 0		@ translation table base addr
	mcr	p15, 0, r6, c13, c0, 0		@ PID

	b	resume_turn_on_mmu		@ cache align execution

	.align 5
resume_turn_on_mmu:
	mcr 	p15, 0, r7, c1, c0, 0		@ turn on MMU, caches, etc.
	mov	pc, r2				@ jump to virtual addr
	nop
	nop
	nop
	nop
	
resume_vector_addr:
	.word	CONFIG_RESUME_VECTOR
	
resume_vector_backup:
	.word	0
	
sleep_save_sp:
	.word	0				@ preserve stack phys ptr here

resume_after_mmu:
	mov	r0, #0
	ldmfd	sp!, {r4 - r12, pc}		@ return to caller
