/*
 *
 *    Copyright (c) 2004 Fujitsu Limited
 *
 *    Module name: booke_wdt.h
 *    Author:      Takeharu KATO<kato.takeharu@jp.fujitsu.com>
 *    Description:
 *      Header file for PowerPC Book E(PPC 4xx/e500) watchdog driver.
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License
 *    as published by the Free Software Foundation; either version 2
 *    of the License, or (at your option) any later version.
 *
 *    Neither Takeharu KATO nor Fujitsu Ltd. admit liability nor provide
 *    warranty for any of this software.
 *
 */
#ifndef _DRIVERS_CHAR_WATCHDOG_BOOKE_WDT_H
#define _DRIVERS_CHAR_WATCHDOG_BOOKE_WDT_H
#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/ptrace.h>
#include <linux/watchdog.h>
#include <asm/reg_booke.h>

/*
 *  Driver state flags(bit position)
 */
#define WDT_STATE_OPEN                  0   /* driver is opend  */
#define WDT_STATE_STOP_ON_CLOSE         1   /* Stop with close is expected */
/*
 * Configurations
 */
#define WDT_TIMO           60000    /* Default timeout = 60000 ms(1min) */
#define WDT_HEARTBEAT_MIN  100      /* Minimum timeout = 100 ms */
#define WDT_HEARTBEAT_MAX  600000   /* Maximum timeout = 600000ms(1hour) */
#ifdef __KERNEL__
//#define BOOKE_WDT_DEBUG                     /*  Debug switch */

/*  Watchdog timer periods can be set on PPC 4xx cpus. */
#if defined(CONFIG_4xx)
/*
 *  For PowerPC 4xx
 */
#define WDT_WP0               0
#define WDT_WP1               1
#define WDT_WP2               2
#define WDT_WP3               3
#define wdt_calculate_tcr(period,reset) ( TCR_WP(period) | TCR_WRC(reset) )
#else
#if defined(CONFIG_E500)
/*
 *  For e500 CPU
 *  Actually, e500 can arbitrary periods can be set,
 *  But this driver uses fix period value as same as PPC440
 *  on purpose for simplicity.
 *  Following values split into WP and WP_EXT parts in booke_wdt.c.
 */
#define WDT_WP0               21
#define WDT_WP1               25
#define WDT_WP2               29
#define WDT_WP3               33
#define WDT_TCR_WP_BITLEN      2 /*  2bit length  */
#define WDT_TCR_WP_BITMSK     ((1<<(WDT_TCR_WP_BITLEN))-1)  /*  mask  */
#define WDT_TCR_WPEXT_BITMSK  0xf  /*  4bit length  */
#define WDT_TCR_WPEXT_SHIFT   17
#define WDT_WPEXT_VAL(period) (((period)>>WDT_TCR_WP_BITLEN)&    \
			       (WDT_TCR_WPEXT_BITMSK))
#define wdt_calculate_tcr(period,reset) \
                         ( TCR_WP((period)&(WDT_TCR_WP_BITMSK))  |\
			   TCR_WRC((reset))                      |\
		   ( WDT_WPEXT_VAL(period) << WDT_TCR_WPEXT_SHIFT))
#else
#error "Book E WDT detects invalid configuration(Unknown CPU)"
#endif  /*  CONFIG_E500  */
#endif  /*  CONFIG_4xx   */
/*
 *  WP relevant values used in our driver.
 *  Note:WDT period must be more than HZ(Timer ticks)
 */
#define WDT_WP                 WDT_WP3

/*
 *  IOCTL commands for comaptiblity for old driver
 */
#define WDIOC_GETPERIOD         WDIOC_GETTIMEOUT
#define WDIOC_SETPERIOD         WDIOC_SETTIMEOUT

/*
 *  output messages
 */
#define __BOOKE_WDT_MSG "BookE-WDT : "
#define booke_mkmsg(str) __BOOKE_WDT_MSG str
#define booke_wdt_info(fmt,arg...) \
    printk(KERN_INFO __BOOKE_WDT_MSG fmt,##arg)
#define booke_wdt_note(fmt,arg...) \
    printk(KERN_NOTICE __BOOKE_WDT_MSG fmt,##arg)
#define booke_wdt_err(fmt,arg...) \
    printk(KERN_ALERT __BOOKE_WDT_MSG fmt,##arg)
#if defined(BOOKE_WDT_DEBUG)
#define booke_wdt_dbg(fmt,arg...) \
    printk(KERN_ALERT __BOOKE_WDT_MSG fmt,##arg)
#else
#define booke_wdt_dbg(fmt,arg...) \
        do{}while(0)
#endif  /*  WDT_DEBUG  */

#endif  /* __KERNEL__  */
#endif  /*  _DRIVERS_CHAR_WATCHDOG_BOOKE_WDT_H  */
