/*-----------------------------------------------------------------------------
 * gpio-cxd4105.c
 *
 * a device driver file for gpio
 *
 * Copyright 2005,2006 Sony Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *---------------------------------------------------------------------------*/
#define __GPIO_CXD4105_C__

/*-----------------------------------------------------------------------------
 * Include file
 *---------------------------------------------------------------------------*/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/spinlock.h>
#include <linux/interrupt.h>
#include <linux/device.h>
#include <linux/pm.h>
#include <linux/proc_fs.h>

#include <asm/io.h>
#include <asm/bitops.h>
#include <asm/arch/irqs.h>
#include <asm/arch/platform.h>

#include <asm/arch/gpio-core.h>
#include <asm/arch/gpio-cxd4105.h>

//#define CHECK_TIME
#ifdef CHECK_TIME
#include <sony/instrumental.h>
#endif
//#define CHECK_SIZE

/*-----------------------------------------------------------------------------
 * Change note
 *
 * 01.00.000    : First release
 * 01.00.001    : changed set_gpio_func() / set_gpioX_func().
 *                changed get_gpio_func() / get_gpioX_func().
 *                fixed bugs in __get_nr_grp().
 *                removed demux routine of interrupt(cxd4105).
 * 01.00.002    : moved definitions of register from header file(cxd4105).
 * 01.00.003    : changed from "Proprietary" driver to "GPL" driver.
 * 01.00.004    : added port data clear codes to __resume_hot() function.
 *                added /proc I/F for debug.
 * 01.00.005    : changed to use target_init() function of kernel(cxd4105).
 *              : changed gpio_cxd4105_init() / gpio_cxd4105_exit().
 * 01.00.006    : changed Vsuspend() / Vresume().
 *                changed function name from __suspend_hot() to __suspend().
 *                changed function name from __resume_hot() to __resume().
 *                deleted __suspend_warm() / __resume_warm().
 *                added size check code.
 *                limited target(cxd4105).
 *                added GPIO_PULLUP / GPIO_PULLDN / GPIO_BUSHD definitions.
 *                changed GPIO_INPUT_PU definition to enable input.
 *                changed GPIO_INPUT_PD definition to enable input.
 *                changed GPIO_INPUT_BH definition to enable input.
 * 01.00.007    : supported GPIO_NOSET in set_gpio_func() / set_gpioX_func().
 * 01.00.008    : added GPIO_FUNC_PERIHZ definition.
 *                added GPIO_FUNC_PERIPU definition.
 *                added GPIO_FUNC_PERIPD definition.
 * 01.00.009    : changed error code of set_gpio_func().
 *                added __adjust_nr_bit().
 * 01.00.010	: fixed ng words.
 * 01.00.011	: removed __suspend/__resume func
*/

/*-----------------------------------------------------------------------------
 * Macro declaration
 *---------------------------------------------------------------------------*/
#define MYNAME		"gpio-cxd4105"
#define MYVERSION	"01.00.011"

#define MAX(a,b)	((a)>=(b)?(a):(b))
#define MIN(a,b)	((a)<=(b)?(a):(b))

/*-----------------------------------------------------------------------------
 * Module infomation
 *---------------------------------------------------------------------------*/
MODULE_AUTHOR("Sony Corporation");
MODULE_DESCRIPTION("CXD4105 gpio driver");
MODULE_VERSION(MYVERSION);
MODULE_LICENSE("GPL");

/*-----------------------------------------------------------------------------
 * Compile switch
 *---------------------------------------------------------------------------*/
#define DEBUG_LEVEL 0

/*-[DEBUG_LEVEL 1]-------------------------------------------------
 * print value when function entry and exit                      */
#if(DEBUG_LEVEL == 1)
#define PDEBUG1(arg...)		{printk(arg);}
#define PDEBUG2(arg...)		{}
#define PDEBUG3(arg...)		{}
#define RETURN1(val,arg)	{printk(arg,val);return(val);}
/*-[DEBUG_LEVEL 2]-------------------------------------------------
 * print value of register                                       */
#elif(DEBUG_LEVEL == 2)
#define PDEBUG1(arg...)		{printk(arg);}
#define PDEBUG2(arg...)		{printk(arg);}
#define PDEBUG3(arg...)		{}
#define RETURN1(val,arg)	{printk(arg,val);__print_register();return(val);}
/*-[DEBUG_LEVEL 3]-------------------------------------------------
 * print value of variable                                       */
#elif(DEBUG_LEVEL == 3)
#define PDEBUG1(arg...)		{printk(arg);}
#define PDEBUG2(arg...)		{printk(arg);}
#define PDEBUG3(arg...)		{printk(arg);}
#define RETURN1(val,arg)	{printk(arg,val);return(val);}
/*-[NO DEBUG]------------------------------------------------------
 * print nothing                                                 */
#else
#define PDEBUG1(arg...)		{}
#define PDEBUG2(arg...)		{}
#define PDEBUG3(arg...)		{}
#define RETURN1(val,arg)	{return(val);}
#endif

/*----------------------------------------------------------------------
 * Macro definition
 *----------------------------------------------------------------------*/
#define GPIO_PROC	"gpio"

#define CMD_GPIOSYS	"sys"
#define CMD_GPIOEASY	"easy"

#define GPIO_NR(x)	(x - GPIO_0)
#define GPIO_IRQ(x)	(IRQ_GPIO0 + GPIO_NR(x))

/*----------------------------------------------------------------------
 * Register base
 *----------------------------------------------------------------------*/
#define GPIO_BASE	(CXD4105_APB2_BASE + SZ_1M * 7 + 0x00002400)
#define GPIO_BASE_(x)	(CXD4105_APB2_BASE + SZ_1M * 7 + 0x00001000 + ((x) << 8))

/*----------------------------------------------------------------------
 * Register offset
 *----------------------------------------------------------------------*/
#define GPS_DIR		0x00000000
#define GPS_RDATA	0x00000004
#define GPS_DATASET	0x00000008
#define GPS_DATACLR	0x0000000C
#define GPS_INTS	0x00000010
#define GPS_INTHE	0x00000014
#define GPS_INTLE	0x00000018
#define GPS_INTMASK	0x0000001C
#define GPS_INTST	0x00000020
#define GPS_INTSTCLR	0x00000024
#define GPS_INEN	0x00000028
#define GPS_RESEN	0x0000002C
#define GPS_PUPD	0x00000030

#define GPE_DIR		0x00000000
#define GPE_RDATA	0x00000004
#define GPE_DATASET	0x00000008
#define GPE_DATACLR	0x0000000C
#define GPE_INEN	0x00000010
#define GPE_PORTSEL0	0x00000014
#define GPE_PORTSEL1	0x00000018
#define GPE_PORTSEL2	0x0000001C
#define GPE_PORTSEL3	0x00000020
#define GPE_PORTSEL4	0x00000024
#define GPE_PORTSEL5	0x00000028
#define GPE_PORTSEL6	0x0000002C
#define GPE_PORTSEL7	0x00000030
#define GPE_PORTSEL8	0x00000034
#define GPE_PORTSEL9	0x00000038
#define GPE_PORTSEL10	0x0000003C
#define GPE_PORTSEL11	0x00000040
#define GPE_PORTSEL12	0x00000044
#define GPE_PORTSEL13	0x00000048
#define GPE_PORTSEL14	0x0000004C
#define GPE_PORTSEL15	0x00000050
#define GPE_HOLDEN	0x00000054
#define GPE_RESEN	0x00000058
#define GPE_PUPD	0x0000005C

#define GPIO_MEMSPACE	0x00001500

/*----------------------------------------------------------------------
 * Register assign
 *----------------------------------------------------------------------*/
#define REG_GPIO_DIR		IO_ADDRESS(GPIO_BASE | GPS_DIR)
#define REG_GPIO_RDATA		IO_ADDRESS(GPIO_BASE | GPS_RDATA)
#define REG_GPIO_DATASET	IO_ADDRESS(GPIO_BASE | GPS_DATASET)
#define REG_GPIO_DATACLR	IO_ADDRESS(GPIO_BASE | GPS_DATACLR)
#define REG_GPIO_INTS		IO_ADDRESS(GPIO_BASE | GPS_INTS)
#define REG_GPIO_INTHE		IO_ADDRESS(GPIO_BASE | GPS_INTHE)
#define REG_GPIO_INTLE		IO_ADDRESS(GPIO_BASE | GPS_INTLE)
#define REG_GPIO_INTMASK	IO_ADDRESS(GPIO_BASE | GPS_INTMASK)
#define REG_GPIO_INTST		IO_ADDRESS(GPIO_BASE | GPS_INTST)
#define REG_GPIO_INTSTCLR	IO_ADDRESS(GPIO_BASE | GPS_INTSTCLR)
#define REG_GPIO_INEN		IO_ADDRESS(GPIO_BASE | GPS_INEN)
#define REG_GPIO_RESEN		IO_ADDRESS(GPIO_BASE | GPS_RESEN)
#define REG_GPIO_PUPD		IO_ADDRESS(GPIO_BASE | GPS_PUPD)

#define REG_GPIO_DIR_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_DIR)
#define REG_GPIO_RDATA_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_RDATA)
#define REG_GPIO_DATASET_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_DATASET)
#define REG_GPIO_DATACLR_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_DATACLR)
#define REG_GPIO_INEN_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_INEN)
#define REG_GPIO_PORTSEL0_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_PORTSEL0)
#define REG_GPIO_PORTSEL1_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_PORTSEL1)
#define REG_GPIO_PORTSEL2_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_PORTSEL2)
#define REG_GPIO_PORTSEL3_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_PORTSEL3)
#define REG_GPIO_PORTSEL4_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_PORTSEL4)
#define REG_GPIO_PORTSEL5_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_PORTSEL5)
#define REG_GPIO_PORTSEL6_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_PORTSEL6)
#define REG_GPIO_PORTSEL7_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_PORTSEL7)
#define REG_GPIO_PORTSEL8_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_PORTSEL8)
#define REG_GPIO_PORTSEL9_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_PORTSEL9)
#define REG_GPIO_PORTSEL10_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_PORTSEL10)
#define REG_GPIO_PORTSEL11_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_PORTSEL11)
#define REG_GPIO_PORTSEL12_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_PORTSEL12)
#define REG_GPIO_PORTSEL13_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_PORTSEL13)
#define REG_GPIO_PORTSEL14_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_PORTSEL14)
#define REG_GPIO_PORTSEL15_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_PORTSEL15)
#define REG_GPIO_HOLDEN_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_HOLDEN)
#define REG_GPIO_RESEN_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_RESEN)
#define REG_GPIO_PUPD_(x)	IO_ADDRESS(GPIO_BASE_(x) | GPE_PUPD)

/*-----------------------------------------------------------------------------
 * Register operation
 *---------------------------------------------------------------------------*/
#define SET_GPREG(nr,reg)	(writew(readw(reg)|(1<<(nr)),reg))
#define CLR_GPREG(nr,reg)	(writew(readw(reg)&~(1<<(nr)),reg))

#define GET_GPREG(nr,reg)	((readw(reg)>>(nr))&1)
#define SET_GPREG2(nr,reg)	(writew(1<<(nr),reg))
#define CLR_GPREG2(nr,reg)	(writew(~(1<<(nr)),reg))

#define GET_SELREG(nr,reg)	((reg)+(nr)*4)

/*-----------------------------------------------------------------------------
 * Function prototype declaration
 *---------------------------------------------------------------------------*/
/* virtual function */
static int Vopen(IN unsigned int nr);
static int Vclose(IN unsigned int nr);
static int Vset_sel_func(IN unsigned int nr, IN unsigned char func, 
    IN unsigned int val);
static int Vget_sel_func(IN unsigned int nr, OUT unsigned char *func);
static int Vwrite(IN unsigned int nr, IN unsigned int val);
static int Vread(IN unsigned int nr);
static int Vset_irq_sense(IN unsigned int nr, IN unsigned char sense);
static int Vget_irq_sense(IN unsigned int nr, OUT unsigned char *sense);
static int Venable_irq(IN unsigned int nr);
static int Vdisable_irq(IN unsigned int nr);
static int Vis_enable_irq(IN unsigned int nr);
static int Vis_irq(IN unsigned int nr);
static int Vclear_irq(IN unsigned int nr);
static int Vregister_handler(IN unsigned int nr);
static int Vunregister_handler(IN unsigned int nr);
static int Venable_wakeup(IN unsigned int nr);
static int Vdisable_wakeup(IN unsigned int nr);
static int Vis_enable_wakeup(IN unsigned int nr);
static int Vsuspend(IN struct amba_device *dev, IN u32 state);
static int Vresume(IN struct amba_device *dev);

static int gpio_cxd4105_read_proc(char *buffer, char **start, off_t offset, 
    int length, int *eof, void *data);
static int gpio_cxd4105_write_proc(struct file *file, const char *buffer, 
    unsigned long count, void *data);
/*static*/ int __init gpio_cxd4105_init(void);
static void __exit gpio_cxd4105_exit(void);
static irqreturn_t handler(int irq, void *dev_id, struct pt_regs *fp);
static int set_gpio_func(IN unsigned int nr, IN unsigned char func, 
    IN unsigned int val);
static int set_gpioX_func(IN unsigned int nr, IN unsigned char func, 
    IN unsigned int val);
static int get_gpio_func(IN unsigned int nr, IN unsigned char *func);
static int get_gpioX_func(IN unsigned int nr, IN unsigned char *func);

static __inline__ int __is_gpio(IN unsigned int nr);
static __inline__ int __get_nr_grp(IN unsigned int nr, OUT unsigned int *grp);
static __inline__ void __adjust_nr_bit(IN unsigned int nr, 
    IN unsigned int grp, OUT unsigned int *bit);
static __inline__ int __get_nr_grpbit(IN unsigned int nr, 
    OUT unsigned int *grp, OUT unsigned int *bit);
static __inline__ int __check_nr_func(IN unsigned int nr, 
    IN unsigned char func);
static __inline__ int __check_func_HiZ(IN unsigned char func);
static __inline__ int __check_func_PullUp(IN unsigned char func);
static __inline__ int __check_func_PullDown(IN unsigned char func);
static __inline__ int __get_nr_func(IN unsigned int nr, 
    OUT unsigned char *func);
static __inline__ int __suspend(IN struct amba_device *dev, IN u32 state);
static __inline__ int __resume(IN struct amba_device *dev);
static __inline__ void __print_register(void);

/*-----------------------------------------------------------------------------
 * Variable declaration
 *---------------------------------------------------------------------------*/
static TGpioPluginOps Vops =
{
    Vopen,
    Vclose,
    Vset_sel_func,
    Vget_sel_func,
    Vwrite,
    Vread,
    Vset_irq_sense,
    Vget_irq_sense,
    Venable_irq,
    Vdisable_irq,
    Vis_enable_irq,
    Vis_irq,
    Vclear_irq,
    Vregister_handler,
    Vunregister_handler,
    Venable_wakeup,
    Vdisable_wakeup,
    Vis_enable_wakeup,
    Vsuspend,
    Vresume,
};

static CGpioPlugin gpio_cxd4105 = {
    MYNAME,
    MYVERSION,
    (PORT_MAX + 1),
    &Vops,
    GPIO_BASE_(0),
    GPIO_BASE_(0) + GPIO_MEMSPACE - 1,
};

static const int nr_base[] = {
    GPIO0_0, GPIO1_0, GPIO2_0, GPIO3_0, GPIO4_0, GPIO5_0, GPIO6_0, GPIO7_0,
    GPIO8_0, GPIO9_0, GPIO10_0, GPIO11_0, GPIO12_0, GPIO13_0, GPIO14_0, 
    GPIO15_0, GPIO16_0,
};

static const unsigned char nr_func[][6] = {
    [GPIO0_0][0]  = GPIO_FUNC_TPU0,
    [GPIO0_0][1]  = -1,
    [GPIO0_0][2]  = -1,
    [GPIO0_0][3]  = -1,
    [GPIO0_0][4]  = -1,
    [GPIO0_0][5]  = -1,
    [GPIO0_1][0]  = GPIO_FUNC_TPU1,
    [GPIO0_1][1]  = -1,
    [GPIO0_1][2]  = -1,
    [GPIO0_1][3]  = -1,
    [GPIO0_1][4]  = -1,
    [GPIO0_1][5]  = -1,
    [GPIO0_2][0]  = GPIO_FUNC_TPU2,
    [GPIO0_2][1]  = -1,
    [GPIO0_2][2]  = -1,
    [GPIO0_2][3]  = -1,
    [GPIO0_2][4]  = -1,
    [GPIO0_2][5]  = -1,
    [GPIO0_3][0]  = GPIO_FUNC_TPU3,
    [GPIO0_3][1]  = -1,
    [GPIO0_3][2]  = -1,
    [GPIO0_3][3]  = -1,
    [GPIO0_3][4]  = -1,
    [GPIO0_3][5]  = -1,
    [GPIO0_4][0]  = GPIO_FUNC_TPU4,
    [GPIO0_4][1]  = -1,
    [GPIO0_4][2]  = -1,
    [GPIO0_4][3]  = -1,
    [GPIO0_4][4]  = -1,
    [GPIO0_4][5]  = -1,
    [GPIO0_5][0]  = GPIO_FUNC_SIR,
    [GPIO0_5][1]  = -1,
    [GPIO0_5][2]  = -1,
    [GPIO0_5][3]  = -1,
    [GPIO0_5][4]  = -1,
    [GPIO0_5][5]  = -1,
    [GPIO0_6][0]  = GPIO_FUNC_SIR,
    [GPIO0_6][1]  = -1,
    [GPIO0_6][2]  = -1,
    [GPIO0_6][3]  = -1,
    [GPIO0_6][4]  = -1,
    [GPIO0_6][5]  = -1,
    [GPIO0_7][0]  = GPIO_FUNC_JOG0,
    [GPIO0_7][1]  = GPIO_FUNC_TPU0,
    [GPIO0_7][2]  = -1,
    [GPIO0_7][3]  = -1,
    [GPIO0_7][4]  = -1,
    [GPIO0_7][5]  = -1,
    [GPIO0_8][0]  = GPIO_FUNC_JOG0,
    [GPIO0_8][1]  = GPIO_FUNC_TPU0,
    [GPIO0_8][2]  = -1,
    [GPIO0_8][3]  = -1,
    [GPIO0_8][4]  = -1,
    [GPIO0_8][5]  = -1,
    [GPIO0_9][0]  = GPIO_FUNC_JOG1,
    [GPIO0_9][1]  = GPIO_FUNC_TPU1,
    [GPIO0_9][2]  = -1,
    [GPIO0_9][3]  = -1,
    [GPIO0_9][4]  = -1,
    [GPIO0_9][5]  = -1,
    [GPIO0_10][0] = GPIO_FUNC_JOG1,
    [GPIO0_10][1] = GPIO_FUNC_TPU1,
    [GPIO0_10][2] = -1,
    [GPIO0_10][3] = -1,
    [GPIO0_10][4] = -1,
    [GPIO0_10][5] = -1,
    [GPIO0_11][0] = GPIO_FUNC_BOOTMODE,
    [GPIO0_11][1] = -1,
    [GPIO0_11][2] = -1,
    [GPIO0_11][3] = -1,
    [GPIO0_11][4] = -1,
    [GPIO0_11][5] = -1,
    [GPIO0_12][0] = GPIO_FUNC_BOOTMODE,
    [GPIO0_12][1] = -1,
    [GPIO0_12][2] = -1,
    [GPIO0_12][3] = -1,
    [GPIO0_12][4] = -1,
    [GPIO0_12][5] = -1,
    [GPIO0_13][0] = GPIO_FUNC_BOOTMODE,
    [GPIO0_13][1] = -1,
    [GPIO0_13][2] = -1,
    [GPIO0_13][3] = -1,
    [GPIO0_13][4] = -1,
    [GPIO0_13][5] = -1,
    [GPIO0_14][0] = GPIO_FUNC_BOOTMODE,
    [GPIO0_14][1] = -1,
    [GPIO0_14][2] = -1,
    [GPIO0_14][3] = -1,
    [GPIO0_14][4] = -1,
    [GPIO0_14][5] = -1,
    [GPIO0_15][0] = GPIO_FUNC_BOOTMODE,
    [GPIO0_15][1] = -1,
    [GPIO0_15][2] = -1,
    [GPIO0_15][3] = -1,
    [GPIO0_15][4] = -1,
    [GPIO0_15][5] = -1,

    [GPIO1_0][0]  = GPIO_FUNC_UART0,
    [GPIO1_0][1]  = -1,
    [GPIO1_0][2]  = -1,
    [GPIO1_0][3]  = -1,
    [GPIO1_0][4]  = -1,
    [GPIO1_0][5]  = -1,
    [GPIO1_1][0]  = GPIO_FUNC_UART0,
    [GPIO1_1][1]  = -1,
    [GPIO1_1][2]  = -1,
    [GPIO1_1][3]  = -1,
    [GPIO1_1][4]  = -1,
    [GPIO1_1][5]  = -1,
    [GPIO1_2][0]  = GPIO_FUNC_UART0,
    [GPIO1_2][1]  = -1,
    [GPIO1_2][2]  = -1,
    [GPIO1_2][3]  = -1,
    [GPIO1_2][4]  = -1,
    [GPIO1_2][5]  = -1,
    [GPIO1_3][0]  = GPIO_FUNC_UART0,
    [GPIO1_3][1]  = -1,
    [GPIO1_3][2]  = -1,
    [GPIO1_3][3]  = -1,
    [GPIO1_3][4]  = -1,
    [GPIO1_3][5]  = -1,
    [GPIO1_4][0]  = GPIO_FUNC_UART1,
    [GPIO1_4][1]  = -1,
    [GPIO1_4][2]  = -1,
    [GPIO1_4][3]  = -1,
    [GPIO1_4][4]  = -1,
    [GPIO1_4][5]  = -1,
    [GPIO1_5][0]  = GPIO_FUNC_UART1,
    [GPIO1_5][1]  = -1,
    [GPIO1_5][2]  = -1,
    [GPIO1_5][3]  = -1,
    [GPIO1_5][4]  = -1,
    [GPIO1_5][5]  = -1,
    [GPIO1_6][0]  = GPIO_FUNC_UART1,
    [GPIO1_6][1]  = -1,
    [GPIO1_6][2]  = -1,
    [GPIO1_6][3]  = -1,
    [GPIO1_6][4]  = -1,
    [GPIO1_6][5]  = -1,
    [GPIO1_7][0]  = GPIO_FUNC_UART1,
    [GPIO1_7][1]  = -1,
    [GPIO1_7][2]  = -1,
    [GPIO1_7][3]  = -1,
    [GPIO1_7][4]  = -1,
    [GPIO1_7][5]  = -1,
    [GPIO1_8][0]  = GPIO_FUNC_UART2,
    [GPIO1_8][1]  = -1,
    [GPIO1_8][2]  = -1,
    [GPIO1_8][3]  = -1,
    [GPIO1_8][4]  = -1,
    [GPIO1_8][5]  = -1,
    [GPIO1_9][0]  = GPIO_FUNC_UART2,
    [GPIO1_9][1]  = -1,
    [GPIO1_9][2]  = -1,
    [GPIO1_9][3]  = -1,
    [GPIO1_9][4]  = -1,
    [GPIO1_9][5]  = -1,
    [GPIO1_10][0] = GPIO_FUNC_UART2,
    [GPIO1_10][1] = -1,
    [GPIO1_10][2] = -1,
    [GPIO1_10][3] = -1,
    [GPIO1_10][4] = -1,
    [GPIO1_10][5] = -1,
    [GPIO1_11][0] = GPIO_FUNC_UART2,
    [GPIO1_11][1] = -1,
    [GPIO1_11][2] = -1,
    [GPIO1_11][3] = -1,
    [GPIO1_11][4] = -1,
    [GPIO1_11][5] = -1,
    [GPIO1_12][0] = GPIO_FUNC_UART3,
    [GPIO1_12][1] = -1,
    [GPIO1_12][2] = -1,
    [GPIO1_12][3] = -1,
    [GPIO1_12][4] = -1,
    [GPIO1_12][5] = -1,
    [GPIO1_13][0] = GPIO_FUNC_UART3,
    [GPIO1_13][1] = -1,
    [GPIO1_13][2] = -1,
    [GPIO1_13][3] = -1,
    [GPIO1_13][4] = -1,
    [GPIO1_13][5] = -1,
    [GPIO1_14][0] = GPIO_FUNC_UART3,
    [GPIO1_14][1] = -1,
    [GPIO1_14][2] = -1,
    [GPIO1_14][3] = -1,
    [GPIO1_14][4] = -1,
    [GPIO1_14][5] = -1,
    [GPIO1_15][0] = GPIO_FUNC_UART3,
    [GPIO1_15][1] = -1,
    [GPIO1_15][2] = -1,
    [GPIO1_15][3] = -1,
    [GPIO1_15][4] = -1,
    [GPIO1_15][5] = -1,

    [GPIO2_0][0]  = GPIO_FUNC_SSP0,
    [GPIO2_0][1]  = -1,
    [GPIO2_0][2]  = -1,
    [GPIO2_0][3]  = -1,
    [GPIO2_0][4]  = -1,
    [GPIO2_0][5]  = -1,
    [GPIO2_1][0]  = GPIO_FUNC_SSP0,
    [GPIO2_1][1]  = -1,
    [GPIO2_1][2]  = -1,
    [GPIO2_1][3]  = -1,
    [GPIO2_1][4]  = -1,
    [GPIO2_1][5]  = -1,
    [GPIO2_2][0]  = GPIO_FUNC_SSP0,
    [GPIO2_2][1]  = -1,
    [GPIO2_2][2]  = -1,
    [GPIO2_2][3]  = -1,
    [GPIO2_2][4]  = -1,
    [GPIO2_2][5]  = -1,
    [GPIO2_3][0]  = GPIO_FUNC_SSP0,
    [GPIO2_3][1]  = -1,
    [GPIO2_3][2]  = -1,
    [GPIO2_3][3]  = -1,
    [GPIO2_3][4]  = -1,
    [GPIO2_3][5]  = -1,
    [GPIO2_4][0]  = GPIO_FUNC_SSP1,
    [GPIO2_4][1]  = -1,
    [GPIO2_4][2]  = -1,
    [GPIO2_4][3]  = -1,
    [GPIO2_4][4]  = -1,
    [GPIO2_4][5]  = -1,
    [GPIO2_5][0]  = GPIO_FUNC_SSP1,
    [GPIO2_5][1]  = -1,
    [GPIO2_5][2]  = -1,
    [GPIO2_5][3]  = -1,
    [GPIO2_5][4]  = -1,
    [GPIO2_5][5]  = -1,
    [GPIO2_6][0]  = GPIO_FUNC_SSP1,
    [GPIO2_6][1]  = -1,
    [GPIO2_6][2]  = -1,
    [GPIO2_6][3]  = -1,
    [GPIO2_6][4]  = -1,
    [GPIO2_6][5]  = -1,
    [GPIO2_7][0]  = GPIO_FUNC_SSP1,
    [GPIO2_7][1]  = -1,
    [GPIO2_7][2]  = -1,
    [GPIO2_7][3]  = -1,
    [GPIO2_7][4]  = -1,
    [GPIO2_7][5]  = -1,
    [GPIO2_8][0]  = GPIO_FUNC_USBVBUS,
    [GPIO2_8][1]  = -1,
    [GPIO2_8][2]  = -1,
    [GPIO2_8][3]  = -1,
    [GPIO2_8][4]  = -1,
    [GPIO2_8][5]  = -1,
    [GPIO2_9][0]  = GPIO_FUNC_USBVBUS,
    [GPIO2_9][1]  = -1,
    [GPIO2_9][2]  = -1,
    [GPIO2_9][3]  = -1,
    [GPIO2_9][4]  = -1,
    [GPIO2_9][5]  = -1,
    [GPIO2_10][0] = GPIO_FUNC_USBVBUS,
    [GPIO2_10][1] = -1,
    [GPIO2_10][2] = -1,
    [GPIO2_10][3] = -1,
    [GPIO2_10][4] = -1,
    [GPIO2_10][5] = -1,
//    [GPIO2_11][0] = ,	/* not used */
//    [GPIO2_11][1] = ,	/* not used */
//    [GPIO2_11][2] = ,	/* not used */
//    [GPIO2_11][3] = ,	/* not used */
//    [GPIO2_11][4] = ,	/* not used */
//    [GPIO2_11][5] = ,	/* not used */
//    [GPIO2_12][0] = ,	/* not used */
//    [GPIO2_12][1] = ,	/* not used */
//    [GPIO2_12][2] = ,	/* not used */
//    [GPIO2_12][3] = ,	/* not used */
//    [GPIO2_12][4] = ,	/* not used */
//    [GPIO2_12][5] = ,	/* not used */
//    [GPIO2_13][0] = ,	/* not used */
//    [GPIO2_13][1] = ,	/* not used */
//    [GPIO2_13][2] = ,	/* not used */
//    [GPIO2_13][3] = ,	/* not used */
//    [GPIO2_13][4] = ,	/* not used */
//    [GPIO2_13][5] = ,	/* not used */
//    [GPIO2_14][0] = ,	/* not used */
//    [GPIO2_14][1] = ,	/* not used */
//    [GPIO2_14][2] = ,	/* not used */
//    [GPIO2_14][3] = ,	/* not used */
//    [GPIO2_14][4] = ,	/* not used */
//    [GPIO2_14][5] = ,	/* not used */
//    [GPIO2_15][0] = ,	/* not used */
//    [GPIO2_15][1] = ,	/* not used */
//    [GPIO2_15][2] = ,	/* not used */
//    [GPIO2_15][3] = ,	/* not used */
//    [GPIO2_15][4] = ,	/* not used */
//    [GPIO2_15][5] = ,	/* not used */

    [GPIO3_0][0]  = GPIO_FUNC_SIO0,
    [GPIO3_0][1]  = -1,
    [GPIO3_0][2]  = -1,
    [GPIO3_0][3]  = -1,
    [GPIO3_0][4]  = -1,
    [GPIO3_0][5]  = -1,
    [GPIO3_1][0]  = GPIO_FUNC_SIO0,
    [GPIO3_1][1]  = -1,
    [GPIO3_1][2]  = -1,
    [GPIO3_1][3]  = -1,
    [GPIO3_1][4]  = -1,
    [GPIO3_1][5]  = -1,
    [GPIO3_2][0]  = GPIO_FUNC_SIO0,
    [GPIO3_2][1]  = -1,
    [GPIO3_2][2]  = -1,
    [GPIO3_2][3]  = -1,
    [GPIO3_2][4]  = -1,
    [GPIO3_2][5]  = -1,
    [GPIO3_3][0]  = GPIO_FUNC_SIO0,
    [GPIO3_3][1]  = -1,
    [GPIO3_3][2]  = -1,
    [GPIO3_3][3]  = -1,
    [GPIO3_3][4]  = -1,
    [GPIO3_3][5]  = -1,
    [GPIO3_4][0]  = GPIO_FUNC_SIO1,
    [GPIO3_4][1]  = -1,
    [GPIO3_4][2]  = -1,
    [GPIO3_4][3]  = -1,
    [GPIO3_4][4]  = -1,
    [GPIO3_4][5]  = -1,
    [GPIO3_5][0]  = GPIO_FUNC_SIO1,
    [GPIO3_5][1]  = -1,
    [GPIO3_5][2]  = -1,
    [GPIO3_5][3]  = -1,
    [GPIO3_5][4]  = -1,
    [GPIO3_5][5]  = -1,
    [GPIO3_6][0]  = GPIO_FUNC_SIO1,
    [GPIO3_6][1]  = -1,
    [GPIO3_6][2]  = -1,
    [GPIO3_6][3]  = -1,
    [GPIO3_6][4]  = -1,
    [GPIO3_6][5]  = -1,
    [GPIO3_7][0]  = GPIO_FUNC_SIO1,
    [GPIO3_7][1]  = -1,
    [GPIO3_7][2]  = -1,
    [GPIO3_7][3]  = -1,
    [GPIO3_7][4]  = -1,
    [GPIO3_7][5]  = -1,
    [GPIO3_8][0]  = GPIO_FUNC_SIO2,
    [GPIO3_8][1]  = -1,
    [GPIO3_8][2]  = -1,
    [GPIO3_8][3]  = -1,
    [GPIO3_8][4]  = -1,
    [GPIO3_8][5]  = -1,
    [GPIO3_9][0]  = GPIO_FUNC_SIO2,
    [GPIO3_9][1]  = -1,
    [GPIO3_9][2]  = -1,
    [GPIO3_9][3]  = -1,
    [GPIO3_9][4]  = -1,
    [GPIO3_9][5]  = -1,
    [GPIO3_10][0] = GPIO_FUNC_SIO2,
    [GPIO3_10][1] = -1,
    [GPIO3_10][2] = -1,
    [GPIO3_10][3] = -1,
    [GPIO3_10][4] = -1,
    [GPIO3_10][5] = -1,
    [GPIO3_11][0] = GPIO_FUNC_SIO2,
    [GPIO3_11][1] = -1,
    [GPIO3_11][2] = -1,
    [GPIO3_11][3] = -1,
    [GPIO3_11][4] = -1,
    [GPIO3_11][5] = -1,
    [GPIO3_12][0] = GPIO_FUNC_SIO3,
    [GPIO3_12][1] = -1,
    [GPIO3_12][2] = -1,
    [GPIO3_12][3] = -1,
    [GPIO3_12][4] = -1,
    [GPIO3_12][5] = -1,
    [GPIO3_13][0] = GPIO_FUNC_SIO3,
    [GPIO3_13][1] = -1,
    [GPIO3_13][2] = -1,
    [GPIO3_13][3] = -1,
    [GPIO3_13][4] = -1,
    [GPIO3_13][5] = -1,
    [GPIO3_14][0] = GPIO_FUNC_SIO3,
    [GPIO3_14][1] = -1,
    [GPIO3_14][2] = -1,
    [GPIO3_14][3] = -1,
    [GPIO3_14][4] = -1,
    [GPIO3_14][5] = -1,
    [GPIO3_15][0] = GPIO_FUNC_SIO3,
    [GPIO3_15][1] = -1,
    [GPIO3_15][2] = -1,
    [GPIO3_15][3] = -1,
    [GPIO3_15][4] = -1,
    [GPIO3_15][5] = -1,

    [GPIO4_0][0]  = GPIO_FUNC_EXIO,
    [GPIO4_0][1]  = GPIO_FUNC_CF,
    [GPIO4_0][2]  = GPIO_FUNC_ATA,
    [GPIO4_0][3]  = GPIO_FUNC_PERIHZ,
    [GPIO4_0][4]  = GPIO_FUNC_PERIPU,
    [GPIO4_0][5]  = GPIO_FUNC_PERIPD,
    [GPIO4_1][0]  = GPIO_FUNC_EXIO,
    [GPIO4_1][1]  = GPIO_FUNC_CF,
    [GPIO4_1][2]  = GPIO_FUNC_ATA,
    [GPIO4_1][3]  = GPIO_FUNC_PERIHZ,
    [GPIO4_1][4]  = GPIO_FUNC_PERIPU,
    [GPIO4_1][5]  = GPIO_FUNC_PERIPD,
    [GPIO4_2][0]  = GPIO_FUNC_EXIO,
    [GPIO4_2][1]  = GPIO_FUNC_CF,
    [GPIO4_2][2]  = GPIO_FUNC_ATA,
    [GPIO4_2][3]  = GPIO_FUNC_PERIHZ,
    [GPIO4_2][4]  = GPIO_FUNC_PERIPU,
    [GPIO4_2][5]  = GPIO_FUNC_PERIPD,
    [GPIO4_3][0]  = GPIO_FUNC_EXIO,
    [GPIO4_3][1]  = GPIO_FUNC_CF,
    [GPIO4_3][2]  = -1,
    [GPIO4_3][3]  = -1,
    [GPIO4_3][4]  = -1,
    [GPIO4_3][5]  = -1,
    [GPIO4_4][0]  = GPIO_FUNC_EXIO,
    [GPIO4_4][1]  = GPIO_FUNC_CF,
    [GPIO4_4][2]  = -1,
    [GPIO4_4][3]  = -1,
    [GPIO4_4][4]  = -1,
    [GPIO4_4][5]  = -1,
    [GPIO4_5][0]  = GPIO_FUNC_EXIO,
    [GPIO4_5][1]  = GPIO_FUNC_CF,
    [GPIO4_5][2]  = -1,
    [GPIO4_5][3]  = -1,
    [GPIO4_5][4]  = -1,
    [GPIO4_5][5]  = -1,
    [GPIO4_6][0]  = GPIO_FUNC_EXIO,
    [GPIO4_6][1]  = GPIO_FUNC_CF,
    [GPIO4_6][2]  = -1,
    [GPIO4_6][3]  = -1,
    [GPIO4_6][4]  = -1,
    [GPIO4_6][5]  = -1,
    [GPIO4_7][0]  = GPIO_FUNC_EXIO,
    [GPIO4_7][1]  = GPIO_FUNC_CF,
    [GPIO4_7][2]  = -1,
    [GPIO4_7][3]  = -1,
    [GPIO4_7][4]  = -1,
    [GPIO4_7][5]  = -1,
    [GPIO4_8][0]  = GPIO_FUNC_EXIO,
    [GPIO4_8][1]  = GPIO_FUNC_CF,
    [GPIO4_8][2]  = -1,
    [GPIO4_8][3]  = -1,
    [GPIO4_8][4]  = -1,
    [GPIO4_8][5]  = -1,
    [GPIO4_9][0]  = GPIO_FUNC_EXIO,
    [GPIO4_9][1]  = GPIO_FUNC_CF,
    [GPIO4_9][2]  = -1,
    [GPIO4_9][3]  = -1,
    [GPIO4_9][4]  = -1,
    [GPIO4_9][5]  = -1,
    [GPIO4_10][0] = GPIO_FUNC_EXIO,
    [GPIO4_10][1] = GPIO_FUNC_CF,
    [GPIO4_10][2] = -1,
    [GPIO4_10][3] = -1,
    [GPIO4_10][4] = -1,
    [GPIO4_10][5] = -1,
    [GPIO4_11][0] = GPIO_FUNC_EXIO,
    [GPIO4_11][1] = -1,
    [GPIO4_11][2] = -1,
    [GPIO4_11][3] = -1,
    [GPIO4_11][4] = -1,
    [GPIO4_11][5] = -1,
    [GPIO4_12][0] = GPIO_FUNC_EXIO,
    [GPIO4_12][1] = -1,
    [GPIO4_12][2] = -1,
    [GPIO4_12][3] = -1,
    [GPIO4_12][4] = -1,
    [GPIO4_12][5] = -1,
    [GPIO4_13][0] = GPIO_FUNC_EXIO,
    [GPIO4_13][1] = -1,
    [GPIO4_13][2] = -1,
    [GPIO4_13][3] = -1,
    [GPIO4_13][4] = -1,
    [GPIO4_13][5] = -1,
    [GPIO4_14][0] = GPIO_FUNC_EXIO,
    [GPIO4_14][1] = -1,
    [GPIO4_14][2] = -1,
    [GPIO4_14][3] = -1,
    [GPIO4_14][4] = -1,
    [GPIO4_14][5] = -1,
    [GPIO4_15][0] = GPIO_FUNC_EXIO,
    [GPIO4_15][1] = -1,
    [GPIO4_15][2] = -1,
    [GPIO4_15][3] = -1,
    [GPIO4_15][4] = -1,
    [GPIO4_15][5] = -1,

    [GPIO5_0][0]  = GPIO_FUNC_EXIO,
    [GPIO5_0][1]  = GPIO_FUNC_CF,
    [GPIO5_0][2]  = GPIO_FUNC_ATA,
    [GPIO5_0][3]  = GPIO_FUNC_PERIHZ,
    [GPIO5_0][4]  = GPIO_FUNC_PERIPU,
    [GPIO5_0][5]  = GPIO_FUNC_PERIPD,
    [GPIO5_1][0]  = GPIO_FUNC_EXIO,
    [GPIO5_1][1]  = GPIO_FUNC_CF,
    [GPIO5_1][2]  = GPIO_FUNC_ATA,
    [GPIO5_1][3]  = GPIO_FUNC_PERIHZ,
    [GPIO5_1][4]  = GPIO_FUNC_PERIPU,
    [GPIO5_1][5]  = GPIO_FUNC_PERIPD,
    [GPIO5_2][0]  = GPIO_FUNC_EXIO,
    [GPIO5_2][1]  = GPIO_FUNC_CF,
    [GPIO5_2][2]  = GPIO_FUNC_ATA,
    [GPIO5_2][3]  = GPIO_FUNC_PERIHZ,
    [GPIO5_2][4]  = GPIO_FUNC_PERIPU,
    [GPIO5_2][5]  = GPIO_FUNC_PERIPD,
    [GPIO5_3][0]  = GPIO_FUNC_EXIO,
    [GPIO5_3][1]  = GPIO_FUNC_CF,
    [GPIO5_3][2]  = GPIO_FUNC_ATA,
    [GPIO5_3][3]  = GPIO_FUNC_PERIHZ,
    [GPIO5_3][4]  = GPIO_FUNC_PERIPU,
    [GPIO5_3][5]  = GPIO_FUNC_PERIPD,
    [GPIO5_4][0]  = GPIO_FUNC_EXIO,
    [GPIO5_4][1]  = GPIO_FUNC_CF,
    [GPIO5_4][2]  = GPIO_FUNC_ATA,
    [GPIO5_4][3]  = GPIO_FUNC_PERIHZ,
    [GPIO5_4][4]  = GPIO_FUNC_PERIPU,
    [GPIO5_4][5]  = GPIO_FUNC_PERIPD,
    [GPIO5_5][0]  = GPIO_FUNC_EXIO,
    [GPIO5_5][1]  = GPIO_FUNC_CF,
    [GPIO5_5][2]  = GPIO_FUNC_ATA,
    [GPIO5_5][3]  = GPIO_FUNC_PERIHZ,
    [GPIO5_5][4]  = GPIO_FUNC_PERIPU,
    [GPIO5_5][5]  = GPIO_FUNC_PERIPD,
    [GPIO5_6][0]  = GPIO_FUNC_EXIO,
    [GPIO5_6][1]  = GPIO_FUNC_CF,
    [GPIO5_6][2]  = GPIO_FUNC_ATA,
    [GPIO5_6][3]  = GPIO_FUNC_PERIHZ,
    [GPIO5_6][4]  = GPIO_FUNC_PERIPU,
    [GPIO5_6][5]  = GPIO_FUNC_PERIPD,
    [GPIO5_7][0]  = GPIO_FUNC_EXIO,
    [GPIO5_7][1]  = GPIO_FUNC_CF,
    [GPIO5_7][2]  = GPIO_FUNC_ATA,
    [GPIO5_7][3]  = GPIO_FUNC_PERIHZ,
    [GPIO5_7][4]  = GPIO_FUNC_PERIPU,
    [GPIO5_7][5]  = GPIO_FUNC_PERIPD,
    [GPIO5_8][0]  = GPIO_FUNC_EXIO,
    [GPIO5_8][1]  = GPIO_FUNC_CF,
    [GPIO5_8][2]  = GPIO_FUNC_ATA,
    [GPIO5_8][3]  = GPIO_FUNC_PERIHZ,
    [GPIO5_8][4]  = GPIO_FUNC_PERIPU,
    [GPIO5_8][5]  = GPIO_FUNC_PERIPD,
    [GPIO5_9][0]  = GPIO_FUNC_EXIO,
    [GPIO5_9][1]  = GPIO_FUNC_CF,
    [GPIO5_9][2]  = GPIO_FUNC_ATA,
    [GPIO5_9][3]  = GPIO_FUNC_PERIHZ,
    [GPIO5_9][4]  = GPIO_FUNC_PERIPU,
    [GPIO5_9][5]  = GPIO_FUNC_PERIPD,
    [GPIO5_10][0] = GPIO_FUNC_EXIO,
    [GPIO5_10][1] = GPIO_FUNC_CF,
    [GPIO5_10][2] = GPIO_FUNC_ATA,
    [GPIO5_10][3] = GPIO_FUNC_PERIHZ,
    [GPIO5_10][4] = GPIO_FUNC_PERIPU,
    [GPIO5_10][5] = GPIO_FUNC_PERIPD,
    [GPIO5_11][0] = GPIO_FUNC_EXIO,
    [GPIO5_11][1] = GPIO_FUNC_CF,
    [GPIO5_11][2] = GPIO_FUNC_ATA,
    [GPIO5_11][3] = GPIO_FUNC_PERIHZ,
    [GPIO5_11][4] = GPIO_FUNC_PERIPU,
    [GPIO5_11][5] = GPIO_FUNC_PERIPD,
    [GPIO5_12][0] = GPIO_FUNC_EXIO,
    [GPIO5_12][1] = GPIO_FUNC_CF,
    [GPIO5_12][2] = GPIO_FUNC_ATA,
    [GPIO5_12][3] = GPIO_FUNC_PERIHZ,
    [GPIO5_12][4] = GPIO_FUNC_PERIPU,
    [GPIO5_12][5] = GPIO_FUNC_PERIPD,
    [GPIO5_13][0] = GPIO_FUNC_EXIO,
    [GPIO5_13][1] = GPIO_FUNC_CF,
    [GPIO5_13][2] = GPIO_FUNC_ATA,
    [GPIO5_13][3] = GPIO_FUNC_PERIHZ,
    [GPIO5_13][4] = GPIO_FUNC_PERIPU,
    [GPIO5_13][5] = GPIO_FUNC_PERIPD,
    [GPIO5_14][0] = GPIO_FUNC_EXIO,
    [GPIO5_14][1] = GPIO_FUNC_CF,
    [GPIO5_14][2] = GPIO_FUNC_ATA,
    [GPIO5_14][3] = GPIO_FUNC_PERIHZ,
    [GPIO5_14][4] = GPIO_FUNC_PERIPU,
    [GPIO5_14][5] = GPIO_FUNC_PERIPD,
    [GPIO5_15][0] = GPIO_FUNC_EXIO,
    [GPIO5_15][1] = GPIO_FUNC_CF,
    [GPIO5_15][2] = GPIO_FUNC_ATA,
    [GPIO5_15][3] = GPIO_FUNC_PERIHZ,
    [GPIO5_15][4] = GPIO_FUNC_PERIPU,
    [GPIO5_15][5] = GPIO_FUNC_PERIPD,

    [GPIO6_0][0]  = GPIO_FUNC_CF,
    [GPIO6_0][1]  = GPIO_FUNC_ATA,
    [GPIO6_0][2]  = GPIO_FUNC_PERIHZ,
    [GPIO6_0][3]  = GPIO_FUNC_PERIPU,
    [GPIO6_0][4]  = GPIO_FUNC_PERIPD,
    [GPIO6_0][5]  = -1,
    [GPIO6_1][0]  = GPIO_FUNC_CF,
    [GPIO6_1][1]  = GPIO_FUNC_ATA,
    [GPIO6_1][2]  = GPIO_FUNC_PERIHZ,
    [GPIO6_1][3]  = GPIO_FUNC_PERIPU,
    [GPIO6_1][4]  = GPIO_FUNC_PERIPD,
    [GPIO6_1][5]  = -1,
    [GPIO6_2][0]  = GPIO_FUNC_CF,
    [GPIO6_2][1]  = -1,
    [GPIO6_2][2]  = -1,
    [GPIO6_2][3]  = -1,
    [GPIO6_2][4]  = -1,
    [GPIO6_2][5]  = -1,
    [GPIO6_3][0]  = GPIO_FUNC_CF,
    [GPIO6_3][1]  = -1,
    [GPIO6_3][2]  = -1,
    [GPIO6_3][3]  = -1,
    [GPIO6_3][4]  = -1,
    [GPIO6_3][5]  = -1,
    [GPIO6_4][0]  = GPIO_FUNC_EXIO,
    [GPIO6_4][1]  = GPIO_FUNC_CF,
    [GPIO6_4][2]  = GPIO_FUNC_ATA,
    [GPIO6_4][3]  = GPIO_FUNC_PERIHZ,
    [GPIO6_4][4]  = GPIO_FUNC_PERIPU,
    [GPIO6_4][5]  = GPIO_FUNC_PERIPD,
    [GPIO6_5][0]  = GPIO_FUNC_EXIO,
    [GPIO6_5][1]  = GPIO_FUNC_CF,
    [GPIO6_5][2]  = GPIO_FUNC_ATA,
    [GPIO6_5][3]  = GPIO_FUNC_PERIHZ,
    [GPIO6_5][4]  = GPIO_FUNC_PERIPU,
    [GPIO6_5][5]  = GPIO_FUNC_PERIPD,
    [GPIO6_6][0]  = GPIO_FUNC_CF,
    [GPIO6_6][1]  = GPIO_FUNC_ATA,
    [GPIO6_6][2]  = GPIO_FUNC_PERIHZ,
    [GPIO6_6][3]  = GPIO_FUNC_PERIPU,
    [GPIO6_6][4]  = GPIO_FUNC_PERIPD,
    [GPIO6_6][5]  = -1,
    [GPIO6_7][0]  = GPIO_FUNC_CF,
    [GPIO6_7][1]  = GPIO_FUNC_ATA,
    [GPIO6_7][2]  = GPIO_FUNC_PERIHZ,
    [GPIO6_7][3]  = GPIO_FUNC_PERIPU,
    [GPIO6_7][4]  = GPIO_FUNC_PERIPD,
    [GPIO6_7][5]  = -1,
    [GPIO6_8][0]  = GPIO_FUNC_CF,
    [GPIO6_8][1]  = GPIO_FUNC_ATA,
    [GPIO6_8][2]  = GPIO_FUNC_PERIHZ,
    [GPIO6_8][3]  = GPIO_FUNC_PERIPU,
    [GPIO6_8][4]  = GPIO_FUNC_PERIPD,
    [GPIO6_8][5]  = -1,
    [GPIO6_9][0]  = GPIO_FUNC_CF,
    [GPIO6_9][1]  = GPIO_FUNC_ATA,
    [GPIO6_9][2]  = GPIO_FUNC_PERIHZ,
    [GPIO6_9][3]  = GPIO_FUNC_PERIPU,
    [GPIO6_9][4]  = GPIO_FUNC_PERIPD,
    [GPIO6_9][5]  = -1,
    [GPIO6_10][0] = GPIO_FUNC_CF,
    [GPIO6_10][1] = GPIO_FUNC_ATA,
    [GPIO6_10][2] = GPIO_FUNC_PERIHZ,
    [GPIO6_10][3] = GPIO_FUNC_PERIPU,
    [GPIO6_10][4] = GPIO_FUNC_PERIPD,
    [GPIO6_10][5] = -1,
    [GPIO6_11][0] = GPIO_FUNC_CF,
    [GPIO6_11][1] = GPIO_FUNC_ATA,
    [GPIO6_11][2] = GPIO_FUNC_PERIHZ,
    [GPIO6_11][3] = GPIO_FUNC_PERIPU,
    [GPIO6_11][4] = GPIO_FUNC_PERIPD,
    [GPIO6_11][5] = -1,
    [GPIO6_12][0] = GPIO_FUNC_CF,
    [GPIO6_12][1] = GPIO_FUNC_ATA,
    [GPIO6_12][2] = -1,
    [GPIO6_12][3] = -1,
    [GPIO6_12][4] = -1,
    [GPIO6_12][5] = -1,
    [GPIO6_13][0] = GPIO_FUNC_CF,
    [GPIO6_13][1] = GPIO_FUNC_ATA,
    [GPIO6_13][2] = GPIO_FUNC_PERIHZ,
    [GPIO6_13][3] = GPIO_FUNC_PERIPU,
    [GPIO6_13][4] = GPIO_FUNC_PERIPD,
    [GPIO6_13][5] = -1,
    [GPIO6_14][0] = GPIO_FUNC_EXIO,
    [GPIO6_14][1] = -1,
    [GPIO6_14][2] = -1,
    [GPIO6_14][3] = -1,
    [GPIO6_14][4] = -1,
    [GPIO6_14][5] = -1,
    [GPIO6_15][0] = GPIO_FUNC_CF,
    [GPIO6_15][1] = -1,
    [GPIO6_15][2] = -1,
    [GPIO6_15][3] = -1,
    [GPIO6_15][4] = -1,
    [GPIO6_15][5] = -1,

    [GPIO7_0][0]  = GPIO_FUNC_EXIO,
    [GPIO7_0][1]  = -1,
    [GPIO7_0][2]  = -1,
    [GPIO7_0][3]  = -1,
    [GPIO7_0][4]  = -1,
    [GPIO7_0][5]  = -1,
    [GPIO7_1][0]  = GPIO_FUNC_EXIO,
    [GPIO7_1][1]  = -1,
    [GPIO7_1][2]  = -1,
    [GPIO7_1][3]  = -1,
    [GPIO7_1][4]  = -1,
    [GPIO7_1][5]  = -1,
    [GPIO7_2][0]  = GPIO_FUNC_EXIO,
    [GPIO7_2][1]  = -1,
    [GPIO7_2][2]  = -1,
    [GPIO7_2][3]  = -1,
    [GPIO7_2][4]  = -1,
    [GPIO7_2][5]  = -1,
    [GPIO7_3][0]  = GPIO_FUNC_EXIO,
    [GPIO7_3][1]  = -1,
    [GPIO7_3][2]  = -1,
    [GPIO7_3][3]  = -1,
    [GPIO7_3][4]  = -1,
    [GPIO7_3][5]  = -1,
    [GPIO7_4][0]  = GPIO_FUNC_EXIO,
    [GPIO7_4][1]  = -1,
    [GPIO7_4][2]  = -1,
    [GPIO7_4][3]  = -1,
    [GPIO7_4][4]  = -1,
    [GPIO7_4][5]  = -1,
    [GPIO7_5][0]  = GPIO_FUNC_EXIO,
    [GPIO7_5][1]  = -1,
    [GPIO7_5][2]  = -1,
    [GPIO7_5][3]  = -1,
    [GPIO7_5][4]  = -1,
    [GPIO7_5][5]  = -1,
//    [GPIO7_6][0]  = ,	/* not used */
//    [GPIO7_6][1]  = ,	/* not used */
//    [GPIO7_6][2]  = ,	/* not used */
//    [GPIO7_6][3]  = ,	/* not used */
//    [GPIO7_6][4]  = ,	/* not used */
//    [GPIO7_6][5]  = ,	/* not used */
//    [GPIO7_7][0]  = ,	/* not used */
//    [GPIO7_7][1]  = ,	/* not used */
//    [GPIO7_7][2]  = ,	/* not used */
//    [GPIO7_7][3]  = ,	/* not used */
//    [GPIO7_7][4]  = ,	/* not used */
//    [GPIO7_7][5]  = ,	/* not used */
//    [GPIO7_8][0]  = ,	/* not used */
//    [GPIO7_8][1]  = ,	/* not used */
//    [GPIO7_8][2]  = ,	/* not used */
//    [GPIO7_8][3]  = ,	/* not used */
//    [GPIO7_8][4]  = ,	/* not used */
//    [GPIO7_8][5]  = ,	/* not used */
//    [GPIO7_9][0]  = ,	/* not used */
//    [GPIO7_9][1]  = ,	/* not used */
//    [GPIO7_9][2]  = ,	/* not used */
//    [GPIO7_9][3]  = ,	/* not used */
//    [GPIO7_9][4]  = ,	/* not used */
//    [GPIO7_9][5]  = ,	/* not used */
//    [GPIO7_10][0] = ,	/* not used */
//    [GPIO7_10][1] = ,	/* not used */
//    [GPIO7_10][2] = ,	/* not used */
//    [GPIO7_10][3] = ,	/* not used */
//    [GPIO7_10][4] = ,	/* not used */
//    [GPIO7_10][5] = ,	/* not used */
//    [GPIO7_11][0] = ,	/* not used */
//    [GPIO7_11][1] = ,	/* not used */
//    [GPIO7_11][2] = ,	/* not used */
//    [GPIO7_11][3] = ,	/* not used */
//    [GPIO7_11][4] = ,	/* not used */
//    [GPIO7_11][5] = ,	/* not used */
//    [GPIO7_12][0] = ,	/* not used */
//    [GPIO7_12][1] = ,	/* not used */
//    [GPIO7_12][2] = ,	/* not used */
//    [GPIO7_12][3] = ,	/* not used */
//    [GPIO7_12][4] = ,	/* not used */
//    [GPIO7_12][5] = ,	/* not used */
//    [GPIO7_13][0] = ,	/* not used */
//    [GPIO7_13][1] = ,	/* not used */
//    [GPIO7_13][2] = ,	/* not used */
//    [GPIO7_13][3] = ,	/* not used */
//    [GPIO7_13][4] = ,	/* not used */
//    [GPIO7_13][5] = ,	/* not used */
//    [GPIO7_14][0] = ,	/* not used */
//    [GPIO7_14][1] = ,	/* not used */
//    [GPIO7_14][2] = ,	/* not used */
//    [GPIO7_14][3] = ,	/* not used */
//    [GPIO7_14][4] = ,	/* not used */
//    [GPIO7_14][5] = ,	/* not used */
//    [GPIO7_15][0] = ,	/* not used */
//    [GPIO7_15][1] = ,	/* not used */
//    [GPIO7_15][2] = ,	/* not used */
//    [GPIO7_15][3] = ,	/* not used */
//    [GPIO7_15][4] = ,	/* not used */
//    [GPIO7_15][5] = ,	/* not used */

    [GPIO8_0][0]  = GPIO_FUNC_LCD,
    [GPIO8_0][1]  = -1,
    [GPIO8_0][2]  = -1,
    [GPIO8_0][3]  = -1,
    [GPIO8_0][4]  = -1,
    [GPIO8_0][5]  = -1,
    [GPIO8_1][0]  = GPIO_FUNC_LCD,
    [GPIO8_1][1]  = -1,
    [GPIO8_1][2]  = -1,
    [GPIO8_1][3]  = -1,
    [GPIO8_1][4]  = -1,
    [GPIO8_1][5]  = -1,
    [GPIO8_2][0]  = GPIO_FUNC_LCD,
    [GPIO8_2][1]  = -1,
    [GPIO8_2][2]  = -1,
    [GPIO8_2][3]  = -1,
    [GPIO8_2][4]  = -1,
    [GPIO8_2][5]  = -1,
    [GPIO8_3][0]  = GPIO_FUNC_LCD,
    [GPIO8_3][1]  = -1,
    [GPIO8_3][2]  = -1,
    [GPIO8_3][3]  = -1,
    [GPIO8_3][4]  = -1,
    [GPIO8_3][5]  = -1,
    [GPIO8_4][0]  = GPIO_FUNC_LCD,
    [GPIO8_4][1]  = -1,
    [GPIO8_4][2]  = -1,
    [GPIO8_4][3]  = -1,
    [GPIO8_4][4]  = -1,
    [GPIO8_4][5]  = -1,
    [GPIO8_5][0]  = GPIO_FUNC_LCD,
    [GPIO8_5][1]  = -1,
    [GPIO8_5][2]  = -1,
    [GPIO8_5][3]  = -1,
    [GPIO8_5][4]  = -1,
    [GPIO8_5][5]  = -1,
    [GPIO8_6][0]  = GPIO_FUNC_LCD,
    [GPIO8_6][1]  = -1,
    [GPIO8_6][2]  = -1,
    [GPIO8_6][3]  = -1,
    [GPIO8_6][4]  = -1,
    [GPIO8_6][5]  = -1,
    [GPIO8_7][0]  = GPIO_FUNC_LCD,
    [GPIO8_7][1]  = -1,
    [GPIO8_7][2]  = -1,
    [GPIO8_7][3]  = -1,
    [GPIO8_7][4]  = -1,
    [GPIO8_7][5]  = -1,
    [GPIO8_8][0]  = GPIO_FUNC_LCD,
    [GPIO8_8][1]  = -1,
    [GPIO8_8][2]  = -1,
    [GPIO8_8][3]  = -1,
    [GPIO8_8][4]  = -1,
    [GPIO8_8][5]  = -1,
    [GPIO8_9][0]  = GPIO_FUNC_LCD,
    [GPIO8_9][1]  = -1,
    [GPIO8_9][2]  = -1,
    [GPIO8_9][3]  = -1,
    [GPIO8_9][4]  = -1,
    [GPIO8_9][5]  = -1,
    [GPIO8_10][0] = GPIO_FUNC_LCD,
    [GPIO8_10][1] = -1,
    [GPIO8_10][2] = -1,
    [GPIO8_10][3] = -1,
    [GPIO8_10][4] = -1,
    [GPIO8_10][5] = -1,
    [GPIO8_11][0] = GPIO_FUNC_LCD,
    [GPIO8_11][1] = -1,
    [GPIO8_11][2] = -1,
    [GPIO8_11][3] = -1,
    [GPIO8_11][4] = -1,
    [GPIO8_11][5] = -1,
    [GPIO8_12][0] = GPIO_FUNC_LCD,
    [GPIO8_12][1] = -1,
    [GPIO8_12][2] = -1,
    [GPIO8_12][3] = -1,
    [GPIO8_12][4] = -1,
    [GPIO8_12][5] = -1,
    [GPIO8_13][0] = GPIO_FUNC_LCD,
    [GPIO8_13][1] = -1,
    [GPIO8_13][2] = -1,
    [GPIO8_13][3] = -1,
    [GPIO8_13][4] = -1,
    [GPIO8_13][5] = -1,
    [GPIO8_14][0] = GPIO_FUNC_LCD,
    [GPIO8_14][1] = -1,
    [GPIO8_14][2] = -1,
    [GPIO8_14][3] = -1,
    [GPIO8_14][4] = -1,
    [GPIO8_14][5] = -1,
    [GPIO8_15][0] = GPIO_FUNC_LCD,
    [GPIO8_15][1] = -1,
    [GPIO8_15][2] = -1,
    [GPIO8_15][3] = -1,
    [GPIO8_15][4] = -1,
    [GPIO8_15][5] = -1,

    [GPIO9_0][0]  = GPIO_FUNC_LCD,
    [GPIO9_0][1]  = -1,
    [GPIO9_0][2]  = -1,
    [GPIO9_0][3]  = -1,
    [GPIO9_0][4]  = -1,
    [GPIO9_0][5]  = -1,
    [GPIO9_1][0]  = GPIO_FUNC_LCD,
    [GPIO9_1][1]  = -1,
    [GPIO9_1][2]  = -1,
    [GPIO9_1][3]  = -1,
    [GPIO9_1][4]  = -1,
    [GPIO9_1][5]  = -1,
    [GPIO9_2][0]  = GPIO_FUNC_LCD,
    [GPIO9_2][1]  = -1,
    [GPIO9_2][2]  = -1,
    [GPIO9_2][3]  = -1,
    [GPIO9_2][4]  = -1,
    [GPIO9_2][5]  = -1,
    [GPIO9_3][0]  = GPIO_FUNC_LCD,
    [GPIO9_3][1]  = -1,
    [GPIO9_3][2]  = -1,
    [GPIO9_3][3]  = -1,
    [GPIO9_3][4]  = -1,
    [GPIO9_3][5]  = -1,
    [GPIO9_4][0]  = GPIO_FUNC_LCD,
    [GPIO9_4][1]  = -1,
    [GPIO9_4][2]  = -1,
    [GPIO9_4][3]  = -1,
    [GPIO9_4][4]  = -1,
    [GPIO9_4][5]  = -1,
    [GPIO9_5][0]  = GPIO_FUNC_LCD,
    [GPIO9_5][1]  = -1,
    [GPIO9_5][2]  = -1,
    [GPIO9_5][3]  = -1,
    [GPIO9_5][4]  = -1,
    [GPIO9_5][5]  = -1,
    [GPIO9_6][0]  = GPIO_FUNC_LCD,
    [GPIO9_6][1]  = -1,
    [GPIO9_6][2]  = -1,
    [GPIO9_6][3]  = -1,
    [GPIO9_6][4]  = -1,
    [GPIO9_6][5]  = -1,
    [GPIO9_7][0]  = GPIO_FUNC_LCD,
    [GPIO9_7][1]  = -1,
    [GPIO9_7][2]  = -1,
    [GPIO9_7][3]  = -1,
    [GPIO9_7][4]  = -1,
    [GPIO9_7][5]  = -1,
    [GPIO9_8][0]  = GPIO_FUNC_LCD,
    [GPIO9_8][1]  = -1,
    [GPIO9_8][2]  = -1,
    [GPIO9_8][3]  = -1,
    [GPIO9_8][4]  = -1,
    [GPIO9_8][5]  = -1,
    [GPIO9_9][0]  = GPIO_FUNC_LCD,
    [GPIO9_9][1]  = -1,
    [GPIO9_9][2]  = -1,
    [GPIO9_9][3]  = -1,
    [GPIO9_9][4]  = -1,
    [GPIO9_9][5]  = -1,
    [GPIO9_10][0] = GPIO_FUNC_LCD,
    [GPIO9_10][1] = -1,
    [GPIO9_10][2] = -1,
    [GPIO9_10][3] = -1,
    [GPIO9_10][4] = -1,
    [GPIO9_10][5] = -1,
    [GPIO9_11][0] = GPIO_FUNC_CODEC,
    [GPIO9_11][1] = -1,
    [GPIO9_11][2] = -1,
    [GPIO9_11][3] = -1,
    [GPIO9_11][4] = -1,
    [GPIO9_11][5] = -1,
//    [GPIO9_12][0] = ,	/* not used */
//    [GPIO9_12][1] = ,	/* not used */
//    [GPIO9_12][2] = ,	/* not used */
//    [GPIO9_12][3] = ,	/* not used */
//    [GPIO9_12][4] = ,	/* not used */
//    [GPIO9_12][5] = ,	/* not used */
//    [GPIO9_13][0] = ,	/* not used */
//    [GPIO9_13][1] = ,	/* not used */
//    [GPIO9_13][2] = ,	/* not used */
//    [GPIO9_13][3] = ,	/* not used */
//    [GPIO9_13][4] = ,	/* not used */
//    [GPIO9_13][5] = ,	/* not used */
//    [GPIO9_14][0] = ,	/* not used */
//    [GPIO9_14][1] = ,	/* not used */
//    [GPIO9_14][2] = ,	/* not used */
//    [GPIO9_14][3] = ,	/* not used */
//    [GPIO9_14][4] = ,	/* not used */
//    [GPIO9_14][5] = ,	/* not used */
//    [GPIO9_15][0] = ,	/* not used */
//    [GPIO9_15][1] = ,	/* not used */
//    [GPIO9_15][2] = ,	/* not used */
//    [GPIO9_15][3] = ,	/* not used */
//    [GPIO9_15][4] = ,	/* not used */
//    [GPIO9_15][5] = ,	/* not used */

    [GPIO10_0][0]  = GPIO_FUNC_CODEC,
    [GPIO10_0][1]  = -1,
    [GPIO10_0][2]  = -1,
    [GPIO10_0][3]  = -1,
    [GPIO10_0][4]  = -1,
    [GPIO10_0][5]  = -1,
    [GPIO10_1][0]  = GPIO_FUNC_CODEC,
    [GPIO10_1][1]  = -1,
    [GPIO10_1][2]  = -1,
    [GPIO10_1][3]  = -1,
    [GPIO10_1][4]  = -1,
    [GPIO10_1][5]  = -1,
//    [GPIO10_2][0]  = ,	/* not used */
//    [GPIO10_2][1]  = ,	/* not used */
//    [GPIO10_2][2]  = ,	/* not used */
//    [GPIO10_2][3]  = ,	/* not used */
//    [GPIO10_2][4]  = ,	/* not used */
//    [GPIO10_2][5]  = ,	/* not used */
    /* __adjust_nr_bit() for GPIO10_3 - GPIO10_9 */
    [GPIO10_3][0]  = GPIO_FUNC_CK,
    [GPIO10_3][1]  = -1,
    [GPIO10_3][2]  = -1,
    [GPIO10_3][3]  = -1,
    [GPIO10_3][4]  = -1,
    [GPIO10_3][5]  = -1,
    [GPIO10_4][0]  = GPIO_FUNC_CK,
    [GPIO10_4][1]  = -1,
    [GPIO10_4][2]  = -1,
    [GPIO10_4][3]  = -1,
    [GPIO10_4][4]  = -1,
    [GPIO10_4][5]  = -1,
    [GPIO10_5][0]  = GPIO_FUNC_SD,
    [GPIO10_5][1]  = -1,
    [GPIO10_5][2]  = -1,
    [GPIO10_5][3]  = -1,
    [GPIO10_5][4]  = -1,
    [GPIO10_5][5]  = -1,
    [GPIO10_6][0]  = GPIO_FUNC_SD,
    [GPIO10_6][1]  = -1,
    [GPIO10_6][2]  = -1,
    [GPIO10_6][3]  = -1,
    [GPIO10_6][4]  = -1,
    [GPIO10_6][5]  = -1,
    [GPIO10_7][0]  = GPIO_FUNC_SD,
    [GPIO10_7][1]  = -1,
    [GPIO10_7][2]  = -1,
    [GPIO10_7][3]  = -1,
    [GPIO10_7][4]  = -1,
    [GPIO10_7][5]  = -1,
    [GPIO10_8][0]  = GPIO_FUNC_SD,
    [GPIO10_8][1]  = -1,
    [GPIO10_8][2]  = -1,
    [GPIO10_8][3]  = -1,
    [GPIO10_8][4]  = -1,
    [GPIO10_8][5]  = -1,
    [GPIO10_9][0]  = GPIO_FUNC_SD,
    [GPIO10_9][1]  = -1,
    [GPIO10_9][2]  = -1,
    [GPIO10_9][3]  = -1,
    [GPIO10_9][4]  = -1,
    [GPIO10_9][5]  = -1,
//    [GPIO10_10][0] = ,	/* not used */
//    [GPIO10_10][1] = ,	/* not used */
//    [GPIO10_10][2] = ,	/* not used */
//    [GPIO10_10][3] = ,	/* not used */
//    [GPIO10_10][4] = ,	/* not used */
//    [GPIO10_10][5] = ,	/* not used */
    /* __adjust_nr_bit() for GPIO10_11 - GPIO10_15 */
    [GPIO10_11][0] = GPIO_FUNC_CAMPERI,
    [GPIO10_11][1] = -1,
    [GPIO10_11][2] = -1,
    [GPIO10_11][3] = -1,
    [GPIO10_11][4] = -1,
    [GPIO10_11][5] = -1,
    [GPIO10_12][0] = GPIO_FUNC_CAMPERI,
    [GPIO10_12][1] = -1,
    [GPIO10_12][2] = -1,
    [GPIO10_12][3] = -1,
    [GPIO10_12][4] = -1,
    [GPIO10_12][5] = -1,
    [GPIO10_13][0] = GPIO_FUNC_CAMPERI,
    [GPIO10_13][1] = -1,
    [GPIO10_13][2] = -1,
    [GPIO10_13][3] = -1,
    [GPIO10_13][4] = -1,
    [GPIO10_13][5] = -1,
    [GPIO10_14][0] = GPIO_FUNC_CAMPERI,
    [GPIO10_14][1] = -1,
    [GPIO10_14][2] = -1,
    [GPIO10_14][3] = -1,
    [GPIO10_14][4] = -1,
    [GPIO10_14][5] = -1,
    [GPIO10_15][0] = GPIO_FUNC_CAMPERI,
    [GPIO10_15][1] = -1,
    [GPIO10_15][2] = -1,
    [GPIO10_15][3] = -1,
    [GPIO10_15][4] = -1,
    [GPIO10_15][5] = -1,

    [GPIO11_0][0]  = GPIO_FUNC_CAMPERI,
    [GPIO11_0][1]  = -1,
    [GPIO11_0][2]  = -1,
    [GPIO11_0][3]  = -1,
    [GPIO11_0][4]  = -1,
    [GPIO11_0][5]  = -1,
    [GPIO11_1][0]  = GPIO_FUNC_CAMPERI,
    [GPIO11_1][1]  = -1,
    [GPIO11_1][2]  = -1,
    [GPIO11_1][3]  = -1,
    [GPIO11_1][4]  = -1,
    [GPIO11_1][5]  = -1,
    [GPIO11_2][0]  = GPIO_FUNC_CAMPERI,
    [GPIO11_2][1]  = -1,
    [GPIO11_2][2]  = -1,
    [GPIO11_2][3]  = -1,
    [GPIO11_2][4]  = -1,
    [GPIO11_2][5]  = -1,
    [GPIO11_3][0]  = GPIO_FUNC_CAMPERI,
    [GPIO11_3][1]  = -1,
    [GPIO11_3][2]  = -1,
    [GPIO11_3][3]  = -1,
    [GPIO11_3][4]  = -1,
    [GPIO11_3][5]  = -1,
    [GPIO11_4][0]  = GPIO_FUNC_CAMPERI,
    [GPIO11_4][1]  = -1,
    [GPIO11_4][2]  = -1,
    [GPIO11_4][3]  = -1,
    [GPIO11_4][4]  = -1,
    [GPIO11_4][5]  = -1,
    [GPIO11_5][0]  = GPIO_FUNC_CAMPERI,
    [GPIO11_5][1]  = -1,
    [GPIO11_5][2]  = -1,
    [GPIO11_5][3]  = -1,
    [GPIO11_5][4]  = -1,
    [GPIO11_5][5]  = -1,
    [GPIO11_6][0]  = GPIO_FUNC_CAMPERI,
    [GPIO11_6][1]  = -1,
    [GPIO11_6][2]  = -1,
    [GPIO11_6][3]  = -1,
    [GPIO11_6][4]  = -1,
    [GPIO11_6][5]  = -1,
    [GPIO11_7][0]  = GPIO_FUNC_CAMPERI,
    [GPIO11_7][1]  = -1,
    [GPIO11_7][2]  = -1,
    [GPIO11_7][3]  = -1,
    [GPIO11_7][4]  = -1,
    [GPIO11_7][5]  = -1,
    [GPIO11_8][0]  = GPIO_FUNC_CAMPERI,
    [GPIO11_8][1]  = -1,
    [GPIO11_8][2]  = -1,
    [GPIO11_8][3]  = -1,
    [GPIO11_8][4]  = -1,
    [GPIO11_8][5]  = -1,
    [GPIO11_9][0]  = GPIO_FUNC_CAMPERI,
    [GPIO11_9][1]  = -1,
    [GPIO11_9][2]  = -1,
    [GPIO11_9][3]  = -1,
    [GPIO11_9][4]  = -1,
    [GPIO11_9][5]  = -1,
    [GPIO11_10][0] = GPIO_FUNC_CAMPERI,
    [GPIO11_10][1] = -1,
    [GPIO11_10][2] = -1,
    [GPIO11_10][3] = -1,
    [GPIO11_10][4] = -1,
    [GPIO11_10][5] = -1,
    [GPIO11_11][0] = GPIO_FUNC_CAMPERI,
    [GPIO11_11][1] = -1,
    [GPIO11_11][2] = -1,
    [GPIO11_11][3] = -1,
    [GPIO11_11][4] = -1,
    [GPIO11_11][5] = -1,
//    [GPIO11_12][0] = ,	/* not used */
//    [GPIO11_12][1] = ,	/* not used */
//    [GPIO11_12][2] = ,	/* not used */
//    [GPIO11_12][3] = ,	/* not used */
//    [GPIO11_12][4] = ,	/* not used */
//    [GPIO11_12][5] = ,	/* not used */
//    [GPIO11_13][0] = ,	/* not used */
//    [GPIO11_13][1] = ,	/* not used */
//    [GPIO11_13][2] = ,	/* not used */
//    [GPIO11_13][3] = ,	/* not used */
//    [GPIO11_13][4] = ,	/* not used */
//    [GPIO11_13][5] = ,	/* not used */
//    [GPIO11_14][0] = ,	/* not used */
//    [GPIO11_14][1] = ,	/* not used */
//    [GPIO11_14][2] = ,	/* not used */
//    [GPIO11_14][3] = ,	/* not used */
//    [GPIO11_14][4] = ,	/* not used */
//    [GPIO11_14][5] = ,	/* not used */
//    [GPIO11_15][0] = ,	/* not used */
//    [GPIO11_15][1] = ,	/* not used */
//    [GPIO11_15][2] = ,	/* not used */
//    [GPIO11_15][3] = ,	/* not used */
//    [GPIO11_15][4] = ,	/* not used */
//    [GPIO11_15][5] = ,	/* not used */

    [GPIO12_0][0]  = GPIO_FUNC_VIN,
    [GPIO12_0][1]  = -1,
    [GPIO12_0][2]  = -1,
    [GPIO12_0][3]  = -1,
    [GPIO12_0][4]  = -1,
    [GPIO12_0][5]  = -1,
    [GPIO12_1][0]  = GPIO_FUNC_VIN,
    [GPIO12_1][1]  = -1,
    [GPIO12_1][2]  = -1,
    [GPIO12_1][3]  = -1,
    [GPIO12_1][4]  = -1,
    [GPIO12_1][5]  = -1,
    [GPIO12_2][0]  = GPIO_FUNC_VIN,
    [GPIO12_2][1]  = -1,
    [GPIO12_2][2]  = -1,
    [GPIO12_2][3]  = -1,
    [GPIO12_2][4]  = -1,
    [GPIO12_2][5]  = -1,
    [GPIO12_3][0]  = GPIO_FUNC_VIN,
    [GPIO12_3][1]  = -1,
    [GPIO12_3][2]  = -1,
    [GPIO12_3][3]  = -1,
    [GPIO12_3][4]  = -1,
    [GPIO12_3][5]  = -1,
    [GPIO12_4][0]  = GPIO_FUNC_VIN,
    [GPIO12_4][1]  = -1,
    [GPIO12_4][2]  = -1,
    [GPIO12_4][3]  = -1,
    [GPIO12_4][4]  = -1,
    [GPIO12_4][5]  = -1,
    [GPIO12_5][0]  = GPIO_FUNC_VIN,
    [GPIO12_5][1]  = -1,
    [GPIO12_5][2]  = -1,
    [GPIO12_5][3]  = -1,
    [GPIO12_5][4]  = -1,
    [GPIO12_5][5]  = -1,
    [GPIO12_6][0]  = GPIO_FUNC_VIN,
    [GPIO12_6][1]  = -1,
    [GPIO12_6][2]  = -1,
    [GPIO12_6][3]  = -1,
    [GPIO12_6][4]  = -1,
    [GPIO12_6][5]  = -1,
    [GPIO12_7][0]  = GPIO_FUNC_VIN,
    [GPIO12_7][1]  = -1,
    [GPIO12_7][2]  = -1,
    [GPIO12_7][3]  = -1,
    [GPIO12_7][4]  = -1,
    [GPIO12_7][5]  = -1,
    [GPIO12_8][0]  = GPIO_FUNC_VIN,
    [GPIO12_8][1]  = -1,
    [GPIO12_8][2]  = -1,
    [GPIO12_8][3]  = -1,
    [GPIO12_8][4]  = -1,
    [GPIO12_8][5]  = -1,
    [GPIO12_9][0]  = GPIO_FUNC_VIN,
    [GPIO12_9][1]  = -1,
    [GPIO12_9][2]  = -1,
    [GPIO12_9][3]  = -1,
    [GPIO12_9][4]  = -1,
    [GPIO12_9][5]  = -1,
    [GPIO12_10][0] = GPIO_FUNC_VIN,
    [GPIO12_10][1] = -1,
    [GPIO12_10][2] = -1,
    [GPIO12_10][3] = -1,
    [GPIO12_10][4] = -1,
    [GPIO12_10][5] = -1,
    [GPIO12_11][0] = GPIO_FUNC_VIN,
    [GPIO12_11][1] = -1,
    [GPIO12_11][2] = -1,
    [GPIO12_11][3] = -1,
    [GPIO12_11][4] = -1,
    [GPIO12_11][5] = -1,
    [GPIO12_12][0] = GPIO_FUNC_VIN,
    [GPIO12_12][1] = -1,
    [GPIO12_12][2] = -1,
    [GPIO12_12][3] = -1,
    [GPIO12_12][4] = -1,
    [GPIO12_12][5] = -1,
    [GPIO12_13][0] = GPIO_FUNC_VIN,
    [GPIO12_13][1] = -1,
    [GPIO12_13][2] = -1,
    [GPIO12_13][3] = -1,
    [GPIO12_13][4] = -1,
    [GPIO12_13][5] = -1,
    [GPIO12_14][0] = GPIO_FUNC_VIN,
    [GPIO12_14][1] = -1,
    [GPIO12_14][2] = -1,
    [GPIO12_14][3] = -1,
    [GPIO12_14][4] = -1,
    [GPIO12_14][5] = -1,
    [GPIO12_15][0] = GPIO_FUNC_VIN,
    [GPIO12_15][1] = -1,
    [GPIO12_15][2] = -1,
    [GPIO12_15][3] = -1,
    [GPIO12_15][4] = -1,
    [GPIO12_15][5] = -1,

    [GPIO13_0][0]  = GPIO_FUNC_VIN,
    [GPIO13_0][1]  = -1,
    [GPIO13_0][2]  = -1,
    [GPIO13_0][3]  = -1,
    [GPIO13_0][4]  = -1,
    [GPIO13_0][5]  = -1,
    [GPIO13_1][0]  = GPIO_FUNC_VIN,
    [GPIO13_1][1]  = -1,
    [GPIO13_1][2]  = -1,
    [GPIO13_1][3]  = -1,
    [GPIO13_1][4]  = -1,
    [GPIO13_1][5]  = -1,
    [GPIO13_2][0]  = GPIO_FUNC_VIN,
    [GPIO13_2][1]  = -1,
    [GPIO13_2][2]  = -1,
    [GPIO13_2][3]  = -1,
    [GPIO13_2][4]  = -1,
    [GPIO13_2][5]  = -1,
    [GPIO13_3][0]  = GPIO_FUNC_IMG,
    [GPIO13_3][1]  = -1,
    [GPIO13_3][2]  = -1,
    [GPIO13_3][3]  = -1,
    [GPIO13_3][4]  = -1,
    [GPIO13_3][5]  = -1,
    [GPIO13_4][0]  = GPIO_FUNC_IMG,
    [GPIO13_4][1]  = -1,
    [GPIO13_4][2]  = -1,
    [GPIO13_4][3]  = -1,
    [GPIO13_4][4]  = -1,
    [GPIO13_4][5]  = -1,
    [GPIO13_5][0]  = GPIO_FUNC_IMG,
    [GPIO13_5][1]  = -1,
    [GPIO13_5][2]  = -1,
    [GPIO13_5][3]  = -1,
    [GPIO13_5][4]  = -1,
    [GPIO13_5][5]  = -1,
    [GPIO13_6][0]  = GPIO_FUNC_EXT,
    [GPIO13_6][1]  = -1,
    [GPIO13_6][2]  = -1,
    [GPIO13_6][3]  = -1,
    [GPIO13_6][4]  = -1,
    [GPIO13_6][5]  = -1,
//    [GPIO13_7][0]  = ,	/* not used */
//    [GPIO13_7][1]  = ,	/* not used */
//    [GPIO13_7][2]  = ,	/* not used */
//    [GPIO13_7][3]  = ,	/* not used */
//    [GPIO13_7][4]  = ,	/* not used */
//    [GPIO13_7][5]  = ,	/* not used */
//    [GPIO13_8][0]  = ,	/* not used */
//    [GPIO13_8][1]  = ,	/* not used */
//    [GPIO13_8][2]  = ,	/* not used */
//    [GPIO13_8][3]  = ,	/* not used */
//    [GPIO13_8][4]  = ,	/* not used */
//    [GPIO13_8][5]  = ,	/* not used */
//    [GPIO13_9][0]  = ,	/* not used */
//    [GPIO13_9][1]  = ,	/* not used */
//    [GPIO13_9][2]  = ,	/* not used */
//    [GPIO13_9][3]  = ,	/* not used */
//    [GPIO13_9][4]  = ,	/* not used */
//    [GPIO13_9][5]  = ,	/* not used */
//    [GPIO13_10][0] = ,	/* not used */
//    [GPIO13_10][1] = ,	/* not used */
//    [GPIO13_10][2] = ,	/* not used */
//    [GPIO13_10][3] = ,	/* not used */
//    [GPIO13_10][4] = ,	/* not used */
//    [GPIO13_10][5] = ,	/* not used */
//    [GPIO13_11][0] = ,	/* not used */
//    [GPIO13_11][1] = ,	/* not used */
//    [GPIO13_11][2] = ,	/* not used */
//    [GPIO13_11][3] = ,	/* not used */
//    [GPIO13_11][4] = ,	/* not used */
//    [GPIO13_11][5] = ,	/* not used */
//    [GPIO13_12][0] = ,	/* not used */
//    [GPIO13_12][1] = ,	/* not used */
//    [GPIO13_12][2] = ,	/* not used */
//    [GPIO13_12][3] = ,	/* not used */
//    [GPIO13_12][4] = ,	/* not used */
//    [GPIO13_12][5] = ,	/* not used */
//    [GPIO13_13][0] = ,	/* not used */
//    [GPIO13_13][1] = ,	/* not used */
//    [GPIO13_13][2] = ,	/* not used */
//    [GPIO13_13][3] = ,	/* not used */
//    [GPIO13_13][4] = ,	/* not used */
//    [GPIO13_13][5] = ,	/* not used */
//    [GPIO13_14][0] = ,	/* not used */
//    [GPIO13_14][1] = ,	/* not used */
//    [GPIO13_14][2] = ,	/* not used */
//    [GPIO13_14][3] = ,	/* not used */
//    [GPIO13_14][4] = ,	/* not used */
//    [GPIO13_14][5] = ,	/* not used */
//    [GPIO13_15][0] = ,	/* not used */
//    [GPIO13_15][1] = ,	/* not used */
//    [GPIO13_15][2] = ,	/* not used */
//    [GPIO13_15][3] = ,	/* not used */
//    [GPIO13_15][4] = ,	/* not used */
//    [GPIO13_15][5] = ,	/* not used */

    [GPIO14_0][0]  = GPIO_FUNC_STL,
    [GPIO14_0][1]  = GPIO_FUNC_CCD,
    [GPIO14_0][2]  = -1,
    [GPIO14_0][3]  = -1,
    [GPIO14_0][4]  = -1,
    [GPIO14_0][5]  = -1,
    [GPIO14_1][0]  = GPIO_FUNC_STL,
    [GPIO14_1][1]  = GPIO_FUNC_CCD,
    [GPIO14_1][2]  = -1,
    [GPIO14_1][3]  = -1,
    [GPIO14_1][4]  = -1,
    [GPIO14_1][5]  = -1,
    [GPIO14_2][0]  = GPIO_FUNC_STL,
    [GPIO14_2][1]  = GPIO_FUNC_CCD,
    [GPIO14_2][2]  = -1,
    [GPIO14_2][3]  = -1,
    [GPIO14_2][4]  = -1,
    [GPIO14_2][5]  = -1,
    [GPIO14_3][0]  = GPIO_FUNC_STL,
    [GPIO14_3][1]  = GPIO_FUNC_CCD,
    [GPIO14_3][2]  = -1,
    [GPIO14_3][3]  = -1,
    [GPIO14_3][4]  = -1,
    [GPIO14_3][5]  = -1,
    [GPIO14_4][0]  = GPIO_FUNC_STL,
    [GPIO14_4][1]  = GPIO_FUNC_CCD,
    [GPIO14_4][2]  = -1,
    [GPIO14_4][3]  = -1,
    [GPIO14_4][4]  = -1,
    [GPIO14_4][5]  = -1,
    [GPIO14_5][0]  = GPIO_FUNC_STL,
    [GPIO14_5][1]  = GPIO_FUNC_CCD,
    [GPIO14_5][2]  = -1,
    [GPIO14_5][3]  = -1,
    [GPIO14_5][4]  = -1,
    [GPIO14_5][5]  = -1,
    [GPIO14_6][0]  = GPIO_FUNC_STL,
    [GPIO14_6][1]  = GPIO_FUNC_CCD,
    [GPIO14_6][2]  = -1,
    [GPIO14_6][3]  = -1,
    [GPIO14_6][4]  = -1,
    [GPIO14_6][5]  = -1,
    [GPIO14_7][0]  = GPIO_FUNC_STL,
    [GPIO14_7][1]  = GPIO_FUNC_CCD,
    [GPIO14_7][2]  = -1,
    [GPIO14_7][3]  = -1,
    [GPIO14_7][4]  = -1,
    [GPIO14_7][5]  = -1,
    [GPIO14_8][0]  = GPIO_FUNC_STL,
    [GPIO14_8][1]  = GPIO_FUNC_CCD,
    [GPIO14_8][2]  = -1,
    [GPIO14_8][3]  = -1,
    [GPIO14_8][4]  = -1,
    [GPIO14_8][5]  = -1,
    [GPIO14_9][0]  = GPIO_FUNC_STL,
    [GPIO14_9][1]  = GPIO_FUNC_CCD,
    [GPIO14_9][2]  = -1,
    [GPIO14_9][3]  = -1,
    [GPIO14_9][4]  = -1,
    [GPIO14_9][5]  = -1,
    [GPIO14_10][0] = GPIO_FUNC_STL,
    [GPIO14_10][1] = GPIO_FUNC_CCD,
    [GPIO14_10][2] = -1,
    [GPIO14_10][3] = -1,
    [GPIO14_10][4] = -1,
    [GPIO14_10][5] = -1,
    [GPIO14_11][0] = GPIO_FUNC_STL,
    [GPIO14_11][1] = GPIO_FUNC_CCD,
    [GPIO14_11][2] = -1,
    [GPIO14_11][3] = -1,
    [GPIO14_11][4] = -1,
    [GPIO14_11][5] = -1,
    [GPIO14_12][0] = GPIO_FUNC_STL,
    [GPIO14_12][1] = GPIO_FUNC_CCD,
    [GPIO14_12][2] = -1,
    [GPIO14_12][3] = -1,
    [GPIO14_12][4] = -1,
    [GPIO14_12][5] = -1,
    [GPIO14_13][0] = GPIO_FUNC_STL,
    [GPIO14_13][1] = GPIO_FUNC_CCD,
    [GPIO14_13][2] = -1,
    [GPIO14_13][3] = -1,
    [GPIO14_13][4] = -1,
    [GPIO14_13][5] = -1,
    [GPIO14_14][0] = GPIO_FUNC_STL,
    [GPIO14_14][1] = -1,
    [GPIO14_14][2] = -1,
    [GPIO14_14][3] = -1,
    [GPIO14_14][4] = -1,
    [GPIO14_14][5] = -1,
    [GPIO14_15][0] = GPIO_FUNC_STL,
    [GPIO14_15][1] = -1,
    [GPIO14_15][2] = -1,
    [GPIO14_15][3] = -1,
    [GPIO14_15][4] = -1,
    [GPIO14_15][5] = -1,

    [GPIO15_0][0]  = GPIO_FUNC_STL,
    [GPIO15_0][1]  = -1,
    [GPIO15_0][2]  = -1,
    [GPIO15_0][3]  = -1,
    [GPIO15_0][4]  = -1,
    [GPIO15_0][5]  = -1,
    [GPIO15_1][0]  = GPIO_FUNC_STL,
    [GPIO15_1][1]  = -1,
    [GPIO15_1][2]  = -1,
    [GPIO15_1][3]  = -1,
    [GPIO15_1][4]  = -1,
    [GPIO15_1][5]  = -1,
    [GPIO15_2][0]  = GPIO_FUNC_STL,
    [GPIO15_2][1]  = -1,
    [GPIO15_2][2]  = -1,
    [GPIO15_2][3]  = -1,
    [GPIO15_2][4]  = -1,
    [GPIO15_2][5]  = -1,
    [GPIO15_3][0]  = GPIO_FUNC_STL,
    [GPIO15_3][1]  = -1,
    [GPIO15_3][2]  = -1,
    [GPIO15_3][3]  = -1,
    [GPIO15_3][4]  = -1,
    [GPIO15_3][5]  = -1,
    [GPIO15_4][0]  = GPIO_FUNC_STL,
    [GPIO15_4][1]  = -1,
    [GPIO15_4][2]  = -1,
    [GPIO15_4][3]  = -1,
    [GPIO15_4][4]  = -1,
    [GPIO15_4][5]  = -1,
    [GPIO15_5][0]  = GPIO_FUNC_STL,
    [GPIO15_5][1]  = -1,
    [GPIO15_5][2]  = -1,
    [GPIO15_5][3]  = -1,
    [GPIO15_5][4]  = -1,
    [GPIO15_5][5]  = -1,
    [GPIO15_6][0]  = GPIO_FUNC_STL,
    [GPIO15_6][1]  = -1,
    [GPIO15_6][2]  = -1,
    [GPIO15_6][3]  = -1,
    [GPIO15_6][4]  = -1,
    [GPIO15_6][5]  = -1,
    [GPIO15_7][0]  = GPIO_FUNC_STL,
    [GPIO15_7][1]  = -1,
    [GPIO15_7][2]  = -1,
    [GPIO15_7][3]  = -1,
    [GPIO15_7][4]  = -1,
    [GPIO15_7][5]  = -1,
    [GPIO15_8][0]  = GPIO_FUNC_STL,
    [GPIO15_8][1]  = -1,
    [GPIO15_8][2]  = -1,
    [GPIO15_8][3]  = -1,
    [GPIO15_8][4]  = -1,
    [GPIO15_8][5]  = -1,
    [GPIO15_9][0]  = GPIO_FUNC_STL,
    [GPIO15_9][1]  = -1,
    [GPIO15_9][2]  = -1,
    [GPIO15_9][3]  = -1,
    [GPIO15_9][4]  = -1,
    [GPIO15_9][5]  = -1,
    [GPIO15_10][0] = GPIO_FUNC_STL,
    [GPIO15_10][1] = -1,
    [GPIO15_10][2] = -1,
    [GPIO15_10][3] = -1,
    [GPIO15_10][4] = -1,
    [GPIO15_10][5] = -1,
    [GPIO15_11][0] = GPIO_FUNC_STL,
    [GPIO15_11][1] = -1,
    [GPIO15_11][2] = -1,
    [GPIO15_11][3] = -1,
    [GPIO15_11][4] = -1,
    [GPIO15_11][5] = -1,
    [GPIO15_12][0] = GPIO_FUNC_STL,
    [GPIO15_12][1] = -1,
    [GPIO15_12][2] = -1,
    [GPIO15_12][3] = -1,
    [GPIO15_12][4] = -1,
    [GPIO15_12][5] = -1,
    [GPIO15_13][0] = GPIO_FUNC_STL,
    [GPIO15_13][1] = -1,
    [GPIO15_13][2] = -1,
    [GPIO15_13][3] = -1,
    [GPIO15_13][4] = -1,
    [GPIO15_13][5] = -1,
    [GPIO15_14][0] = GPIO_FUNC_STL,
    [GPIO15_14][1] = -1,
    [GPIO15_14][2] = -1,
    [GPIO15_14][3] = -1,
    [GPIO15_14][4] = -1,
    [GPIO15_14][5] = -1,
    [GPIO15_15][0] = GPIO_FUNC_STL,
    [GPIO15_15][1] = -1,
    [GPIO15_15][2] = -1,
    [GPIO15_15][3] = -1,
    [GPIO15_15][4] = -1,
    [GPIO15_15][5] = -1,

    [GPIO16_0][0]  = GPIO_FUNC_STL,
    [GPIO16_0][1]  = -1,
    [GPIO16_0][2]  = -1,
    [GPIO16_0][3]  = -1,
    [GPIO16_0][4]  = -1,
    [GPIO16_0][5]  = -1,
    [GPIO16_1][0]  = GPIO_FUNC_STL,
    [GPIO16_1][1]  = -1,
    [GPIO16_1][2]  = -1,
    [GPIO16_1][3]  = -1,
    [GPIO16_1][4]  = -1,
    [GPIO16_1][5]  = -1,
    [GPIO16_2][0]  = GPIO_FUNC_STL,
    [GPIO16_2][1]  = -1,
    [GPIO16_2][2]  = -1,
    [GPIO16_2][3]  = -1,
    [GPIO16_2][4]  = -1,
    [GPIO16_2][5]  = -1,
    [GPIO16_3][0]  = GPIO_FUNC_STL,
    [GPIO16_3][1]  = -1,
    [GPIO16_3][2]  = -1,
    [GPIO16_3][3]  = -1,
    [GPIO16_3][4]  = -1,
    [GPIO16_3][5]  = -1,
    [GPIO16_4][0]  = GPIO_FUNC_STL,
    [GPIO16_4][1]  = -1,
    [GPIO16_4][2]  = -1,
    [GPIO16_4][3]  = -1,
    [GPIO16_4][4]  = -1,
    [GPIO16_4][5]  = -1,
    [GPIO16_5][0]  = GPIO_FUNC_CCD,
    [GPIO16_5][1]  = -1,
    [GPIO16_5][2]  = -1,
    [GPIO16_5][3]  = -1,
    [GPIO16_5][4]  = -1,
    [GPIO16_5][5]  = -1,
    [GPIO16_6][0]  = GPIO_FUNC_CCD,
    [GPIO16_6][1]  = -1,
    [GPIO16_6][2]  = -1,
    [GPIO16_6][3]  = -1,
    [GPIO16_6][4]  = -1,
    [GPIO16_6][5]  = -1,
    [GPIO16_7][0]  = GPIO_FUNC_CCD,
    [GPIO16_7][1]  = -1,
    [GPIO16_7][2]  = -1,
    [GPIO16_7][3]  = -1,
    [GPIO16_7][4]  = -1,
    [GPIO16_7][5]  = -1,
//    [GPIO16_8][0]  = ,	/* not used */
//    [GPIO16_8][1]  = ,	/* not used */
//    [GPIO16_8][2]  = ,	/* not used */
//    [GPIO16_8][3]  = ,	/* not used */
//    [GPIO16_8][4]  = ,	/* not used */
//    [GPIO16_8][5]  = ,	/* not used */
    /* __adjust_nr_bit() for GPIO16_9, GPIO16_10 */
    [GPIO16_9][0]  = GPIO_FUNC_CLAMP,
    [GPIO16_9][1]  = -1,
    [GPIO16_9][2]  = -1,
    [GPIO16_9][3]  = -1,
    [GPIO16_9][4]  = -1,
    [GPIO16_9][5]  = -1,
    [GPIO16_10][0] = GPIO_FUNC_CLAMP,
    [GPIO16_10][1] = -1,
    [GPIO16_10][2] = -1,
    [GPIO16_10][3] = -1,
    [GPIO16_10][4] = -1,
    [GPIO16_10][5] = -1,
//    [GPIO16_11][0] = ,	/* not used */
//    [GPIO16_11][1] = ,	/* not used */
//    [GPIO16_11][2] = ,	/* not used */
//    [GPIO16_11][3] = ,	/* not used */
//    [GPIO16_11][4] = ,	/* not used */
//    [GPIO16_11][5] = ,	/* not used */
//    [GPIO16_12][0] = ,	/* not used */
//    [GPIO16_12][1] = ,	/* not used */
//    [GPIO16_12][2] = ,	/* not used */
//    [GPIO16_12][3] = ,	/* not used */
//    [GPIO16_12][4] = ,	/* not used */
//    [GPIO16_12][5] = ,	/* not used */
//    [GPIO16_13][0] = ,	/* not used */
//    [GPIO16_13][1] = ,	/* not used */
//    [GPIO16_13][2] = ,	/* not used */
//    [GPIO16_13][3] = ,	/* not used */
//    [GPIO16_13][4] = ,	/* not used */
//    [GPIO16_13][5] = ,	/* not used */
//    [GPIO16_14][0] = ,	/* not used */
//    [GPIO16_14][1] = ,	/* not used */
//    [GPIO16_14][2] = ,	/* not used */
//    [GPIO16_14][3] = ,	/* not used */
//    [GPIO16_14][4] = ,	/* not used */
//    [GPIO16_14][5] = ,	/* not used */
//    [GPIO16_15][0] = ,	/* not used */
//    [GPIO16_15][1] = ,	/* not used */
//    [GPIO16_15][2] = ,	/* not used */
//    [GPIO16_15][3] = ,	/* not used */
//    [GPIO16_15][4] = ,	/* not used */
//    [GPIO16_15][5] = ,	/* not used */
};

static spinlock_t gpio_spinlock = SPIN_LOCK_UNLOCKED;

static RegGpio *reggpio = 0;

static char cmd_proc[8];

/*======================================================================
 *
 * Main function body
 *
 *======================================================================*/
/*-----------------------------------------------------------------------------
 * init_module
 *---------------------------------------------------------------------------*/
/*static*/ int __init gpio_cxd4105_init(void)
{
#ifdef CHECK_TIME
    u32 t = tick();
#endif
    int err;
    struct proc_dir_entry *ent;
    PDEBUG1(" "MYNAME"::gpio_cxd4105_init() : Entry\n");

    if((err = gpio_attach_plugin(&gpio_cxd4105)) < 0){
        RETURN1(err, " "MYNAME"::gpio_cxd4105_init() : Return(%x)\n");
    }
    if(! (ent = create_proc_entry(GPIO_PROC, S_IFREG | S_IRUGO | S_IWUSR, 
        proc_root_driver))){
        gpio_detach_plugin(&gpio_cxd4105);
        RETURN1(-EACCES," "MYNAME"::gpio_cxd4105_init() : Return(%x)\n");
    }
    ent->read_proc = &gpio_cxd4105_read_proc;
    ent->write_proc = &gpio_cxd4105_write_proc;
    ent->data = (void *)cmd_proc;

#ifdef CHECK_TIME
    tprintk(tick() - t, "done\n");
#endif
    RETURN1(err, " "MYNAME"::gpio_cxd4105_init() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * exit_module
 *---------------------------------------------------------------------------*/
static void __exit gpio_cxd4105_exit(void)
{
#ifdef CHECK_TIME
    u32 t = tick();
#endif
    PDEBUG1(" "MYNAME"::gpio_cxd4105_exit() : Entry\n");

    remove_proc_entry(GPIO_PROC, proc_root_driver);
    gpio_detach_plugin(&gpio_cxd4105);

    PDEBUG1(" "MYNAME"::gpio_cxd4105_exit() : Return\n");
#ifdef CHECK_TIME
    tprintk(tick() - t, "done\n");
#endif
    return;
}

/*-----------------------------------------------------------------------------
 * open
 *---------------------------------------------------------------------------*/
static int Vopen(IN unsigned int nr)
{
    PDEBUG1(" "MYNAME"::Vopen(%d) : Entry\n",nr);
    RETURN1(0, " "MYNAME"::Vopen() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * close
 *---------------------------------------------------------------------------*/
static int Vclose(IN unsigned int nr)
{
    PDEBUG1(" "MYNAME"::Vclose(%d) : Entry\n",nr);
    RETURN1(0, " "MYNAME"::Vclose() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * set_sel_func
 *---------------------------------------------------------------------------*/
static int Vset_sel_func(IN unsigned int nr, IN unsigned char func, 
    IN unsigned int val)
{
    int err;
    PDEBUG1(" "MYNAME"::Vset_sel_func(%d,%x,%x) : Entry\n", nr, func, val);

    if(__is_gpio(nr)){
        err = set_gpio_func(nr, func, val);
    }else{
        err = set_gpioX_func(nr, func, val);
    }

    RETURN1(err, " "MYNAME"::Vset_sel_func() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * get_sel_func
 *---------------------------------------------------------------------------*/
static int Vget_sel_func(IN unsigned int nr, OUT unsigned char *func)
{
    int err;
    PDEBUG1(" "MYNAME"::Vget_sel_func(%d,%x) : Entry\n", nr, (int)func);

    if(__is_gpio(nr)){
        err = get_gpio_func(nr, func);
    }else{
        err = get_gpioX_func(nr, func);
    }

    RETURN1(err, " "MYNAME"::Vget_sel_func() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * write
 *---------------------------------------------------------------------------*/
static int Vwrite(IN unsigned int nr, IN unsigned int val)
{
    unsigned int nr_bit;
    unsigned int reg;
    int err;
    unsigned int nr_grp;
    PDEBUG1(" "MYNAME"::Vwrite(%d,%x) : Entry\n", nr, val);

    if(__is_gpio(nr)){
        nr_bit = GPIO_NR(nr);
        if(val){
            reg = REG_GPIO_DATASET;
        }else{
            reg = REG_GPIO_DATACLR;
        }
    }else{
        if((err = __get_nr_grpbit(nr, &nr_grp, &nr_bit)) < 0){
            RETURN1(err, " "MYNAME"::Vwrite() : Return(%x)\n");
        }
        if(val){
            reg = REG_GPIO_DATASET_(nr_grp);
        }else{
            reg = REG_GPIO_DATACLR_(nr_grp);
        }
    }
    /* Set register */
    SET_GPREG2(nr_bit, reg);

    RETURN1(0, " "MYNAME"::Vwrite() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * read
 *---------------------------------------------------------------------------*/
static int Vread(IN unsigned int nr)
{
    unsigned int nr_bit;
    unsigned int reg;
    int err;
    unsigned int nr_grp;
    PDEBUG1(" "MYNAME"::Vread(%d) : Entry\n", nr);

    if(__is_gpio(nr)){
        nr_bit = GPIO_NR(nr);
        reg = REG_GPIO_RDATA;
    }else{
        if((err = __get_nr_grpbit(nr, &nr_grp, &nr_bit)) < 0){
            RETURN1(err, " "MYNAME"::Vread() : Return(%x)\n");
        }
        reg = REG_GPIO_RDATA_(nr_grp);
    }
    /* Read register */
    if(GET_GPREG(nr_bit, reg)){
        RETURN1(1, " "MYNAME"::Vread() : Return(%x)\n");
    }

    RETURN1(0, " "MYNAME"::Vread() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * set_irq_sense
 *---------------------------------------------------------------------------*/
static int Vset_irq_sense(IN unsigned int nr, IN unsigned char sense)
{
    unsigned int nr_bit;
    unsigned long flags;
    PDEBUG1(" "MYNAME"::Vset_irq_sense(%d,%x) : Entry\n", nr, sense);

    if(! __is_gpio(nr)){
        /* gpio0, gpio1, ..., gpio16 don't support interrupt */
        RETURN1(-ENOEXEC, " "MYNAME"::Vset_irq_sense() : Return(%x)\n");
    }
    nr_bit    = GPIO_NR(nr);
    spin_lock_irqsave(&gpio_spinlock, flags);
    switch(sense){
    case GPIO_INT_DOWN_EDGE:
        CLR_GPREG(nr_bit, REG_GPIO_INTS);	/* edge */
        CLR_GPREG(nr_bit, REG_GPIO_INTHE);	/* invalid high/up interrupt */
        SET_GPREG(nr_bit, REG_GPIO_INTLE);	/* valid low/down interrupt */
        break;
    case GPIO_INT_UP_EDGE:
        CLR_GPREG(nr_bit, REG_GPIO_INTS);	/* edge */
        SET_GPREG(nr_bit, REG_GPIO_INTHE);	/* valid high/up interrupt */
        CLR_GPREG(nr_bit, REG_GPIO_INTLE);	/* invalid low/down interrupt */
        break;
    case GPIO_INT_UPDOWN_EDGE:
        CLR_GPREG(nr_bit, REG_GPIO_INTS);	/* edge */
        SET_GPREG(nr_bit, REG_GPIO_INTHE);	/* valid high/up interrupt */
        SET_GPREG(nr_bit, REG_GPIO_INTLE);	/* valid low/down interrupt */
        break;
    case GPIO_INT_L_LEVEL:
        SET_GPREG(nr_bit, REG_GPIO_INTS);	/* level */
        CLR_GPREG(nr_bit, REG_GPIO_INTHE);	/* invalid high/up interrupt */
        SET_GPREG(nr_bit, REG_GPIO_INTLE);	/* valid low/down interrupt */
        break;
    case GPIO_INT_H_LEVEL:
        SET_GPREG(nr_bit, REG_GPIO_INTS);	/* level */
        SET_GPREG(nr_bit, REG_GPIO_INTHE);	/* valid high/up interrupt */
        CLR_GPREG(nr_bit, REG_GPIO_INTLE);	/* invalid low/down interrupt */
        break;
    default:
        spin_unlock_irqrestore(&gpio_spinlock, flags);
        RETURN1(-EINVAL, " "MYNAME"::Vset_irq_sense() : Return(%x)\n");
    }
    spin_unlock_irqrestore(&gpio_spinlock, flags);

    RETURN1(0, " "MYNAME"::Vset_irq_sense() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * get_irq_sense
 *---------------------------------------------------------------------------*/
static int Vget_irq_sense(IN unsigned int nr, OUT unsigned char *sense)
{
    unsigned int nr_bit;
    PDEBUG1(" "MYNAME"::Vget_irq_sense(%d,%x) : Entry\n", nr, (int)sense);

    if(! __is_gpio(nr)){ 
        /* gpio0, gpio1, ..., gpio16 don't support interrupt */
        RETURN1(-ENOEXEC, " "MYNAME"::Vget_irq_sense() : Return(%x)\n");
    }
    nr_bit = GPIO_NR(nr);
    if(GET_GPREG(nr_bit, REG_GPIO_INTS)){	/* level */
        if(GET_GPREG(nr_bit, REG_GPIO_INTHE)){
            if(GET_GPREG(nr_bit, REG_GPIO_INTLE)){
                RETURN1(-EINVAL, " "MYNAME"::Vget_irq_sense() : Return(%x)\n");
            }else{
                *sense = GPIO_INT_H_LEVEL;
            }
        }else{
            if(GET_GPREG(nr_bit, REG_GPIO_INTLE)){
                *sense = GPIO_INT_L_LEVEL;
            }else{
                *sense = GPIO_INT_NOSET;
            }
        }
    }else{	/* edge */
        if(GET_GPREG(nr_bit, REG_GPIO_INTHE)){
            if(GET_GPREG(nr_bit, REG_GPIO_INTLE)){
                *sense = GPIO_INT_UPDOWN_EDGE;
            }else{
                *sense = GPIO_INT_UP_EDGE;
            }
        }else{
            if(GET_GPREG(nr_bit, REG_GPIO_INTLE)){
                *sense = GPIO_INT_DOWN_EDGE;
            }else{
                *sense = GPIO_INT_NOSET;
            }
        }
    }

    RETURN1(0, " "MYNAME"::Vget_irq_sense() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * enable_irq
 *---------------------------------------------------------------------------*/
static int Venable_irq(IN unsigned int nr)
{
    unsigned long flags;
    PDEBUG1(" "MYNAME"::Venable_irq(%d) : Entry\n", nr);

    /* Clear irq status */
    Vclear_irq(nr);
    if(! __is_gpio(nr)){
        /* gpio0, gpio1, ..., gpio16 don't support interrupt */
        RETURN1(-ENOEXEC, " "MYNAME"::Venable_irq() : Return(%x)\n");
    }
    spin_lock_irqsave(&gpio_spinlock, flags);
    SET_GPREG(GPIO_NR(nr), REG_GPIO_INTMASK);
    spin_unlock_irqrestore(&gpio_spinlock, flags);

    RETURN1(0, " "MYNAME"::Venable_irq() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * disable_irq
 *---------------------------------------------------------------------------*/
static int Vdisable_irq(IN unsigned int nr)
{
    unsigned long flags;
    PDEBUG1(" "MYNAME"::Vdisable_irq(%d) : Entry\n", nr);

    if(! __is_gpio(nr)){
        /* gpio0, gpio1, ..., gpio16 don't support interrupt */
        RETURN1(-ENOEXEC, " "MYNAME"::Vdisable_irq() : Return(%x)\n");
    }
    spin_lock_irqsave(&gpio_spinlock, flags);
    CLR_GPREG(GPIO_NR(nr), REG_GPIO_INTMASK);
    spin_unlock_irqrestore(&gpio_spinlock, flags);

    RETURN1(0, " "MYNAME"::Vdisable_irq() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * is_enable_irq
 *---------------------------------------------------------------------------*/
static int Vis_enable_irq(IN unsigned int nr)
{
    PDEBUG1(" "MYNAME"::Vis_enable_irq(%d) : Entry\n", nr);

    if(! __is_gpio(nr)){
        /* gpio0, gpio1, ..., gpio16 don't support interrupt */
        RETURN1(-ENOEXEC, " "MYNAME"::Vis_enable_irq() : Return(%x)\n");
    }
    if(GET_GPREG(GPIO_NR(nr), REG_GPIO_INTMASK)){	/* enable */
        RETURN1(1, " "MYNAME"::Vis_enable_irq() : Return(%x)\n");
    }

    RETURN1(0, " "MYNAME"::Vis_enable_irq() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * is_irq
 *---------------------------------------------------------------------------*/
static int Vis_irq(IN unsigned int nr)
{
    PDEBUG1(" "MYNAME"::Vis_irq(%d) : Entry\n", nr);

    if(! __is_gpio(nr)){
        /* gpio0, gpio1, ..., gpio16 don't support interrupt */
        RETURN1(-ENOEXEC, " "MYNAME"::Vis_irq() : Return(%x)\n");
    }
    if(GET_GPREG(GPIO_NR(nr), REG_GPIO_INTST)){	/* request */
        RETURN1(1, " "MYNAME"::Vis_irq() : Return(%x)\n");
    }

    RETURN1(0, " "MYNAME"::Vis_irq() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * clear_irq
 *---------------------------------------------------------------------------*/
static int Vclear_irq(IN unsigned int nr)
{
    PDEBUG1(" "MYNAME"::Vclear_irq(%d) : Entry\n", nr);

    if(! __is_gpio(nr)){
        /* gpio0, gpio1, ..., gpio16 don't support interrupt */
        RETURN1(-ENOEXEC, " "MYNAME"::Vclear_irq() : Return(%x)\n");
    }
    SET_GPREG2(GPIO_NR(nr), REG_GPIO_INTSTCLR);

    RETURN1(0, " "MYNAME"::Vclear_irq() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * register_handler
 *---------------------------------------------------------------------------*/
static int Vregister_handler(IN unsigned int nr)
{
    int err;
    PDEBUG1(" "MYNAME"::Vregister_handler(%d) : Entry\n", nr);

    if(! __is_gpio(nr)){
        /* gpio0, gpio1, ..., gpio16 don't support interrupt */
        RETURN1(-ENOEXEC, " "MYNAME"::Vregister_handler() : Return(%x)\n");
    }
    err = request_irq(GPIO_IRQ(nr), handler, 0, MYNAME, 0);

    RETURN1(err, " "MYNAME"::Vregister_handler() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * unregister_handler
 *---------------------------------------------------------------------------*/
static int Vunregister_handler(IN unsigned int nr)
{
    PDEBUG1(" "MYNAME"::Vunregister_handler(%d) : Entry\n", nr);

    if(! __is_gpio(nr)){
        /* gpio0, gpio1, ..., gpio16 don't support interrupt */
        RETURN1(-ENOEXEC, " "MYNAME"::Vunregister_handler() : Return(%x)\n");
    }
    free_irq(GPIO_IRQ(nr), 0);

    RETURN1(0, " "MYNAME"::Vunregister_handler() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * enable_wakeup
 *---------------------------------------------------------------------------*/
static int Venable_wakeup(IN unsigned int nr)
{
    PDEBUG1(" "MYNAME"::Venable_wakeup(%d) : Entry\n", nr);
    /* gpio, gpio0, gpio1, ..., gpio16 don't support wakeup */
    RETURN1(-ENOEXEC, " "MYNAME"::Venable_wakeup() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * disable_wakeup
 *---------------------------------------------------------------------------*/
static int Vdisable_wakeup(IN unsigned int nr)
{
    PDEBUG1(" "MYNAME"::Vdisable_wakeup(%d) : Entry\n", nr);
    /* gpio, gpio0, gpio1, ..., gpio16 don't support wakeup */
    RETURN1(-ENOEXEC, " "MYNAME"::Vdisable_wakeup() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * is_enable_wakeup
 *---------------------------------------------------------------------------*/
static int Vis_enable_wakeup(IN unsigned int nr)
{
    PDEBUG1(" "MYNAME"::Vis_enable_wakeup(%d) : Entry\n", nr);
    /* gpio, gpio0, gpio1, ..., gpio16 don't support wakeup */
    RETURN1(-ENOEXEC, " "MYNAME"::Vis_enable_wakeup() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * suspend
 *---------------------------------------------------------------------------*/
static int Vsuspend(IN struct amba_device *dev, IN u32 state)
{
    suspend_state_t pm = pm_get_state();
    int err = 0;
    PDEBUG1(" "MYNAME"::Vsuspend(%x,%x) : Entry\n", (int)dev, state);

//    err = __suspend(dev, state);
    if(pm == PM_SUSPEND_MEM){
        PDEBUG1(" "MYNAME"::Vsuspend() : suspend\n");
    }else if(pm == PM_SUSPEND_DISK){
        PDEBUG1(" "MYNAME"::Vsuspend() : hibernate\n");
    }

    RETURN1(err, " "MYNAME"::Vsuspend() : Return(%d)\n");
}

/*-----------------------------------------------------------------------------
 * resume
 *---------------------------------------------------------------------------*/
static int Vresume(IN struct amba_device *dev)
{
    suspend_state_t pm = pm_get_state();
    int err = 0;
    PDEBUG1(" "MYNAME"::Vresume(%x) : Entry\n", (int)dev);

    if(pm == PM_SUSPEND_MEM){
        PDEBUG1(" "MYNAME"::Vresume() : hot boot\n");
    }else if(pm == PM_SUSPEND_DISK){
        PDEBUG1(" "MYNAME"::Vresume() : warm boot\n");
    }
//    err = __resume(dev);

    RETURN1(err, " "MYNAME"::Vresume() : Return(%d)\n");
}

/*-----------------------------------------------------------------------------
 * read_proc
 *---------------------------------------------------------------------------*/
static int gpio_cxd4105_read_proc(char *buffer, char **start, off_t offset, 
    int length, int *eof, void *data)
{
    int len = 0;
    char cmd[3];
    int num = -1;
    unsigned short portsel;
    PDEBUG1(" "MYNAME"::gpio_cxd4105_read_proc() : Entry\n");

    /* get register name */
    if(! strcmp(CMD_GPIOSYS, (const char *)data)){
        len += sprintf(buffer + len, "GPIOSYS\n");
        len += sprintf(buffer + len, "DIR    :0x%04x\n", readw(REG_GPIO_DIR));
        len += sprintf(buffer + len, "RDATA  :0x%04x\n", readw(REG_GPIO_RDATA));
        len += sprintf(buffer + len, "INTS   :0x%04x\n", readw(REG_GPIO_INTS));
        len += sprintf(buffer + len, "INTHE  :0x%04x\n", readw(REG_GPIO_INTHE));
        len += sprintf(buffer + len, "INTLE  :0x%04x\n", readw(REG_GPIO_INTLE));
        len += sprintf(buffer + len, "INTMASK:0x%04x\n", readw(REG_GPIO_INTMASK));
        len += sprintf(buffer + len, "INEN   :0x%04x\n", readw(REG_GPIO_INEN));
        len += sprintf(buffer + len, "RESEN  :0x%04x\n", readw(REG_GPIO_RESEN));
        len += sprintf(buffer + len, "PUPD   :0x%04x\n", readw(REG_GPIO_PUPD));
        len -= offset;
    }else{
        cmd[0] = *((char *)data + 4);
        cmd[1] = *((char *)data + 5);
        cmd[2] = *((char *)data + 6);
        if(cmd[1] == '\0'){
            num = cmd[0] - '0';
        }else if(cmd[0] != '0' && cmd[2] == '\0'){
            num = (cmd[0] - '0') * 10 + cmd[1] - '0';
        }
        *((char *)data + 4) = '\0';
        if((! strcmp(CMD_GPIOEASY, (const char *)data)) && num >= 0 && 
            num <= 16){
            *((char *)data + 4) = cmd[0];
            len += sprintf(buffer + len, "GPIOEASY%d\n",  num);
            len += sprintf(buffer + len, "DIR    :0x%04x\n", 
                readw(REG_GPIO_DIR_(num)));
            len += sprintf(buffer + len, "RDATA  :0x%04x\n", 
                readw(REG_GPIO_RDATA_(num)));
            len += sprintf(buffer + len, "INEN   :0x%04x\n", 
                readw(REG_GPIO_INEN_(num)));
	    portsel = readw(REG_GPIO_PORTSEL15_(num));
            portsel <<= 1;
	    portsel |= readw(REG_GPIO_PORTSEL14_(num));
            portsel <<= 1;
	    portsel |= readw(REG_GPIO_PORTSEL13_(num));
            portsel <<= 1;
	    portsel |= readw(REG_GPIO_PORTSEL12_(num));
            portsel <<= 1;
	    portsel |= readw(REG_GPIO_PORTSEL11_(num));
            portsel <<= 1;
	    portsel |= readw(REG_GPIO_PORTSEL10_(num));
            portsel <<= 1;
	    portsel |= readw(REG_GPIO_PORTSEL9_(num));
            portsel <<= 1;
	    portsel |= readw(REG_GPIO_PORTSEL8_(num));
            portsel <<= 1;
	    portsel |= readw(REG_GPIO_PORTSEL7_(num));
            portsel <<= 1;
	    portsel |= readw(REG_GPIO_PORTSEL6_(num));
            portsel <<= 1;
	    portsel |= readw(REG_GPIO_PORTSEL5_(num));
            portsel <<= 1;
	    portsel |= readw(REG_GPIO_PORTSEL4_(num));
            portsel <<= 1;
	    portsel |= readw(REG_GPIO_PORTSEL3_(num));
            portsel <<= 1;
	    portsel |= readw(REG_GPIO_PORTSEL2_(num));
            portsel <<= 1;
	    portsel |= readw(REG_GPIO_PORTSEL1_(num));
            portsel <<= 1;
	    portsel |= readw(REG_GPIO_PORTSEL0_(num));
            len += sprintf(buffer + len, "PORTSEL:0x%04x\n", portsel);
            len += sprintf(buffer + len, "HOLDEN :0x%04x\n", 
                readw(REG_GPIO_HOLDEN_(num)));
            len += sprintf(buffer + len, "RESEN  :0x%04x\n", 
                readw(REG_GPIO_RESEN_(num)));
            len += sprintf(buffer + len, "PUPD   :0x%04x\n", 
                readw(REG_GPIO_PUPD_(num)));
	    len -= offset;
        }
    }
    if(len < length){
        *eof = 1;
        if(len <= 0){
            RETURN1(0, " "MYNAME"::gpio_cxd4105_read_proc() : Return(%x)\n");
        }
    }else{
        len = length;
    }
    *start = buffer + offset;

    RETURN1(len, " "MYNAME"::gpio_cxd4105_read_proc() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * write_proc
 *---------------------------------------------------------------------------*/
static int gpio_cxd4105_write_proc(struct file *file, const char *buffer, 
    unsigned long count, void *data)
{
    unsigned char i;
    PDEBUG1(" "MYNAME"::gpio_cxd4105_write_proc() : Entry\n");

    for(i = 0; i < MIN(sizeof(cmd_proc), count); i++){
        *(cmd_proc + i) = *(buffer + i);
    }
    *(cmd_proc + MIN(sizeof(cmd_proc), count) - 1) = '\0';

    RETURN1(count, " "MYNAME"::gpio_cxd4105_write_proc() : Return(%lx)\n");
}

/*-----------------------------------------------------------------------------
 * handler
 *---------------------------------------------------------------------------*/
static irqreturn_t handler(int irq, void *dev_id, struct pt_regs *fp)
{
    unsigned int nr;
    PDEBUG1(" "MYNAME"::handler(%d) : Entry\n", irq);

    nr = GPIO_NR(irq - IRQ_GPIO0);
    if(__is_gpio(nr)){
        /* Clear irq status */
        Vclear_irq(nr);
        PDEBUG1(" "MYNAME"::handler() : call gpio_handler(%d)\n", nr);
        gpio_handler(nr);
    }

    RETURN1(IRQ_HANDLED, " "MYNAME"::handler() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * set_gpio_func
 *---------------------------------------------------------------------------*/
static int set_gpio_func(IN unsigned int nr, IN unsigned char func, 
    IN unsigned int val)
{
    unsigned int nr_bit;
    unsigned long flags;
    PDEBUG1(" "MYNAME"::set_gpio_func(%d,%x,%x) : Entry\n", nr, func, val);

    nr_bit = GPIO_NR(nr);
    spin_lock_irqsave(&gpio_spinlock, flags);
    switch(func){
    case GPIO_OUTPUT:
        CLR_GPREG(nr_bit, REG_GPIO_INEN);
        CLR_GPREG(nr_bit, REG_GPIO_RESEN);
        /* Set output setting */
        if(val){
            SET_GPREG2(nr_bit, REG_GPIO_DATASET);
        }else{
            SET_GPREG2(nr_bit, REG_GPIO_DATACLR);
        }
        SET_GPREG(nr_bit, REG_GPIO_DIR);
        break;
    case GPIO_INPUT:
        CLR_GPREG(nr_bit, REG_GPIO_RESEN);
        /* Set input setting */
        CLR_GPREG(nr_bit, REG_GPIO_DIR);
        SET_GPREG(nr_bit, REG_GPIO_INEN);
        break;
    case GPIO_INPUT_PU:
    case GPIO_INPUT_PD:
        /* Set input setting */
        CLR_GPREG(nr_bit, REG_GPIO_DIR);
        if(func == GPIO_INPUT_PU){
            /* Set pull up setting */
            SET_GPREG(nr_bit, REG_GPIO_PUPD);
            SET_GPREG(nr_bit, REG_GPIO_RESEN);
        }else{
            /* Set pull down setting */
            CLR_GPREG(nr_bit, REG_GPIO_PUPD);
            SET_GPREG(nr_bit, REG_GPIO_RESEN);
        }
        SET_GPREG(nr_bit, REG_GPIO_INEN);
        break;
    case GPIO_PULLUP:
    case GPIO_PULLDN:
        CLR_GPREG(nr_bit, REG_GPIO_INEN);
        /* Set input setting */
        CLR_GPREG(nr_bit, REG_GPIO_DIR);
        if(func == GPIO_PULLUP){
            /* Set pull up setting */
            SET_GPREG(nr_bit, REG_GPIO_PUPD);
            SET_GPREG(nr_bit, REG_GPIO_RESEN);
        }else{
            /* Set pull down setting */
            CLR_GPREG(nr_bit, REG_GPIO_PUPD);
            SET_GPREG(nr_bit, REG_GPIO_RESEN);
        }
        break;
    case GPIO_NOSET:
        CLR_GPREG(nr_bit, REG_GPIO_INEN);
        CLR_GPREG(nr_bit, REG_GPIO_RESEN);
        /* Set input setting */
        CLR_GPREG(nr_bit, REG_GPIO_DIR);
        break;
    default:
        spin_unlock_irqrestore(&gpio_spinlock, flags);
        RETURN1(-ENOEXEC, " "MYNAME"::set_gpio_func() : Return(%x)\n");
    }
    spin_unlock_irqrestore(&gpio_spinlock, flags);

    RETURN1(0, " "MYNAME"::set_gpio_func() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * set_gpioX_func
 *---------------------------------------------------------------------------*/
static int set_gpioX_func(IN unsigned int nr, IN unsigned char func, 
    IN unsigned int val)
{
    int err;
    unsigned int nr_bit;
    unsigned int nr_grp;
    unsigned long flags;
    PDEBUG1(" "MYNAME"::set_gpioX_func(%d,%x,%x) : Entry\n", nr, func, val);

    if((err = __get_nr_grpbit(nr, &nr_grp, &nr_bit)) < 0){
        RETURN1(err, " "MYNAME"::set_gpioX_func() : Return(%x)\n");
    }
    spin_lock_irqsave(&gpio_spinlock, flags);
    switch(func){
    case GPIO_OUTPUT:
        CLR_GPREG(nr_bit, REG_GPIO_INEN_(nr_grp));
        /* Clear other setting */
        CLR_GPREG2(0, GET_SELREG(nr_bit, REG_GPIO_PORTSEL0_(nr_grp)));	// __clr_gpioX_func_en()
        CLR_GPREG(nr_bit, REG_GPIO_HOLDEN_(nr_grp));
        CLR_GPREG(nr_bit, REG_GPIO_RESEN_(nr_grp));
        /* Set output setting */
        if(val){
            SET_GPREG2(nr_bit, REG_GPIO_DATASET_(nr_grp));
        }else{
            SET_GPREG2(nr_bit, REG_GPIO_DATACLR_(nr_grp));
        }
        SET_GPREG(nr_bit, REG_GPIO_DIR_(nr_grp));
        break;
    case GPIO_INPUT:
        /* Clear other setting */
        CLR_GPREG2(0, GET_SELREG(nr_bit, REG_GPIO_PORTSEL0_(nr_grp)));	// __clr_gpioX_func_en()
        CLR_GPREG(nr_bit, REG_GPIO_HOLDEN_(nr_grp));
        CLR_GPREG(nr_bit, REG_GPIO_RESEN_(nr_grp));
        /* Set intput setting */
        CLR_GPREG(nr_bit, REG_GPIO_DIR_(nr_grp));
        SET_GPREG(nr_bit, REG_GPIO_INEN_(nr_grp));
        break;
    case GPIO_INPUT_PU:
    case GPIO_INPUT_PD:
    case GPIO_INPUT_BH:
        /* Clear other setting */
        CLR_GPREG2(0, GET_SELREG(nr_bit, REG_GPIO_PORTSEL0_(nr_grp)));	// __clr_gpioX_func_en()
        /* Set intput setting */
        CLR_GPREG(nr_bit, REG_GPIO_DIR_(nr_grp));
        if(func == GPIO_INPUT_PU){
            /* Set pull up setting */
            CLR_GPREG(nr_bit, REG_GPIO_HOLDEN_(nr_grp));
            SET_GPREG(nr_bit, REG_GPIO_PUPD_(nr_grp));
            SET_GPREG(nr_bit, REG_GPIO_RESEN_(nr_grp));
        }else if(func == GPIO_INPUT_PD){
            /* Set pull down setting */
            CLR_GPREG(nr_bit, REG_GPIO_HOLDEN_(nr_grp));
            CLR_GPREG(nr_bit, REG_GPIO_PUPD_(nr_grp));
            SET_GPREG(nr_bit, REG_GPIO_RESEN_(nr_grp));
        }else{
            /* Set bus holder setting */
            CLR_GPREG(nr_bit, REG_GPIO_RESEN_(nr_grp));
            SET_GPREG(nr_bit, REG_GPIO_HOLDEN_(nr_grp));
        }
        SET_GPREG(nr_bit, REG_GPIO_INEN_(nr_grp));
        break;
    case GPIO_PULLUP:
    case GPIO_PULLDN:
    case GPIO_BUSHD:
        CLR_GPREG(nr_bit, REG_GPIO_INEN_(nr_grp));
        /* Clear other setting */
        CLR_GPREG2(0, GET_SELREG(nr_bit, REG_GPIO_PORTSEL0_(nr_grp)));	// __clr_gpioX_func_en()
        /* Set intput setting */
        CLR_GPREG(nr_bit, REG_GPIO_DIR_(nr_grp));
        if(func == GPIO_PULLUP){
            /* Set pull up setting */
            CLR_GPREG(nr_bit, REG_GPIO_HOLDEN_(nr_grp));
            SET_GPREG(nr_bit, REG_GPIO_PUPD_(nr_grp));
            SET_GPREG(nr_bit, REG_GPIO_RESEN_(nr_grp));
        }else if(func == GPIO_PULLDN){
            /* Set pull down setting */
            CLR_GPREG(nr_bit, REG_GPIO_HOLDEN_(nr_grp));
            CLR_GPREG(nr_bit, REG_GPIO_PUPD_(nr_grp));
            SET_GPREG(nr_bit, REG_GPIO_RESEN_(nr_grp));
        }else{
            /* Set bus holder setting */
            CLR_GPREG(nr_bit, REG_GPIO_RESEN_(nr_grp));
            SET_GPREG(nr_bit, REG_GPIO_HOLDEN_(nr_grp));
        }
        break;
    case GPIO_NOSET:
        CLR_GPREG(nr_bit, REG_GPIO_INEN_(nr_grp));
        /* Clear other setting */
        CLR_GPREG2(0, GET_SELREG(nr_bit, REG_GPIO_PORTSEL0_(nr_grp)));	// __clr_gpioX_func_en()
        CLR_GPREG(nr_bit, REG_GPIO_HOLDEN_(nr_grp));
        CLR_GPREG(nr_bit, REG_GPIO_RESEN_(nr_grp));
        /* Set intput setting */
        CLR_GPREG(nr_bit, REG_GPIO_DIR_(nr_grp));
        break;
    default:
        if(__check_nr_func(nr, func)){
            /* Use port as peripheral */
            SET_GPREG2(0, GET_SELREG(nr_bit, REG_GPIO_PORTSEL0_(nr_grp)));
            if(__check_func_HiZ(func)){
                CLR_GPREG(nr_bit, REG_GPIO_INEN_(nr_grp));
                CLR_GPREG(nr_bit, REG_GPIO_HOLDEN_(nr_grp));
                CLR_GPREG(nr_bit, REG_GPIO_RESEN_(nr_grp));
                /* Set intput setting */
                CLR_GPREG(nr_bit, REG_GPIO_DIR_(nr_grp));
            }
            if(__check_func_PullUp(func)){
                CLR_GPREG(nr_bit, REG_GPIO_INEN_(nr_grp));
                CLR_GPREG(nr_bit, REG_GPIO_HOLDEN_(nr_grp));
                SET_GPREG(nr_bit, REG_GPIO_PUPD_(nr_grp));
                SET_GPREG(nr_bit, REG_GPIO_RESEN_(nr_grp));
                /* Set intput setting */
                CLR_GPREG(nr_bit, REG_GPIO_DIR_(nr_grp));
            }
            if(__check_func_PullDown(func)){
                CLR_GPREG(nr_bit, REG_GPIO_INEN_(nr_grp));
                CLR_GPREG(nr_bit, REG_GPIO_HOLDEN_(nr_grp));
                CLR_GPREG(nr_bit, REG_GPIO_PUPD_(nr_grp));
                SET_GPREG(nr_bit, REG_GPIO_RESEN_(nr_grp));
                /* Set intput setting */
                CLR_GPREG(nr_bit, REG_GPIO_DIR_(nr_grp));
            }
            break;
        }
        spin_unlock_irqrestore(&gpio_spinlock, flags);
        RETURN1(-ENOEXEC, " "MYNAME"::set_gpioX_func() : Return(%x)\n");
    }
    spin_unlock_irqrestore(&gpio_spinlock, flags);

    RETURN1(0, " "MYNAME"::set_gpioX_func() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * get_gpio_func
 *---------------------------------------------------------------------------*/
static int get_gpio_func(IN unsigned int nr, IN unsigned char *func)
{
    unsigned int nr_bit;
    PDEBUG1(" "MYNAME"::get_gpio_func(%d,%x) : Entry\n", nr, (int)func);

    nr_bit = GPIO_NR(nr);
    /* Check direction */
    if(GET_GPREG(nr_bit, REG_GPIO_DIR)){
        if(GET_GPREG(nr_bit, REG_GPIO_INEN)){
            RETURN1(-EINVAL, " "MYNAME"::get_gpio_func() : Return(%x)\n");
        }else{
            *func = GPIO_OUTPUT;
        }
    }else{
        if(GET_GPREG(nr_bit, REG_GPIO_RESEN)){
            if(GET_GPREG(nr_bit, REG_GPIO_INEN)){
                if(GET_GPREG(nr_bit, REG_GPIO_PUPD)){
                    *func = GPIO_INPUT_PU;
                }else{
                    *func = GPIO_INPUT_PD;
                }
            }else{
                if(GET_GPREG(nr_bit, REG_GPIO_PUPD)){
                    *func = GPIO_PULLUP;
                }else{
                    *func = GPIO_PULLDN;
                }
            }
        }else{
            if(GET_GPREG(nr_bit, REG_GPIO_INEN)){
                *func = GPIO_INPUT;
            }else{
                *func = GPIO_NOSET;
            }
        }
    }

    RETURN1(0, " "MYNAME"::get_gpio_func() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * get_gpioX_func
 *---------------------------------------------------------------------------*/
static int get_gpioX_func(IN unsigned int nr, IN unsigned char *func)
{
    int err;
    unsigned int nr_bit;
    unsigned int nr_grp;
    unsigned char sel_func;
    PDEBUG1(" "MYNAME"::get_gpioX_func(%d,%x) : Entry\n", nr, (int)func);

    if((err = __get_nr_grpbit(nr, &nr_grp, &nr_bit)) < 0){
        RETURN1(err, " "MYNAME"::get_gpioX_func() : Return(%x)\n");
    }
    if((err = __get_nr_func(nr, &sel_func)) < 0){
        RETURN1(err, " "MYNAME"::get_gpioX_func() : Return(%x)\n");
    }
    /* Check port use */
    if(GET_GPREG(0, GET_SELREG(nr_bit, REG_GPIO_PORTSEL0_(nr_grp)))){
        if(! GET_GPREG(nr_bit, REG_GPIO_DIR_(nr_grp))){
            if(! GET_GPREG(nr_bit, REG_GPIO_HOLDEN_(nr_grp))){
                if(! GET_GPREG(nr_bit, REG_GPIO_RESEN_(nr_grp))){
                    if(! GET_GPREG(nr_bit, REG_GPIO_INEN_(nr_grp))){
                        sel_func = GPIO_FUNC_PERIHZ;
                    }
                }else{
                    if(! GET_GPREG(nr_bit, REG_GPIO_INEN_(nr_grp))){
                        if(GET_GPREG(nr_bit, REG_GPIO_PUPD)){
                            sel_func = GPIO_FUNC_PERIPU;
                        }else{
                            sel_func = GPIO_FUNC_PERIPU;
                        }
                    }
                }
            }
        }
        *func = sel_func;
        RETURN1(0, " "MYNAME"::get_gpioX_func() : Return(%x)\n");
    }
    /* Check direction */
    if(GET_GPREG(nr_bit, REG_GPIO_DIR_(nr_grp))){
        if(GET_GPREG(nr_bit, REG_GPIO_INEN_(nr_grp))){
            RETURN1(-EINVAL, " "MYNAME"::get_gpioX_func() : Return(%x)\n");
        }else{
            *func = GPIO_OUTPUT;
        }
    }else{
        if(GET_GPREG(nr_bit, REG_GPIO_HOLDEN_(nr_grp))){
            if(GET_GPREG(nr_bit, REG_GPIO_INEN_(nr_grp))){
                *func = GPIO_INPUT_BH;
            }else{
                *func = GPIO_BUSHD;
            }
        }else{
            if(GET_GPREG(nr_bit, REG_GPIO_RESEN_(nr_grp))){
                if(GET_GPREG(nr_bit, REG_GPIO_INEN_(nr_grp))){
                    if(GET_GPREG(nr_bit, REG_GPIO_PUPD_(nr_grp))){
                        *func = GPIO_INPUT_PU;
                    }else{
                        *func = GPIO_INPUT_PD;
                    }
                }else{
                    if(GET_GPREG(nr_bit, REG_GPIO_PUPD_(nr_grp))){
                        *func = GPIO_PULLUP;
                    }else{
                        *func = GPIO_PULLDN;
                    }
                }
            }else{
                if(GET_GPREG(nr_bit, REG_GPIO_INEN_(nr_grp))){
                    *func = GPIO_INPUT;
                }else{
                    *func = GPIO_NOSET;
                }
            }
        }
    }

    RETURN1(0, " "MYNAME"::get_gpioX_func() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * inline is_gpio
 *---------------------------------------------------------------------------*/
static __inline__ int __is_gpio(IN unsigned int nr)
{
    if(nr >= GPIO_0 && nr <= GPIO_15){
        return 1;
    }
    return 0;
}

/*-----------------------------------------------------------------------------
 * inline get_nr_grp
 *---------------------------------------------------------------------------*/
static __inline__ int __get_nr_grp(IN unsigned int nr, OUT unsigned int *grp)
{
    int i;
    if(nr > PORT_MAX){
        return -EINVAL;
    }
    for(i = ARRAY_SIZE(nr_base); i > 0; i--){
        if(nr >= nr_base[i - 1]){
            *grp = i - 1;
            return 0;
        }
    }
    return -EINVAL;
}

/*-----------------------------------------------------------------------------
 * inline adjust_nr_bit
 *---------------------------------------------------------------------------*/
static __inline__ void __adjust_nr_bit(IN unsigned int nr, 
    IN unsigned int grp, OUT unsigned int *bit)
{
    if(grp == 10){	/* GPIO10 */
        if(*bit >= 2 && *bit < 9){	/* GPIO10_2 not used */
            *bit += 1;
        }else if(*bit >= 9 && *bit < 14){	/* GPIO10_10 not used */
            *bit += 2;
        }
    }else if(grp == 16){	/* GPIO16 */
        if(*bit >= 8 && *bit < 10){	/* GPIO16_8 not used */
            *bit += 1;
        }
    }
    return;
}

/*-----------------------------------------------------------------------------
 * inline get_nr_grpbit
 *---------------------------------------------------------------------------*/
static __inline__ int __get_nr_grpbit(IN unsigned int nr, 
    OUT unsigned int *grp, OUT unsigned int *bit)
{
    int err;
    if((err = __get_nr_grp(nr, grp)) < 0){
        return err;
    }
    *bit = nr - nr_base[*grp];
    __adjust_nr_bit(nr, *grp, bit);
    return 0;
}

/*-----------------------------------------------------------------------------
 * inline check_nr_func
 *---------------------------------------------------------------------------*/
static __inline__ int __check_nr_func(IN unsigned int nr, 
    IN unsigned char func)
{
    int i;
    if(nr > PORT_MAX){
        return -EINVAL;
    }
    for(i = 0; i < ARRAY_SIZE(nr_func[nr]); i++){
        if(nr_func[nr][i] == func){
            return 1;
        }
    }
    return 0;
}

/*-----------------------------------------------------------------------------
 * inline check_func_HiZ
 *---------------------------------------------------------------------------*/
static __inline__ int __check_func_HiZ(IN unsigned char func)
{
    if(func == GPIO_FUNC_PERIHZ){
        return 1;
    }
    return 0;
}

/*-----------------------------------------------------------------------------
 * inline check_func_PullUp
 *---------------------------------------------------------------------------*/
static __inline__ int __check_func_PullUp(IN unsigned char func)
{
    if(func == GPIO_FUNC_PERIPU){
        return 1;
    }
    return 0;
}

/*-----------------------------------------------------------------------------
 * inline check_func_PullDown
 *---------------------------------------------------------------------------*/
static __inline__ int __check_func_PullDown(IN unsigned char func)
{
    if(func == GPIO_FUNC_PERIPD){
        return 1;
    }
    return 0;
}

/*-----------------------------------------------------------------------------
 * inline get_nr_func
 *---------------------------------------------------------------------------*/
static __inline__ int __get_nr_func(IN unsigned int nr, 
    OUT unsigned char *func)
{
    if(nr > PORT_MAX){
        return -EINVAL;
    }
    *func = nr_func[nr][0];
    return 0;
}

/*-----------------------------------------------------------------------------
 * inline suspend
 *---------------------------------------------------------------------------*/
static __inline__ int __suspend(IN struct amba_device *dev, IN u32 state)
{
    unsigned int i;
    PDEBUG1(" "MYNAME"::__suspend(%x) : Entry\n", (int)dev);

#ifdef CHECK_SIZE
    printk("__suspend() : sizeof(RegGpio) -> %d\n", sizeof(RegGpio));
#endif
    if(! (reggpio = kmalloc(sizeof(RegGpio), GFP_KERNEL | GFP_USER))){
        PDEBUG3(" "MYNAME"::__suspend() : kmalloc() Err\n");
        RETURN1(-ENOSPC, " "MYNAME"::__suspend() : Return(%x)\n");
    }
    /* gpio sys */
    reggpio->sys.dir = readw(REG_GPIO_DIR);
    reggpio->sys.rdata = readw(REG_GPIO_RDATA);
    reggpio->sys.intmask = readw(REG_GPIO_INTMASK);
    reggpio->sys.intle = readw(REG_GPIO_INTLE);
    reggpio->sys.inthe = readw(REG_GPIO_INTHE);
    reggpio->sys.ints = readw(REG_GPIO_INTS);
    reggpio->sys.inen = readw(REG_GPIO_INEN);
    reggpio->sys.resen = readw(REG_GPIO_RESEN);
    reggpio->sys.pupd = readw(REG_GPIO_PUPD);
    /* gpio easy */
    for(i = 0; i < ARRAY_SIZE(nr_base); i++){
        reggpio->easy[i].dir = readw(REG_GPIO_DIR_(i));
        reggpio->easy[i].rdata = readw(REG_GPIO_RDATA_(i));
        reggpio->easy[i].inen = readw(REG_GPIO_INEN_(i));
        reggpio->easy[i].portsel = readw(REG_GPIO_PORTSEL15_(i));
        reggpio->easy[i].portsel <<= 1;
        reggpio->easy[i].portsel |= readw(REG_GPIO_PORTSEL14_(i));
        reggpio->easy[i].portsel <<= 1;
        reggpio->easy[i].portsel |= readw(REG_GPIO_PORTSEL13_(i));
        reggpio->easy[i].portsel <<= 1;
        reggpio->easy[i].portsel |= readw(REG_GPIO_PORTSEL12_(i));
        reggpio->easy[i].portsel <<= 1;
        reggpio->easy[i].portsel |= readw(REG_GPIO_PORTSEL11_(i));
        reggpio->easy[i].portsel <<= 1;
        reggpio->easy[i].portsel |= readw(REG_GPIO_PORTSEL10_(i));
        reggpio->easy[i].portsel <<= 1;
        reggpio->easy[i].portsel |= readw(REG_GPIO_PORTSEL9_(i));
        reggpio->easy[i].portsel <<= 1;
        reggpio->easy[i].portsel |= readw(REG_GPIO_PORTSEL8_(i));
        reggpio->easy[i].portsel <<= 1;
        reggpio->easy[i].portsel |= readw(REG_GPIO_PORTSEL7_(i));
        reggpio->easy[i].portsel <<= 1;
        reggpio->easy[i].portsel |= readw(REG_GPIO_PORTSEL6_(i));
        reggpio->easy[i].portsel <<= 1;
        reggpio->easy[i].portsel |= readw(REG_GPIO_PORTSEL5_(i));
        reggpio->easy[i].portsel <<= 1;
        reggpio->easy[i].portsel |= readw(REG_GPIO_PORTSEL4_(i));
        reggpio->easy[i].portsel <<= 1;
        reggpio->easy[i].portsel |= readw(REG_GPIO_PORTSEL3_(i));
        reggpio->easy[i].portsel <<= 1;
        reggpio->easy[i].portsel |= readw(REG_GPIO_PORTSEL2_(i));
        reggpio->easy[i].portsel <<= 1;
        reggpio->easy[i].portsel |= readw(REG_GPIO_PORTSEL1_(i));
        reggpio->easy[i].portsel <<= 1;
        reggpio->easy[i].portsel |= readw(REG_GPIO_PORTSEL0_(i));
        reggpio->easy[i].holden = readw(REG_GPIO_HOLDEN_(i));
        reggpio->easy[i].resen = readw(REG_GPIO_RESEN_(i));
        reggpio->easy[i].pupd = readw(REG_GPIO_PUPD_(i));
    }

    RETURN1(0, " "MYNAME"::__suspend() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * inline resume
 *---------------------------------------------------------------------------*/
static __inline__ int __resume(IN struct amba_device *dev)
{
    unsigned int i;
    PDEBUG1(" "MYNAME"::__resume(%x) : Entry\n", (int)dev);

    if(reggpio){
        /* gpio sys */
        writew(reggpio->sys.pupd, REG_GPIO_PUPD);
        writew(reggpio->sys.resen, REG_GPIO_RESEN);
        writew(reggpio->sys.inen, REG_GPIO_INEN);
        writew(reggpio->sys.ints, REG_GPIO_INTS);
        writew(reggpio->sys.inthe, REG_GPIO_INTHE);
        writew(reggpio->sys.intle, REG_GPIO_INTLE);
        writew(reggpio->sys.intmask, REG_GPIO_INTMASK);
        writew(reggpio->sys.rdata, REG_GPIO_DATASET);
        writew(~reggpio->sys.rdata, REG_GPIO_DATACLR);
        writew(reggpio->sys.dir, REG_GPIO_DIR);
	/* gpio easy */
        for(i = 0; i < ARRAY_SIZE(nr_base); i++){
            writew(reggpio->easy[i].pupd, REG_GPIO_PUPD_(i));
            writew(reggpio->easy[i].resen, REG_GPIO_RESEN_(i));
            writew(reggpio->easy[i].holden, REG_GPIO_HOLDEN_(i));
            writew(reggpio->easy[i].portsel, REG_GPIO_PORTSEL0_(i));
            reggpio->easy[i].portsel >>= 1;
            writew(reggpio->easy[i].portsel, REG_GPIO_PORTSEL1_(i));
            reggpio->easy[i].portsel >>= 1;
            writew(reggpio->easy[i].portsel, REG_GPIO_PORTSEL2_(i));
            reggpio->easy[i].portsel >>= 1;
            writew(reggpio->easy[i].portsel, REG_GPIO_PORTSEL3_(i));
            reggpio->easy[i].portsel >>= 1;
            writew(reggpio->easy[i].portsel, REG_GPIO_PORTSEL4_(i));
            reggpio->easy[i].portsel >>= 1;
            writew(reggpio->easy[i].portsel, REG_GPIO_PORTSEL5_(i));
            reggpio->easy[i].portsel >>= 1;
            writew(reggpio->easy[i].portsel, REG_GPIO_PORTSEL6_(i));
            reggpio->easy[i].portsel >>= 1;
            writew(reggpio->easy[i].portsel, REG_GPIO_PORTSEL7_(i));
            reggpio->easy[i].portsel >>= 1;
            writew(reggpio->easy[i].portsel, REG_GPIO_PORTSEL8_(i));
            reggpio->easy[i].portsel >>= 1;
            writew(reggpio->easy[i].portsel, REG_GPIO_PORTSEL9_(i));
            reggpio->easy[i].portsel >>= 1;
            writew(reggpio->easy[i].portsel, REG_GPIO_PORTSEL10_(i));
            reggpio->easy[i].portsel >>= 1;
            writew(reggpio->easy[i].portsel, REG_GPIO_PORTSEL11_(i));
            reggpio->easy[i].portsel >>= 1;
            writew(reggpio->easy[i].portsel, REG_GPIO_PORTSEL12_(i));
            reggpio->easy[i].portsel >>= 1;
            writew(reggpio->easy[i].portsel, REG_GPIO_PORTSEL13_(i));
            reggpio->easy[i].portsel >>= 1;
            writew(reggpio->easy[i].portsel, REG_GPIO_PORTSEL14_(i));
            reggpio->easy[i].portsel >>= 1;
            writew(reggpio->easy[i].portsel, REG_GPIO_PORTSEL15_(i));
            writew(reggpio->easy[i].inen, REG_GPIO_INEN_(i));
            writew(reggpio->easy[i].rdata, REG_GPIO_DATASET_(i));
            writew(~reggpio->easy[i].rdata, REG_GPIO_DATACLR_(i));
            writew(reggpio->easy[i].dir, REG_GPIO_DIR_(i));
        }
        kfree(reggpio);
        reggpio = 0;
    }

    RETURN1(0, " "MYNAME"::__resume() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * inline print_register
 *--------------------------------------------------------------------------*/
static __inline__ void __print_register(void)
{
    unsigned int i;
    PDEBUG2("gpio_dir          = 0x%04x\n", readw(REG_GPIO_DIR));
    PDEBUG2("gpio_rdata        = 0x%04x\n", readw(REG_GPIO_RDATA));
    PDEBUG2("gpio_ints         = 0x%04x\n", readw(REG_GPIO_INTS));
    PDEBUG2("gpio_inthe        = 0x%04x\n", readw(REG_GPIO_INTHE));
    PDEBUG2("gpio_intle        = 0x%04x\n", readw(REG_GPIO_INTLE));
    PDEBUG2("gpio_intmask      = 0x%04x\n", readw(REG_GPIO_INTMASK));
    PDEBUG2("gpio_intst        = 0x%04x\n", readw(REG_GPIO_INTST));
    PDEBUG2("gpio_inen         = 0x%04x\n", readw(REG_GPIO_INEN));
    PDEBUG2("gpio_resen        = 0x%04x\n", readw(REG_GPIO_RESEN));
    PDEBUG2("gpio_pupd         = 0x%04x\n", readw(REG_GPIO_PUPD));
    for(i = 0; i < ARRAY_SIZE(nr_base); i++){
        PDEBUG2("gpio%d_dir        = 0x%04x\n", i, readw(REG_GPIO_DIR_(i)));
        PDEBUG2("gpio%d_rdata      = 0x%04x\n", i, readw(REG_GPIO_RDATA_(i)));
        PDEBUG2("gpio%d_inen       = 0x%04x\n", i, readw(REG_GPIO_INEN_(i)));
        PDEBUG2("gpio%d_portsel0   = 0x%04x\n", i, readw(REG_GPIO_PORTSEL0_(i)));
        PDEBUG2("gpio%d_portsel1   = 0x%04x\n", i, readw(REG_GPIO_PORTSEL1_(i)));
        PDEBUG2("gpio%d_portsel2   = 0x%04x\n", i, readw(REG_GPIO_PORTSEL2_(i)));
        PDEBUG2("gpio%d_portsel3   = 0x%04x\n", i, readw(REG_GPIO_PORTSEL3_(i)));
        PDEBUG2("gpio%d_portsel4   = 0x%04x\n", i, readw(REG_GPIO_PORTSEL4_(i)));
        PDEBUG2("gpio%d_portsel5   = 0x%04x\n", i, readw(REG_GPIO_PORTSEL5_(i)));
        PDEBUG2("gpio%d_portsel6   = 0x%04x\n", i, readw(REG_GPIO_PORTSEL6_(i)));
        PDEBUG2("gpio%d_portsel7   = 0x%04x\n", i, readw(REG_GPIO_PORTSEL7_(i)));
        PDEBUG2("gpio%d_portsel8   = 0x%04x\n", i, readw(REG_GPIO_PORTSEL8_(i)));
        PDEBUG2("gpio%d_portsel9   = 0x%04x\n", i, readw(REG_GPIO_PORTSEL9_(i)));
        PDEBUG2("gpio%d_portsel10  = 0x%04x\n", i, readw(REG_GPIO_PORTSEL10_(i)));
        PDEBUG2("gpio%d_portsel11  = 0x%04x\n", i, readw(REG_GPIO_PORTSEL11_(i)));
        PDEBUG2("gpio%d_portsel12  = 0x%04x\n", i, readw(REG_GPIO_PORTSEL12_(i)));
        PDEBUG2("gpio%d_portsel13  = 0x%04x\n", i, readw(REG_GPIO_PORTSEL13_(i)));
        PDEBUG2("gpio%d_portsel14  = 0x%04x\n", i, readw(REG_GPIO_PORTSEL14_(i)));
        PDEBUG2("gpio%d_portsel15  = 0x%04x\n", i, readw(REG_GPIO_PORTSEL15_(i)));
        PDEBUG2("gpio%d_holden     = 0x%04x\n", i, readw(REG_GPIO_HOLDEN_(i)));
        PDEBUG2("gpio%d_resen      = 0x%04x\n", i, readw(REG_GPIO_RESEN_(i)));
        PDEBUG2("gpio%d_pupd       = 0x%04x\n", i, readw(REG_GPIO_PUPD_(i)));
    }
    return;
}

/*=============================================================================
 * Export symbols
 *===========================================================================*/

//module_init(gpio_cxd4105_init);
module_exit(gpio_cxd4105_exit);
