/*-----------------------------------------------------------------------------
 * gpio-cxd4108.c
 *
 * a device driver file for gpio
 *
 * Copyright 2005,2006 Sony Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * Change note
 *
 * 01.00.000    : First release
 * 01.00.001    : changed name cxd4105 to cxd4108.
 *                added GPIO_FUNC_PERIHZ definition.
 *                added GPIO_FUNC_PERIPU definition.
 +                added GPIO_FUNC_PERIPD definition.
 *                added vbusdat / vbusaddr variables.
 *                changed set_gpio_func() / get_gpio_func() functions.
 *                changed set_gpioe_func() / get_gpioe_func() functions.
 *                changed set_gpioX_func() / get_gpioX_func() functions.
 * 01.00.002    : deleted vbusaddr variable.
 *                changed gpio_cxd4108_init() function.
 * 01.00.003    : deleted unnecessary definitions in header file.
 * 01.00.004    : changed Vsuspend() / Vresume() functions.
 *---------------------------------------------------------------------------*/
#define __GPIO_CXD4108_C__

/*-----------------------------------------------------------------------------
 * Include file
 *---------------------------------------------------------------------------*/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/spinlock.h>
#include <linux/interrupt.h>
#include <linux/device.h>
#include <linux/pm.h>
#include <linux/proc_fs.h>

#include <asm/io.h>
#include <asm/bitops.h>
#include <asm/arch/irqs.h>
#include <asm/arch/platform.h>

#include <asm/arch/gpio-core.h>
#include <asm/arch/gpio-cxd4108.h>

//#define CHECK_TIME
#ifdef CHECK_TIME
#include <sony/instrumental.h>
#endif
//#define CHECK_SIZE

/*-----------------------------------------------------------------------------
 * Macro declaration
 *---------------------------------------------------------------------------*/
#define MYNAME		"gpio-cxd4108"
#define MYVERSION	"01.00.004"

#define GPIO_PROC	"gpio"

#define CMD_GPIOSYS	"sys"
#define CMD_GPIOEASY	"easy"
#define CMD_GPIOCTRL	"ctrl"

#define MAX(a,b)	((a)>=(b)?(a):(b))
#define MIN(a,b)	((a)<=(b)?(a):(b))

#define GPIO_NR(x)	(x - GPIO_0)
#define GPIOE_NR(x)	(x - GPIOE_0)
#define GPIO_IRQ(x)	(IRQ_GPIO0 + GPIO_NR(x))

/*-----------------------------------------------------------------------------
 * Module infomation
 *---------------------------------------------------------------------------*/
MODULE_AUTHOR("Sony Corporation");
MODULE_DESCRIPTION("CXD4108 gpio driver");
MODULE_VERSION(MYVERSION);
MODULE_LICENSE("GPL");

/*-----------------------------------------------------------------------------
 * Compile switch
 *---------------------------------------------------------------------------*/
#define DEBUG_LEVEL 0

/*-[DEBUG_LEVEL 1]-------------------------------------------------
 * print value when function entry and exit                      */
#if(DEBUG_LEVEL == 1)
#define PDEBUG1(arg...)		{printk(arg);}
#define PDEBUG2(arg...)		{}
#define PDEBUG3(arg...)		{}
#define RETURN1(val,arg)	{printk(arg,val);return(val);}
/*-[DEBUG_LEVEL 2]-------------------------------------------------
 * print value of register                                       */
#elif(DEBUG_LEVEL == 2)
#define PDEBUG1(arg...)		{printk(arg);}
#define PDEBUG2(arg...)		{printk(arg);}
#define PDEBUG3(arg...)		{}
#define RETURN1(val,arg)	{printk(arg,val);__print_register();return(val);}
/*-[DEBUG_LEVEL 3]-------------------------------------------------
 * print value of variable                                       */
#elif(DEBUG_LEVEL == 3)
#define PDEBUG1(arg...)		{printk(arg);}
#define PDEBUG2(arg...)		{printk(arg);}
#define PDEBUG3(arg...)		{printk(arg);}
#define RETURN1(val,arg)	{printk(arg,val);return(val);}
/*-[NO DEBUG]------------------------------------------------------
 * print nothing                                                 */
#else
#define PDEBUG1(arg...)		{}
#define PDEBUG2(arg...)		{}
#define PDEBUG3(arg...)		{}
#define RETURN1(val,arg)	{return(val);}
#endif

/*-----------------------------------------------------------------------------
 * Parameter declaration
 *---------------------------------------------------------------------------*/
enum{
    CTRL_0 = 0,	/* reg */
    CTRL_1,
    CTRL_2,
    CTRL_3,
    CTRL_4,
    CTRL_5,
    CTRL_6,
    CTRL_7,
    CTRL_8,
    CTRL_9,
    CTRL_10,
    CTRL_11,
    CTRL_12,
    CTRL_13,
};

#define CTRL_MAX	CTRL_13
#define CTRL_NUM	(CTRL_13 - CTRL_0 + 1)

enum{
    BIT_0 = 0,	/* bit */
    BIT_1,
    BIT_2,
    BIT_3,
    BIT_4,
    BIT_5,
    BIT_6,
    BIT_7,
    BIT_8,
    BIT_9,
    BIT_10,
    BIT_11,
    BIT_12,
    BIT_13,
    BIT_14,
    BIT_15,
};

enum{
    NO_PU = 0,	/* pupd */
    NO_PD,
};

/*-----------------------------------------------------------------------------
 * Extern declaration
 *---------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 * Function prototype declaration
 *---------------------------------------------------------------------------*/
/* virtual function */
static int Vopen(IN unsigned int nr);
static int Vclose(IN unsigned int nr);
static int Vset_sel_func(IN unsigned int nr, IN unsigned char func, 
    IN unsigned int val);
static int Vget_sel_func(IN unsigned int nr, OUT unsigned char *func);
static int Vwrite(IN unsigned int nr, IN unsigned int val);
static int Vread(IN unsigned int nr);
static int Vset_irq_sense(IN unsigned int nr, IN unsigned char sense);
static int Vget_irq_sense(IN unsigned int nr, OUT unsigned char *sense);
static int Venable_irq(IN unsigned int nr);
static int Vdisable_irq(IN unsigned int nr);
static int Vis_enable_irq(IN unsigned int nr);
static int Vis_irq(IN unsigned int nr);
static int Vclear_irq(IN unsigned int nr);
static int Vregister_handler(IN unsigned int nr);
static int Vunregister_handler(IN unsigned int nr);
static int Venable_wakeup(IN unsigned int nr);
static int Vdisable_wakeup(IN unsigned int nr);
static int Vis_enable_wakeup(IN unsigned int nr);
static int Vsuspend(IN struct amba_device *dev, IN u32 state);
static int Vresume(IN struct amba_device *dev);

static int gpio_cxd4108_read_proc(char *buffer, char **start, off_t offset, 
    int length, int *eof, void *data);
static int gpio_cxd4108_write_proc(struct file *file, const char *buffer, 
    unsigned long count, void *data);
/*static*/ int __init gpio_cxd4108_init(void);
static void __exit gpio_cxd4108_exit(void);
static irqreturn_t handler(int irq, void *dev_id, struct pt_regs *fp);
static int set_gpio_func(IN unsigned int nr, IN unsigned char func, 
    IN unsigned int val);
static int set_gpioe_func(IN unsigned int nr, IN unsigned char func, 
    IN unsigned int val);
static int set_gpioX_func(IN unsigned int nr, IN unsigned char func, 
    IN unsigned int val);
static int get_gpio_func(IN unsigned int nr, IN unsigned char *func);
static int get_gpioe_func(IN unsigned int nr, IN unsigned char *func);
static int get_gpioX_func(IN unsigned int nr, IN unsigned char *func);

static __inline__ int __is_gpio(IN unsigned int nr);
static __inline__ int __is_gpioe(IN unsigned int nr);
static __inline__ int __get_nr_grp(IN unsigned int nr, OUT unsigned int *grp);
static __inline__ int __get_nr_grpbit(IN unsigned int nr, 
    OUT unsigned int *grp, OUT unsigned int *bit);
static __inline__ int __check_nr_func(IN unsigned int nr, 
    IN unsigned char func);
static __inline__ int __check_nr_ctrl(IN unsigned int nr);
static __inline__ int __check_nr_pupd(IN unsigned int nr, 
    IN unsigned char pupd);
static __inline__ int __check_func_HiZ(IN unsigned char func);
static __inline__ int __check_func_PullUp(IN unsigned char func);
static __inline__ int __check_func_PullDown(IN unsigned char func);
static __inline__ int __get_nr_func(IN unsigned int nr, 
    OUT unsigned char *func);
static __inline__ int __suspend(IN struct amba_device *dev, IN u32 state);
static __inline__ int __resume(IN struct amba_device *dev);
static __inline__ void __print_register(void);

/*-----------------------------------------------------------------------------
 * Register base
 *---------------------------------------------------------------------------*/
#ifndef CXD4108_APB2_BASE
#define CXD4108_APB2_BASE CXD4105_APB2_BASE
#endif

#define GPS_BASE	(CXD4108_APB2_BASE + SZ_1M * 7 + 0x00090000)
#define GPE_BASE_(x)	(CXD4108_APB2_BASE + SZ_1M * 7 + 0x00010000 + ((x) << 16))
#define GPE_BASE	(CXD4108_APB2_BASE + SZ_1M * 7 + 0x00080000)
#define GPC_BASE_(x)	(CXD4108_APB2_BASE + SZ_1M * 7 + 0x000A0000 + ((x) << 8))
#define GPP_BASE_(x)	(CXD4108_APB2_BASE + SZ_1M * 7 + 0x00070000 + ((x) << 4))
#define GPM_BASE	(CXD4108_APB2_BASE + SZ_1M * 7 + 0x000B0000)

/*-----------------------------------------------------------------------------
 * Register offset
 *---------------------------------------------------------------------------*/
#define INT_GSINT	0x00000370
#define INT_GSINTCLR	0x00000374

#define GPS_DIR		0x00000000
#define GPS_RDATA	0x00000004
#define GPS_DATASET	0x00000008
#define GPS_DATACLR	0x0000000C
#define GPS_INTS	0x00000010
#define GPS_INTHE	0x00000014
#define GPS_INTLE	0x00000018
#define GPS_INTMASK	0x0000001C
#define GPS_INTST	0x00000020
#define GPS_INTSTCLR	0x00000024
#define GPS_INEN	0x00000028

#define GPE_DIR		0x00000000
#define GPE_RDATA	0x00000004
#define GPE_DATASET	0x00000008
#define GPE_DATACLR	0x0000000C
#define GPE_INEN	0x00000010

#define GPC_DRV2	0x00000000
#define GPC_DRV2SET	0x00000004
#define GPC_DRV2CLR	0x00000008
#define GPC_DRV1	0x0000000C
#define GPC_DRV1SET	0x00000010
#define GPC_DRV1CLR	0x00000014
#define GPC_RESEN	0x00000018
#define GPC_RESENSET	0x0000001C
#define GPC_RESENCLR	0x00000020
#define GPC_PUPD	0x00000024
#define GPC_PUPDSET	0x00000028
#define GPC_PUPDCLR	0x0000002C

#define GPP_PORTSEL	0x00000000
#define GPP_PORTSET	0x00000004
#define GPP_PORTCLR	0x00000008

#define GPM_MISC3	0x00000300
#define GPM_MISC3SET	0x00000304
#define GPM_MISC3CLR	0x00000308

#define GPIO_MEMSPACE	0x000B0000

/*-----------------------------------------------------------------------------
 * Register assign
 *---------------------------------------------------------------------------*/
#define REG_INT_GSINT		INT_CTRL(INT_GSINT)
#define REG_INT_GSINTCLR	INT_CTRL(INT_GSINTCLR)

#define REG_GPS_DIR		IO_ADDRESS(GPS_BASE | GPS_DIR)
#define REG_GPS_RDATA		IO_ADDRESS(GPS_BASE | GPS_RDATA)
#define REG_GPS_DATASET		IO_ADDRESS(GPS_BASE | GPS_DATASET)
#define REG_GPS_DATACLR		IO_ADDRESS(GPS_BASE | GPS_DATACLR)
#define REG_GPS_INTS		IO_ADDRESS(GPS_BASE | GPS_INTS)
#define REG_GPS_INTHE		IO_ADDRESS(GPS_BASE | GPS_INTHE)
#define REG_GPS_INTLE		IO_ADDRESS(GPS_BASE | GPS_INTLE)
#define REG_GPS_INTMASK		IO_ADDRESS(GPS_BASE | GPS_INTMASK)
#define REG_GPS_INTST		IO_ADDRESS(GPS_BASE | GPS_INTST)
#define REG_GPS_INTSTCLR	IO_ADDRESS(GPS_BASE | GPS_INTSTCLR)
#define REG_GPS_INEN		IO_ADDRESS(GPS_BASE | GPS_INEN)

#define REG_GPE_DIR_(x)		IO_ADDRESS(GPE_BASE_(x) | GPE_DIR)
#define REG_GPE_RDATA_(x)	IO_ADDRESS(GPE_BASE_(x) | GPE_RDATA)
#define REG_GPE_DATASET_(x)	IO_ADDRESS(GPE_BASE_(x) | GPE_DATASET)
#define REG_GPE_DATACLR_(x)	IO_ADDRESS(GPE_BASE_(x) | GPE_DATACLR)
#define REG_GPE_INEN_(x)	IO_ADDRESS(GPE_BASE_(x) | GPE_INEN)
#define REG_GPE_PORTSEL_(x)	IO_ADDRESS(GPP_BASE_(x) | GPP_PORTSEL)
#define REG_GPE_PORTSET_(x)	IO_ADDRESS(GPP_BASE_(x) | GPP_PORTSET)
#define REG_GPE_PORTCLR_(x)	IO_ADDRESS(GPP_BASE_(x) | GPP_PORTCLR)

#define REG_GPE_DIR		IO_ADDRESS(GPE_BASE | GPE_DIR)
#define REG_GPE_RDATA		IO_ADDRESS(GPE_BASE | GPE_RDATA)
#define REG_GPE_DATASET		IO_ADDRESS(GPE_BASE | GPE_DATASET)
#define REG_GPE_DATACLR		IO_ADDRESS(GPE_BASE | GPE_DATACLR)
#define REG_GPE_INEN		IO_ADDRESS(GPE_BASE | GPE_INEN)

#define REG_GPC_DRV2_(x)	IO_ADDRESS(GPC_BASE_(x) | GPC_DRV2)
#define REG_GPC_DRV2SET_(x)	IO_ADDRESS(GPC_BASE_(x) | GPC_DRV2SET)
#define REG_GPC_DRV2CLR_(x)	IO_ADDRESS(GPC_BASE_(x) | GPC_DRV2CLR)
#define REG_GPC_DRV1_(x)	IO_ADDRESS(GPC_BASE_(x) | GPC_DRV1)
#define REG_GPC_DRV1SET_(x)	IO_ADDRESS(GPC_BASE_(x) | GPC_DRV1SET)
#define REG_GPC_DRV1CLR_(x)	IO_ADDRESS(GPC_BASE_(x) | GPC_DRV1CLR)
#define REG_GPC_RESEN_(x)	IO_ADDRESS(GPC_BASE_(x) | GPC_RESEN)
#define REG_GPC_RESENSET_(x)	IO_ADDRESS(GPC_BASE_(x) | GPC_RESENSET)
#define REG_GPC_RESENCLR_(x)	IO_ADDRESS(GPC_BASE_(x) | GPC_RESENCLR)
#define REG_GPC_PUPD_(x)	IO_ADDRESS(GPC_BASE_(x) | GPC_PUPD)
#define REG_GPC_PUPDSET_(x)	IO_ADDRESS(GPC_BASE_(x) | GPC_PUPDSET)
#define REG_GPC_PUPDCLR_(x)	IO_ADDRESS(GPC_BASE_(x) | GPC_PUPDCLR)

#define REG_GPM_MISC3		IO_ADDRESS(GPM_BASE | GPM_MISC3)
#define REG_GPM_MISC3SET	IO_ADDRESS(GPM_BASE | GPM_MISC3SET)
#define REG_GPM_MISC3CLR	IO_ADDRESS(GPM_BASE | GPM_MISC3CLR)

/*-----------------------------------------------------------------------------
 * Register mask
 *---------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 * Register operation
 *---------------------------------------------------------------------------*/
#define SET_GPREG(nr,reg)	(writew(readw(reg)|(1<<(nr)),reg))
#define CLR_GPREG(nr,reg)	(writew(readw(reg)&~(1<<(nr)),reg))

#define GET_GPREG(nr,reg)	((readw(reg)>>(nr))&1)
#define SET_GPREG2(nr,reg)	(writew(1<<(nr),reg))

/*-----------------------------------------------------------------------------
 * Variable declaration
 *---------------------------------------------------------------------------*/
static TGpioPluginOps Vops = 
{
    Vopen,
    Vclose,
    Vset_sel_func,
    Vget_sel_func,
    Vwrite,
    Vread,
    Vset_irq_sense,
    Vget_irq_sense,
    Venable_irq,
    Vdisable_irq,
    Vis_enable_irq,
    Vis_irq,
    Vclear_irq,
    Vregister_handler,
    Vunregister_handler,
    Venable_wakeup,
    Vdisable_wakeup,
    Vis_enable_wakeup,
    Vsuspend,
    Vresume,
};

static CGpioPlugin gpio_cxd4108 = 
{
    MYNAME,
    MYVERSION,
    (PORT_MAX + 1),
    &Vops,
    GPE_BASE_(0),
    GPE_BASE_(0) + GPIO_MEMSPACE - 1,
};

static const int nr_base[] = 
{
    GPIO0_0, GPIO1_0, GPIO2_0, GPIO3_0, GPIO4_0, GPIO5_0,
};

static const unsigned char nr_func[][4] = 
{
    [GPIO0_0][0]  = GPIO_FUNC_TPU0,	[GPIO0_0][1]  = GPIO_FUNC_PERIHZ,
    [GPIO0_0][2]  = GPIO_FUNC_PERIPU,	[GPIO0_0][3]  = GPIO_FUNC_PERIPD,
    [GPIO0_1][0]  = GPIO_FUNC_TPU1,	[GPIO0_1][1]  = GPIO_FUNC_PERIHZ,
    [GPIO0_1][2]  = GPIO_FUNC_PERIPU,	[GPIO0_1][3]  = GPIO_FUNC_PERIPD,
    [GPIO0_2][0]  = GPIO_FUNC_TPU2,	[GPIO0_2][1]  = GPIO_FUNC_PERIHZ,
    [GPIO0_2][2]  = GPIO_FUNC_PERIPU,	[GPIO0_2][3]  = GPIO_FUNC_PERIPD,
    [GPIO0_3][0]  = GPIO_FUNC_TPU3,	[GPIO0_3][1]  = GPIO_FUNC_PERIHZ,
    [GPIO0_3][2]  = GPIO_FUNC_PERIPU,	[GPIO0_3][3]  = GPIO_FUNC_PERIPD,
    [GPIO0_4][0]  = GPIO_FUNC_TPU0,	[GPIO0_4][1]  = GPIO_FUNC_PERIHZ,
    [GPIO0_4][2]  = GPIO_FUNC_PERIPU,	[GPIO0_4][3]  = GPIO_FUNC_PERIPD,
    [GPIO0_5][0]  = GPIO_FUNC_TPU0,	[GPIO0_5][1]  = GPIO_FUNC_PERIHZ,
    [GPIO0_5][2]  = GPIO_FUNC_PERIPU,	[GPIO0_5][3]  = GPIO_FUNC_PERIPD,
    [GPIO0_6][0]  = GPIO_FUNC_UART0,	[GPIO0_6][1]  = GPIO_FUNC_PERIHZ,
    [GPIO0_6][2]  = GPIO_FUNC_PERIPU,	[GPIO0_6][3]  = GPIO_FUNC_PERIPD,
    [GPIO0_7][0]  = GPIO_FUNC_UART0,	[GPIO0_7][1]  = GPIO_FUNC_PERIHZ,
    [GPIO0_7][2]  = GPIO_FUNC_PERIPU,	[GPIO0_7][3]  = GPIO_FUNC_PERIPD,
    [GPIO0_8][0]  = GPIO_FUNC_UART0,	[GPIO0_8][1]  = GPIO_FUNC_PERIHZ,
    [GPIO0_8][2]  = GPIO_FUNC_PERIPU,	[GPIO0_8][3]  = GPIO_FUNC_PERIPD,
    [GPIO0_9][0]  = GPIO_FUNC_UART0,	[GPIO0_9][1]  = GPIO_FUNC_PERIHZ,
    [GPIO0_9][2]  = GPIO_FUNC_PERIPU,	[GPIO0_9][3]  = GPIO_FUNC_PERIPD,
    [GPIO0_10][0] = GPIO_FUNC_UART1,	[GPIO0_10][1] = GPIO_FUNC_PERIHZ,
    [GPIO0_10][2] = GPIO_FUNC_PERIPU,	[GPIO0_10][3] = GPIO_FUNC_PERIPD,
    [GPIO0_11][0] = GPIO_FUNC_UART1,	[GPIO0_11][1] = GPIO_FUNC_PERIHZ,
    [GPIO0_11][2] = GPIO_FUNC_PERIPU,	[GPIO0_11][3] = GPIO_FUNC_PERIPD,
    [GPIO0_12][0] = GPIO_FUNC_UART1,	[GPIO0_12][1] = GPIO_FUNC_PERIHZ,
    [GPIO0_12][2] = GPIO_FUNC_PERIPU,	[GPIO0_12][3] = GPIO_FUNC_PERIPD,
    [GPIO0_13][0] = GPIO_FUNC_UART1,	[GPIO0_13][1] = GPIO_FUNC_PERIHZ,
    [GPIO0_13][2] = GPIO_FUNC_PERIPU,	[GPIO0_13][3] = GPIO_FUNC_PERIPD,
    [GPIO0_14][0] = GPIO_FUNC_I2C,	[GPIO0_14][1] = GPIO_FUNC_PERIHZ,
    [GPIO0_14][2] = -1,			[GPIO0_14][3] = -1,
    [GPIO0_15][0] = GPIO_FUNC_I2C,	[GPIO0_15][1] = GPIO_FUNC_PERIHZ,
    [GPIO0_15][2] = -1,			[GPIO0_15][3] = -1,

    [GPIO1_0][0]  = GPIO_FUNC_SIO0,	[GPIO1_0][1]  = GPIO_FUNC_PERIHZ,
    [GPIO1_0][2]  = GPIO_FUNC_PERIPU,	[GPIO1_0][3]  = GPIO_FUNC_PERIPD,
    [GPIO1_1][0]  = GPIO_FUNC_SIO0,	[GPIO1_1][1]  = GPIO_FUNC_PERIHZ,
    [GPIO1_1][2]  = GPIO_FUNC_PERIPU,	[GPIO1_1][3]  = GPIO_FUNC_PERIPD,
    [GPIO1_2][0]  = GPIO_FUNC_SIO0,	[GPIO1_2][1]  = GPIO_FUNC_PERIHZ,
    [GPIO1_2][2]  = GPIO_FUNC_PERIPU,	[GPIO1_2][3]  = GPIO_FUNC_PERIPD,
    [GPIO1_3][0]  = GPIO_FUNC_SIO0,	[GPIO1_3][1]  = GPIO_FUNC_PERIHZ,
    [GPIO1_3][2]  = GPIO_FUNC_PERIPU,	[GPIO1_3][3]  = GPIO_FUNC_PERIPD,
    [GPIO1_4][0]  = GPIO_FUNC_SIO1,	[GPIO1_4][1]  = GPIO_FUNC_PERIHZ,
    [GPIO1_4][2]  = GPIO_FUNC_PERIPU,	[GPIO1_4][3]  = GPIO_FUNC_PERIPD,
    [GPIO1_5][0]  = GPIO_FUNC_SIO1,	[GPIO1_5][1]  = GPIO_FUNC_PERIHZ,
    [GPIO1_5][2]  = GPIO_FUNC_PERIPU,	[GPIO1_5][3]  = GPIO_FUNC_PERIPD,
    [GPIO1_6][0]  = GPIO_FUNC_SIO1,	[GPIO1_6][1]  = GPIO_FUNC_PERIHZ,
    [GPIO1_6][2]  = GPIO_FUNC_PERIPU,	[GPIO1_6][3]  = GPIO_FUNC_PERIPD,
    [GPIO1_7][0]  = GPIO_FUNC_SIO2,	[GPIO1_7][1]  = GPIO_FUNC_PERIHZ,
    [GPIO1_7][2]  = GPIO_FUNC_PERIPU,	[GPIO1_7][3]  = GPIO_FUNC_PERIPD,
    [GPIO1_8][0]  = GPIO_FUNC_SIO2,	[GPIO1_8][1]  = GPIO_FUNC_PERIHZ,
    [GPIO1_8][2]  = GPIO_FUNC_PERIPU,	[GPIO1_8][3]  = GPIO_FUNC_PERIPD,
    [GPIO1_9][0]  = GPIO_FUNC_SIO2,	[GPIO1_9][1]  = GPIO_FUNC_PERIHZ,
    [GPIO1_9][2]  = GPIO_FUNC_PERIPU,	[GPIO1_9][3]  = GPIO_FUNC_PERIPD,
    [GPIO1_10][0] = GPIO_FUNC_SIO3,	[GPIO1_10][1] = GPIO_FUNC_PERIHZ,
    [GPIO1_10][2] = GPIO_FUNC_PERIPU,	[GPIO1_10][3] = GPIO_FUNC_PERIPD,
    [GPIO1_11][0] = GPIO_FUNC_SIO3,	[GPIO1_11][1] = GPIO_FUNC_PERIHZ,
    [GPIO1_11][2] = GPIO_FUNC_PERIPU,	[GPIO1_11][3] = GPIO_FUNC_PERIPD,
    [GPIO1_12][0] = GPIO_FUNC_SIO3,	[GPIO1_12][1] = GPIO_FUNC_PERIHZ,
    [GPIO1_12][2] = GPIO_FUNC_PERIPU,	[GPIO1_12][3] = GPIO_FUNC_PERIPD,
    [GPIO1_13][0] = GPIO_FUNC_SIR,	[GPIO1_13][1] = GPIO_FUNC_PERIHZ,
    [GPIO1_13][2] = GPIO_FUNC_PERIPU,	[GPIO1_13][3] = GPIO_FUNC_PERIPD,
    [GPIO1_14][0] = GPIO_FUNC_USBVBUS,	[GPIO1_14][1] = GPIO_FUNC_PERIHZ,
    [GPIO1_14][2] = GPIO_FUNC_PERIPU,	[GPIO1_14][3] = GPIO_FUNC_PERIPD,
//    [GPIO1_15] = ,	/* not used */

    [GPIO2_0][0]  = GPIO_FUNC_EMC,	[GPIO2_0][1]  = GPIO_FUNC_PERIHZ,
    [GPIO2_0][2]  = GPIO_FUNC_PERIPD,	[GPIO2_0][3]  = -1,
    [GPIO2_1][0]  = GPIO_FUNC_EMC,	[GPIO2_1][1]  = GPIO_FUNC_PERIHZ,
    [GPIO2_1][2]  = GPIO_FUNC_PERIPD,	[GPIO2_1][3]  = -1,
    [GPIO2_2][0]  = GPIO_FUNC_EMC,	[GPIO2_2][1]  = GPIO_FUNC_PERIHZ,
    [GPIO2_2][2]  = GPIO_FUNC_PERIPD,	[GPIO2_2][3]  = -1,
    [GPIO2_3][0]  = GPIO_FUNC_EMC,	[GPIO2_3][1]  = GPIO_FUNC_PERIHZ,
    [GPIO2_3][2]  = GPIO_FUNC_PERIPD,	[GPIO2_3][3]  = -1,
    [GPIO2_4][0]  = GPIO_FUNC_EMC,	[GPIO2_4][1]  = GPIO_FUNC_PERIHZ,
    [GPIO2_4][2]  = GPIO_FUNC_PERIPD,	[GPIO2_4][3]  = -1,
    [GPIO2_5][0]  = GPIO_FUNC_EMC,	[GPIO2_5][1]  = GPIO_FUNC_PERIHZ,
    [GPIO2_5][2]  = GPIO_FUNC_PERIPD,	[GPIO2_5][3]  = -1,
    [GPIO2_6][0]  = GPIO_FUNC_EMC,	[GPIO2_6][1]  = GPIO_FUNC_PERIHZ,
    [GPIO2_6][2]  = GPIO_FUNC_PERIPD,	[GPIO2_6][3]  = -1,
    [GPIO2_7][0]  = GPIO_FUNC_EMC,	[GPIO2_7][1]  = GPIO_FUNC_PERIHZ,
    [GPIO2_7][2]  = GPIO_FUNC_PERIPD,	[GPIO2_7][3]  = -1,
    [GPIO2_8][0]  = GPIO_FUNC_EMC,	[GPIO2_8][1]  = GPIO_FUNC_PERIHZ,
    [GPIO2_8][2]  = GPIO_FUNC_PERIPD,	[GPIO2_8][3]  = -1,
    [GPIO2_9][0]  = GPIO_FUNC_EMC,	[GPIO2_9][1]  = GPIO_FUNC_PERIHZ,
    [GPIO2_9][2]  = GPIO_FUNC_PERIPD,	[GPIO2_9][3]  = -1,
    [GPIO2_10][0] = GPIO_FUNC_EMC,	[GPIO2_10][1] = GPIO_FUNC_PERIHZ,
    [GPIO2_10][2] = GPIO_FUNC_PERIPD,	[GPIO2_10][3] = -1,
    [GPIO2_11][0] = GPIO_FUNC_EMC,	[GPIO2_11][1] = GPIO_FUNC_PERIHZ,
    [GPIO2_11][2] = GPIO_FUNC_PERIPD,	[GPIO2_11][3] = -1,
    [GPIO2_12][0] = GPIO_FUNC_EMC,	[GPIO2_12][1] = GPIO_FUNC_PERIHZ,
    [GPIO2_12][2] = GPIO_FUNC_PERIPD,	[GPIO2_12][3] = -1,
    [GPIO2_13][0] = GPIO_FUNC_EMC,	[GPIO2_13][1] = GPIO_FUNC_PERIHZ,
    [GPIO2_13][2] = GPIO_FUNC_PERIPD,	[GPIO2_13][3] = -1,
    [GPIO2_14][0] = GPIO_FUNC_CAMPERI,	[GPIO2_14][1] = GPIO_FUNC_PERIHZ,
    [GPIO2_14][2] = GPIO_FUNC_PERIPU,	[GPIO2_14][3] = GPIO_FUNC_PERIPD,
//    [GPIO2_15] = ,	/* not used */

    [GPIO3_0][0]  = GPIO_FUNC_ETC,	[GPIO3_0][1]  = GPIO_FUNC_PERIHZ,
    [GPIO3_0][2]  = GPIO_FUNC_PERIPU,	[GPIO3_0][3]  = GPIO_FUNC_PERIPD,
    [GPIO3_1][0]  = GPIO_FUNC_ETC,	[GPIO3_1][1]  = GPIO_FUNC_PERIHZ,
    [GPIO3_1][2]  = GPIO_FUNC_PERIPU,	[GPIO3_1][3]  = GPIO_FUNC_PERIPD,
    [GPIO3_2][0]  = GPIO_FUNC_ETC,	[GPIO3_2][1]  = GPIO_FUNC_PERIHZ,
    [GPIO3_2][2]  = GPIO_FUNC_PERIPU,	[GPIO3_2][3]  = GPIO_FUNC_PERIPD,
    [GPIO3_3][0]  = GPIO_FUNC_ETC,	[GPIO3_3][1]  = GPIO_FUNC_PERIHZ,
    [GPIO3_3][2]  = GPIO_FUNC_PERIPU,	[GPIO3_3][3]  = GPIO_FUNC_PERIPD,
    [GPIO3_4][0]  = GPIO_FUNC_ETC,	[GPIO3_4][1]  = GPIO_FUNC_PERIHZ,
    [GPIO3_4][2]  = GPIO_FUNC_PERIPU,	[GPIO3_4][3]  = GPIO_FUNC_PERIPD,
    [GPIO3_5][0]  = GPIO_FUNC_AU,	[GPIO3_5][1]  = GPIO_FUNC_PERIHZ,
    [GPIO3_5][2]  = GPIO_FUNC_PERIPU,	[GPIO3_5][3]  = GPIO_FUNC_PERIPD,
    [GPIO3_6][0]  = GPIO_FUNC_AU,	[GPIO3_6][1]  = GPIO_FUNC_PERIHZ,
    [GPIO3_6][2]  = GPIO_FUNC_PERIPU,	[GPIO3_6][3]  = GPIO_FUNC_PERIPD,
    [GPIO3_7][0]  = GPIO_FUNC_AU,	[GPIO3_7][1]  = GPIO_FUNC_PERIHZ,
    [GPIO3_7][2]  = GPIO_FUNC_PERIPU,	[GPIO3_7][3]  = GPIO_FUNC_PERIPD,
    [GPIO3_8][0]  = GPIO_FUNC_AU,	[GPIO3_8][1]  = GPIO_FUNC_PERIHZ,
    [GPIO3_8][2]  = GPIO_FUNC_PERIPU,	[GPIO3_8][3]  = GPIO_FUNC_PERIPD,
    [GPIO3_9][0]  = GPIO_FUNC_AU,	[GPIO3_9][1]  = GPIO_FUNC_PERIHZ,
    [GPIO3_9][2]  = GPIO_FUNC_PERIPU,	[GPIO3_9][3]  = GPIO_FUNC_PERIPD,
    [GPIO3_10][0] = GPIO_FUNC_AU,	[GPIO3_10][1] = GPIO_FUNC_PERIHZ,
    [GPIO3_10][2] = GPIO_FUNC_PERIPU,	[GPIO3_10][3] = GPIO_FUNC_PERIPD,
    [GPIO3_11][0] = GPIO_FUNC_SA,	[GPIO3_11][1] = GPIO_FUNC_PERIHZ,
    [GPIO3_11][2] = GPIO_FUNC_PERIPU,	[GPIO3_11][3] = GPIO_FUNC_PERIPD,
    [GPIO3_12][0] = GPIO_FUNC_SA,	[GPIO3_12][1] = GPIO_FUNC_PERIHZ,
    [GPIO3_12][2] = GPIO_FUNC_PERIPU,	[GPIO3_12][3] = GPIO_FUNC_PERIPD,
//    [GPIO3_13] = ,	/* not used */
//    [GPIO3_14] = ,	/* not used */
//    [GPIO3_15] = ,	/* not used */

    [GPIO4_0][0]  = GPIO_FUNC_BR3,	[GPIO4_0][1]  = GPIO_FUNC_PERIHZ,
    [GPIO4_0][2]  = GPIO_FUNC_PERIPU,	[GPIO4_0][3]  = GPIO_FUNC_PERIPD,
    [GPIO4_1][0]  = GPIO_FUNC_BR3,	[GPIO4_1][1]  = GPIO_FUNC_PERIHZ,
    [GPIO4_1][2]  = GPIO_FUNC_PERIPU,	[GPIO4_1][3]  = GPIO_FUNC_PERIPD,
    [GPIO4_2][0]  = GPIO_FUNC_BR3,	[GPIO4_2][1]  = GPIO_FUNC_PERIHZ,
    [GPIO4_2][2]  = GPIO_FUNC_PERIPU,	[GPIO4_2][3]  = GPIO_FUNC_PERIPD,
    [GPIO4_3][0]  = GPIO_FUNC_BR3,	[GPIO4_3][1]  = GPIO_FUNC_PERIHZ,
    [GPIO4_3][2]  = GPIO_FUNC_PERIPU,	[GPIO4_3][3]  = GPIO_FUNC_PERIPD,
    [GPIO4_4][0]  = GPIO_FUNC_BR3,	[GPIO4_4][1]  = GPIO_FUNC_PERIHZ,
    [GPIO4_4][2]  = GPIO_FUNC_PERIPU,	[GPIO4_4][3]  = GPIO_FUNC_PERIPD,
    [GPIO4_5][0]  = GPIO_FUNC_BR3,	[GPIO4_5][1]  = GPIO_FUNC_PERIHZ,
    [GPIO4_5][2]  = GPIO_FUNC_PERIPU,	[GPIO4_5][3]  = GPIO_FUNC_PERIPD,
    [GPIO4_6][0]  = GPIO_FUNC_BR3,	[GPIO4_6][1]  = GPIO_FUNC_PERIHZ,
    [GPIO4_6][2]  = GPIO_FUNC_PERIPU,	[GPIO4_6][3]  = GPIO_FUNC_PERIPD,
    [GPIO4_7][0]  = GPIO_FUNC_BR3,	[GPIO4_7][1]  = GPIO_FUNC_PERIHZ,
    [GPIO4_7][2]  = GPIO_FUNC_PERIPU,	[GPIO4_7][3]  = GPIO_FUNC_PERIPD,
    [GPIO4_8][0]  = GPIO_FUNC_BR3,	[GPIO4_8][1]  = GPIO_FUNC_PERIHZ,
    [GPIO4_8][2]  = GPIO_FUNC_PERIPU,	[GPIO4_8][3]  = GPIO_FUNC_PERIPD,
    [GPIO4_9][0]  = GPIO_FUNC_BR3,	[GPIO4_9][1]  = GPIO_FUNC_PERIHZ,
    [GPIO4_9][2]  = GPIO_FUNC_PERIPU,	[GPIO4_9][3]  = GPIO_FUNC_PERIPD,
    [GPIO4_10][0] = GPIO_FUNC_BR3,	[GPIO4_10][1] = GPIO_FUNC_PERIHZ,
    [GPIO4_10][2] = GPIO_FUNC_PERIPU,	[GPIO4_10][3] = GPIO_FUNC_PERIPD,
    [GPIO4_11][0] = GPIO_FUNC_BR3,	[GPIO4_11][1] = GPIO_FUNC_PERIHZ,
    [GPIO4_11][2] = GPIO_FUNC_PERIPU,	[GPIO4_11][3] = GPIO_FUNC_PERIPD,
    [GPIO4_12][0] = GPIO_FUNC_BR3,	[GPIO4_12][1] = GPIO_FUNC_PERIHZ,
    [GPIO4_12][2] = GPIO_FUNC_PERIPU,	[GPIO4_12][3] = GPIO_FUNC_PERIPD,
    [GPIO4_13][0] = GPIO_FUNC_BR3,	[GPIO4_13][1] = GPIO_FUNC_PERIHZ,
    [GPIO4_13][2] = GPIO_FUNC_PERIPU,	[GPIO4_13][3] = GPIO_FUNC_PERIPD,
    [GPIO4_14][0] = GPIO_FUNC_TEST,	[GPIO4_14][1] = GPIO_FUNC_PERIHZ,
    [GPIO4_14][2] = GPIO_FUNC_PERIPU,	[GPIO4_14][3] = GPIO_FUNC_PERIPD,
    [GPIO4_15][0] = GPIO_FUNC_TEST,	[GPIO4_15][1] = GPIO_FUNC_PERIHZ,
    [GPIO4_15][2] = GPIO_FUNC_PERIPU,	[GPIO4_15][3] = GPIO_FUNC_PERIPD,

    [GPIO5_0][0]  = GPIO_FUNC_MS,	[GPIO5_0][1]  = GPIO_FUNC_PERIHZ,
    [GPIO5_0][2]  = GPIO_FUNC_PERIPU,	[GPIO5_0][3]  = GPIO_FUNC_PERIPD,
    [GPIO5_1][0]  = GPIO_FUNC_MS,	[GPIO5_1][1]  = GPIO_FUNC_PERIHZ,
    [GPIO5_1][2]  = GPIO_FUNC_PERIPU,	[GPIO5_1][3]  = GPIO_FUNC_PERIPD,
    [GPIO5_2][0]  = GPIO_FUNC_MS,	[GPIO5_2][1]  = GPIO_FUNC_PERIHZ,
    [GPIO5_2][2]  = GPIO_FUNC_PERIPU,	[GPIO5_2][3]  = GPIO_FUNC_PERIPD,
    [GPIO5_3][0]  = GPIO_FUNC_MS,	[GPIO5_3][1]  = GPIO_FUNC_PERIHZ,
    [GPIO5_3][2]  = GPIO_FUNC_PERIPU,	[GPIO5_3][3]  = GPIO_FUNC_PERIPD,
    [GPIO5_4][0]  = GPIO_FUNC_MS,	[GPIO5_4][1]  = GPIO_FUNC_PERIHZ,
    [GPIO5_4][2]  = GPIO_FUNC_PERIPU,	[GPIO5_4][3]  = GPIO_FUNC_PERIPD,
    [GPIO5_5][0]  = GPIO_FUNC_MS,	[GPIO5_5][1]  = GPIO_FUNC_PERIHZ,
    [GPIO5_5][2]  = GPIO_FUNC_PERIPU,	[GPIO5_5][3]  = GPIO_FUNC_PERIPD,
    [GPIO5_6][0]  = GPIO_FUNC_SD,	[GPIO5_6][1]  = GPIO_FUNC_PERIHZ,
    [GPIO5_6][2]  = GPIO_FUNC_PERIPU,	[GPIO5_6][3]  = GPIO_FUNC_PERIPD,
    [GPIO5_7][0]  = GPIO_FUNC_SD,	[GPIO5_7][1]  = GPIO_FUNC_PERIHZ,
    [GPIO5_7][2]  = GPIO_FUNC_PERIPU,	[GPIO5_7][3]  = GPIO_FUNC_PERIPD,
    [GPIO5_8][0]  = GPIO_FUNC_SD,	[GPIO5_8][1]  = GPIO_FUNC_PERIHZ,
    [GPIO5_8][2]  = GPIO_FUNC_PERIPU,	[GPIO5_8][3]  = GPIO_FUNC_PERIPD,
    [GPIO5_9][0]  = GPIO_FUNC_SD,	[GPIO5_9][1]  = GPIO_FUNC_PERIHZ,
    [GPIO5_9][2]  = GPIO_FUNC_PERIPU,	[GPIO5_9][3]  = GPIO_FUNC_PERIPD,
    [GPIO5_10][0] = GPIO_FUNC_SD,	[GPIO5_10][1] = GPIO_FUNC_PERIHZ,
    [GPIO5_10][2] = GPIO_FUNC_PERIPU,	[GPIO5_10][3] = GPIO_FUNC_PERIPD,
    [GPIO5_11][0] = GPIO_FUNC_SD,	[GPIO5_11][1] = GPIO_FUNC_PERIHZ,
    [GPIO5_11][2] = GPIO_FUNC_PERIPU,	[GPIO5_11][3] = GPIO_FUNC_PERIPD,
    [GPIO5_12][0] = GPIO_FUNC_SD,	[GPIO5_12][1] = GPIO_FUNC_PERIHZ,
    [GPIO5_12][2] = GPIO_FUNC_PERIPU,	[GPIO5_12][3] = GPIO_FUNC_PERIPD,
    [GPIO5_13][0] = GPIO_FUNC_SD,	[GPIO5_13][1] = GPIO_FUNC_PERIHZ,
    [GPIO5_13][2] = GPIO_FUNC_PERIPU,	[GPIO5_13][3] = GPIO_FUNC_PERIPD,
//    [GPIO5_14] = ,	/* not used */
//    [GPIO5_15] = ,	/* not used */
};

static const unsigned char nr_ctrl[][3] = 
{
/*--/reg------------------------/bit--------------------/pupd-----------------*/
    [GPIO_0][0]   = CTRL_1,	[GPIO_0][1]   = BIT_0,	[GPIO_0][2]   = -1,
    [GPIO_1][0]   = CTRL_1,	[GPIO_1][1]   = BIT_1,	[GPIO_1][2]   = -1,
    [GPIO_2][0]   = CTRL_1,	[GPIO_2][1]   = BIT_2,	[GPIO_2][2]   = -1,
    [GPIO_3][0]   = CTRL_1,	[GPIO_3][1]   = BIT_3,	[GPIO_3][2]   = -1,
    [GPIO_4][0]   = CTRL_1,	[GPIO_4][1]   = BIT_4,	[GPIO_4][2]   = -1,
    [GPIO_5][0]   = CTRL_1,	[GPIO_5][1]   = BIT_5,	[GPIO_5][2]   = -1,
    [GPIO_6][0]   = CTRL_1,	[GPIO_6][1]   = BIT_6,	[GPIO_6][2]   = -1,
    [GPIO_7][0]   = CTRL_1,	[GPIO_7][1]   = BIT_7,	[GPIO_7][2]   = -1,
    [GPIO_8][0]   = CTRL_1,	[GPIO_8][1]   = BIT_8,	[GPIO_8][2]   = -1,
    [GPIO_9][0]   = CTRL_1,	[GPIO_9][1]   = BIT_9,	[GPIO_9][2]   = -1,
    [GPIO_10][0]  = CTRL_1,	[GPIO_10][1]  = BIT_10,	[GPIO_10][2]  = -1,
    [GPIO_11][0]  = CTRL_1,	[GPIO_11][1]  = BIT_11,	[GPIO_11][2]  = -1,
    [GPIO_12][0]  = CTRL_1,	[GPIO_12][1]  = BIT_12,	[GPIO_12][2]  = -1,
    [GPIO_13][0]  = CTRL_1,	[GPIO_13][1]  = BIT_13,	[GPIO_13][2]  = -1,
    [GPIO_14][0]  = CTRL_1,	[GPIO_14][1]  = BIT_14,	[GPIO_14][2]  = -1,
    [GPIO_15][0]  = CTRL_7,	[GPIO_15][1]  = BIT_15,	[GPIO_15][2]  = NO_PU,

    [GPIO0_0][0]  = CTRL_0,	[GPIO0_0][1]  = BIT_0,	[GPIO0_0][2]  = -1,
    [GPIO0_1][0]  = CTRL_0,	[GPIO0_1][1]  = BIT_1,	[GPIO0_1][2]  = -1,
    [GPIO0_2][0]  = CTRL_0,	[GPIO0_2][1]  = BIT_2,	[GPIO0_2][2]  = -1,
    [GPIO0_3][0]  = CTRL_0,	[GPIO0_3][1]  = BIT_3,	[GPIO0_3][2]  = -1,
    [GPIO0_4][0]  = CTRL_0,	[GPIO0_4][1]  = BIT_4,	[GPIO0_4][2]  = -1,
    [GPIO0_5][0]  = CTRL_0,	[GPIO0_5][1]  = BIT_5,	[GPIO0_5][2]  = -1,
    [GPIO0_6][0]  = CTRL_0,	[GPIO0_6][1]  = BIT_8,	[GPIO0_6][2]  = -1,
    [GPIO0_7][0]  = CTRL_0,	[GPIO0_7][1]  = BIT_9,	[GPIO0_7][2]  = -1,
    [GPIO0_8][0]  = CTRL_0,	[GPIO0_8][1]  = BIT_10,	[GPIO0_8][2]  = -1,
    [GPIO0_9][0]  = CTRL_0,	[GPIO0_9][1]  = BIT_11,	[GPIO0_9][2]  = -1,
    [GPIO0_10][0] = CTRL_0,	[GPIO0_10][1] = BIT_12,	[GPIO0_10][2] = -1,
    [GPIO0_11][0] = CTRL_0,	[GPIO0_11][1] = BIT_13,	[GPIO0_11][2] = -1,
    [GPIO0_12][0] = CTRL_0,	[GPIO0_12][1] = BIT_14,	[GPIO0_12][2] = -1,
    [GPIO0_13][0] = CTRL_0,	[GPIO0_13][1] = BIT_15,	[GPIO0_13][2] = -1,
    [GPIO0_14][0] = -1,		[GPIO0_14][1] = -1,	[GPIO0_14][2] = -1,
    [GPIO0_15][0] = -1,		[GPIO0_15][1] = -1,	[GPIO0_15][2] = -1,

    [GPIO1_0][0]  = CTRL_5,	[GPIO1_0][1]  = BIT_0,	[GPIO1_0][2]  = -1,
    [GPIO1_1][0]  = CTRL_5,	[GPIO1_1][1]  = BIT_1,	[GPIO1_1][2]  = -1,
    [GPIO1_2][0]  = CTRL_5,	[GPIO1_2][1]  = BIT_2,	[GPIO1_2][2]  = -1,
    [GPIO1_3][0]  = CTRL_5,	[GPIO1_3][1]  = BIT_3,	[GPIO1_3][2]  = -1,
    [GPIO1_4][0]  = CTRL_5,	[GPIO1_4][1]  = BIT_4,	[GPIO1_4][2]  = -1,
    [GPIO1_5][0]  = CTRL_5,	[GPIO1_5][1]  = BIT_5,	[GPIO1_5][2]  = -1,
    [GPIO1_6][0]  = CTRL_5,	[GPIO1_6][1]  = BIT_6,	[GPIO1_6][2]  = -1,
    [GPIO1_7][0]  = CTRL_5,	[GPIO1_7][1]  = BIT_7,	[GPIO1_7][2]  = -1,
    [GPIO1_8][0]  = CTRL_5,	[GPIO1_8][1]  = BIT_8,	[GPIO1_8][2]  = -1,
    [GPIO1_9][0]  = CTRL_5,	[GPIO1_9][1]  = BIT_9,	[GPIO1_9][2]  = -1,
    [GPIO1_10][0] = CTRL_5,	[GPIO1_10][1] = BIT_10,	[GPIO1_10][2] = -1,
    [GPIO1_11][0] = CTRL_5,	[GPIO1_11][1] = BIT_11,	[GPIO1_11][2] = -1,
    [GPIO1_12][0] = CTRL_5,	[GPIO1_12][1] = BIT_12,	[GPIO1_12][2] = -1,
    [GPIO1_13][0] = CTRL_5,	[GPIO1_13][1] = BIT_13,	[GPIO1_13][2] = -1,
    [GPIO1_14][0] = CTRL_4,	[GPIO1_14][1] = BIT_8,	[GPIO1_14][2] = -1,
//    [GPIO1_15][0] = ,[GPIO1_15][1] = ,[GPIO1_15][2] = ,/* not used */

    [GPIO2_0][0]  = CTRL_6,	[GPIO2_0][1]  = BIT_0,	[GPIO2_0][2]  = NO_PU,
    [GPIO2_1][0]  = CTRL_6,	[GPIO2_1][1]  = BIT_1,	[GPIO2_1][2]  = NO_PU,
    [GPIO2_2][0]  = CTRL_6,	[GPIO2_2][1]  = BIT_2,	[GPIO2_2][2]  = NO_PU,
    [GPIO2_3][0]  = CTRL_6,	[GPIO2_3][1]  = BIT_3,	[GPIO2_3][2]  = NO_PU,
    [GPIO2_4][0]  = CTRL_6,	[GPIO2_4][1]  = BIT_4,	[GPIO2_4][2]  = NO_PU,
    [GPIO2_5][0]  = CTRL_6,	[GPIO2_5][1]  = BIT_5,	[GPIO2_5][2]  = NO_PU,
    [GPIO2_6][0]  = CTRL_6,	[GPIO2_6][1]  = BIT_6,	[GPIO2_6][2]  = NO_PU,
    [GPIO2_7][0]  = CTRL_6,	[GPIO2_7][1]  = BIT_7,	[GPIO2_7][2]  = NO_PU,
    [GPIO2_8][0]  = CTRL_6,	[GPIO2_8][1]  = BIT_8,	[GPIO2_8][2]  = NO_PU,
    [GPIO2_9][0]  = CTRL_7,	[GPIO2_9][1]  = BIT_10,	[GPIO2_9][2]  = NO_PU,
    [GPIO2_10][0] = CTRL_7,	[GPIO2_10][1] = BIT_11,	[GPIO2_10][2] = NO_PU,
    [GPIO2_11][0] = CTRL_7,	[GPIO2_11][1] = BIT_12,	[GPIO2_11][2] = NO_PU,
    [GPIO2_12][0] = CTRL_7,	[GPIO2_12][1] = BIT_13,	[GPIO2_12][2] = NO_PU,
    [GPIO2_13][0] = CTRL_7,	[GPIO2_13][1] = BIT_14,	[GPIO2_13][2] = NO_PU,
    [GPIO2_14][0] = CTRL_9,	[GPIO2_14][1] = BIT_1,	[GPIO2_14][2] = -1,
//    [GPIO2_15][0] = ,[GPIO2_15][1] = ,[GPIO2_15][2] = ,/* not used */

    [GPIO3_0][0]  = CTRL_10,	[GPIO3_0][1]  = BIT_0,	[GPIO3_0][2]  = -1,
    [GPIO3_1][0]  = CTRL_10,	[GPIO3_1][1]  = BIT_1,	[GPIO3_1][2]  = -1,
    [GPIO3_2][0]  = CTRL_10,	[GPIO3_2][1]  = BIT_2,	[GPIO3_2][2]  = -1,
    [GPIO3_3][0]  = CTRL_10,	[GPIO3_3][1]  = BIT_3,	[GPIO3_3][2]  = -1,
    [GPIO3_4][0]  = CTRL_10,	[GPIO3_4][1]  = BIT_4,	[GPIO3_4][2]  = -1,
    [GPIO3_5][0]  = CTRL_11,	[GPIO3_5][1]  = BIT_4,	[GPIO3_5][2]  = -1,
    [GPIO3_6][0]  = CTRL_11,	[GPIO3_6][1]  = BIT_5,	[GPIO3_6][2]  = -1,
    [GPIO3_7][0]  = CTRL_11,	[GPIO3_7][1]  = BIT_6,	[GPIO3_7][2]  = -1,
    [GPIO3_8][0]  = CTRL_11,	[GPIO3_8][1]  = BIT_7,	[GPIO3_8][2]  = -1,
    [GPIO3_9][0]  = CTRL_11,	[GPIO3_9][1]  = BIT_8,	[GPIO3_9][2]  = -1,
    [GPIO3_10][0] = CTRL_11,	[GPIO3_10][1] = BIT_9,	[GPIO3_10][2] = -1,
    [GPIO3_11][0] = CTRL_12,	[GPIO3_11][1] = BIT_0,	[GPIO3_11][2] = -1,
    [GPIO3_12][0] = CTRL_12,	[GPIO3_12][1] = BIT_1,	[GPIO3_12][2] = -1,
//    [GPIO3_13][0] = ,[GPIO3_13][1] = ,[GPIO3_13][2] = ,/* not used */
//    [GPIO3_14][0] = ,[GPIO3_14][1] = ,[GPIO3_14][2] = ,/* not used */
//    [GPIO3_15][0] = ,[GPIO3_15][1] = ,[GPIO3_15][2] = ,/* not used */

    [GPIO4_0][0]  = CTRL_10,	[GPIO4_0][1]  = BIT_5,	[GPIO4_0][2]  = -1,
    [GPIO4_1][0]  = CTRL_10,	[GPIO4_1][1]  = BIT_6,	[GPIO4_1][2]  = -1,
    [GPIO4_2][0]  = CTRL_10,	[GPIO4_2][1]  = BIT_7,	[GPIO4_2][2]  = -1,
    [GPIO4_3][0]  = CTRL_10,	[GPIO4_3][1]  = BIT_8,	[GPIO4_3][2]  = -1,
    [GPIO4_4][0]  = CTRL_10,	[GPIO4_4][1]  = BIT_9,	[GPIO4_4][2]  = -1,
    [GPIO4_5][0]  = CTRL_10,	[GPIO4_5][1]  = BIT_10,	[GPIO4_5][2]  = -1,
    [GPIO4_6][0]  = CTRL_10,	[GPIO4_6][1]  = BIT_11,	[GPIO4_6][2]  = -1,
    [GPIO4_7][0]  = CTRL_10,	[GPIO4_7][1]  = BIT_12,	[GPIO4_7][2]  = -1,
    [GPIO4_8][0]  = CTRL_10,	[GPIO4_8][1]  = BIT_13,	[GPIO4_8][2]  = -1,
    [GPIO4_9][0]  = CTRL_10,	[GPIO4_9][1]  = BIT_14,	[GPIO4_9][2]  = -1,
    [GPIO4_10][0] = CTRL_11,	[GPIO4_10][1] = BIT_0,	[GPIO4_10][2] = -1,
    [GPIO4_11][0] = CTRL_11,	[GPIO4_11][1] = BIT_1,	[GPIO4_11][2] = -1,
    [GPIO4_12][0] = CTRL_11,	[GPIO4_12][1] = BIT_2,	[GPIO4_12][2] = -1,
    [GPIO4_13][0] = CTRL_11,	[GPIO4_13][1] = BIT_3,	[GPIO4_13][2] = -1,
    [GPIO4_14][0] = CTRL_13,	[GPIO4_14][1] = BIT_3,	[GPIO4_14][2] = -1,
    [GPIO4_15][0] = CTRL_13,	[GPIO4_15][1] = BIT_4,	[GPIO4_15][2] = -1,

    [GPIO5_0][0]  = CTRL_3,	[GPIO5_0][1]  = BIT_0,	[GPIO5_0][2]  = -1,
    [GPIO5_1][0]  = CTRL_3,	[GPIO5_1][1]  = BIT_1,	[GPIO5_1][2]  = -1,
    [GPIO5_2][0]  = CTRL_3,	[GPIO5_2][1]  = BIT_2,	[GPIO5_2][2]  = -1,
    [GPIO5_3][0]  = CTRL_3,	[GPIO5_3][1]  = BIT_3,	[GPIO5_3][2]  = -1,
    [GPIO5_4][0]  = CTRL_3,	[GPIO5_4][1]  = BIT_4,	[GPIO5_4][2]  = -1,
    [GPIO5_5][0]  = CTRL_3,	[GPIO5_5][1]  = BIT_5,	[GPIO5_5][2]  = -1,
    [GPIO5_6][0]  = CTRL_4,	[GPIO5_6][1]  = BIT_0,	[GPIO5_6][2]  = -1,
    [GPIO5_7][0]  = CTRL_4,	[GPIO5_7][1]  = BIT_1,	[GPIO5_7][2]  = -1,
    [GPIO5_8][0]  = CTRL_4,	[GPIO5_8][1]  = BIT_2,	[GPIO5_8][2]  = -1,
    [GPIO5_9][0]  = CTRL_4,	[GPIO5_9][1]  = BIT_3,	[GPIO5_9][2]  = -1,
    [GPIO5_10][0] = CTRL_4,	[GPIO5_10][1] = BIT_4,	[GPIO5_10][2] = -1,
    [GPIO5_11][0] = CTRL_4,	[GPIO5_11][1] = BIT_5,	[GPIO5_11][2] = -1,
    [GPIO5_12][0] = CTRL_4,	[GPIO5_12][1] = BIT_6,	[GPIO5_12][2] = -1,
    [GPIO5_13][0] = CTRL_4,	[GPIO5_13][1] = BIT_7,	[GPIO5_13][2] = -1,
//    [GPIO5_14][0] = ,[GPIO5_14][1] = ,[GPIO5_14][2] = ,/* not used */
//    [GPIO5_15][0] = ,[GPIO5_15][1] = ,[GPIO5_15][2] = ,/* not used */

    [GPIOE_0][0]  = CTRL_2,	[GPIOE_0][1]  = BIT_0,	[GPIOE_0][2]  = -1,
    [GPIOE_1][0]  = CTRL_2,	[GPIOE_1][1]  = BIT_1,	[GPIOE_1][2]  = -1,
    [GPIOE_2][0]  = CTRL_2,	[GPIOE_2][1]  = BIT_2,	[GPIOE_2][2]  = -1,
    [GPIOE_3][0]  = CTRL_2,	[GPIOE_3][1]  = BIT_3,	[GPIOE_3][2]  = -1,
    [GPIOE_4][0]  = CTRL_2,	[GPIOE_4][1]  = BIT_4,	[GPIOE_4][2]  = -1,
    [GPIOE_5][0]  = CTRL_2,	[GPIOE_5][1]  = BIT_5,	[GPIOE_5][2]  = -1,
    [GPIOE_6][0]  = CTRL_2,	[GPIOE_6][1]  = BIT_6,	[GPIOE_6][2]  = -1,
    [GPIOE_7][0]  = CTRL_2,	[GPIOE_7][1]  = BIT_7,	[GPIOE_7][2]  = -1,
    [GPIOE_8][0]  = CTRL_2,	[GPIOE_8][1]  = BIT_8,	[GPIOE_8][2]  = -1,
    [GPIOE_9][0]  = CTRL_2,	[GPIOE_9][1]  = BIT_9,	[GPIOE_9][2]  = -1,
    [GPIOE_10][0] = CTRL_2,	[GPIOE_10][1] = BIT_10,	[GPIOE_10][2] = -1,
    [GPIOE_11][0] = CTRL_2,	[GPIOE_11][1] = BIT_11,	[GPIOE_11][2] = -1,
    [GPIOE_12][0] = CTRL_2,	[GPIOE_12][1] = BIT_12,	[GPIOE_12][2] = -1,
    [GPIOE_13][0] = CTRL_2,	[GPIOE_13][1] = BIT_13,	[GPIOE_13][2] = -1,
    [GPIOE_14][0] = CTRL_2,	[GPIOE_14][1] = BIT_14,	[GPIOE_14][2] = -1,
    [GPIOE_15][0] = CTRL_2,	[GPIOE_15][1] = BIT_15,	[GPIOE_15][2] = -1,
};

static spinlock_t gpio_spinlock = SPIN_LOCK_UNLOCKED;

static RegGpio *reggpio = 0;

static char cmd_proc[8];

/* for USB_VBUS detection */
static unsigned char irqreq = 0;
static unsigned char vbusdat = 0;

/*======================================================================
 *
 * Main function body
 *
 *======================================================================*/
/*-----------------------------------------------------------------------------
 * init_module
 *---------------------------------------------------------------------------*/
/*static*/ int __init gpio_cxd4108_init(void)
{
#ifdef CHECK_TIME
    u32 t = tick();
#endif
    int err;
    struct proc_dir_entry *ent;
    unsigned int nr_grp;
    unsigned int nr_bit;
    PDEBUG1(" "MYNAME"::gpio_cxd4108_init() : Entry\n");

    if((err = gpio_attach_plugin(&gpio_cxd4108)) < 0){
        RETURN1(err, " "MYNAME"::gpio_cxd4108_init() : Return(%x)\n");
    }
    if(! (ent = create_proc_entry(GPIO_PROC, S_IFREG | S_IRUGO | S_IWUSR, 
        proc_root_driver))){
        gpio_detach_plugin(&gpio_cxd4108);
        RETURN1(-EACCES," "MYNAME"::gpio_cxd4108_init() : Return(%x)\n");
    }
    ent->read_proc = &gpio_cxd4108_read_proc;
    ent->write_proc = &gpio_cxd4108_write_proc;
    ent->data = (void *)cmd_proc;
    /* for USB_VBUS detection */
    {
        __get_nr_grpbit(GPIO0_6, &nr_grp, &nr_bit);
        /* Set output setting */
        SET_GPREG(nr_bit, REG_GPE_DIR_(nr_grp));
    }

#ifdef CHECK_TIME
    tprintk(tick() - t, "done\n");
#endif
    RETURN1(err, " "MYNAME"::gpio_cxd4108_init() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * exit_module
 *---------------------------------------------------------------------------*/
static void __exit gpio_cxd4108_exit(void)
{
#ifdef CHECK_TIME
    u32 t = tick();
#endif
    PDEBUG1(" "MYNAME"::gpio_cxd4108_exit() : Entry\n");

    remove_proc_entry(GPIO_PROC, proc_root_driver);
    gpio_detach_plugin(&gpio_cxd4108);

    PDEBUG1(" "MYNAME"::gpio_cxd4108_exit() : Return\n");
#ifdef CHECK_TIME
    tprintk(tick() - t, "done\n");
#endif
    return;
}

/*-----------------------------------------------------------------------------
 * open
 *---------------------------------------------------------------------------*/
static int Vopen(IN unsigned int nr)
{
    PDEBUG1(" "MYNAME"::Vopen(%d) : Entry\n",nr);
    RETURN1(0, " "MYNAME"::Vopen() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * close
 *---------------------------------------------------------------------------*/
static int Vclose(IN unsigned int nr)
{
    PDEBUG1(" "MYNAME"::Vclose(%d) : Entry\n",nr);
    RETURN1(0, " "MYNAME"::Vclose() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * set_sel_func
 *---------------------------------------------------------------------------*/
static int Vset_sel_func(IN unsigned int nr, IN unsigned char func, 
    IN unsigned int val)
{
    int err;
    PDEBUG1(" "MYNAME"::Vset_sel_func(%d,%x,%x) : Entry\n", nr, func, val);

    /* for USB_VBUS detection */
    if(nr == GPIO_14){
        if(func == GPIO_INPUT_PU){
            RETURN1(0, " "MYNAME"::Vset_sel_func() : Return(%x)\n");
        }
        RETURN1(-EINVAL, " "MYNAME"::Vset_sel_func() : Return(%x)\n");
    }
    if(__is_gpio(nr)){
        err = set_gpio_func(nr, func, val);
    }else if(__is_gpioe(nr)){
        err = set_gpioe_func(nr, func, val);
    }else{
        err = set_gpioX_func(nr, func, val);
    }

    RETURN1(err, " "MYNAME"::Vset_sel_func() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * get_sel_func
 *---------------------------------------------------------------------------*/
static int Vget_sel_func(IN unsigned int nr, OUT unsigned char *func)
{
    int err;
    PDEBUG1(" "MYNAME"::Vget_sel_func(%d,%x) : Entry\n", nr, (int)func);

    /* for USB_VBUS detection */
    if(nr == GPIO_14){
        *func = GPIO_INPUT_PU;
        RETURN1(0, " "MYNAME"::Vget_sel_func() : Return(%x)\n");
    }
    if(__is_gpio(nr)){
        err = get_gpio_func(nr, func);
    }else if(__is_gpioe(nr)){
        err = get_gpioe_func(nr, func);
    }else{
        err = get_gpioX_func(nr, func);
    }

    RETURN1(err, " "MYNAME"::Vget_sel_func() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * write
 *---------------------------------------------------------------------------*/
static int Vwrite(IN unsigned int nr, IN unsigned int val)
{
    unsigned int nr_bit;
    unsigned int reg;
    int err;
    unsigned int nr_grp;
    PDEBUG1(" "MYNAME"::Vwrite(%d,%x) : Entry\n", nr, val);

    /* for USB_VBUS detection */
    if(nr == GPIO_14){
        RETURN1(0, " "MYNAME"::Vwrite() : Return(%x)\n");
    }
    if(__is_gpio(nr)){
        nr_bit = GPIO_NR(nr);
        if(val){
            reg = REG_GPS_DATASET;
        }else{
            reg = REG_GPS_DATACLR;
        }
    }else if(__is_gpioe(nr)){
        nr_bit = GPIOE_NR(nr);
        if(val){
            reg = REG_GPE_DATASET;
        }else{
            reg = REG_GPE_DATACLR;
        }
    }else{
        if((err = __get_nr_grpbit(nr, &nr_grp, &nr_bit)) < 0){
            RETURN1(err, " "MYNAME"::Vwrite() : Return(%x)\n");
        }
        if(val){
            reg = REG_GPE_DATASET_(nr_grp);
        }else{
            reg = REG_GPE_DATACLR_(nr_grp);
        }
    }
    /* Set register */
    SET_GPREG2(nr_bit, reg);

    RETURN1(0, " "MYNAME"::Vwrite() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * read
 *---------------------------------------------------------------------------*/
static int Vread(IN unsigned int nr)
{
    unsigned int nr_bit;
    unsigned int reg;
    int err;
    unsigned int nr_grp;
    PDEBUG1(" "MYNAME"::Vread(%d) : Entry\n", nr);

    /* for USB_VBUS detection */
    if(nr == GPIO_14){
        RETURN1(vbusdat, " "MYNAME"::Vread() : Return(%x)\n");
    }
    if(__is_gpio(nr)){
        nr_bit = GPIO_NR(nr);
        reg = REG_GPS_RDATA;
    }else if(__is_gpioe(nr)){
        nr_bit = GPIOE_NR(nr);
        reg = REG_GPE_RDATA;
    }else{
        if((err = __get_nr_grpbit(nr, &nr_grp, &nr_bit)) < 0){
            RETURN1(err, " "MYNAME"::Vread() : Return(%x)\n");
        }
        reg = REG_GPE_RDATA_(nr_grp);
    }
    /* Read register */
    if(GET_GPREG(nr_bit, reg)){
        RETURN1(1, " "MYNAME"::Vread() : Return(%x)\n");
    }

    RETURN1(0, " "MYNAME"::Vread() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * set_irq_sense
 *---------------------------------------------------------------------------*/
static int Vset_irq_sense(IN unsigned int nr, IN unsigned char sense)
{
    unsigned int nr_bit;
    unsigned long flags;
    PDEBUG1(" "MYNAME"::Vset_irq_sense(%d,%x) : Entry\n", nr, sense);

    /* for USB_VBUS detection */
    if(nr == GPIO_14){
        if(sense == GPIO_INT_UPDOWN_EDGE){
            RETURN1(0, " "MYNAME"::Vset_irq_sense() : Return(%x)\n");
        }
        RETURN1(-EINVAL, " "MYNAME"::Vset_irq_sense() : Return(%x)\n");
    }
    if(! __is_gpio(nr)){
        /* gpio0, gpio1, ..., gpio5, gpioe don't support interrupt */
        RETURN1(-ENOEXEC, " "MYNAME"::Vset_irq_sense() : Return(%x)\n");
    }
    nr_bit = GPIO_NR(nr);
    spin_lock_irqsave(&gpio_spinlock, flags);
    switch(sense){
    case GPIO_INT_DOWN_EDGE:
        CLR_GPREG(nr_bit, REG_GPS_INTS);	/* edge */
        CLR_GPREG(nr_bit, REG_GPS_INTHE);	/* invalid high/up interrupt */
        SET_GPREG(nr_bit, REG_GPS_INTLE);	/* valid low/down interrupt */
        break;
    case GPIO_INT_UP_EDGE:
        CLR_GPREG(nr_bit, REG_GPS_INTS);	/* edge */
        SET_GPREG(nr_bit, REG_GPS_INTHE);	/* valid high/up interrupt */
        CLR_GPREG(nr_bit, REG_GPS_INTLE);	/* invalid low/down interrupt */
        break;
    case GPIO_INT_UPDOWN_EDGE:
        CLR_GPREG(nr_bit, REG_GPS_INTS);	/* edge */
        SET_GPREG(nr_bit, REG_GPS_INTHE);	/* valid high/up interrupt */
        SET_GPREG(nr_bit, REG_GPS_INTLE);	/* valid low/down interrupt */
        break;
    case GPIO_INT_L_LEVEL:
        SET_GPREG(nr_bit, REG_GPS_INTS);	/* level */
        CLR_GPREG(nr_bit, REG_GPS_INTHE);	/* invalid high/up interrupt */
        SET_GPREG(nr_bit, REG_GPS_INTLE);	/* valid low/down interrupt */
        break;
    case GPIO_INT_H_LEVEL:
        SET_GPREG(nr_bit, REG_GPS_INTS);	/* level */
        SET_GPREG(nr_bit, REG_GPS_INTHE);	/* valid high/up interrupt */
        CLR_GPREG(nr_bit, REG_GPS_INTLE);	/* invalid low/down interrupt */
        break;
    default:
        spin_unlock_irqrestore(&gpio_spinlock, flags);
        RETURN1(-EINVAL, " "MYNAME"::Vset_irq_sense() : Return(%x)\n");
    }
    spin_unlock_irqrestore(&gpio_spinlock, flags);

    RETURN1(0, " "MYNAME"::Vset_irq_sense() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * get_irq_sense
 *---------------------------------------------------------------------------*/
static int Vget_irq_sense(IN unsigned int nr, OUT unsigned char *sense)
{
    unsigned int nr_bit;
    PDEBUG1(" "MYNAME"::Vget_irq_sense(%d,%x) : Entry\n", nr, (int)sense);

    /* for USB_VBUS detection */
    if(nr == GPIO_14){
        *sense = GPIO_INT_UPDOWN_EDGE;
        RETURN1(0, " "MYNAME"::Vget_irq_sense() : Return(%x)\n");
    }
    if(! __is_gpio(nr)){ 
        /* gpio0, gpio1, ..., gpio5, gpioe don't support interrupt */
        RETURN1(-ENOEXEC, " "MYNAME"::Vget_irq_sense() : Return(%x)\n");
    }
    nr_bit = GPIO_NR(nr);
    if(GET_GPREG(nr_bit, REG_GPS_INTS)){	/* level */
        if(GET_GPREG(nr_bit, REG_GPS_INTHE)){
            if(GET_GPREG(nr_bit, REG_GPS_INTLE)){
                RETURN1(-EINVAL, " "MYNAME"::Vget_irq_sense() : Return(%x)\n");
            }else{
                *sense = GPIO_INT_H_LEVEL;
            }
        }else{
            if(GET_GPREG(nr_bit, REG_GPS_INTLE)){
                *sense = GPIO_INT_L_LEVEL;
            }else{
                *sense = GPIO_INT_NOSET;
            }
        }
    }else{	/* edge */
        if(GET_GPREG(nr_bit, REG_GPS_INTHE)){
            if(GET_GPREG(nr_bit, REG_GPS_INTLE)){
                *sense = GPIO_INT_UPDOWN_EDGE;
            }else{
                *sense = GPIO_INT_UP_EDGE;
            }
        }else{
            if(GET_GPREG(nr_bit, REG_GPS_INTLE)){
                *sense = GPIO_INT_DOWN_EDGE;
            }else{
                *sense = GPIO_INT_NOSET;
            }
        }
    }

    RETURN1(0, " "MYNAME"::Vget_irq_sense() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * enable_irq
 *---------------------------------------------------------------------------*/
static int Venable_irq(IN unsigned int nr)
{
    int err;
    unsigned long flags;
    PDEBUG1(" "MYNAME"::Venable_irq(%d) : Entry\n", nr);

    /* for USB_VBUS detection */
    if(nr == GPIO_14){
        if(! (err = request_irq(GPIO_IRQ(nr), handler, 0, MYNAME, 0))){
	    irqreq = 1;
	}
        RETURN1(err, " "MYNAME"::Venable_irq() : Return(%x)\n");
    }
    /* Clear irq status */
    Vclear_irq(nr);
    if(! __is_gpio(nr)){
        /* gpio0, gpio1, ..., gpio5, gpioe don't support interrupt */
        RETURN1(-ENOEXEC, " "MYNAME"::Venable_irq() : Return(%x)\n");
    }
    spin_lock_irqsave(&gpio_spinlock, flags);
    SET_GPREG(GPIO_NR(nr), REG_GPS_INTMASK);
    spin_unlock_irqrestore(&gpio_spinlock, flags);

    RETURN1(0, " "MYNAME"::Venable_irq() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * disable_irq
 *---------------------------------------------------------------------------*/
static int Vdisable_irq(IN unsigned int nr)
{
    unsigned long flags;
    PDEBUG1(" "MYNAME"::Vdisable_irq(%d) : Entry\n", nr);

    /* for USB_VBUS detection */
    if(nr == GPIO_14){
        free_irq(GPIO_IRQ(nr), 0);
        irqreq = 0;
        RETURN1(0, " "MYNAME"::Vdisable_irq() : Return(%x)\n");
    }
    if(! __is_gpio(nr)){
        /* gpio0, gpio1, ..., gpio5, gpioe don't support interrupt */
        RETURN1(-ENOEXEC, " "MYNAME"::Vdisable_irq() : Return(%x)\n");
    }
    spin_lock_irqsave(&gpio_spinlock, flags);
    CLR_GPREG(GPIO_NR(nr), REG_GPS_INTMASK);
    spin_unlock_irqrestore(&gpio_spinlock, flags);

    RETURN1(0, " "MYNAME"::Vdisable_irq() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * is_enable_irq
 *---------------------------------------------------------------------------*/
static int Vis_enable_irq(IN unsigned int nr)
{
    PDEBUG1(" "MYNAME"::Vis_enable_irq(%d) : Entry\n", nr);

    /* for USB_VBUS detection */
    if(nr == GPIO_14){
        if(irqreq){	/* enable */
            RETURN1(1, " "MYNAME"::Vis_enable_irq() : Return(%x)\n");
        }
        RETURN1(0, " "MYNAME"::Vis_enable_irq() : Return(%x)\n");
    }
    if(! __is_gpio(nr)){
        /* gpio0, gpio1, ..., gpio5, gpioe don't support interrupt */
        RETURN1(-ENOEXEC, " "MYNAME"::Vis_enable_irq() : Return(%x)\n");
    }
    if(GET_GPREG(GPIO_NR(nr), REG_GPS_INTMASK)){	/* enable */
        RETURN1(1, " "MYNAME"::Vis_enable_irq() : Return(%x)\n");
    }

    RETURN1(0, " "MYNAME"::Vis_enable_irq() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * is_irq
 *---------------------------------------------------------------------------*/
static int Vis_irq(IN unsigned int nr)
{
    PDEBUG1(" "MYNAME"::Vis_irq(%d) : Entry\n", nr);

    /* for USB_VBUS detection */
    if(nr == GPIO_14){
        if(GET_GPREG(GPIO_NR(nr), REG_INT_GSINT)){	/* request */
            RETURN1(1, " "MYNAME"::Vis_irq() : Return(%x)\n");
	}
        RETURN1(0, " "MYNAME"::Vis_irq() : Return(%x)\n");
    }
    if(! __is_gpio(nr)){
        /* gpio0, gpio1, ..., gpio5, gpioe don't support interrupt */
        RETURN1(-ENOEXEC, " "MYNAME"::Vis_irq() : Return(%x)\n");
    }
    if(GET_GPREG(GPIO_NR(nr), REG_GPS_INTST)){	/* request */
        RETURN1(1, " "MYNAME"::Vis_irq() : Return(%x)\n");
    }

    RETURN1(0, " "MYNAME"::Vis_irq() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * clear_irq
 *---------------------------------------------------------------------------*/
static int Vclear_irq(IN unsigned int nr)
{
    unsigned int nr_grp;
    unsigned int nr_bit;
    PDEBUG1(" "MYNAME"::Vclear_irq(%d) : Entry\n", nr);

    /* for USB_VBUS detection */
    if(nr == GPIO_14){
        __get_nr_grpbit(GPIO0_6, &nr_grp, &nr_bit);
        vbusdat = (unsigned char)GET_GPREG(nr_bit, REG_GPE_RDATA_(nr_grp));
        SET_GPREG2(GPIO_NR(nr), REG_INT_GSINTCLR);
        RETURN1(0, " "MYNAME"::Vclear_irq() : Return(%x)\n");
    }
    if(! __is_gpio(nr)){
        /* gpio0, gpio1, ..., gpio5, gpioe don't support interrupt */
        RETURN1(-ENOEXEC, " "MYNAME"::Vclear_irq() : Return(%x)\n");
    }
    SET_GPREG2(GPIO_NR(nr), REG_GPS_INTSTCLR);

    RETURN1(0, " "MYNAME"::Vclear_irq() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * register_handler
 *---------------------------------------------------------------------------*/
static int Vregister_handler(IN unsigned int nr)
{
    int err;
    PDEBUG1(" "MYNAME"::Vregister_handler(%d) : Entry\n", nr);

    /* for USB_VBUS detection */
    if(nr == GPIO_14){
        RETURN1(0, " "MYNAME"::Vregister_handler() : Return(%x)\n");
    }
    if(! __is_gpio(nr)){
        /* gpio0, gpio1, ..., gpio5, gpioe don't support interrupt */
        RETURN1(-ENOEXEC, " "MYNAME"::Vregister_handler() : Return(%x)\n");
    }
    err = request_irq(GPIO_IRQ(nr), handler, 0, MYNAME, 0);

    RETURN1(err, " "MYNAME"::Vregister_handler() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * unregister_handler
 *---------------------------------------------------------------------------*/
static int Vunregister_handler(IN unsigned int nr)
{
    PDEBUG1(" "MYNAME"::Vunregister_handler(%d) : Entry\n", nr);

    /* for USB_VBUS detection */
    if(nr == GPIO_14){
        RETURN1(0, " "MYNAME"::Vunregister_handler() : Return(%x)\n");
    }
    if(! __is_gpio(nr)){
        /* gpio0, gpio1, ..., gpio5, gpioe don't support interrupt */
        RETURN1(-ENOEXEC, " "MYNAME"::Vunregister_handler() : Return(%x)\n");
    }
    free_irq(GPIO_IRQ(nr), 0);

    RETURN1(0, " "MYNAME"::Vunregister_handler() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * enable_wakeup
 *---------------------------------------------------------------------------*/
static int Venable_wakeup(IN unsigned int nr)
{
    PDEBUG1(" "MYNAME"::Venable_wakeup(%d) : Entry\n", nr);
    /* gpio, gpio0, gpio1, ..., gpio5, gpioe don't support wakeup */
    RETURN1(-ENOEXEC, " "MYNAME"::Venable_wakeup() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * disable_wakeup
 *---------------------------------------------------------------------------*/
static int Vdisable_wakeup(IN unsigned int nr)
{
    PDEBUG1(" "MYNAME"::Vdisable_wakeup(%d) : Entry\n", nr);
    /* gpio, gpio0, gpio1, ..., gpio5, gpioe don't support wakeup */
    RETURN1(-ENOEXEC, " "MYNAME"::Vdisable_wakeup() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * is_enable_wakeup
 *---------------------------------------------------------------------------*/
static int Vis_enable_wakeup(IN unsigned int nr)
{
    PDEBUG1(" "MYNAME"::Vis_enable_wakeup(%d) : Entry\n", nr);
    /* gpio, gpio0, gpio1, ..., gpio5, gpioe don't support wakeup */
    RETURN1(-ENOEXEC, " "MYNAME"::Vis_enable_wakeup() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * suspend
 *---------------------------------------------------------------------------*/
static int Vsuspend(IN struct amba_device *dev, IN u32 state)
{
    suspend_state_t pm = pm_get_state();
    int err = 0;
    PDEBUG1(" "MYNAME"::Vsuspend(%x,%x) : Entry\n", (int)dev, state);

// unnecessary process in cxd4108
//    err = __suspend(dev, state);
    if(pm == PM_SUSPEND_MEM){
        PDEBUG1(" "MYNAME"::Vsuspend() : suspend\n");
    }else if(pm == PM_SUSPEND_DISK){
        PDEBUG1(" "MYNAME"::Vsuspend() : hibernate\n");
    }

    RETURN1(err, " "MYNAME"::Vsuspend() : Return(%d)\n");
}

/*-----------------------------------------------------------------------------
 * resume
 *---------------------------------------------------------------------------*/
static int Vresume(IN struct amba_device *dev)
{
    suspend_state_t pm = pm_get_state();
    int err = 0;
    unsigned int nr_grp;
    unsigned int nr_bit;
    PDEBUG1(" "MYNAME"::Vresume(%x) : Entry\n", (int)dev);

    if(pm == PM_SUSPEND_MEM){
        PDEBUG1(" "MYNAME"::Vresume() : hot boot\n");
    }else if(pm == PM_SUSPEND_DISK){
        PDEBUG1(" "MYNAME"::Vresume() : warm boot\n");
    }
// unnecessary process in cxd4108
//    err = __resume(dev);
    /* for USB_VBUS detection */
    {
        __get_nr_grpbit(GPIO0_6, &nr_grp, &nr_bit);
        /* Set output setting */
        SET_GPREG(nr_bit, REG_GPE_DIR_(nr_grp));
    }

    RETURN1(err, " "MYNAME"::Vresume() : Return(%d)\n");
}

/*-----------------------------------------------------------------------------
 * read_proc
 *---------------------------------------------------------------------------*/
static int gpio_cxd4108_read_proc(char *buffer, char **start, off_t offset, 
    int length, int *eof, void *data)
{
    int len = 0;
    char cmd[3];
    int num = -1;
    PDEBUG1(" "MYNAME"::gpio_cxd4108_read_proc() : Entry\n");

    /* get register name */
    if(! strcmp(CMD_GPIOSYS, (const char *)data)){
        len += sprintf(buffer + len, "GPIOSYS\n");
        len += sprintf(buffer + len, "DIR    :0x%04x\n", readw(REG_GPS_DIR));
        len += sprintf(buffer + len, "RDATA  :0x%04x\n", readw(REG_GPS_RDATA));
        len += sprintf(buffer + len, "INTS   :0x%04x\n", readw(REG_GPS_INTS));
        len += sprintf(buffer + len, "INTHE  :0x%04x\n", readw(REG_GPS_INTHE));
        len += sprintf(buffer + len, "INTLE  :0x%04x\n", readw(REG_GPS_INTLE));
        len += sprintf(buffer + len, "INTMASK:0x%04x\n", readw(REG_GPS_INTMASK));
        len += sprintf(buffer + len, "INEN   :0x%04x\n", readw(REG_GPS_INEN));
        len -= offset;
    }else{
        cmd[0] = *((char *)data + 4);
        cmd[1] = *((char *)data + 5);
        cmd[2] = *((char *)data + 6);
        if(cmd[0] == '\0'){
            num = -1;
        }else if(cmd[1] == '\0'){
            num = cmd[0] - '0';
        }else if(cmd[0] != '0' && cmd[2] == '\0'){
            num = (cmd[0] - '0') * 10 + cmd[1] - '0';
        }
        *((char *)data + 4) = '\0';
        if((! strcmp(CMD_GPIOEASY, (const char *)data)) && num >= -1 && 
            num <= 5){
            *((char *)data + 4) = cmd[0];
            if(num == -1){
                len += sprintf(buffer + len, "GPIOEASY\n");
                len += sprintf(buffer + len, "DIR    :0x%04x\n", 
                    readw(REG_GPE_DIR));
                len += sprintf(buffer + len, "RDATA  :0x%04x\n", 
                    readw(REG_GPE_RDATA));
                len += sprintf(buffer + len, "INEN   :0x%04x\n", 
                    readw(REG_GPE_INEN));
	        len -= offset;
            }else{
                len += sprintf(buffer + len, "GPIOEASY%d\n",  num);
                len += sprintf(buffer + len, "DIR    :0x%04x\n", 
                    readw(REG_GPE_DIR_(num)));
                len += sprintf(buffer + len, "RDATA  :0x%04x\n", 
                    readw(REG_GPE_RDATA_(num)));
                len += sprintf(buffer + len, "INEN   :0x%04x\n", 
                    readw(REG_GPE_INEN_(num)));
                len += sprintf(buffer + len, "PORTSEL:0x%04x\n", 
                    readw(REG_GPE_PORTSEL_(num)));
	        len -= offset;
            }
        }else if((! strcmp(CMD_GPIOCTRL, (const char *)data)) && num >= 0 && 
            num <= 13){
            *((char *)data + 4) = cmd[0];
            len += sprintf(buffer + len, "GPIOCTRL%d\n",  num);
            len += sprintf(buffer + len, "DRV2   :0x%04x\n", 
                readw(REG_GPC_DRV2_(num)));
            len += sprintf(buffer + len, "DRV1   :0x%04x\n", 
                readw(REG_GPC_DRV1_(num)));
            len += sprintf(buffer + len, "RESEN  :0x%04x\n", 
                readw(REG_GPC_RESEN_(num)));
            len += sprintf(buffer + len, "PUPD   :0x%04x\n", 
                readw(REG_GPC_PUPD_(num)));
	    len -= offset;
        }
    }
    if(len < length){
        *eof = 1;
        if(len <= 0){
            RETURN1(0, " "MYNAME"::gpio_cxd4108_read_proc() : Return(%x)\n");
        }
    }else{
        len = length;
    }
    *start = buffer + offset;

    RETURN1(len, " "MYNAME"::gpio_cxd4108_read_proc() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * write_proc
 *---------------------------------------------------------------------------*/
static int gpio_cxd4108_write_proc(struct file *file, const char *buffer, 
    unsigned long count, void *data)
{
    unsigned char i;
    PDEBUG1(" "MYNAME"::gpio_cxd4108_write_proc() : Entry\n");

    for(i = 0; i < MIN(sizeof(cmd_proc), count); i++){
        *(cmd_proc + i) = *(buffer + i);
    }
    *(cmd_proc + MIN(sizeof(cmd_proc), count) - 1) = '\0';

    RETURN1(count, " "MYNAME"::gpio_cxd4108_write_proc() : Return(%lx)\n");
}

/*-----------------------------------------------------------------------------
 * handler
 *---------------------------------------------------------------------------*/
static irqreturn_t handler(int irq, void *dev_id, struct pt_regs *fp)
{
    unsigned int nr;
    PDEBUG1(" "MYNAME"::handler(%d) : Entry\n", irq);

    nr = GPIO_NR(irq - IRQ_GPIO0);
    if(__is_gpio(nr)){
        /* Clear irq status */
        Vclear_irq(nr);
        PDEBUG1(" "MYNAME"::handler() : call gpio_handler(%d)\n", nr);
        gpio_handler(nr);
    }

    RETURN1(IRQ_HANDLED, " "MYNAME"::handler() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * set_gpio_func
 *---------------------------------------------------------------------------*/
static int set_gpio_func(IN unsigned int nr, IN unsigned char func, 
    IN unsigned int val)
{
    unsigned int nr_bit;
    unsigned long flags;
    PDEBUG1(" "MYNAME"::set_gpio_func(%d,%x,%x) : Entry\n", nr, func, val);

    nr_bit = GPIO_NR(nr);
    spin_lock_irqsave(&gpio_spinlock, flags);
    switch(func){
    case GPIO_OUTPUT:
        CLR_GPREG(nr_bit, REG_GPS_INEN);
        if(__check_nr_ctrl(nr)){
            SET_GPREG2(nr_ctrl[nr][1], REG_GPC_RESENSET_(nr_ctrl[nr][0]));
        }
        /* Set output setting */
        if(val){
            SET_GPREG2(nr_bit, REG_GPS_DATASET);
        }else{
            SET_GPREG2(nr_bit, REG_GPS_DATACLR);
        }
        SET_GPREG(nr_bit, REG_GPS_DIR);
        break;
    case GPIO_INPUT:
        if(__check_nr_ctrl(nr)){
            SET_GPREG2(nr_ctrl[nr][1], REG_GPC_RESENSET_(nr_ctrl[nr][0]));
        }
        /* Set input setting */
        CLR_GPREG(nr_bit, REG_GPS_DIR);
        SET_GPREG(nr_bit, REG_GPS_INEN);
        break;
    case GPIO_INPUT_PU:
    case GPIO_PULLUP:
        if((! __check_nr_ctrl(nr)) || __check_nr_pupd(nr, NO_PU)){
            spin_unlock_irqrestore(&gpio_spinlock, flags);
            RETURN1(-ENOEXEC, " "MYNAME"::set_gpio_func() : Return(%x)\n");
        }
        if(func == GPIO_PULLUP){
            CLR_GPREG(nr_bit, REG_GPS_INEN);
        }
        /* Set input setting */
        CLR_GPREG(nr_bit, REG_GPS_DIR);
        /* Set pull up setting */
        SET_GPREG2(nr_ctrl[nr][1], REG_GPC_PUPDSET_(nr_ctrl[nr][0]));
        SET_GPREG2(nr_ctrl[nr][1], REG_GPC_RESENCLR_(nr_ctrl[nr][0]));
        if(func == GPIO_INPUT_PU){
            SET_GPREG(nr_bit, REG_GPS_INEN);
        }
        break;
    case GPIO_INPUT_PD:
    case GPIO_PULLDN:
        if((! __check_nr_ctrl(nr)) || __check_nr_pupd(nr, NO_PD)){
            spin_unlock_irqrestore(&gpio_spinlock, flags);
            RETURN1(-ENOEXEC, " "MYNAME"::set_gpio_func() : Return(%x)\n");
        }
        if(func == GPIO_PULLDN){
            CLR_GPREG(nr_bit, REG_GPS_INEN);
        }
        /* Set input setting */
        CLR_GPREG(nr_bit, REG_GPS_DIR);
        /* Set pull down setting */
        SET_GPREG2(nr_ctrl[nr][1], REG_GPC_PUPDCLR_(nr_ctrl[nr][0]));
        SET_GPREG2(nr_ctrl[nr][1], REG_GPC_RESENCLR_(nr_ctrl[nr][0]));
        if(func == GPIO_INPUT_PD){
            SET_GPREG(nr_bit, REG_GPS_INEN);
        }
        break;
    case GPIO_NOSET:
        CLR_GPREG(nr_bit, REG_GPS_INEN);
        if(__check_nr_ctrl(nr)){
            SET_GPREG2(nr_ctrl[nr][1], REG_GPC_RESENSET_(nr_ctrl[nr][0]));
	}
        /* Set input setting */
        CLR_GPREG(nr_bit, REG_GPS_DIR);
        break;
    default:
        spin_unlock_irqrestore(&gpio_spinlock, flags);
        RETURN1(-ENOEXEC, " "MYNAME"::set_gpio_func() : Return(%x)\n");
    }
    spin_unlock_irqrestore(&gpio_spinlock, flags);

    RETURN1(0, " "MYNAME"::set_gpio_func() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * set_gpioe_func
 *---------------------------------------------------------------------------*/
static int set_gpioe_func(IN unsigned int nr, IN unsigned char func, 
    IN unsigned int val)
{
    unsigned int nr_bit;
    unsigned long flags;
    PDEBUG1(" "MYNAME"::set_gpioe_func(%d,%x,%x) : Entry\n", nr, func, val);

    nr_bit = GPIOE_NR(nr);
    spin_lock_irqsave(&gpio_spinlock, flags);
    switch(func){
    case GPIO_OUTPUT:
        CLR_GPREG(nr_bit, REG_GPE_INEN);
        if(__check_nr_ctrl(nr)){
            SET_GPREG2(nr_ctrl[nr][1], REG_GPC_RESENSET_(nr_ctrl[nr][0]));
        }
        /* Set output setting */
        if(val){
            SET_GPREG2(nr_bit, REG_GPE_DATASET);
        }else{
            SET_GPREG2(nr_bit, REG_GPE_DATACLR);
        }
        SET_GPREG(nr_bit, REG_GPE_DIR);
        break;
    case GPIO_INPUT:
        if(__check_nr_ctrl(nr)){
            SET_GPREG2(nr_ctrl[nr][1], REG_GPC_RESENSET_(nr_ctrl[nr][0]));
        }
        /* Set intput setting */
        CLR_GPREG(nr_bit, REG_GPE_DIR);
        SET_GPREG(nr_bit, REG_GPE_INEN);
        break;
    case GPIO_INPUT_PU:
    case GPIO_PULLUP:
        if((! __check_nr_ctrl(nr)) || __check_nr_pupd(nr, NO_PU)){
            spin_unlock_irqrestore(&gpio_spinlock, flags);
            RETURN1(-ENOEXEC, " "MYNAME"::set_gpioe_func() : Return(%x)\n");
        }
        if(func == GPIO_PULLUP){
            CLR_GPREG(nr_bit, REG_GPE_INEN);
        }
        /* Set intput setting */
        CLR_GPREG(nr_bit, REG_GPE_DIR);
        /* Set pull up setting */
        SET_GPREG2(nr_ctrl[nr][1], REG_GPC_PUPDSET_(nr_ctrl[nr][0]));
        SET_GPREG2(nr_ctrl[nr][1], REG_GPC_RESENCLR_(nr_ctrl[nr][0]));
        if(func == GPIO_INPUT_PU){
            SET_GPREG(nr_bit, REG_GPE_INEN);
        }
        break;
    case GPIO_INPUT_PD:
    case GPIO_PULLDN:
        if((! __check_nr_ctrl(nr)) || __check_nr_pupd(nr, NO_PD)){
            spin_unlock_irqrestore(&gpio_spinlock, flags);
            RETURN1(-ENOEXEC, " "MYNAME"::set_gpioe_func() : Return(%x)\n");
        }
        if(func == GPIO_PULLDN){
            CLR_GPREG(nr_bit, REG_GPE_INEN);
        }
        /* Set intput setting */
        CLR_GPREG(nr_bit, REG_GPE_DIR);
        /* Set pull down setting */
        SET_GPREG2(nr_ctrl[nr][1], REG_GPC_PUPDCLR_(nr_ctrl[nr][0]));
        SET_GPREG2(nr_ctrl[nr][1], REG_GPC_RESENCLR_(nr_ctrl[nr][0]));
        if(func == GPIO_INPUT_PD){
            SET_GPREG(nr_bit, REG_GPE_INEN);
        }
        break;
    case GPIO_NOSET:
        CLR_GPREG(nr_bit, REG_GPE_INEN);
        if(__check_nr_ctrl(nr)){
            SET_GPREG2(nr_ctrl[nr][1], REG_GPC_RESENSET_(nr_ctrl[nr][0]));
        }
        /* Set intput setting */
        CLR_GPREG(nr_bit, REG_GPE_DIR);
        break;
    default:
        spin_unlock_irqrestore(&gpio_spinlock, flags);
        RETURN1(-ENOEXEC, " "MYNAME"::set_gpioe_func() : Return(%x)\n");
    }
    spin_unlock_irqrestore(&gpio_spinlock, flags);

    RETURN1(0, " "MYNAME"::set_gpioe_func() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * set_gpioX_func
 *---------------------------------------------------------------------------*/
static int set_gpioX_func(IN unsigned int nr, IN unsigned char func, 
    IN unsigned int val)
{
    int err;
    unsigned int nr_bit;
    unsigned int nr_grp;
    unsigned long flags;
    PDEBUG1(" "MYNAME"::set_gpioX_func(%d,%x,%x) : Entry\n", nr, func, val);

    if((err = __get_nr_grpbit(nr, &nr_grp, &nr_bit)) < 0){
        RETURN1(err, " "MYNAME"::set_gpioX_func() : Return(%x)\n");
    }
    spin_lock_irqsave(&gpio_spinlock, flags);
    switch(func){
    case GPIO_OUTPUT:
        CLR_GPREG(nr_bit, REG_GPE_INEN_(nr_grp));
        /* Clear other setting */
        SET_GPREG2(nr_bit, REG_GPE_PORTSET_(nr_grp));	// __clr_gpioX_func_en()
        if(__check_nr_ctrl(nr)){
            SET_GPREG2(nr_ctrl[nr][1], REG_GPC_RESENSET_(nr_ctrl[nr][0]));
        }
        /* Set output setting */
        if(val){
            SET_GPREG2(nr_bit, REG_GPE_DATASET_(nr_grp));
        }else{
            SET_GPREG2(nr_bit, REG_GPE_DATACLR_(nr_grp));
        }
        SET_GPREG(nr_bit, REG_GPE_DIR_(nr_grp));
        break;
    case GPIO_INPUT:
        /* Clear other setting */
        SET_GPREG2(nr_bit, REG_GPE_PORTSET_(nr_grp));	// __clr_gpioX_func_en()
        if(__check_nr_ctrl(nr)){
            SET_GPREG2(nr_ctrl[nr][1], REG_GPC_RESENSET_(nr_ctrl[nr][0]));
        }
        /* Set intput setting */
        CLR_GPREG(nr_bit, REG_GPE_DIR_(nr_grp));
        SET_GPREG(nr_bit, REG_GPE_INEN_(nr_grp));
        break;
    case GPIO_INPUT_PU:
    case GPIO_PULLUP:
        if((! __check_nr_ctrl(nr)) || __check_nr_pupd(nr, NO_PU)){
            spin_unlock_irqrestore(&gpio_spinlock, flags);
            RETURN1(-ENOEXEC, " "MYNAME"::set_gpioX_func() : Return(%x)\n");
        }
        if(func == GPIO_PULLUP){
            CLR_GPREG(nr_bit, REG_GPE_INEN_(nr_grp));
        }
        /* Clear other setting */
        SET_GPREG2(nr_bit, REG_GPE_PORTSET_(nr_grp));	// __clr_gpioX_func_en()
        /* Set intput setting */
        CLR_GPREG(nr_bit, REG_GPE_DIR_(nr_grp));
        /* Set pull up setting */
        SET_GPREG2(nr_ctrl[nr][1], REG_GPC_PUPDSET_(nr_ctrl[nr][0]));
        SET_GPREG2(nr_ctrl[nr][1], REG_GPC_RESENCLR_(nr_ctrl[nr][0]));
        if(func == GPIO_INPUT_PU){
            SET_GPREG(nr_bit, REG_GPE_INEN_(nr_grp));
        }
        break;
    case GPIO_INPUT_PD:
    case GPIO_PULLDN:
        if((! __check_nr_ctrl(nr)) || __check_nr_pupd(nr, NO_PD)){
            spin_unlock_irqrestore(&gpio_spinlock, flags);
            RETURN1(-ENOEXEC, " "MYNAME"::set_gpioX_func() : Return(%x)\n");
        }
        if(func == GPIO_PULLDN){
            CLR_GPREG(nr_bit, REG_GPE_INEN_(nr_grp));
        }
        /* Clear other setting */
        SET_GPREG2(nr_bit, REG_GPE_PORTSET_(nr_grp));	// __clr_gpioX_func_en()
        /* Set intput setting */
        CLR_GPREG(nr_bit, REG_GPE_DIR_(nr_grp));
        /* Set pull down setting */
        SET_GPREG2(nr_ctrl[nr][1], REG_GPC_PUPDCLR_(nr_ctrl[nr][0]));
        SET_GPREG2(nr_ctrl[nr][1], REG_GPC_RESENCLR_(nr_ctrl[nr][0]));
        if(func == GPIO_INPUT_PD){
            SET_GPREG(nr_bit, REG_GPE_INEN_(nr_grp));
        }
        break;
    case GPIO_NOSET:
        CLR_GPREG(nr_bit, REG_GPE_INEN_(nr_grp));
        /* Clear other setting */
        SET_GPREG2(nr_bit, REG_GPE_PORTSET_(nr_grp));	// __clr_gpioX_func_en()
        if(__check_nr_ctrl(nr)){
            SET_GPREG2(nr_ctrl[nr][1], REG_GPC_RESENSET_(nr_ctrl[nr][0]));
        }
        /* Set intput setting */
        CLR_GPREG(nr_bit, REG_GPE_DIR_(nr_grp));
        break;
    default:
        if(__check_nr_func(nr, func)){
            /* Use port as peripheral */
            SET_GPREG2(nr_bit, REG_GPE_PORTCLR_(nr_grp));
            if(__check_func_HiZ(func)){
                CLR_GPREG(nr_bit, REG_GPE_INEN_(nr_grp));
                if(__check_nr_ctrl(nr)){
                    SET_GPREG2(nr_ctrl[nr][1], REG_GPC_RESENSET_(nr_ctrl[nr][0]));
                }
                /* Set intput setting */
                CLR_GPREG(nr_bit, REG_GPE_DIR_(nr_grp));
            }
            if(__check_func_PullUp(func)){
                CLR_GPREG(nr_bit, REG_GPE_INEN_(nr_grp));
                if(__check_nr_ctrl(nr) && (! __check_nr_pupd(nr, NO_PU))){
                    /* Set pull up setting */
                    SET_GPREG2(nr_ctrl[nr][1], REG_GPC_PUPDSET_(nr_ctrl[nr][0]));
                    SET_GPREG2(nr_ctrl[nr][1], REG_GPC_RESENCLR_(nr_ctrl[nr][0]));
		}
                /* Set intput setting */
                CLR_GPREG(nr_bit, REG_GPE_DIR_(nr_grp));
            }
            if(__check_func_PullDown(func)){
                CLR_GPREG(nr_bit, REG_GPE_INEN_(nr_grp));
                if(__check_nr_ctrl(nr) && (! __check_nr_pupd(nr, NO_PD))){
                    /* Set pull down setting */
                    SET_GPREG2(nr_ctrl[nr][1], REG_GPC_PUPDCLR_(nr_ctrl[nr][0]));
                    SET_GPREG2(nr_ctrl[nr][1], REG_GPC_RESENCLR_(nr_ctrl[nr][0]));
		}
                /* Set intput setting */
                CLR_GPREG(nr_bit, REG_GPE_DIR_(nr_grp));
            }
            break;
        }
        spin_unlock_irqrestore(&gpio_spinlock, flags);
        RETURN1(-ENOEXEC, " "MYNAME"::set_gpioX_func() : Return(%x)\n");
    }
    spin_unlock_irqrestore(&gpio_spinlock, flags);

    RETURN1(0, " "MYNAME"::set_gpioX_func() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * get_gpio_func
 *---------------------------------------------------------------------------*/
static int get_gpio_func(IN unsigned int nr, IN unsigned char *func)
{
    unsigned int nr_bit;
    PDEBUG1(" "MYNAME"::get_gpio_func(%d,%x) : Entry\n", nr, (int)func);

    nr_bit = GPIO_NR(nr);
    /* Check direction */
    if(GET_GPREG(nr_bit, REG_GPS_DIR)){
        if(GET_GPREG(nr_bit, REG_GPS_INEN)){
            RETURN1(-EINVAL, " "MYNAME"::get_gpio_func() : Return(%x)\n");
        }else{
            *func = GPIO_OUTPUT;
        }
    }else{
        if(__check_nr_ctrl(nr) && 
            (! GET_GPREG(nr_ctrl[nr][1], REG_GPC_RESEN_(nr_ctrl[nr][0])))){
            if(GET_GPREG(nr_bit, REG_GPS_INEN)){
                if(GET_GPREG(nr_ctrl[nr][1], REG_GPC_PUPD_(nr_ctrl[nr][0]))){
                    *func = GPIO_INPUT_PU;
                }else{
                    *func = GPIO_INPUT_PD;
                }
            }else{
                if(GET_GPREG(nr_ctrl[nr][1], REG_GPC_PUPD_(nr_ctrl[nr][0]))){
                    *func = GPIO_PULLUP;
                }else{
                    *func = GPIO_PULLDN;
                }
            }
        }else{
            if(GET_GPREG(nr_bit, REG_GPS_INEN)){
                *func = GPIO_INPUT;
            }else{
                *func = GPIO_NOSET;
            }
        }
    }

    RETURN1(0, " "MYNAME"::get_gpio_func() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * get_gpioe_func
 *---------------------------------------------------------------------------*/
static int get_gpioe_func(IN unsigned int nr, IN unsigned char *func)
{
    unsigned int nr_bit;
    PDEBUG1(" "MYNAME"::get_gpioe_func(%d,%x) : Entry\n", nr, (int)func);

    nr_bit = GPIOE_NR(nr);
    /* Check direction */
    if(GET_GPREG(nr_bit, REG_GPE_DIR)){
        if(GET_GPREG(nr_bit, REG_GPE_INEN)){
            RETURN1(-EINVAL, " "MYNAME"::get_gpioe_func() : Return(%x)\n");
        }else{
            *func = GPIO_OUTPUT;
        }
    }else{
        if(__check_nr_ctrl(nr) && 
            (! GET_GPREG(nr_ctrl[nr][1], REG_GPC_RESEN_(nr_ctrl[nr][0])))){
            if(GET_GPREG(nr_bit, REG_GPE_INEN)){
                if(GET_GPREG(nr_ctrl[nr][1], REG_GPC_PUPD_(nr_ctrl[nr][0]))){
                    *func = GPIO_INPUT_PU;
                }else{
                    *func = GPIO_INPUT_PD;
                }
            }else{
                if(GET_GPREG(nr_ctrl[nr][1], REG_GPC_PUPD_(nr_ctrl[nr][0]))){
                    *func = GPIO_PULLUP;
                }else{
                    *func = GPIO_PULLDN;
                }
            }
        }else{
            if(GET_GPREG(nr_bit, REG_GPE_INEN)){
                *func = GPIO_INPUT;
            }else{
                *func = GPIO_NOSET;
            }
        }
    }

    RETURN1(0, " "MYNAME"::get_gpioe_func() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * get_gpioX_func
 *---------------------------------------------------------------------------*/
static int get_gpioX_func(IN unsigned int nr, IN unsigned char *func)
{
    int err;
    unsigned int nr_bit;
    unsigned int nr_grp;
    unsigned char sel_func;
    PDEBUG1(" "MYNAME"::get_gpioX_func(%d,%x) : Entry\n", nr, (int)func);

    if((err = __get_nr_grpbit(nr, &nr_grp, &nr_bit)) < 0){
        RETURN1(err, " "MYNAME"::get_gpioX_func() : Return(%x)\n");
    }
    if((err = __get_nr_func(nr, &sel_func)) < 0){
        RETURN1(err, " "MYNAME"::get_gpioX_func() : Return(%x)\n");
    }
    /* Check port use */
    if(! GET_GPREG(nr_bit, REG_GPE_PORTSEL_(nr_grp))){
        if(! GET_GPREG(nr_bit, REG_GPE_DIR_(nr_grp))){
            if(__check_nr_ctrl(nr) && 
                (! GET_GPREG(nr_ctrl[nr][1], REG_GPC_RESEN_(nr_ctrl[nr][0])))){
                if(! GET_GPREG(nr_bit, REG_GPE_INEN_(nr_grp))){
                    if(GET_GPREG(nr_ctrl[nr][1], REG_GPC_PUPD_(nr_ctrl[nr][0]))){
                        sel_func = GPIO_FUNC_PERIPU;
                    }else{
                        sel_func = GPIO_FUNC_PERIPD;
                    }
                }
            }else{
                if(! GET_GPREG(nr_bit, REG_GPE_INEN_(nr_grp))){
                    sel_func = GPIO_FUNC_PERIHZ;
                }
            }
        }
        *func = sel_func;
        RETURN1(0, " "MYNAME"::get_gpioX_func() : Return(%x)\n");
    }
    /* Check direction */
    if(GET_GPREG(nr_bit, REG_GPE_DIR_(nr_grp))){
        if(GET_GPREG(nr_bit, REG_GPE_INEN_(nr_grp))){
            RETURN1(-EINVAL, " "MYNAME"::get_gpioX_func() : Return(%x)\n");
        }else{
            *func = GPIO_OUTPUT;
        }
    }else{
        if(__check_nr_ctrl(nr) && 
            (! GET_GPREG(nr_ctrl[nr][1], REG_GPC_RESEN_(nr_ctrl[nr][0])))){
            if(GET_GPREG(nr_bit, REG_GPE_INEN_(nr_grp))){
                if(GET_GPREG(nr_ctrl[nr][1], REG_GPC_PUPD_(nr_ctrl[nr][0]))){
                    *func = GPIO_INPUT_PU;
                }else{
                    *func = GPIO_INPUT_PD;
                }
            }else{
                if(GET_GPREG(nr_ctrl[nr][1], REG_GPC_PUPD_(nr_ctrl[nr][0]))){
                    *func = GPIO_PULLUP;
                }else{
                     *func = GPIO_PULLDN;
                }
            }
        }else{
            if(GET_GPREG(nr_bit, REG_GPE_INEN_(nr_grp))){
                *func = GPIO_INPUT;
            }else{
                *func = GPIO_NOSET;
            }
        }
    }

    RETURN1(0, " "MYNAME"::get_gpioX_func() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * inline is_gpio
 *---------------------------------------------------------------------------*/
static __inline__ int __is_gpio(IN unsigned int nr)
{
    if(nr >= GPIO_0 && nr <= GPIO_15){
        return 1;
    }
    return 0;
}

/*-----------------------------------------------------------------------------
 * inline is_gpioe
 *---------------------------------------------------------------------------*/
static __inline__ int __is_gpioe(IN unsigned int nr)
{
    if(nr >= GPIOE_0 && nr <= GPIOE_15){
        return 1;
    }
    return 0;
}

/*-----------------------------------------------------------------------------
 * inline get_nr_grp
 *---------------------------------------------------------------------------*/
static __inline__ int __get_nr_grp(IN unsigned int nr, OUT unsigned int *grp)
{
    int i;
    if(nr > PORT_MAX){
        return -EINVAL;
    }
    for(i = ARRAY_SIZE(nr_base); i > 0; i--){
        if(nr >= nr_base[i - 1]){
            *grp = i - 1;
            return 0;
        }
    }
    return -EINVAL;
}

/*-----------------------------------------------------------------------------
 * inline get_nr_grpbit
 *---------------------------------------------------------------------------*/
static __inline__ int __get_nr_grpbit(IN unsigned int nr, 
    OUT unsigned int *grp, OUT unsigned int *bit)
{
    int err;
    if((err = __get_nr_grp(nr, grp)) < 0){
        return err;
    }
    *bit = nr - nr_base[*grp];
    return 0;
}

/*-----------------------------------------------------------------------------
 * inline check_nr_func
 *---------------------------------------------------------------------------*/
static __inline__ int __check_nr_func(IN unsigned int nr, 
    IN unsigned char func)
{
    int i;
    if(nr > PORT_MAX){
        return -EINVAL;
    }
    for(i = 0; i < ARRAY_SIZE(nr_func[nr]); i++){
        if(nr_func[nr][i] == func){
            return 1;
        }
    }
    return 0;
}

/*-----------------------------------------------------------------------------
 * inline check_nr_ctrl
 *---------------------------------------------------------------------------*/
static __inline__ int __check_nr_ctrl(IN unsigned int nr)
{
    if(nr > PORT_MAX){
        return -EINVAL;
    }
    if(nr_ctrl[nr][0] <= CTRL_MAX){
        return 1;
    }
    return 0;
}

/*-----------------------------------------------------------------------------
 * inline check_nr_pupd
 *---------------------------------------------------------------------------*/
static __inline__ int __check_nr_pupd(IN unsigned int nr, 
    IN unsigned char pupd)
{
    if(nr > PORT_MAX){
        return -EINVAL;
    }
    if(nr_ctrl[nr][2] == pupd){
        return 1;
    }
    return 0;
}

/*-----------------------------------------------------------------------------
 * inline check_func_HiZ
 *---------------------------------------------------------------------------*/
static __inline__ int __check_func_HiZ(IN unsigned char func)
{
    if(func == GPIO_FUNC_PERIHZ){
        return 1;
    }
    return 0;
}

/*-----------------------------------------------------------------------------
 * inline check_func_PullUp
 *---------------------------------------------------------------------------*/
static __inline__ int __check_func_PullUp(IN unsigned char func)
{
    if(func == GPIO_FUNC_PERIPU){
        return 1;
    }
    return 0;
}

/*-----------------------------------------------------------------------------
 * inline check_func_PullDown
 *---------------------------------------------------------------------------*/
static __inline__ int __check_func_PullDown(IN unsigned char func)
{
    if(func == GPIO_FUNC_PERIPD){
        return 1;
    }
    return 0;
}

/*-----------------------------------------------------------------------------
 * inline get_nr_func
 *---------------------------------------------------------------------------*/
static __inline__ int __get_nr_func(IN unsigned int nr, 
    OUT unsigned char *func)
{
    if(nr > PORT_MAX){
        return -EINVAL;
    }
    *func = nr_func[nr][0];
    return 0;
}

/*-----------------------------------------------------------------------------
 * inline suspend
 *---------------------------------------------------------------------------*/
static __inline__ int __suspend(IN struct amba_device *dev, IN u32 state)
{
    unsigned int i;
    PDEBUG1(" "MYNAME"::__suspend(%x) : Entry\n", (int)dev);

#ifdef CHECK_SIZE
    printk("__suspend() : sizeof(RegGpio) -> %d\n", sizeof(RegGpio));
#endif
    if(! (reggpio = kmalloc(sizeof(RegGpio), GFP_KERNEL | GFP_USER))){
        PDEBUG3(" "MYNAME"::__suspend() : kmalloc() Err\n");
        RETURN1(-ENOSPC, " "MYNAME"::__suspend() : Return(%x)\n");
    }
    /* gpio sys */
    reggpio->sys.dir = readw(REG_GPS_DIR);
    reggpio->sys.rdata = readw(REG_GPS_RDATA);
    reggpio->sys.intmask = readw(REG_GPS_INTMASK);
    reggpio->sys.intle = readw(REG_GPS_INTLE);
    reggpio->sys.inthe = readw(REG_GPS_INTHE);
    reggpio->sys.ints = readw(REG_GPS_INTS);
    reggpio->sys.inen = readw(REG_GPS_INEN);
    /* gpio easyX */
    for(i = 0; i < ARRAY_SIZE(nr_base); i++){
        reggpio->easy[i].dir = readw(REG_GPE_DIR_(i));
        reggpio->easy[i].rdata = readw(REG_GPE_RDATA_(i));
        reggpio->easy[i].inen = readw(REG_GPE_INEN_(i));
        reggpio->easy[i].portsel = readw(REG_GPE_PORTSEL_(i));
    }
    /* gpio easy */
    reggpio->easy[i].dir = readw(REG_GPE_DIR);
    reggpio->easy[i].rdata = readw(REG_GPE_RDATA);
    reggpio->easy[i].inen = readw(REG_GPE_INEN);
    /* gpio ctrl */
    for(i = 0; i < CTRL_NUM; i++){
        reggpio->ctrl[i].drv2 = readw(REG_GPC_DRV2_(i));
        reggpio->ctrl[i].drv1 = readw(REG_GPC_DRV1_(i));
        reggpio->ctrl[i].resen = readw(REG_GPC_RESEN_(i));
        reggpio->ctrl[i].pupd = readw(REG_GPC_PUPD_(i));
    }
    /* gpio misc */
    reggpio->misc.misc3 = readw(REG_GPM_MISC3);

    RETURN1(0, " "MYNAME"::__suspend() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * inline resume
 *---------------------------------------------------------------------------*/
static __inline__ int __resume(IN struct amba_device *dev)
{
    unsigned int i;
    PDEBUG1(" "MYNAME"::__resume(%x) : Entry\n", (int)dev);

    if(reggpio){
        /* gpio sys */
        writew(reggpio->sys.inen, REG_GPS_INEN);
        writew(reggpio->sys.ints, REG_GPS_INTS);
        writew(reggpio->sys.inthe, REG_GPS_INTHE);
        writew(reggpio->sys.intle, REG_GPS_INTLE);
        writew(reggpio->sys.intmask, REG_GPS_INTMASK);
        writew(reggpio->sys.rdata, REG_GPS_DATASET);
        writew(~reggpio->sys.rdata, REG_GPS_DATACLR);
        writew(reggpio->sys.dir, REG_GPS_DIR);
        /* gpio easyX */
        for(i = 0; i < ARRAY_SIZE(nr_base); i++){
            writew(reggpio->easy[i].portsel, REG_GPE_PORTSET_(i));
            writew(~reggpio->easy[i].portsel, REG_GPE_PORTCLR_(i));
            writew(reggpio->easy[i].inen, REG_GPE_INEN_(i));
            writew(reggpio->easy[i].rdata, REG_GPE_DATASET_(i));
            writew(~reggpio->easy[i].rdata, REG_GPE_DATACLR_(i));
            writew(reggpio->easy[i].dir, REG_GPE_DIR_(i));
        }
        /* gpio easy */
        writew(reggpio->easy[i].inen, REG_GPE_INEN);
        writew(reggpio->easy[i].rdata, REG_GPE_DATASET);
        writew(~reggpio->easy[i].rdata, REG_GPE_DATACLR);
        writew(reggpio->easy[i].dir, REG_GPE_DIR);
        /* gpio ctrl */
        for(i = 0; i < CTRL_NUM; i++){
            writew(reggpio->ctrl[i].pupd, REG_GPC_PUPDSET_(i));
            writew(~reggpio->ctrl[i].pupd, REG_GPC_PUPDCLR_(i));
            writew(reggpio->ctrl[i].resen, REG_GPC_RESENSET_(i));
            writew(~reggpio->ctrl[i].resen, REG_GPC_RESENCLR_(i));
            writew(reggpio->ctrl[i].drv1, REG_GPC_DRV1SET_(i));
            writew(~reggpio->ctrl[i].drv1, REG_GPC_DRV1CLR_(i));
            writew(reggpio->ctrl[i].drv2, REG_GPC_DRV2SET_(i));
            writew(~reggpio->ctrl[i].drv2, REG_GPC_DRV2CLR_(i));
        }
        /* gpio misc */
        writew(reggpio->misc.misc3, REG_GPM_MISC3SET);
        writew(~reggpio->misc.misc3, REG_GPM_MISC3CLR);
        kfree(reggpio);
        reggpio = 0;
    }

    RETURN1(0, " "MYNAME"::__resume() : Return(%x)\n");
}

/*-----------------------------------------------------------------------------
 * inline print_register
 *--------------------------------------------------------------------------*/
static __inline__ void __print_register(void)
{
    unsigned int i;
    PDEBUG2("gpio_dir      = 0x%04x\n", readw(REG_GPS_DIR));
    PDEBUG2("gpio_rdata    = 0x%04x\n", readw(REG_GPS_RDATA));
    PDEBUG2("gpio_ints     = 0x%04x\n", readw(REG_GPS_INTS));
    PDEBUG2("gpio_inthe    = 0x%04x\n", readw(REG_GPS_INTHE));
    PDEBUG2("gpio_intle    = 0x%04x\n", readw(REG_GPS_INTLE));
    PDEBUG2("gpio_intmask  = 0x%04x\n", readw(REG_GPS_INTMASK));
    PDEBUG2("gpio_intst    = 0x%04x\n", readw(REG_GPS_INTST));
    PDEBUG2("gpio_inen     = 0x%04x\n", readw(REG_GPS_INEN));
    for(i = 0; i < ARRAY_SIZE(nr_base); i++){
        PDEBUG2("gpio%d_dir     = 0x%04x\n", i, readw(REG_GPE_DIR_(i)));
        PDEBUG2("gpio%d_rdata   = 0x%04x\n", i, readw(REG_GPE_RDATA_(i)));
        PDEBUG2("gpio%d_inen    = 0x%04x\n", i, readw(REG_GPE_INEN_(i)));
        PDEBUG2("gpio%d_portsel = 0x%04x\n", i, readw(REG_GPE_PORTSEL_(i)));
    }
    PDEBUG2("gpioe_dir     = 0x%04x\n", readw(REG_GPE_DIR));
    PDEBUG2("gpioe_rdata   = 0x%04x\n", readw(REG_GPE_RDATA));
    PDEBUG2("gpioe_inen    = 0x%04x\n", readw(REG_GPE_INEN));
    for(i = 0; i < CTRL_NUM; i++){
        PDEBUG2("ctrl%d_drv2    = 0x%04x\n", i, readw(REG_GPC_DRV2_(i)));
        PDEBUG2("ctrl%d_drv1    = 0x%04x\n", i, readw(REG_GPC_DRV1_(i)));
        PDEBUG2("ctrl%d_resen   = 0x%04x\n", i, readw(REG_GPC_RESEN_(i)));
        PDEBUG2("ctrl%d_pupd    = 0x%04x\n", i, readw(REG_GPC_PUPD_(i)));
    }
    PDEBUG2("gpio_misc3    = 0x%04x\n", i, readw(REG_GPM_MISC3));
    return;
}

/*=============================================================================
 * Export symbols
 *===========================================================================*/

//module_init(gpio_cxd4108_init);
module_exit(gpio_cxd4108_exit);
