/*
 * usb_otgcore.c
 * 
 * Copyright 2005,2006 Sony Corporation
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 */
 
/*-----------------------------------------------------------------------------
 * Include file
 *---------------------------------------------------------------------------*/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/ioctl.h>
#include <linux/wait.h>
#include <linux/slab.h>
#include <linux/smp_lock.h>
#include <linux/proc_fs.h>
#include <linux/moduleparam.h>
#include <linux/errno.h>
#include <asm/semaphore.h>
#include <asm/uaccess.h>
#include <linux/device.h>
#include <linux/usb_ch9.h>
#include <asm/arch/sonymisc.h>
#include <linux/usb/mcd/usb_otg_control.h>
#include <linux/usb/gcore/usb_event.h>
#include <linux/usb/gcore/usb_otgcore.h>

#include "usb_otgcore_cfg.h"
#include "usb_otgcore_pvt.h"

/*-----------------------------------------------------------------------------
 * Module infomation
 *---------------------------------------------------------------------------*/
MODULE_AUTHOR("Sony Corporation");
MODULE_DESCRIPTION(USBOTGCORE_NAME
                   " driver ver " USBOTGCORE_VERSION);
MODULE_LICENSE("GPL");

#define MYDRIVER_NAME    USBOTGCORE_NAME

/*-----------------------------------------------------------------------------
 * Function prototype declaration
 *---------------------------------------------------------------------------*/
#if !(defined(CONFIG_ARCH_CXD4105) && defined(CONFIG_USB_OTG_CORE))
  #if !defined(CONFIG_USB_OTG_CORE_MODULE)
   #error "Error: Kconfig"
  #endif
  static
#endif
int __init usb_otgcore_module_init(void);
static void __exit usb_otgcore_module_exit(void);
static int proc_read(char *, char **, off_t, int, int *, void *);

static int __init otg_core_alloc(void);
static void otg_core_free(struct otg_core_drv *otg_core);
static int otg_core_bind(struct usb_otg_control *otg_control);
static int otg_core_unbind(struct usb_otg_control *otg_control);
static int otg_core_notify(union usb_otg_event *otg_event);

static int event_cid_exec(unsigned int);
static int event_vbus_exec(enum usb_otg_vbus_stat);
static int event_vbus_error_exec(void);
static int event_receive_srp_exec(void);
static int event_rchost_end_exec(unsigned int);
static int event_rcgadget_end_exec(unsigned int);

static int transition_to_idle(void);
static int transition_to_stop(void);

/*-----------------------------------------------------------------------------
 * Variable declaration
 *---------------------------------------------------------------------------*/
static struct otg_core_drv *the_otg_core;

static struct usb_otg_core otg_core_driver =
{
    .bind   = otg_core_bind,
    .unbind = otg_core_unbind,
    .notify = otg_core_notify,
};

/*=============================================================================
 *
 * Main function body
 *
 *===========================================================================*/
int usb_otgcore_gadget_suspend(void)
{
    struct otg_core_drv *otg_core = the_otg_core;
    unsigned long flags;
    int res;
    
    PDEBUG("%s call\n", __func__);
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    if(test_bit(USB_OTGCORE_ATOMIC_ENABLE_RCHOST, &otg_core->bitflags)){
        /* enable_rchost()ѤߤʤRollChangeγϤȽ */
        res = USB_OTGCORE_RES_START_RCHOST;
    }else{
        /* enable_rchost()ѤߤǤʤ̾suspendȽ */
        res = USB_OTGCORE_RES_SUSPEND;
    }
    
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    if(res == USB_OTGCORE_RES_START_RCHOST){
        PVERBOSE("result: USB_OTGCORE_RES_START_RCHOST\n");
    }else{
        PVERBOSE("result: USB_OTGCORE_RES_SUSPEND\n");
    }
    
    return res;
}

unsigned char usb_otgcore_get_hs_disable(void)
{
    struct otg_core_drv *otg_core = the_otg_core;
    unsigned long flags;
    unsigned char hs_disable;
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    hs_disable = otg_core->hs_disable;
    
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    return hs_disable;
}

int usb_otgcore_gadget_set_feature(__le16 feature_selector)
{
    struct otg_core_drv *otg_core = the_otg_core;
    unsigned long flags;
    int res = 0;
    
    PDEBUG("%s call\n", __func__);
    PVERBOSE(" feature_selector: %04x\n", feature_selector);
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    /* Ǥϡb_hnp_enableΤߤ */
    if(feature_selector != USB_DEVICE_B_HNP_ENABLE){
        goto exit;
    }
    
    /* Hostˤä HNP Ĥ줿ȤϿ */
    set_bit(USB_OTGCORE_ATOMIC_HNP_ENABLE_HOST, &otg_core->bitflags);
    
    /* Userˤä HNP ĤƤ             &&
       usb_otg_control_enable_rchost() ȯԺѤߤǤʤ    */
    if(test_bit(USB_OTGCORE_ATOMIC_HNP_ENABLE_USER, &otg_core->bitflags) &&
       test_bit(USB_OTGCORE_ATOMIC_ENABLE_RCHOST, &otg_core->bitflags) == 0){
        
        set_bit(USB_OTGCORE_ATOMIC_ENABLE_RCHOST, &otg_core->bitflags);
        
        /* rchostĤ */
        PDEBUG("usb_otg_control_enable_rchost()\n");
        res = usb_otg_control_enable_rchost(otg_core->otg_control);
        if(res != 0){
            /* Ԥ usb_otg_control_enable_rchost() ȯԺѤߤǤʤ 
               ֤᤹ */
            clear_bit(USB_OTGCORE_ATOMIC_ENABLE_RCHOST, &otg_core->bitflags);
        }
    }
    
exit:
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    if(res != 0){
        PERR("usb_otg_control_enable_rchost()\n");
    }
    
    return 0;
}

void usb_otgcore_gadget_disconnect(void)
{
    struct otg_core_drv *otg_core = the_otg_core;
    unsigned long flags;
    int res_testmode = 0, res_rchost = 0;
    
    PDEBUG("%s call\n", __func__);
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    /* TestModeʤNormal᤹ */
    if(otg_core->test_mode != USB_OTGCORE_TEST_MODE_NORMAL){
        res_testmode = usb_otg_control_set_test_mode(otg_core->otg_control, 
                                                     USB_OTG_TEST_MODE_NORMAL);
        if(res_testmode == 0){
            otg_core->test_mode = USB_OTGCORE_TEST_MODE_NORMAL;
        }
    }
    
    /* enable_rchost() ȯԺѤߤʤdisable_rchost() Ƥ */
    if(test_and_clear_bit(USB_OTGCORE_ATOMIC_ENABLE_RCHOST, &otg_core->bitflags)){
        res_rchost = usb_otg_control_disable_rchost(otg_core->otg_control);
    }
    
    /* Host ˤhnp_enable򥯥ꥢ */
    clear_bit(USB_OTGCORE_ATOMIC_HNP_ENABLE_HOST, &otg_core->bitflags);
    
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    if(res_testmode != 0){
        PERR("usb_otg_control_set_test_mode()\n");
    }
    
    if(res_rchost != 0){
        PERR("usb_otg_control_disable_rchost()\n");
    }
    
    return;
}

/*-------------------------------------------------------------------------*/
static int
ioctl_probe(void *arg)
{
    struct otg_core_drv *otg_core = the_otg_core;
    struct usb_otgcore_probe_info tmp_info;
    unsigned long flags;
    unsigned long res;
    
    PDEBUG("%s call\n", __func__);
    
    /* User֤饳ԡ */
    res = copy_from_user(&tmp_info, arg, sizeof(tmp_info));
    if(res != 0){
        PWARN("error: probe failed\n");
        return -EFAULT;
    }
    
    /* ǧ */
    if(tmp_info.hndl == 0){
        PWARN("error: handle is 0\n");
        return -EINVAL;
    }
    
    /* ǤprobeѤߤǧ */
    if(otg_core->otg_probe.hndl != 0){
        PWARN("error: probe failed\n");
        /* ǤprobeѤߤʤХ顼 */
        return -EBUSY;
    }
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    /* tmp饳ԡ */
    memcpy(&otg_core->otg_probe,
           &tmp_info,
           sizeof(struct usb_otgcore_probe_info));
    
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    PDEBUG("otg_probe.hndl  :   %lu\n", otg_core->otg_probe.hndl);
    PDEBUG("otg_probe.event : 0x%lx\n", (unsigned long)&otg_core->otg_probe.event);
    PDEBUG(" event.cid : 0x%lx\n", (unsigned long)otg_core->otg_probe.event.cid);
    PDEBUG(" event.vbus: 0x%lx\n", (unsigned long)otg_core->otg_probe.event.vbus);
    PDEBUG(" event.vbus_error : 0x%lx\n", (unsigned long)otg_core->otg_probe.event.vbus_error);
    PDEBUG(" event.pullup : 0x%lx\n", (unsigned long)otg_core->otg_probe.event.pullup);
    PDEBUG(" event.receive_srp : 0x%lx\n", (unsigned long)otg_core->otg_probe.event.receive_srp);
    PDEBUG(" event.rchost : 0x%lx\n", (unsigned long)otg_core->otg_probe.event.rchost);
    PDEBUG(" event.rcgadget : 0x%lx\n", (unsigned long)otg_core->otg_probe.event.rcgadget);
    PDEBUG(" event.set_feature : 0x%lx\n", (unsigned long)otg_core->otg_probe.event.set_feature);
    
    return 0;
}

/*-------------------------------------------------------------------------*/
static int
ioctl_remove(void *arg)
{
    struct otg_core_drv *otg_core = the_otg_core;
    unsigned long flags;
    
    PDEBUG("%s call\n", __func__);
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    /* Handle0ˤ */
    otg_core->otg_probe.hndl = 0;
    
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    return 0;
}

/*-------------------------------------------------------------------------*/
static int transition_to_stop(void)
{
    struct otg_core_drv *otg_core = the_otg_core;
    unsigned long flags;
    int mode;
    int res;
    
    PDEBUG("%s call\n", __func__);
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    /* ߤΥ⡼ɤ */
    mode = usb_otg_control_get_mode(otg_core->otg_control);
    
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    /* IDLE֤Ǥ뤳Ȥǧ */
    if(mode != USB_OTG_CONTROL_IDLE){
        PERR("mode error\n");
        res = -EBUSY;
        goto exit;
    }
    
    /* controllerߤ */
    res = usb_otg_control_stop_control(otg_core->otg_control);
    if(res != 0){
        PERR("usb_otg_control_stop_control()\n");
    }
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    /* ݻƤVBUS֤UNKNOWN˽ */
    otg_core->vbus = USB_OTGCORE_VBUS_UNKNOWN;
    
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
exit:
    return res;
}

static int
ioctl_stop(void *arg)
{
    return transition_to_stop();
}

/*-------------------------------------------------------------------------*/
static int
transition_to_idle(void)
{
    struct otg_core_drv *otg_core = the_otg_core;
    unsigned long flags;
    int mode;
    int res = 0;
    
    PDEBUG("%s call\n", __func__);
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    /* ߤΥ⡼ɤ */
    mode = usb_otg_control_get_mode(otg_core->otg_control);
    
    /* Gadget֤λΤߤstop */
    if(mode == USB_OTG_CONTROL_GADGET){
    
        /* usb_otg_control_enable_rchost() ȯԺѤߤʤФdisableˤ */
        if(test_and_clear_bit(USB_OTGCORE_ATOMIC_ENABLE_RCHOST, &otg_core->bitflags)){
            /* rchostԵĤˤ */
            usb_otg_control_disable_rchost(otg_core->otg_control);
        }
        
        /* TestMode ʤв */
        if(otg_core->test_mode != USB_OTGCORE_TEST_MODE_NORMAL){
        
            /* TestModeNORMALˤ */
            PDEBUG("usb_otg_control_set_test_mode(USB_OTG_TEST_MODE_NORMAL)\n");
            res = usb_otg_control_set_test_mode(otg_core->otg_control, 
                                                USB_OTG_TEST_MODE_NORMAL);
            if(res == 0){
                otg_core->test_mode = USB_OTGCORE_TEST_MODE_NORMAL;
            }else{
                PERR("usb_otg_control_set_test_mode(USB_OTG_TEST_MODE_NORMAL)\n");
            }
        }
        
        /* Gadgetλ hs_disable  DISABLE ˤʤ */
        otg_core->hs_disable = USB_OTGCORE_HS_DISABLE;
        
        /* Gadgetλ */
        res = usb_otg_control_stop_gadget(otg_core->otg_control);
    }
    
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    if(mode == USB_OTG_CONTROL_IDLE     ||
       mode == USB_OTG_CONTROL_HOST     ||
       mode == USB_OTG_CONTROL_RCHOST   ||
       mode == USB_OTG_CONTROL_RCGADGET   ){
        
        if(otg_core->test_mode != USB_OTGCORE_TEST_MODE_NORMAL         ||
           test_bit(USB_OTGCORE_ATOMIC_E_HOST, &otg_core->bitflags)   ){
            
            /* TestModeNORMALˤ */
            PDEBUG("usb_otg_control_set_test_mode(USB_OTG_TEST_MODE_NORMAL)\n");
            res = usb_otg_control_set_test_mode(otg_core->otg_control, 
                                                USB_OTG_TEST_MODE_NORMAL);
            if(res == 0){
                otg_core->test_mode = USB_OTGCORE_TEST_MODE_NORMAL;
                clear_bit(USB_OTGCORE_ATOMIC_E_HOST, &otg_core->bitflags);
            }else{
                PERR("usb_otg_control_set_test_mode(USB_OTG_TEST_MODE_NORMAL)\n");
            }
        }
    }
    
    /* ⡼ɤ˱ƽ򤹤 */
    switch(mode){
      case USB_OTG_CONTROL_STOP:
        /* Controllerstart */
        res = usb_otg_control_start_control(otg_core->otg_control);
        break;
        
      case USB_OTG_CONTROL_IDLE:
        /* ⤷ʤ */
        res = 0;
        break;
        
      case USB_OTG_CONTROL_GADGET:
        /* ǤstopƤΤǲ⤷ʤ */
        break;
        
      case USB_OTG_CONTROL_HOST:
        /* Hostߤ */
        res = usb_otg_control_stop_host(otg_core->otg_control);
        break;
        
      case USB_OTG_CONTROL_RCGADGET:
        /* RCGadgetߤ */
        res = usb_otg_control_stop_rcgadget(otg_core->otg_control, 0);
        break;
        
      case USB_OTG_CONTROL_RCHOST:
        /* RCHostߤ */
        res = usb_otg_control_stop_rchost(otg_core->otg_control);
        break;
        
      default:
        res = -EINVAL;
        break;
    }
    
    if(res != 0){
        PERR("transition to IDLE mode\n");
    }
    return res;
}

/*-------------------------------------------------------------------------*/
static int
ioctl_idle(void *arg)
{
    return transition_to_idle();
}

/*-------------------------------------------------------------------------*/
static int
ioctl_start_gadget(void *arg)
{
    struct otg_core_drv *otg_core = the_otg_core;
    unsigned long flags;
    int mode;
    int res;
    
    PDEBUG("%s call\n", __func__);
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    /* ߤΥ⡼ɤ */
    mode = usb_otg_control_get_mode(otg_core->otg_control);
    
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    /* ߤξ֤IDLEǧ */
    if(mode != USB_OTG_CONTROL_IDLE){
        PERR("mode error\n");
        res = -EBUSY;
        goto exit;
    }
    
    /* Gadget 򳫻Ϥ */
    res = usb_otg_control_start_gadget(otg_core->otg_control);
    if(res != 0){
        PERR("transfer to Gadget\n");
        goto exit;
    }
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    /* Gadget򳫻Ϥ hs_disable  ENABLE ˤʤ */
    otg_core->hs_disable = USB_OTGCORE_HS_ENABLE;
    
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
exit:
    return res;
}

/*-------------------------------------------------------------------------*/
static int
ioctl_start_host(void *arg)
{
    struct otg_core_drv *otg_core = the_otg_core;
    unsigned long flags;
    int mode;
    int res;
    
    PDEBUG("%s call\n", __func__);
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    /* ߤΥ⡼ɤ */
    mode = usb_otg_control_get_mode(otg_core->otg_control);
    
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    /* ߤξ֤IDLEǧ */
    if(mode != USB_OTG_CONTROL_IDLE){
        PERR("mode error\n");
        res = -EBUSY;
        goto exit;
    }
    
    /* Host 򳫻Ϥ */
    res = usb_otg_control_start_host(otg_core->otg_control);
    if(res != 0){
        PERR("transfer to Host\n");
        goto exit;
    }
    
exit:
    return res;
}

/*-------------------------------------------------------------------------*/
static int
ioctl_start_e_host(void *arg)
{
    struct otg_core_drv *otg_core = the_otg_core;
    unsigned long flags;
    int mode;
    int res = 0;
    
    PDEBUG("%s call\n", __func__);
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    /* ߤΥ⡼ɤ */
    mode = usb_otg_control_get_mode(otg_core->otg_control);
    
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    /* IDLE֤Ǥ뤳Ȥǧ */
    if(mode != USB_OTG_CONTROL_IDLE){
        PERR("mode error\n");
        res = -EBUSY;
        goto exit;
    }
    
    res = usb_otg_control_set_test_mode(otg_core->otg_control, 
                                        USB_OTG_TEST_MODE_H_AS);
    if(res != 0){
        PERR("usb_otg_control_set_test_mode(USB_OTG_TEST_MODE_H_AS)\n");
        goto exit;
    }
    
    /* Host 򳫻Ϥ */
    res = usb_otg_control_start_host(otg_core->otg_control);
    if(res != 0){
        PERR("transfer to Host\n");
        goto exit;
    }
    
    set_bit(USB_OTGCORE_ATOMIC_E_HOST, &otg_core->bitflags);
    
exit:
    return res;
}

/*-------------------------------------------------------------------------*/
static int
ioctl_enable_rchost(void *arg)
{
    struct otg_core_drv *otg_core = the_otg_core;
    unsigned long flags;
    int mode;
    int res = 0;
    
    PDEBUG("%s call\n", __func__);
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    /* ߤΥ⡼ɤ */
    mode = usb_otg_control_get_mode(otg_core->otg_control);
    
    /* ߤξ֤Gadgetǧ */
    if(mode != USB_OTG_CONTROL_GADGET){
        PERR("mode error\n");
        res = -EBUSY;
        goto exit;
    }
    
    /* UserˤHNPĤ줿Ȥ¸ */
    set_bit(USB_OTGCORE_ATOMIC_HNP_ENABLE_USER, &otg_core->bitflags);
    
    /* HostHNP ĤƤ &&
       usb_otg_control_enable_rchost() ȯԺѤߤǤʤ    */
    if(test_bit(USB_OTGCORE_ATOMIC_HNP_ENABLE_HOST, &otg_core->bitflags) &&
       test_bit(USB_OTGCORE_ATOMIC_ENABLE_RCHOST, &otg_core->bitflags) == 0){
        
        /* rchostĤ */
        PDEBUG("usb_otg_control_enable_rchost()\n");
        res = usb_otg_control_enable_rchost(otg_core->otg_control);
        if(res == 0){
            /*  usb_otg_control_enable_rchost() ȯԺѤ߾֤ˤ */
            set_bit(USB_OTGCORE_ATOMIC_ENABLE_RCHOST, &otg_core->bitflags);
        }
    }
    
exit:
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    return res;
}

/*-------------------------------------------------------------------------*/
static int
ioctl_disable_rchost(void *arg)
{
    struct otg_core_drv *otg_core = the_otg_core;
    unsigned long flags;
    int mode;
    int res = 0;
    
    PDEBUG("%s call\n", __func__);
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    /* ߤΥ⡼ɤ */
    mode = usb_otg_control_get_mode(otg_core->otg_control);
    
    /* ߤξ֤Gadgetǧ */
    if(mode != USB_OTG_CONTROL_GADGET){
        PERR("mode error\n");
        res = -EBUSY;
        
        goto exit;
    }
    
    /* UserˤHNPԵĤˤ줿Ȥ¸ */
    clear_bit(USB_OTGCORE_ATOMIC_HNP_ENABLE_USER, &otg_core->bitflags);
    
    /* usb_otg_control_enable_rchost() ȯԺѤ    */
    if(test_bit(USB_OTGCORE_ATOMIC_ENABLE_RCHOST, &otg_core->bitflags)){
        
        /* rchostԵĤˤ */
        PDEBUG("usb_otg_control_disable_rchost()\n");
        res = usb_otg_control_disable_rchost(otg_core->otg_control);
        if(res == 0){
            /*  usb_otg_control_enable_rchost() ȯԺѤ߾֤ */
            clear_bit(USB_OTGCORE_ATOMIC_ENABLE_RCHOST, &otg_core->bitflags);
        }
    }
    
exit:
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    return res;
}

/*-------------------------------------------------------------------------*/
static int
ioctl_start_rcgadget(void *arg)
{
    struct otg_core_drv *otg_core = the_otg_core;
    unsigned long flags;
    int mode;
    int res;
    
    PDEBUG("%s call\n", __func__);
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    /* ߤΥ⡼ɤ */
    mode = usb_otg_control_get_mode(otg_core->otg_control);
    
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    /* ߤξ֤HOSTǧ */
    if(mode != USB_OTG_CONTROL_HOST){
        PERR("mode error\n");
        res = -EBUSY;
        goto exit;
    }
    
    /* RCGadget 򳫻Ϥ */
    res = usb_otg_control_start_rcgadget(otg_core->otg_control, 0);
    if(res != 0){
        PERR("transfer to Gadget\n");
    }
    
exit:
    return res;
}

/*-------------------------------------------------------------------------*/
static int
ioctl_set_speed(void *arg)
{
    struct otg_core_drv *otg_core = the_otg_core;
    unsigned long flags;
    unsigned char otg_core_speed;
    unsigned int  ctrl_speed;
    
    PDEBUG("%s call\n", __func__);
    
    /* User֤饳ԡ */
    if(copy_from_user(&otg_core_speed, arg, sizeof(otg_core_speed))){
        return -EFAULT;
    }
    
    /* OTG Core ͤ OTG Controller ͤѴ */
    switch(otg_core_speed){
      case USB_OTGCORE_HS_ENABLE:
        ctrl_speed = USB_OTG_HS_ENABLE;
        break;
        
      case USB_OTGCORE_HS_DISABLE:
        ctrl_speed = USB_OTG_HS_DISABLE;
        break;
        
      default:
        return -EINVAL;
    }
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    otg_core->hs_disable = otg_core_speed;
    
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    /* Controller Ƥspeed */
    return usb_otg_control_set_speed(otg_core->otg_control, ctrl_speed);
}

/*-------------------------------------------------------------------------*/
static int
ioctl_get_speed(void *arg)
{
    struct otg_core_drv *otg_core = the_otg_core;
    unsigned long flags;
    enum usb_otg_control_speed ctrl_speed;
    unsigned char otg_core_speed;
    
    PDEBUG("%s call\n", __func__);
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    /* OTG Controller  BusSpeed */
    ctrl_speed = usb_otg_control_get_speed(otg_core->otg_control);
    
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    /* OTG Controller ͤ OTG Core ͤѴ */
    switch(ctrl_speed){
      case USB_OTG_SPEED_LS:
        otg_core_speed = USB_OTGCORE_SPEED_LS;
        break;
        
      case USB_OTG_SPEED_FS:
        otg_core_speed = USB_OTGCORE_SPEED_FS;
        break;
        
      case USB_OTG_SPEED_HS:
        otg_core_speed = USB_OTGCORE_SPEED_HS;
        break;
      
      default:
        return -EFAULT;
    }
    
    /* User֤˥ԡ */
    if(copy_to_user(arg, &otg_core_speed, sizeof(otg_core_speed))){
        return -EFAULT;
    }
    return 0;
}

/*-------------------------------------------------------------------------*/
static int
ioctl_set_test_mode(void *arg)
{
    struct otg_core_drv *otg_core = the_otg_core;
    unsigned char tmp_mode;
    int res;
    enum usb_otg_control_test_mode test_mode;
    
    PDEBUG("%s call\n", __func__);
    
    /* User֤饳ԡ */
    res = copy_from_user(&tmp_mode, 
                         (unsigned char*)arg,
                         sizeof(unsigned char));
    if(res != 0){
        PERR("error: copy_from_user\n");
        return -EFAULT;
    }
    
    res = 0;
    switch(tmp_mode){
      case USB_OTGCORE_TEST_MODE_NORMAL:
        PDEBUG("USB_OTG_TEST_MODE_NORMAL\n");
        test_mode = USB_OTG_TEST_MODE_NORMAL;
        break;
        
      case USB_OTGCORE_TEST_MODE_G_PACKET:
        PDEBUG("USB_OTG_TEST_MODE_G_PACKET\n");
        test_mode = USB_OTG_TEST_MODE_G_PACKET;
        break;
        
      case USB_OTGCORE_TEST_MODE_G_K:
        PDEBUG("USB_OTG_TEST_MODE_G_K\n");
        test_mode = USB_OTG_TEST_MODE_G_K;
        break;
        
      case USB_OTGCORE_TEST_MODE_G_J:
        PDEBUG("USB_OTG_TEST_MODE_G_J\n");
        test_mode = USB_OTG_TEST_MODE_G_J;
        break;
        
      case USB_OTGCORE_TEST_MODE_G_SE0_NAK:
        PDEBUG("USB_OTG_TEST_MODE_G_SE0_NAK\n");
        test_mode = USB_OTG_TEST_MODE_G_SE0_NAK;
        break;
        
      case USB_OTGCORE_TEST_MODE_G_FORCE_PACKET:
        PDEBUG("USB_OTG_TEST_MODE_G_FORCE_PACKET\n");
        test_mode = USB_OTG_TEST_MODE_G_FORCE_PACKET;
        break;
        
      case USB_OTGCORE_TEST_MODE_H_LS:
        PDEBUG("USB_OTG_TEST_MODE_H_LS\n");
        test_mode = USB_OTG_TEST_MODE_H_LS;
        break;
        
      case USB_OTGCORE_TEST_MODE_H_FS:
        PDEBUG("USB_OTG_TEST_MODE_H_FS\n");
        test_mode = USB_OTG_TEST_MODE_H_FS;
        break;
        
      case USB_OTGCORE_TEST_MODE_H_HS:
        PDEBUG("USB_OTG_TEST_MODE_H_HS\n");
        test_mode = USB_OTG_TEST_MODE_H_HS;
        break;
        
      default:
        PDEBUG("Error\n");
        res = -EINVAL;
        break;
    }
    
    if(res == 0){
        res = usb_otg_control_set_test_mode(otg_core->otg_control, test_mode);
        if(res == 0){
            otg_core->test_mode = tmp_mode;
        }
    }
    
    if(res != 0){
        PWARN("error: TestMode\n");
    }
    
    return 0;
}

/*-------------------------------------------------------------------------*/
static int
ioctl_send_srp(void *arg)
{
    struct otg_core_drv *otg_core = the_otg_core;
    unsigned long flags;
    int mode;
    int res;
    
    PDEBUG("%s call\n", __func__);
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    /* ߤΥ⡼ɤ */
    mode = usb_otg_control_get_mode(otg_core->otg_control);
    
    /* IDLE֤Ǥ뤳Ȥǧ */
    if(mode != USB_OTG_CONTROL_IDLE){
        /***** ԥå *****/
        spin_unlock_irqrestore(&otg_core->lock, flags);
        
        PERR("mode error\n");
        res = -EBUSY;
        goto exit;
    }
    
    /* CIDBǤ뤳Ȥǧ */
    if(otg_core->cid != USB_OTGCORE_CID_B){
        /***** ԥå *****/
        spin_unlock_irqrestore(&otg_core->lock, flags);
        
        PERR("cid error\n");
        res = -EBUSY;
        goto exit;
    }
    
    /* SRPѥ륷󥰤׵ */
    res = usb_otg_control_request_session(otg_core->otg_control);
    
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    if(res != 0){
        PERR("usb_otg_control_request_session()\n");
        res = -EINVAL;
        goto exit;
    }
    
exit:
    return res;
}

/*-------------------------------------------------------------------------*/
static int
usb_otgcore_ioctl(struct inode *inode,
                      struct file *filp,
                      unsigned int cmd,
                      unsigned long arg)
{
    struct otg_core_drv *otg_core = the_otg_core;
    int res;
    void __user *argp = (void __user *)arg;
    
    /* ޥե */
    if(down_interruptible(&otg_core->sem) !=0){
        return -ERESTARTSYS;
    }
    
    /* bindѤߤǧ */
    if(test_bit(USB_OTGCORE_ATOMIC_BIND, &otg_core->bitflags) == 0){
        /* ޥե */
        up(&otg_core->sem);
        
        PERR("not bind\n");
        return -EBUSY;
    }
    
    switch(cmd){
      /*----------------------------------------*/
      case USB_IOC_OTGCORE_PROBE:
        res = ioctl_probe(argp);
        break;
        
      /*----------------------------------------*/
      case USB_IOC_OTGCORE_REMOVE:
        res = ioctl_remove(argp);
        break;
        
      /*----------------------------------------*/
      case USB_IOC_OTGCORE_STOP:
        res = ioctl_stop(argp);
        break;
        
      /*----------------------------------------*/
      case USB_IOC_OTGCORE_IDLE:
        res = ioctl_idle(argp);
        break;
        
      /*----------------------------------------*/
      case USB_IOC_OTGCORE_START_GADGET:
        res = ioctl_start_gadget(argp);
        break;
        
      /*----------------------------------------*/
      case USB_IOC_OTGCORE_START_HOST:
        res = ioctl_start_host(argp);
        break;
        
      /*----------------------------------------*/
      case USB_IOC_OTGCORE_START_E_HOST:
        res = ioctl_start_e_host(argp);
        break;
        
      /*----------------------------------------*/
      case USB_IOC_OTGCORE_ENABLE_RCHOST:
        res = ioctl_enable_rchost(argp);
        break;
        
      /*----------------------------------------*/
      case USB_IOC_OTGCORE_DISABLE_RCHOST:
        res = ioctl_disable_rchost(argp);
        break;
        
      /*----------------------------------------*/
      case USB_IOC_OTGCORE_START_RCGADGET:
        res = ioctl_start_rcgadget(argp);
        break;
        
      /*----------------------------------------*/
      case USB_IOC_OTGCORE_SET_SPEED:
        res = ioctl_set_speed(argp);
        break;
        
      /*----------------------------------------*/
      case USB_IOC_OTGCORE_GET_SPEED:
        res = ioctl_get_speed(argp);
        break;
        
      /*----------------------------------------*/
      case USB_IOC_OTGCORE_SET_TEST_MODE:
        res = ioctl_set_test_mode(argp);
        break;
        
      /*----------------------------------------*/
      case USB_IOC_OTGCORE_SEND_SRP:
        res = ioctl_send_srp(argp);
        break;
        
      /*----------------------------------------*/
      default:
        PWARN("usb_otgcore_ioctl(none) call\n");
        res = -ENOTTY;
    }
    
    /* ޥե */
    up(&otg_core->sem);
    
    return res;
}

/*-------------------------------------------------------------------------*/
static int
usb_otgcore_open(struct inode *inode, struct file *filp)
{
    struct otg_core_drv *otg_core = the_otg_core;
    
    /* ¾processopenƤʤǧ */
    if(test_and_set_bit(USB_OTGCORE_ATOMIC_FD_LOCK, &otg_core->bitflags) != 0){
        PWARN("usb_otgcore_open() failed\n");
        return -EBUSY;
    }
    
    PDEBUG("usb_otgcore_open() success\n");
    
    return 0;
}

/*-------------------------------------------------------------------------*/
static int
usb_otgcore_release(struct inode *inode, struct file *filp)
{
    struct otg_core_drv *otg_core = the_otg_core;
    
    PDEBUG("usb_otgcore_release() success\n");
    
    /*  */
    clear_bit(USB_OTGCORE_ATOMIC_FD_LOCK, &otg_core->bitflags);
    
    return 0;
}

/*-------------------------------------------------------------------------*/
static struct file_operations usb_otgcore_fops = 
{
    .owner  = THIS_MODULE,
    .open   = usb_otgcore_open,
    .release = usb_otgcore_release,
    .ioctl  = usb_otgcore_ioctl,
};

static struct sonymisc_device usb_otgcore_device = 
{
    .class  = DEV_CLASS_CHR,
    .minor  = USBOTGCORE_MINOR_NUMBER,
    .name   = USBOTGCORE_NAME,
    .fops   = &usb_otgcore_fops,
    .gd     = NULL,
};

/*-------------------------------------------------------------------------*/
static int otg_core_bind(struct usb_otg_control *otg_control)
{
    struct otg_core_drv *otg_core = the_otg_core;
    
    PDEBUG("%s call\n", __func__);
    
    /* otg_control¸ */
    otg_core->otg_control = otg_control;
    
    /* bindѤߥե饰򥻥å */
    set_bit(USB_OTGCORE_ATOMIC_BIND, &otg_core->bitflags);
    
    return 0;
}

static int otg_core_unbind(struct usb_otg_control *otg_control)
{
    struct otg_core_drv *otg_core = the_otg_core;
    
    PDEBUG("%s call\n", __func__);
    
    /* bindѤߥե饰򥯥ꥢ */
    clear_bit(USB_OTGCORE_ATOMIC_BIND, &otg_core->bitflags);
    
    /* otg_control˴ */
    otg_core->otg_control = NULL;
    
    return 0;
}

/*-------------------------------------------------------------------------*/
static int
event_cid_exec(unsigned int value)
{
    struct otg_core_drv *otg_core = the_otg_core;
    struct usb_kevent_arg_otgcore_cid cid;
    int rc;
    
    switch(value){
      case 0:
        PDEBUG("  USB_OTG_EVENT_TYPE_CID: a-device\n");
        otg_core->cid = USB_OTGCORE_CID_A;
        cid.value = USB_OTGCORE_CID_A;
        break;
      
      case 1:
        PDEBUG("  USB_OTG_EVENT_TYPE_CID: b-device\n");
        otg_core->cid = USB_OTGCORE_CID_B;
        cid.value = USB_OTGCORE_CID_B;
        break;
      
      default:
        return -EINVAL;
    }
    
    /* probeƤơcb褬¸ߤ eventaddqueue */
    rc = 0;
    if(otg_core->otg_probe.hndl && otg_core->otg_probe.event.cid ){
        rc = usb_event_add_queue(USB_EVENT_PRI_NORMAL,
                                 otg_core->otg_probe.event.cid,
                                 otg_core->otg_probe.hndl,
                                 USB_KEVENT_ID_OTGCORE_CID,
                                 sizeof(cid),
                                 (void*)&cid );
    }
    
    return rc;
}

static int
event_vbus_exec(enum usb_otg_vbus_stat stat)
{
    struct otg_core_drv *otg_core = the_otg_core;
    struct usb_kevent_arg_otgcore_vbus vbus;
    int rc;
    int old_vbus = otg_core->vbus;
    
    switch(stat){
      case USB_OTG_VBUS_STAT_OFF:
        PDEBUG("  USB_OTG_EVENT_TYPE_VBUS: USB_OTG_VBUS_STAT_OFF\n");
        otg_core->vbus = USB_OTGCORE_VBUS_OFF;
        break;
        
      case USB_OTG_VBUS_STAT_LOW:
        PDEBUG("  USB_OTG_EVENT_TYPE_VBUS: USB_OTG_VBUS_STAT_LOW\n");
        otg_core->vbus = USB_OTGCORE_VBUS_OFF;
        break;
        
      case USB_OTG_VBUS_STAT_SESS:
        PDEBUG("  USB_OTG_EVENT_TYPE_VBUS: USB_OTG_VBUS_STAT_SESS\n");
        otg_core->vbus = USB_OTGCORE_VBUS_ON;
        break;
        
      case USB_OTG_VBUS_STAT_VALID:
        PDEBUG("  USB_OTG_EVENT_TYPE_VBUS: USB_OTG_VBUS_STAT_VALID\n");
        otg_core->vbus = USB_OTGCORE_VBUS_ON;
        break;
        
      default:
        return -EINVAL;
    }
    
    /* VBUSξ֤󤫤ѲƤо̤ */
    rc = 0;
    if(old_vbus != otg_core->vbus){
        vbus.value = otg_core->vbus;
        PDEBUG("AddQueue\n");
        if(otg_core->otg_probe.hndl && otg_core->otg_probe.event.vbus ){
            rc = usb_event_add_queue(USB_EVENT_PRI_NORMAL,
                                     otg_core->otg_probe.event.vbus,
                                     otg_core->otg_probe.hndl,
                                     USB_KEVENT_ID_OTGCORE_VBUS,
                                     sizeof(vbus),
                                     (void*)&vbus );
        }
    }
    
    return rc;
}

static int
event_vbus_error_exec(void)
{
    struct otg_core_drv *otg_core = the_otg_core;
    int rc;
    
    PDEBUG("  USB_OTG_EVENT_TYPE_VBUSERROR\n");
    
    /* ̤ */
    rc = 0;
    if(otg_core->otg_probe.hndl && otg_core->otg_probe.event.vbus_error ){
        rc = usb_event_add_queue(USB_EVENT_PRI_NORMAL,
                                 otg_core->otg_probe.event.vbus_error,
                                 otg_core->otg_probe.hndl,
                                 USB_KEVENT_ID_OTGCORE_VBUS_ERROR,
                                 0,
                                 NULL );
    }
    
    return rc;
}

static int
event_pullup_exec(unsigned int value)
{
    struct otg_core_drv *otg_core = the_otg_core;
    struct usb_kevent_arg_otgcore_pullup pullup;
    int rc;
    
    switch(value){
      case 0:
        PDEBUG("  USB_OTG_EVENT_TYPE_CON: disconnect\n");
        pullup.value = USB_OTGCORE_PULLUP_OFF;
        break;
        
      case 1:
        PDEBUG("  USB_OTG_EVENT_TYPE_CON: connect\n");
        pullup.value = USB_OTGCORE_PULLUP_ON;
        break;
      
      default:
        return -EINVAL;
    }
    
    /* probeƤơcb褬¸ߤ eventaddqueue */
    rc = 0;
    if(otg_core->otg_probe.hndl && otg_core->otg_probe.event.pullup ){
        rc = usb_event_add_queue(USB_EVENT_PRI_NORMAL,
                                 otg_core->otg_probe.event.pullup,
                                 otg_core->otg_probe.hndl,
                                 USB_KEVENT_ID_OTGCORE_PULLUP,
                                 sizeof(pullup),
                                 (void*)&pullup );
    }
    
    return rc;
}

static int
event_receive_srp_exec(void)
{
    struct otg_core_drv *otg_core = the_otg_core;
    int rc;
    
    PDEBUG("  USB_OTG_EVENT_TYPE_SRP\n");
    
    /* ̤ */
    rc = 0;
    if(otg_core->otg_probe.hndl && otg_core->otg_probe.event.receive_srp ){
        rc = usb_event_add_queue(USB_EVENT_PRI_NORMAL,
                                 otg_core->otg_probe.event.receive_srp,
                                 otg_core->otg_probe.hndl,
                                 USB_KEVENT_ID_OTGCORE_RECEIVE_SRP,
                                 0,
                                 NULL );
    }
    
    return rc;
}

static int
event_rchost_end_exec(unsigned int status)
{
    struct otg_core_drv *otg_core = the_otg_core;
    struct usb_kevent_arg_otgcore_rchost rchost;
    unsigned long flags;
    int mode;
    int rc;
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    /* ߤΥ⡼ɤ */
    mode = usb_otg_control_get_mode(otg_core->otg_control);
    
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    switch(status){
      case USB_OTG_SUCCESS:
        PDEBUG("  USB_OTG_EVENT_TYPE_CON: Success\n");
        
        /* enable_rchost()Ѥߤ */
        clear_bit(USB_OTGCORE_ATOMIC_ENABLE_RCHOST, &otg_core->bitflags);
        
        rchost.result = USB_OTGCORE_SUCCESS;
        
        /* RCHostǤʤä餹ǤmodeڤؤäƤΤ
           ⤷ʤǽλ */
        if(mode != USB_OTG_CONTROL_RCHOST){
            return 0;
        }
        
        break;
        
      case USB_OTG_FAIL_T_B_ASE0_BRST:
        PDEBUG("  USB_OTG_EVENT_TYPE_CON: TimeOut t_b_ase0_brst\n");
        
        rchost.result = USB_OTGCORE_FAIL;
        
        /* GadgetǤʤä餹ǤmodeڤؤäƤΤ
           ⤷ʤǽλ */
        if(mode != USB_OTG_CONTROL_GADGET){
            return 0;
        }
        
        break;
      
      default:
        return -EINVAL;
    }
    
    /* probeƤơcb褬¸ߤ eventaddqueue */
    rc = 0;
    if(otg_core->otg_probe.hndl && otg_core->otg_probe.event.rchost ){
        rc = usb_event_add_queue(USB_EVENT_PRI_NORMAL,
                                 otg_core->otg_probe.event.rchost,
                                 otg_core->otg_probe.hndl,
                                 USB_KEVENT_ID_OTGCORE_RCHOST,
                                 sizeof(rchost),
                                 (void*)&rchost );
    }
    
    return rc;
}

static int
event_rcgadget_end_exec(unsigned int status)
{
    struct otg_core_drv *otg_core = the_otg_core;
    struct usb_kevent_arg_otgcore_rcgadget rcgadget;
    unsigned long flags;
    int mode;
    int rc;
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    /* ߤΥ⡼ɤ */
    mode = usb_otg_control_get_mode(otg_core->otg_control);
    
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    switch(status){
      case USB_OTG_SUCCESS:
        PDEBUG("  USB_OTG_EVENT_TYPE_RCGADGET: Success\n");
        
        rcgadget.result = USB_OTGCORE_SUCCESS;
        
        break;
      
      case USB_OTG_FAIL_T_A_AIDL_BDIS:
        PDEBUG("  USB_OTG_EVENT_TYPE_RCGADGET: TimeOut t_a_aidl_bdis\n");
        
        rcgadget.result = USB_OTGCORE_FAIL;
        
        break;
      
      default:
        return -EINVAL;
        break;
    }
    
    /* RCGadgetǤʤä餹ǤmodeڤؤäƤΤ
       ⤷ʤǽλ */
    if(mode != USB_OTG_CONTROL_RCGADGET){
        return 0;
    }
    
    /* probeƤơcb褬¸ߤ eventaddqueue */
    rc = 0;
    if(otg_core->otg_probe.hndl && otg_core->otg_probe.event.rcgadget ){
        rc = usb_event_add_queue(USB_EVENT_PRI_NORMAL,
                                 otg_core->otg_probe.event.rcgadget,
                                 otg_core->otg_probe.hndl,
                                 USB_KEVENT_ID_OTGCORE_RCGADGET,
                                 sizeof(rcgadget),
                                 (void*)&rcgadget );
    }
    
    return rc;
}

static int otg_core_notify(union usb_otg_event *otg_event)
{
    int rc;
    
    /* ۾ǤʤȤǧ */
    if(!otg_event){
        return -EINVAL;
    }
    
    PDEBUG("event notify:\n");
    
    switch(otg_event->type){
      
      /* CID Ѳ */
      case USB_OTG_EVENT_TYPE_CID:
        rc = event_cid_exec(otg_event->cid.value);
        break;
        
      /* VBUS Ѳ */
      case USB_OTG_EVENT_TYPE_VBUS:
        rc = event_vbus_exec(otg_event->vbus.vbus_stat);
        break;
        
      /* VBUS Error  */
      case USB_OTG_EVENT_TYPE_VBUSERROR:
        rc = event_vbus_error_exec();
        break;
        
      /* PullUP Ѳ */
      case USB_OTG_EVENT_TYPE_CON:
        rc = event_pullup_exec(otg_event->con.value);
        break;
      
      /* SRP  */
      case USB_OTG_EVENT_TYPE_SRP:
        rc = event_receive_srp_exec();
        break;
      
      /* RollChange to Host λ */
      case USB_OTG_EVENT_TYPE_RCHOST:
        rc = event_rchost_end_exec(otg_event->rchost.status);
        break;
      
      /* RollChange to Gadget λ */
      case USB_OTG_EVENT_TYPE_RCGADGET:
        rc = event_rcgadget_end_exec(otg_event->rcgadget.status);
        break;
      
      default:
        return -EINVAL;
    }
    
    return 0;
}

/*-------------------------------------------------------------------------*/
static int __init
otg_core_alloc(void)
{
    struct otg_core_drv *otg_core;
    
    /* g_core Ѥΰ */
    otg_core = kmalloc(sizeof *otg_core, GFP_KERNEL);
    if(!otg_core) return -ENOMEM;
    
    /* g_core  */
    memset(otg_core, 0, sizeof *otg_core);
    
    /* VBUS֤UNKNOWN˽ */
    otg_core->vbus = USB_OTGCORE_VBUS_UNKNOWN;
    
    /* TestMode Ƥʤ֤˥å */
    otg_core->test_mode = USB_OTGCORE_TEST_MODE_NORMAL;
    
    /* HS Enable˥å */
    otg_core->hs_disable = USB_OTGCORE_HS_ENABLE;
    
    the_otg_core = otg_core;
    
    PDEBUG("the_otg_core : 0x%lx\n", (unsigned long)the_otg_core);
    
    return 0;
}

/*-------------------------------------------------------------------------*/
static void
otg_core_free(struct otg_core_drv *otg_core)
{
    kfree(otg_core);
}

/*-------------------------------------------------------------------------*/
static int
proc_read(char *page, char **start, off_t off, int count, int *eof, void *data)
{
    struct otg_core_drv *otg_core = the_otg_core;
    unsigned long flags;
    int len = 0, write_size;
    int mode;
    unsigned char cid = 0;
    unsigned char vbus = 0;
    enum usb_otg_control_speed bus_speed = 0;
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    /* ߤΥ⡼ɤ */
    mode = usb_otg_control_get_mode(otg_core->otg_control);
    
    if(mode != USB_OTG_CONTROL_STOP){
        /* CID */
        cid = otg_core->cid;
        
        /* VBUS */
        vbus = otg_core->vbus;
        
        /* OTG Controller  BusSpeed */
        bus_speed = usb_otg_control_get_speed(otg_core->otg_control);
    }
    
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    /* modeSTOPʳʤ顢BusSpeedOTGCoreƤͤѴ */
    if(mode != USB_OTG_CONTROL_STOP){
        switch(bus_speed){
          case USB_OTG_SPEED_LS:
            bus_speed = USB_OTGCORE_SPEED_LS;
            break;
            
          case USB_OTG_SPEED_FS:
            bus_speed = USB_OTGCORE_SPEED_FS;
            break;
            
          case USB_OTG_SPEED_HS:
            bus_speed = USB_OTGCORE_SPEED_HS;
            break;
          
          default:
            bus_speed = 0;
        }
    }
    
    /* modeɵ */
    write_size = snprintf(page+len, count, "%02d ", mode);
    if(write_size > count){
        len += count;
        goto exit;
    }
    len += write_size;
    count -= write_size;
    
    /* CIDɵ */
    write_size = snprintf(page+len, count, "%02d ", cid);
    if(write_size > count){
        len += count;
        goto exit;
    }
    len += write_size;
    count -= write_size;
    
    /* VBUSɵ */
    write_size = snprintf(page+len, count, "%02d ", vbus);
    if(write_size > count){
        len += count;
        goto exit;
    }
    len += write_size;
    count -= write_size;
    
    /* BusSpeedɵ */
    write_size = snprintf(page+len, count, "%02d ", bus_speed);
    if(write_size > count){
        len += count;
        goto exit;
    }
    len += write_size;
    
exit:
    *eof = 1;
    
    return len;
}

/*-------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------*/
#if !(defined(CONFIG_ARCH_CXD4105) && defined(CONFIG_USB_OTG_CORE))
  #if !defined(CONFIG_USB_OTG_CORE_MODULE)
   #error "Error: Kconfig"
  #endif
  static
#endif
int __init usb_otgcore_module_init(void)
{
    struct otg_core_drv *otg_core;
    int res;
    
    PINFO("USB OTGCore driver ver " USBOTGCORE_VERSION "\n");
    
    /* otg_core_drvѤΰ */
    res = otg_core_alloc();
    if(res != 0){
        res = -EINVAL;
        goto exit;
    }
    
    /* OTG Core Driver μΤ */
    otg_core = the_otg_core;
    
    /* ޥե */
    sema_init(&otg_core->sem, 1);
    
    /* spin_lockѿ */
    spin_lock_init(&otg_core->lock);
    
    /* Controller  regist  */
    res = usb_otg_control_register_core(&otg_core_driver);
    if(res != 0){
        PDEBUG("usb_otg_control_register_core() failed\n");
        
        /* otg_core¤Τ */
        otg_core_free(otg_core);
        res = -EINVAL;
        goto exit;
    }
    
    /* sonymiscCharacterDeviceȤϿ */
    res = sonymisc_register( &usb_otgcore_device );
    if(res != 0){
        PDEBUG("sonymisc_register() failed\n");
        
        /* Controller unregist */
        usb_otg_control_unregister_core(&otg_core_driver);
        
        /* otg_core¤Τ */
        otg_core_free(otg_core);
        res = -EINVAL;
        goto exit;
    }
    
    /* procfileϿ */
    create_proc_read_entry(OTGCORE_PROCFILE, 0, NULL, proc_read, NULL);
    res = 0;
    
    PDEBUG("usb_otgcore_module_init() success\n");
    
exit:
    return res;
}

/*-------------------------------------------------------------------------*/
static void __exit usb_otgcore_module_exit(void)
{
    struct otg_core_drv *otg_core = the_otg_core;
    unsigned long flags;
    int mode;
    int res;
    
    PDEBUG("%s call\n", __func__);
    
    /* procե */
    remove_proc_entry(OTGCORE_PROCFILE, NULL);
    
    /***** ԥå *****/
    spin_lock_irqsave(&otg_core->lock, flags);
    
    /* ߤΥ⡼ɤ */
    mode = usb_otg_control_get_mode(otg_core->otg_control);
    
    /***** ԥå *****/
    spin_unlock_irqrestore(&otg_core->lock, flags);
    
    
    /* stop֤Ǥʤästop */
    if(mode != USB_OTG_CONTROL_STOP){
        res = transition_to_idle();
        PDEBUG("transition_to_idle() rc: %d\n", res);
        if(res == 0){
            res = transition_to_stop();
            PDEBUG("transition_to_stop() rc: %d\n", res);
        }
    }
    
    /* sonymiscϿ */
    res = sonymisc_deregister( &usb_otgcore_device );
    if(res != 0){
        PDEBUG("sonymisc_deregister() failed\n");
    }
    
    /* Controller  unregist  */
    res = usb_otg_control_unregister_core(&otg_core_driver);
    if(res != 0){
        PDEBUG("usb_otg_control_unregister_core() failed\n");
    }
    
    /* otg_core¤Τ */
    otg_core_free(otg_core);
}

/*-------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------*/
/*=============================================================================
 * Export symbols
 *===========================================================================*/
EXPORT_SYMBOL(usb_otgcore_get_hs_disable);
EXPORT_SYMBOL(usb_otgcore_gadget_set_feature);
EXPORT_SYMBOL(usb_otgcore_gadget_suspend);
EXPORT_SYMBOL(usb_otgcore_gadget_disconnect);

#if !(defined(CONFIG_ARCH_CXD4105) && defined(CONFIG_USB_OTG_CORE))
  #if !defined(CONFIG_USB_OTG_CORE_MODULE)
   #error "Error: Kconfig"
  #endif
  module_init(usb_otgcore_module_init);
#endif
module_exit(usb_otgcore_module_exit);
