/*--------------------------------------------------------------------------*
 * drivers/usb/mcd/usb_mcd_board_cxd4105.c
 * 
 * USB controller driver
 * 
 * Copyright 2005,2006 Sony Corporation
 *
 * This file is part of the Inventra Controller Driver for Linux.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *---------------------------------------------------------------------------*/

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/list.h>
#include <linux/proc_fs.h>
#include <asm/irq.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <linux/usb.h>
#include <asm/arch/gpio-core.h>
#if defined(CONFIG_ARCH_CXD4105)
#include <asm/arch/gpio-cxd4105.h>
#elif defined(CONFIG_ARCH_CXD4108)
#include <asm/arch/gpio-cxd4108.h>
#endif  /* ARCH */
#include "usb_mcd_defs.h"

/* UIDTYPE0 */
#if defined(CONFIG_ARCH_CXD4108)
#define UIDTYPE0 0x767c0000
static unsigned short uidtype0;
#endif

/* PHYCLK ON */
#define CLKBASE		0x77400000
#define USBPHYCKEN	0x64
#define USBPHY		(1UL<<0)
#define PHYCLK_ON() { \
    unsigned long reg; \
    reg = (unsigned long)ioremap_nocache(CLKBASE, 0x100); \
    if (reg <= 0) err("USBPHYCK ioremap failed\n"); \
    *(unsigned long*)(reg+USBPHYCKEN) = USBPHY; \
    iounmap((void*)reg); \
}

/* Pѥ᡼ѹprocե̾ */
#define PPARAM_PROCFILE			"usb_mcd_pparam"
#define PPARAM_WRITE_BUFSIZE	32

/* Pѥ᡼PIOݡ */
#define _GPIO17		0
#define _GPIO18		1
#define _GPIO19		2
/* GPIOEASY3 */
#define _GPIOEASY3	3

#if defined(CONFIG_ARCH_CXD4105)
#define _XVBUS_OVC_DET_GPIO_ GPIO_2
static const unsigned long _gpio_pparam_adrs[] = {
    0x76702100,	/* _GPIO17 */
    0x76702200,	/* _GPIO18 */
    0x76702300,	/* _GPIO19 */
    0x76700300,	/* _GPIOEASY3 */
};
#elif defined(CONFIG_ARCH_CXD4108)
#define _XVBUS_OVC_DET_GPIO_ GPIO_14
static const unsigned long _gpio_pparam_adrs[] = {
    0x767B0800,	/* _GPIO17 */
    0x767B0900,	/* _GPIO18 */
    0x767B0A00,	/* _GPIO19 */
    0x76700300,	/* _GPIOEASY3 */
};
#endif  /* ARCH */

static unsigned long _gpio_log_adrs[sizeof(_gpio_pparam_adrs) /
                                    sizeof _gpio_pparam_adrs[0]];

#define _READ32(ADR) *(unsigned int*)(ADR)
#define _WRITE32(ADR, VAL) *(unsigned int*)(ADR) = (unsigned int)(VAL)

/*  _GPIO_INIT ... GPIO ioremap */
#define _GPIO_INIT(NR) { \
    _gpio_log_adrs[(NR)] \
        = (unsigned long)ioremap_nocache(_gpio_pparam_adrs[(NR)], 0x100); \
};

/*  _GPIO_EXIT ... GPIO iounmap */
#define _GPIO_EXIT(NR) { \
    if (_gpio_log_adrs[(NR)]) { \
        iounmap((void*)_gpio_log_adrs[(NR)]); \
        _gpio_log_adrs[(NR)] = 0; \
    } \
};

/*  _GPIO_SET_MON ... 1bitǡ˥Ϥ */
#define _GPIO_SET_MON(NR, BIT) { \
    unsigned short reg; \
    reg = _READ32(_gpio_log_adrs[(NR)]+0x00); /*  */ \
    reg &= ~(1<<(BIT));         /*  */ \
    _WRITE32(_gpio_log_adrs[(NR)]+0x00, reg); /*  */ \
}

/*  _GPIO_SET_IN ... 1bitǡϤ */
#define _GPIO_SET_IN(NR, BIT) { \
    unsigned short reg; \
    reg = _READ32(_gpio_log_adrs[(NR)]+0x00); /*  */ \
    reg &= ~(1<<(BIT));         /*  */ \
    _WRITE32(_gpio_log_adrs[(NR)]+0x00, reg); /*  */ \
    _WRITE32(_gpio_log_adrs[(NR)]+0x14+(BIT)*4, 0); /* ѥݡ */ \
}

/*  _GPIO_SET_OUT ... 1bitǡ */
#define _GPIO_SET_OUT(NR, BIT, DATA) { \
    unsigned short reg; \
    _WRITE32(_gpio_log_adrs[(NR)]+0x14+(BIT)*4, 0); /* ѥݡ */ \
    reg = 1<<(BIT);             /* ϥǡ */ \
    /* DATAONGPIODATASet, OFFGPIODATAClear */ \
    _WRITE32(_gpio_log_adrs[(NR)]+((DATA)?0x08:0x0c), reg); \
    reg = _READ32(_gpio_log_adrs[(NR)]+0x00); /*  */ \
    reg |= 1<<(BIT);            /*  */ \
    _WRITE32(_gpio_log_adrs[(NR)]+0x00, reg); /*  */ \
}

/*  _GPIO_SET_FUNC ... 1bitǽԥ */
#define _GPIO_SET_FUNC(NR, BIT) { \
    unsigned short reg; \
    _WRITE32(_gpio_log_adrs[(NR)]+0x14+(BIT)*4, 1); /* ǽݡ */ \
    reg = _READ32(_gpio_log_adrs[(NR)]+0x00); /*  */ \
    reg &= ~(1<<(BIT));         /*  */ \
    _WRITE32(_gpio_log_adrs[(NR)]+0x00, reg); /*  */ \
}

/*  _GPIO_READ ... 1bitǡ */
#define _GPIO_READ(NR, BIT)    (1&((_READ32(_gpio_log_adrs[(NR)]+0x04))>>(BIT)))

/*  _GPIO_READ32 ... 32bitǡ */
#define _GPIO_READ32(NR)  (_READ32(_gpio_log_adrs[(NR)]+0x04))

/*  _GPIO_SET_OUT16 ... 16ӥåȥǡν */
#define _GPIO_SET_OUT32(NR, DATA) { \
    _WRITE32(_gpio_log_adrs[(NR)]+0x00, 0xffff); /*  */ \
    _WRITE32(_gpio_log_adrs[(NR)]+0x08, DATA); /* ϥǡ */ \
}

/*  ON/OFF */
#ifdef ON
#undef ON
#endif
#define ON	1
#ifdef OFF
#undef OFF
#endif
#define OFF	0

/* mcd_resources, mcd_dma_mask, mcd_dma_mask ... ǥХ */

static void nop_release(struct device* dev);
static int pparam_proc_write(struct file *file, const char *buffer, 
                               unsigned long count, void *data);
static char *pparam_parse(char *str, unsigned short *value);

static struct resource mcd_resources[] = {
    [0] = {
        .start	= 0x70200000,
        .end	= 0x70201000,
        .flags	= IORESOURCE_IO,
    },
    [1] = {
        .name	= "USB_HS",
        .start	= IRQ_USB_HS,
        .end	= 0,
        .flags	= IORESOURCE_IRQ,
    },
    [2] = {
        .name	= "USB_DMAC",
        .start	= IRQ_USB_DMAC,
        .end	= 0,
        .flags	= IORESOURCE_IRQ,
    },
};

static u64 mcd_dma_mask = 0xffffffffUL;

static struct platform_device mcd_device = {
    .name	= "usb_mcd",
    .id		= 0,
    .dev	= {
        .dma_mask = &mcd_dma_mask,
        .coherent_dma_mask = 0xffffffffUL,
        .release = nop_release,
    },
    .num_resources = ARRAY_SIZE (mcd_resources),
    .resource	= mcd_resources,
};

#if defined(CONFIG_ARCH_CXD4105)
static unsigned short pparam[] = { 0x2224, 0x0492, 0x8924 };
#elif defined(CONFIG_ARCH_CXD4108)
static unsigned short pparam[] = { 0x0888, 0x4894, 0x4894 };
#endif  /* ARCH */

static struct usb_otg_control* cxd4105_otgc;

#if defined(CONFIG_USB_MCD_GPIO_VBUS) || defined(CONFIG_USB_MCD_GPIO_OC)
/* VbusΤ⤷OverCurrentGPIOΥϥɥ */
static int xvbus_ovc_det_handle;
#endif /* defined(CONFIG_USB_MCD_GPIO_VBUS) || defined(CONFIG_USB_MCD_GPIO_OC)*/
struct _gpio_func {
    unsigned char func;
    unsigned int data;
};

/***************************** internal functions *****************************/

static void
nop_release(
    struct device *dev)
{
    DBG(3, "%s %s\n", __FUNCTION__, dev->bus_id);
}

#if defined(CONFIG_USB_MCD_OCINT)
/* det_oc ... GPIO interrupt callback to detect OverCurrent */
static void
det_oc(
    unsigned int nr,
    void* param)
{
    DBG(2, "<==\n");

    musb_otg_control_notify_vbuserror(cxd4105_otgc);
    DBG(2, "==>\n");
}
#endif /* defined(CONFIG_USB_MCD_OCINT) */

#if defined(CONFIG_USB_MCD_VBUSINT)
/* det_vbus ... GPIO interrupt callback to change Vbus level */
static void
det_vbus(
    unsigned int nr,
    void* param)
{
    struct musb_otgc_intdat* otgc_intdat;

    DBG(2, "<==\n");

    otgc_intdat = (struct musb_otgc_intdat*)cxd4105_otgc->data;

    if (timer_pending(&otgc_intdat->detector)) {
        return;
    }
    otgc_intdat->detector.expires = jiffies;
    add_timer(&otgc_intdat->detector);
    DBG(2, "==>\n");
}
#endif /* defined(CONFIG_USB_MCD_VBUSINT) */

#if defined(CONFIG_USB_MCD_VBUSINT) || defined(CONFIG_USB_MCD_OCINT)
/* unset_gpio_callback ... unset GPIO callback */
static int
unset_gpio_callback(
    int handle)
{
    int retval;

    DBG(2, "<==\n");

    retval = gpio_unregister_callback(handle);
    if (retval) {
        return retval;
    }

    retval = gpio_disable_callback(handle);

    DBG(2, "==>\n");
    return retval;
}

/* set_gpio_callback ... set GPIO callback */
static int
set_gpio_callback(
    int handle,
    struct gpio_callback *cb_info)
{
    int retval;

    DBG(2, "<==\n");

    retval = gpio_register_callback(handle, cb_info);
    if (retval) {
        DBG(2, "GPIO register_callback error %d\n", retval);
        return retval;
    }

    retval = gpio_enable_callback(handle);

    DBG(2, "==>\n");
    return retval;
}
#endif /* defined(CONFIG_USB_MCD_VBUSINT) || defined(CONFIG_USB_MCD_OCINT) */


#if defined(CONFIG_USB_MCD_HOST)
/* save_gpio_func ... save GPIO function */
static int
save_gpio_func(
    int handle,
    struct _gpio_func* save)
{
    int retval;

    /* ǽ򥻡 */
    retval = gpio_get_io_func(handle, &save->func);
    if (retval) {
        return retval;
    }

    /* ǡ򥻡 */
    save->data = gpio_read(handle);
    if (save->data < 0) {
        return save->data;
    }

    return 0;
}
#endif /* defined(CONFIG_USB_MCD_HOST) */

#if defined(CONFIG_USB_MCD_HOST)
/* set_gpio_func ... set GPIO function */
static int
set_gpio_func(
    unsigned int nr,
    struct _gpio_func* new_func,
    struct _gpio_func* old_func)
{
    int handle;
    int retval;

    DBG(2, "<==\n");

    handle = gpio_open(nr);
    if (handle < 0) {
        return handle;
    }

    if (old_func) {
        retval = save_gpio_func(handle, old_func);
        if (retval) {
            goto error_close;
        }
    }

    retval = gpio_set_io_func(handle, new_func->func, 0);

  error_close:
    gpio_close(handle);

    DBG(2, "==>\n");
    return retval;
}
#endif /* defined(CONFIG_USB_MCD_HOST) */

/* set_pparam ... set pparam */
int
set_pparam(
    unsigned short* data)
{
    _GPIO_SET_OUT32(_GPIO17, data[0]);
    _GPIO_SET_OUT32(_GPIO18, data[1]);
    _GPIO_SET_OUT32(_GPIO19, data[2]);
    DBG(2, "GPIO17 0x%x\n", _READ32(_gpio_log_adrs[_GPIO17]+0x04));
    DBG(2, "GPIO18 0x%x\n", _READ32(_gpio_log_adrs[_GPIO18]+0x04));
    DBG(2, "GPIO19 0x%x\n", _READ32(_gpio_log_adrs[_GPIO19]+0x04));
    return 0;
}

#if defined(CONFIG_USB_MCD_VBUSINT)
static struct gpio_callback vbus_det_cb_info = {
    .sense	= GPIO_INT_UPDOWN_EDGE,
    .timing	= GPIO_CALLBACK_IRQ,
    .function	= det_vbus,
    .param	= NULL,
};
#endif /* defined(CONFIG_USB_MCD_VBUSINT) */

#if defined(CONFIG_USB_MCD_OCINT)
static struct gpio_callback oc_det_cb_info = {
    .sense	= GPIO_INT_DOWN_EDGE,
    .timing	= GPIO_CALLBACK_IRQ,
    .function	= det_oc,
    .param	= NULL,
};
#endif /* defined(CONFIG_USB_MCD_OCINT) */

/**************************** operation functions *****************************/

/*
 * ؿcontrol_vbus
 */
static int control_vbus(
    struct usb_otg_control* otg_control,
    unsigned int control)
{
    DBG(2, "<==\n");

    if (control == USB_OTG_VBUS_ON) {
        _GPIO_SET_OUT(_GPIOEASY3, 0, OFF); /* SESSEND */
        _GPIO_SET_OUT(_GPIOEASY3, 1, ON); /* AVALID */
        _GPIO_SET_OUT(_GPIOEASY3, 2, ON); /* VBUSVALID */
    }
    else if (control == USB_OTG_VBUS_OFF) {
        _GPIO_SET_OUT(_GPIOEASY3, 0, ON); /* SESSEND */
        _GPIO_SET_OUT(_GPIOEASY3, 1, OFF); /* AVALID */
        _GPIO_SET_OUT(_GPIOEASY3, 2, OFF); /* VBUSVALID */
    }
    else if (control == USB_OTG_VBUS_THRU) {
        _GPIO_SET_FUNC(_GPIOEASY3, 0); /* SESSEND */
        _GPIO_SET_FUNC(_GPIOEASY3, 1); /* AVALID */
        _GPIO_SET_FUNC(_GPIOEASY3, 2); /* VBUSVALID */
    }
    else {
        return -EINVAL;
    }

    DBG(2, "==>\n");
    return 0;
}

/*
 * ؿcontrol_test
 * usb_otg_control_set_test_modeˡ쥸˸ƤӽФ롣
 */
static int control_test(
    struct usb_otg_control* otg_control,
    enum usb_otg_control_test_mode test_mode)
{
    DBG(2, "<==\n");

    switch (test_mode) {
#if defined(CONFIG_USB_MCD_FORCE_HOST)
      case USB_OTG_TEST_MODE_H_AS:
        msleep(1);
        _GPIO_SET_OUT(_GPIOEASY3, 3, OFF);
        control_vbus(otg_control, USB_OTG_VBUS_ON);
        DBG(2, "set test mode : H_AS\n");
        break;
#endif
      case USB_OTG_TEST_MODE_NORMAL:
#if defined(CONFIG_USB_MCD_GPIO_VBUS)
        _GPIO_SET_FUNC(_GPIOEASY3, 3); /* IDDIG */
        control_vbus(otg_control, USB_OTG_VBUS_THRU);
#else
        _GPIO_SET_OUT(_GPIOEASY3, 3, ON); /* IDDIG */
        control_vbus(otg_control, USB_OTG_VBUS_ON);
#endif /* defined(CONFIG_USB_MCD_GPIO_VBUS) */
     DBG(2, "set test mode : NORMAL\n");
        break;
      case USB_OTG_TEST_MODE_G_FORCE_PACKET:
        control_vbus(otg_control, USB_OTG_VBUS_ON);
        DBG(2, "set test mode : FORCE_PACKET\n");
        break;
      default:
        DBG(2, "set test mode : Unknown mode\n");
        break;
    }

    DBG(2, "==>\n");
    return 0;
}

/*
 * ؿcontrol_io
 * usb_otg_control_start_control, usb_otg_control_start_control
 * ƤӽФ롣
 */

static int
control_io(
    struct usb_otg_control* otg_control,
    unsigned int control)
{
#if defined(CONFIG_USB_MCD_HOST)
    struct _gpio_func new_func; /* VbusϢGPIOǽԥѰѿ */
#endif /* defined(CONFIG_USB_MCD_HOST) */
#if defined(CONFIG_USB_MCD_VBUSINT) || defined(CONFIG_USB_MCD_OCINT) \
    || defined(CONFIG_USB_MCD_GPIO_VBUS) || defined(CONFIG_USB_MCD_GPIO_OC)
    int ret;
#endif
    DBG(2, "<==\n");

    /* ǧ */
    if (!otg_control) {
        return -EINVAL;
    }

    if (control == USB_OTG_IO_START) {
        /* Ͻ */
        /* PHYåON */
        PHYCLK_ON();
        set_pparam(pparam);
        /* SUSPEND# ǽԥ */
        _GPIO_SET_FUNC(_GPIOEASY3, 8);
#if !defined(CONFIG_USB_MCD_GPIO_VBUS)
        _GPIO_SET_FUNC(_GPIOEASY3, 3); /* IDDIG */
        control_vbus(otg_control, USB_OTG_VBUS_THRU);
        _GPIO_SET_OUT(_GPIOEASY3, 6, OFF); /* VCMPEN# OFF - ͭ */
#else
        _GPIO_SET_OUT(_GPIOEASY3, 3, ON); /* IDDIG */
        control_vbus(otg_control, USB_OTG_VBUS_OFF);
#endif /* !defined(CONFIG_USB_MCD_GPIO_VBUS) */
#if defined(CONFIG_USB_MCD_GPIO_VBUS) || defined(CONFIG_USB_MCD_GPIO_OC)
        xvbus_ovc_det_handle = gpio_open(_XVBUS_OVC_DET_GPIO_);
        if (xvbus_ovc_det_handle < 0) {
            err("cannot open GPIO for xvbus_ovc_det_handle %d\n",
                xvbus_ovc_det_handle);
            return xvbus_ovc_det_handle;
        }
#if defined(CONFIG_USB_MCD_GPIO_VBUS)
        ret = gpio_set_io_func(xvbus_ovc_det_handle, GPIO_INPUT_PU, 0);
#else  /*  defined(CONFIG_USB_MCD_GPIO_OC) */
        ret = gpio_set_io_func(xvbus_ovc_det_handle, GPIO_INPUT, 0);
#endif
        if (ret) {
            err("gpio_set_io_func %d\n", ret);
            return ret;
        }
#endif /* defined(CONFIG_USB_MCD_GPIO_VBUS) || defined(CONFIG_USB_MCD_GPIO_OC)*/
#if defined(CONFIG_USB_MCD_VBUSINT)
        /* Vbusг */
        ret = set_gpio_callback(xvbus_ovc_det_handle, &vbus_det_cb_info);
        if (ret) {
            err("set_gpio_callback of det_vbus %d\n", ret);
            return ret;
        }
#endif
#if defined(CONFIG_USB_MCD_OCINT)
        /* OverCurrentг */
        ret = set_gpio_callback(xvbus_ovc_det_handle, &oc_det_cb_info);
        if (ret) {
            err("set_gpio_callback of det_oc %d\n", ret);
        }
#endif
#if defined(CONFIG_USB_MCD_HOST)
        /* Vbusϵǽԥ */
        new_func.func = GPIO_FUNC_USBVBUS;
        new_func.data = 0;
        set_gpio_func(GPIO2_8, &new_func, NULL); /* VBUSCHG */
        set_gpio_func(GPIO2_9, &new_func, NULL); /* VBUSDISCHG */
        set_gpio_func(GPIO2_10, &new_func, NULL); /* VBUSEN */
        _GPIO_SET_OUT(_GPIOEASY3, 7, ON); /* IDPULLUP ON */
#endif /* defined(CONFIG_USB_MCD_HOST) */
        _GPIO_SET_MON(_GPIOEASY3, 4); // LINESTATE
        _GPIO_SET_MON(_GPIOEASY3, 5); // LINESTATE
        _GPIO_SET_MON(_GPIOEASY3, 8); // SUSPEND#
        _GPIO_SET_MON(_GPIOEASY3, 9); // OPMODE
        _GPIO_SET_MON(_GPIOEASY3, 10); // OPMODE
#if MUSB_DEBUG > 0
        DBG(2, "GPIOEASY3/DIR 0x%x\n",
               _READ32(_gpio_log_adrs[_GPIOEASY3]+0x00));
        DBG(2, "GPIOEASY3/DATA 0x%x\n",
               _READ32(_gpio_log_adrs[_GPIOEASY3]+0x04));
        DBG(2, "GPIOEASY3/INEN 0x%x\n",
               _READ32(_gpio_log_adrs[_GPIOEASY3]+0x10));
        {
            int i;
            for (i = 0; i < 12; i++) {
                DBG(2, "GPIOEASY3/SEL%d 0x%x\n", i,
                    _READ32(_gpio_log_adrs[_GPIOEASY3]+0x14+i*4));
            }
        }
#endif
    }
    else if (control == USB_OTG_IO_STOP) {
        /* λ */
#if !defined(CONFIG_USB_MCD_GPIO_VBUS)
        _GPIO_SET_OUT(_GPIOEASY3, 6, ON); /* VCMPEN# ON */
#endif /* !defined(CONFIG_USB_MCD_GPIO_VBUS) */
#if defined(CONFIG_USB_MCD_HOST)
        _GPIO_SET_OUT(_GPIOEASY3, 7, OFF); /* IDPULLUP OFF */
        /* VbusüLowϥԥ */
        new_func.func = GPIO_OUTPUT;
        new_func.data = 0;
        set_gpio_func(GPIO2_8, &new_func, NULL); /* VBUSCHG */
        set_gpio_func(GPIO2_9, &new_func, NULL); /* VBUSDISCHG */
        set_gpio_func(GPIO2_10, &new_func, NULL); /* VBUSEN */
#endif /* defined(CONFIG_USB_MCD_HOST) */
#if defined(CONFIG_USB_MCD_VBUSINT) || defined(CONFIG_USB_MCD_OCINT)
        /* OverCurrent⤷VbusФγ */
        unset_gpio_callback(xvbus_ovc_det_handle);
        gpio_close(xvbus_ovc_det_handle);
#endif /* defined(CONFIG_USB_MCD_VBUSINT) || defined(CONFIG_USB_MCD_OCINT) */
    }
    else {
        return -EINVAL;
    }
    
    DBG(2, "==>\n");
    return 0;                   // success
}

#if defined(CONFIG_USB_MCD_HOST)
/*
 * ؿcheck_cid
 * ؿƤӽФ롣GPIO˸CID֤ͤ
 */

static int
check_cid(
    struct usb_otg_control* otg_control)
{
    /* Read IDDIG */
    return _GPIO_READ(_GPIOEASY3, 3);
}
#endif /* defined(CONFIG_USB_MCD_HOST) */

/*
 * ؿcheck_vbus
 * ؿƤӽФ롣GPIO˸Vbus֤ͤ
 */

static enum usb_otg_vbus_stat
check_vbus(
    struct usb_otg_control* otg_control)
{
    int data;
    enum usb_otg_vbus_stat vbus_stat;

#if defined(CONFIG_USB_MCD_GPIO_VBUS)
    data = gpio_read(xvbus_ovc_det_handle);
    if (data < 0) {
        DBG(2, "GPIO read error %d\n", data);
        /* Error */
    }

#if !defined(CONFIG_USB_MCD_VBUS_HIGH_ACTIVE)
    data = data ? 0 : 1;
#endif /* defined(USB_MCD_VBUS_HIGH_ACTIVE) */

    if (data) {
        vbus_stat = USB_OTG_VBUS_STAT_SESS;
    }
    else {
        vbus_stat = USB_OTG_VBUS_STAT_OFF;
    }
#else
    /* Vbus GPIOФʤGPIOEASY3ˤVbus */
    data = _GPIO_READ32(_GPIOEASY3) & 0x7;
    /* data = { VBUSVALID, AVALID, SESSEND } by UTMI spec. */
    switch (data) {
      case 0x1:
        vbus_stat = USB_OTG_VBUS_STAT_OFF;
        break;
      case 0x0:
        vbus_stat = USB_OTG_VBUS_STAT_LOW;
        break;
      case 0x2:
        vbus_stat = USB_OTG_VBUS_STAT_SESS;
        break;
      case 0x6:
        vbus_stat = USB_OTG_VBUS_STAT_VALID;
        break;
      default:
        vbus_stat = USB_OTG_VBUS_STAT_UNKNOWN;
        DBG(2, "Unkonwn vbus stat=%d", data);
    }
#endif /* defined(CONFIG_USB_MCD_GPIO_VBUS) */

    return vbus_stat;
}

/*
 * ؿcheck_suspend
 * ǥХΥڥɾ֤å롣
 */

static int
check_suspend(
    struct usb_otg_control* otg_control)
{
    /* Read SUSPEND# */
    return !(_GPIO_READ(_GPIOEASY3, 8));
}

static int pparam_proc_write(struct file *file, const char *buffer, 
                               unsigned long count, void *data)
{
    int res;
    char param[PPARAM_WRITE_BUFSIZE];
    unsigned short tmp[3];
    char *pstr;
    
    count = min(count, (unsigned long)PPARAM_WRITE_BUFSIZE);
    
    /* User֤饳ԡ */
    if(copy_from_user(param, buffer, count)){
        res = -EFAULT;
        goto exit;
    }
    param[count] = '\0';
    res = count;
    
    DBG(2, "count: %ld (%s)\n", count, param);
    
    /* setޥɤǧ */
    if(strncmp(param, "set ", 4) != 0){
        DBG(2, "parse error\n");
        goto exit;
    }
    
    pstr = pparam_parse(&param[4], &tmp[0]);
    if(pstr == NULL || *pstr == '\0'){
        DBG(2, "parse error\n");
        goto exit;
    }
    DBG(2, "set GPIO17: 0x%04x\n", tmp[0]);
    
    pstr = pparam_parse(pstr, &tmp[1]);
    if(pstr == NULL || *pstr == '\0'){
        DBG(2, "parse error\n");
        goto exit;
    }
    DBG(2, "set GPIO18: 0x%04x\n", tmp[1]);
    
    pstr = pparam_parse(pstr, &tmp[2]);
    if(pstr == NULL){
        DBG(2, "parse error\n");
        goto exit;
    }
    DBG(2, "set GPIO19: 0x%04x\n", tmp[2]);
    
    memcpy(pparam, tmp, sizeof tmp );
    
    DBG(2, "copy tmp -> pparam count: %d\n", sizeof tmp);
    
exit:
    return res;
}

static char *pparam_parse(char *str, unsigned short *value)
{
    const char hexchars[] = "0123456789abcdef";
    unsigned char tmp;
    
    *value = 0;
    
    do {
        for(tmp = 0; hexchars[tmp] != '\0'; tmp++){
            if(*str == hexchars[tmp]){
                *value <<= 4;
                *value += tmp;
                break;
            }
        }
        if(hexchars[tmp] == '\0'){
            return NULL;
        }
        str++;
    } while(*str != '\0' && *str != ',');
    
    if(*str == ','){
        str++;
    }
    
    return str;
}

/*
 * ؿmusb_otg_user_init
 * GPIO桼ν桼ؿ
 */

int
musb_otg_user_init(void)
{
    int (*func_check_cid)(struct usb_otg_control* otg_control);
    struct proc_dir_entry *proc_entry;
#if defined(CONFIG_ARCH_CXD4108)
    volatile void* reg;
#endif /* defined(CONFIG_ARCH_CXD4108) */

    DBG(2, "<==\n");

#if defined(CONFIG_ARCH_CXD4108)
    reg = ioremap_nocache(UIDTYPE0, 0x100);

    if (reg) {
        uidtype0 = *(unsigned short*)reg;
        iounmap((void*)reg);
        DBG(1, "mcd uidtype0 remapped_adr=0x%x, value=%d\n", reg, uidtype0);
    }
    else {
        err("cannot ioremap UIDTYPE0\n");
        uidtype0 = 0;
    }
#endif /* defined(CONFIG_ARCH_CXD4108) */

#if !defined(CONFIG_USB_MCD_HOST)
    func_check_cid = NULL;
#else
    func_check_cid = check_cid;
#endif

    /* GPIO ioremap */
    _GPIO_INIT(_GPIO17);
    _GPIO_INIT(_GPIO18);
    _GPIO_INIT(_GPIO19);
    _GPIO_INIT(_GPIOEASY3);
    
    /* Platform deviceȤϿ */
    platform_device_register(&mcd_device);

    /* OTGȥ饪֥Ȥ */
    cxd4105_otgc = musb_otg_control_create_object(
        0,                      // user_data
        10,                     // periodic_delay
        3,                      // cid_filter_count
        3,                      // vbus_filter_count
        func_check_cid,
        check_vbus,
        check_suspend,
        control_io,
        control_vbus,
        control_test,
        "musbhdrc0");
    
    /* pparamprocե */
    proc_entry = create_proc_entry(PPARAM_PROCFILE,
                                    S_IFREG | S_IRUGO | S_IWUSR, NULL);
    if(proc_entry){
        /* set to struct proc_entry */
        proc_entry->data = NULL;
#ifdef MUSB_V26
        proc_entry->owner = THIS_MODULE;
#endif
        proc_entry->write_proc = pparam_proc_write;
    }
    
    DBG(2, "==>\n");

    return 0;
}

/*
 * ؿmusb_otg_user_exit
 * GPIO桼νλ桼ؿ
 */

int
musb_otg_user_exit(void)
{
    DBG(2, "<==\n");
    /* GPIOλ */

    /* pparamprocե */
    remove_proc_entry(PPARAM_PROCFILE, NULL);
    
    /* ץåȥեǥХȤϿ */
    platform_device_unregister(&mcd_device);
    
    /* GPIO iounmap */
    _GPIO_EXIT(_GPIO17);
    _GPIO_EXIT(_GPIO18);
    _GPIO_EXIT(_GPIO19);
    _GPIO_EXIT(_GPIOEASY3);
    
    DBG(2, "==>\n");
    return 0;
}


#if defined(CONFIG_ARCH_CXD4108)
/*
 * ؿmusb_get_uidtype0
 */

unsigned short
musb_get_uidtype0(void)
{
    return uidtype0;
}
#endif /* defined(CONFIG_ARCH_CXD4108) */

void
phy_reset(void)
{
    _GPIO_SET_OUT(_GPIOEASY3, 11, OFF);
    udelay(3);
    _GPIO_SET_OUT(_GPIOEASY3, 11, ON);
}

