/*--------------------------------------------------------------------------*
 * drivers/usb/mcd/usb_mcd_defs.h
 * 
 * USB controller driver
 * 
 * Copyright 2005,2006 Sony Corporation
 * 
 * This code is based on musbdefs.h,
 * Copyright 2005 Mentor Graphics Corporation
 *
 * This file is part of the Inventra Controller Driver for Linux.
 *
 * The Inventra Controller Driver for Linux is free software; you
 * can redistribute it and/or modify it under the terms of the GNU
 * General Public License version 2 as published by the Free Software
 * Foundation.
 *
 * The Inventra Controller Driver for Linux is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with The Inventra Controller Driver for Linux ; if not,
 * write to the Free Software Foundation, Inc., 59 Temple Place,
 * Suite 330, Boston, MA  02111-1307  USA
 *
 * ANY DOWNLOAD, USE, REPRODUCTION, MODIFICATION OR DISTRIBUTION
 * OF THIS DRIVER INDICATES YOUR COMPLETE AND UNCONDITIONAL ACCEPTANCE
 * OF THOSE TERMS.THIS DRIVER IS PROVIDED "AS IS" AND MENTOR GRAPHICS
 * MAKES NO WARRANTIES, EXPRESS OR IMPLIED, RELATED TO THIS DRIVER.
 * MENTOR GRAPHICS SPECIFICALLY DISCLAIMS ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY; FITNESS FOR A PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT.  MENTOR GRAPHICS DOES NOT PROVIDE SUPPORT
 * SERVICES OR UPDATES FOR THIS DRIVER, EVEN IF YOU ARE A MENTOR
 * GRAPHICS SUPPORT CUSTOMER.
 *---------------------------------------------------------------------------*/

#ifndef __MUSB_MUSBDEFS_H__
#define __MUSB_MUSBDEFS_H__

#include <linux/version.h>
#include <linux/slab.h>
#include <linux/list.h>
#include <linux/smp_lock.h>

#ifdef MUSB_CONFIG_PROC_FS
#include <linux/fs.h>
#endif

#define  MUSB_DYNFIFO_USED

/* Board-specific definitions (hard-wired controller locations/IRQs) */
#include "usb_mcd_debug.h"

#include "usb_mcd_plat_cnf.h"
#include "usb_mcd_plat_arc.h"
#include "usb_mcd_musbhdrc.h"
#include "usb_mcd_virthub.h"
#include "usb_mcd_debug.h"
#if defined(MUSB_DMA)||defined(DMA_HOST_CODE)
#include "usb_mcd_dma.h"
#endif  /* MUSB_DMA */

#ifndef MUSB_LINUX_MV21
#define HAS_USB_TT_MULTI
#endif

/* OTG => GADGET */
#ifdef MUSB_OTG
#include <linux/usb/mcd/usb_otg_control.h>
#include "usb_mcd_otg.h"
#ifndef MUSB_GADGET
#define MUGB_GADGET
#endif
#endif

/****************************** CONSTANTS ********************************/
#define MGC_FIFO_MAXSIZE	0xFFFF
#define MGC_PAD_FRONT   0xa5deadfe
#define MGC_PAD_BACK    0xabadcafe
#define MGC_TEST_PACKET_SIZE 53

#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0) && LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) ) 
#undef MUSB_V26

#if !defined(MUSB_V24) 
#define MUSB_V24
#endif 

#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
#undef MUSB_V24
#if !defined(MUSB_V26) 
#define MUSB_V26
#endif
#endif

#ifndef USB_DT_DEVICE_QUALIFIER
#define USB_DT_DEVICE_QUALIFIER 6
#endif

#ifndef USB_DT_DEVICE_QUALIFIER_SIZE
#define USB_DT_DEVICE_QUALIFIER_SIZE 10
#endif

#ifndef USB_DT_OTHER_SPEED
#define USB_DT_OTHER_SPEED 7
#endif

#ifndef USB_MAXCHILDREN
#define USB_MAXCHILDREN         (16)
#endif

/****************************** CONSTANTS ********************************/


/* many common platforms have dma-coherent caches, which means that it's
 * safe to use kmalloc() memory for all i/o buffers without using any
 * cache flushing calls.  (unless you're trying to share cache lines
 * between dma and non-dma activities, which is a slow idea in any case.)
 *
 * other platforms need more care, with 2.5 having a moderately general
 * solution (which falls down for allocations smaller than one page)
 * that improves significantly on the 2.4 PCI allocators by removing
 * the restriction that memory never be freed in_interrupt().
 */
#define	DMA_ADDR_INVALID	(~(dma_addr_t)0)

#undef USE_KMALLOC
#if	defined(CONFIG_X86)
#define USE_KMALLOC

#elif	defined(CONFIG_PPC) && !defined(CONFIG_NOT_COHERENT_CACHE)
#define USE_KMALLOC

#elif	defined(CONFIG_MIPS) && !defined(CONFIG_NONCOHERENT_IO)
#define USE_KMALLOC

/* FIXME there are other cases, including an x86-64 one ...  */
#endif

#define USE_KMALLOC


#if defined(MUSB_V26) || defined(MUSB_V24)
#define MUSB_HAS_BUSNAME
#endif

#ifdef MUSB_OTG
#define MUSB_HAS_OTG
#endif

#ifdef CONFIG_PREEMPT

#endif

#if MUSB_DEBUG > 0
#define STATIC
#define MUSB_PARANOID
#else
#define STATIC static
#endif

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

#ifndef MUSB_C_NUM_EPS
#define MUSB_C_NUM_EPS ((uint8_t)16)
#endif

#ifndef MUSB_MAX_END0_PACKET
#define MUSB_MAX_END0_PACKET ((uint16_t)MGC_END0_FIFOSIZE)
#endif

#define MGC_END0_START  0x0
#define MGC_END0_OUT    0x2
#define MGC_END0_IN     0x4
#define MGC_END0_STATUS 0x8

#define MGC_END0_STAGE_SETUP 		0x0
#define MGC_END0_STAGE_TX		0x2
#define MGC_END0_STAGE_RX		0x4
#define MGC_END0_STAGE_STATUSIN		0x8
#define MGC_END0_STAGE_STATUSOUT        0xf
#define MGC_END0_STAGE_STALL_BIT	0x10

/* obsolete */
#define MGC_END0_STAGE_DATAIN		MGC_END0_STAGE_TX
#define MGC_END0_STAGE_DATAOUT		MGC_END0_STAGE_RX

/* EASY GUESS */
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,0)
#define USB_ALLOC_DEV( _parent, _usb_bus, _port) usb_alloc_dev(_parent, _usb_bus, _port)
#else
/* 2.4, mvl21, bc5 */
#define USB_ALLOC_DEV( _parent, _usb_bus, _port) usb_alloc_dev(_parent, _usb_bus)
#endif


/* 2.4 compatibility */
#ifdef MUSB_V26 

#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,9)
#define USB_HALT_ENDPOINT(_dev, _pipe_ep, _pipe_out) ((_dev)->bus->op->disable(_dev, _pipe_ep))
#define USB_ENDPOINT_HALTED(_dev, _pipe_ep, _pipe_out) ( 0 )
#else
#define USB_HALT_ENDPOINT(_dev, _pipe_ep, _pipe_out) usb_endpoint_halt(_dev, _pipe_ep, _pipe_out)
#define USB_ENDPOINT_HALTED(_dev, _pipe_ep, _pipe_out) usb_endpoint_halted(_dev, _pipe_ep, _pipe_out)
#endif


#define COMPLETE_URB(_pUrb, _p) _pUrb->complete(_pUrb, _p)
#define WAIT_MS(_ms)			msleep(_ms)

#define USB_ISO_ASAP            0x0002
#define USB_ASYNC_UNLINK        0x0008

#define USB_ST_NOERROR          (0)
#define USB_ST_CRC              (-EILSEQ)
#define USB_ST_BITSTUFF         (-EPROTO)
#define USB_ST_NORESPONSE       (-ETIMEDOUT)    /* device not responding/handshaking */
#define USB_ST_DATAOVERRUN      (-EOVERFLOW)
#define USB_ST_DATAUNDERRUN     (-EREMOTEIO)
#define USB_ST_BUFFEROVERRUN    (-ECOMM)
#define USB_ST_BUFFERUNDERRUN   (-ENOSR)
#define USB_ST_INTERNALERROR    (-EPROTO)       /* unknown error */
#define USB_ST_SHORT_PACKET     (-EREMOTEIO)
#define USB_ST_PARTIAL_ERROR    (-EXDEV)        /* ISO transfer only partially completed */
#define USB_ST_URB_KILLED       (-ENOENT)       /* URB canceled by user */
#define USB_ST_URB_PENDING      (-EINPROGRESS)
#define USB_ST_REMOVED          (-ENODEV)       /* device not existing or removed */
#define USB_ST_TIMEOUT          (-ETIMEDOUT)    /* communication timed out, also in urb->status**/
#define USB_ST_NOTSUPPORTED     (-ENOSYS)
#define USB_ST_BANDWIDTH_ERROR  (-ENOSPC)       /* too much bandwidth used */
#define USB_ST_URB_INVALID_ERROR  (-EINVAL)     /* invalid value/transfer type */
#define USB_ST_URB_REQUEST_ERROR  (-ENXIO)      /* invalid endpoint */
#define USB_ST_STALL            (-EPIPE)        /* pipe stalled, also in urb->status*/
                                                                                
#define USB_ZERO_PACKET         0x0040  	/* Finish bulk OUTs always with zero length packet */

#endif

#ifdef MUSB_V24
#define usb_disabled()			0
#define COMPLETE_URB(_pUrb, _p)		_pUrb->complete(_pUrb)
#define WAIT_MS(_ms)			wait_ms(_ms)
#define USB_HALT_ENDPOINT(_dev, _pipe_ep, _pipe_out) usb_endpoint_halt(_dev, _pipe_ep, _pipe_out)
#define USB_ENDPOINT_HALTED(_dev, _pipe_ep, _pipe_out) usb_endpoint_halted(_dev, _pipe_ep, _pipe_out)

#if defined(CONFIG_USB_MCD_HOST)
#ifdef MUSB_LINUX_MV21
#define usb_get_urb(_pUrb) _pUrb
#define usb_put_urb(_pUrb)
#undef MUSB_HAS_BUSNAME
#endif
#endif

#endif

typedef enum
{
  MGC_STATE_DEFAULT,
  MGC_STATE_ADDRESS,
  MGC_STATE_CONFIGURED
} MGC_DeviceState;

/* failure codes */
#define MUSB_ERR_VBUS		-1
#define MUSB_ERR_BABBLE		-2
#define MUSB_ERR_CORRUPTED	-3
#define MUSB_ERR_IRQ		-4
#define MUSB_ERR_SHUTDOWN	-5
#define MUSB_ERR_RESTART	-6

/* EP0 IN status*/
#define MCD_EP0_COMP_END	0
#define MCD_EP0_COMP_NON	1

/*==============================*/
/*	Macro for new function		*/
/*    in Transfer		 		*/
/*==============================*/
#define USB_MCD_FLAG_OFF	0	/* set variable to ZERO */
#define USB_MCD_FLAG_ON		1	/* set variable to 1 	*/

/****************************** FUNCTIONS ********************************/

#define KMALLOC(a,b,c)	{ lock_kernel(); a=kmalloc(b,c); unlock_kernel(); }
#define KFREE(p)	{ lock_kernel(); kfree(p); unlock_kernel(); }

#define MUSB_HST_MODE(_pthis) { (_pthis)->bIsHost=TRUE; (_pthis)->bIsDevice=FALSE; \
	(_pthis)->bFailCode=0; }
#define MUSB_DEV_MODE(_pthis) { (_pthis)->bIsHost=FALSE; (_pthis)->bIsDevice=TRUE; \
	(_pthis)->bFailCode=0; }
#define MUSB_OTG_MODE(_pthis) { (_pthis)->bIsHost=FALSE; (_pthis)->bIsDevice=FALSE; \
	(_pthis)->bFailCode=0; }
#define MUSB_ERR_MODE(_pthis, _cause) { (_pthis)->bFailCode=_cause; }

#define MUSB_MODE(_x) (((_x)->bIsHost) ? ((!(_x)->bIsDevice)?"HOST":"ERROR"): \
	((_x)->bIsDevice)?"FUNCTION":"OTG")

#define MUSB_IS_ERR(_x) ( (_x)->bFailCode )
#define MUSB_IS_HST(_x) ((_x)->bIsHost && !(_x)->bIsDevice )
#define MUSB_IS_DEV(_x) ( !MUSB_IS_HST(_x) )
#define MUSB_IS_OTG(_x) ( !MUSB_IS_ERR(_x) && !(_x)->bIsHost && !(_x)->bIsDevice )


/* indexed vs. flat register model */
#ifdef MUSB_FLAT_REG
#define MGC_SelectEnd(_pBase, _bEnd)
#define MGC_ReadCsr8(_pBase, _bOffset, _bEnd) \
    MGC_Read8(_pBase, MGC_END_OFFSET(_bEnd, _bOffset))
#define MGC_ReadCsr16(_pBase, _bOffset, _bEnd) \
    MGC_Read16(_pBase, MGC_END_OFFSET(_bEnd, _bOffset))
#define MGC_WriteCsr8(_pBase, _bOffset, _bEnd, _bData) \
    MGC_Write8(_pBase, MGC_END_OFFSET(_bEnd, _bOffset), _bData)
#define MGC_WriteCsr16(_pBase, _bOffset, _bEnd, _bData) \
    MGC_Write16(_pBase, MGC_END_OFFSET(_bEnd, _bOffset), _bData)
#else
#define MGC_SelectEnd(_pBase, _bEnd) \
    MGC_Write8(_pBase, MGC_O_HDRC_INDEX, _bEnd)
#define MGC_ReadCsr8(_pBase, _bOffset, _bEnd) \
    MGC_Read8(_pBase, (_bOffset + 0x10))
#define MGC_ReadCsr16(_pBase, _bOffset, _bEnd) \
    MGC_Read16(_pBase, (_bOffset + 0x10))
#define MGC_WriteCsr8(_pBase, _bOffset, _bEnd, _bData) \
    MGC_Write8(_pBase, (_bOffset + 0x10), _bData)
#define MGC_WriteCsr16(_pBase, _bOffset, _bEnd, _bData) \
    MGC_Write16(_pBase, (_bOffset + 0x10), _bData)
#endif

/******************************** TYPES **********************************/


extern const uint8_t MGC_aTestPacket[MGC_TEST_PACKET_SIZE];

#ifdef MUSB_GADGET_API
struct usb_gadget;
#endif

struct urb;
struct usb_device;

typedef struct
{
    uint8_t bmRequestType;
    uint8_t bRequest;
    uint16_t wValue;
    uint16_t wIndex;
    uint16_t wLength;
}  __attribute__ ((packed)) MUSB_DeviceRequest;


#ifdef DMA_HOST_CODE
/*--------------------------------------------------*/
/* for Host 										*/
/* States of transfer								*/
/*--------------------------------------------------*/
typedef enum  dma_host_state{
	DMA_HOST_ST_IDLE,	/*	not transfered 				*/
	DMA_HOST_ST_CPU,	/*	transfered by CPU 			*/
	DMA_HOST_ST_NULL,	/*	NULL transfered by CPU		*/
	DMA_HOST_ST_DMA,	/*	transfered by DMA			*/
	DMA_HOST_ST_CANCEL	/*  needed to cancel transfer	*/
}DMA_HOST_STATE;

/*--------------------------------------------------*/
/*  For Host										*/
/*	DMA_HOST_MODE:DMA mode							*/
/*--------------------------------------------------*/
typedef enum dma_host_mode{ 
	DMA_HOST_MODE0,			/*  MODE0	*/
	DMA_HOST_MODE1,			/*	MODE1	*/
	DMA_HOST_MODE_NON		/*  Initial */
}DMA_HOST_MODE;

/*--------------------------------------------------*/
/*	For Host										*/
/*	USB_MCD_DmaChStatus:local structure in DMA		*/
/*--------------------------------------------------*/
typedef struct dma_ch_status{
	uint8_t				index;	/* channel number */
	uint8_t				Ep;		/* EP number 	*/
	DMA_HOST_STATE		State;	/* transfer state */
	DMA_HOST_MODE		mode;	/* DMA Mode	*/
	uint8_t				dma_trans_flag;	/* DMA transfer flag */
	uint8_t				dma_req_flag;	/* DMA request flag */
	uint8_t				dma_ch_flag;	/* DMA channel flag (whether DMAchannel is) */
	uint32_t			dwStartAddress;	/* DMA start address */
}USB_MCD_DmaChStatus;

#endif

#ifdef HOST_NULL_FLAG
/*--------------------------------------------------*/
/*	USB_MCD_HostInfo : Information for host driver		*/
/*	@field null_flag 								*/
/*--------------------------------------------------*/
typedef struct mcd_hostinfo{
	uint8_t	null_flag;	/* flag to transfer Null data. (1:YES, 0:NO) 	*/
	uint8_t	db_buff;	/* 1:Double Buffer mode							*/
}USB_MCD_HostInfo;
#endif

/**
 * MGC_LinuxLocalEnd.
 * Local endpoint resource.
 * @field Lock spinlock
 * @field pUrb current URB
 * @field urb_list list
 * @field dwOffset current buffer offset
 * @field dwRequestSize how many bytes were last requested to move
 * @field wMaxPacketSizeTx local Tx FIFO size
 * @field wMaxPacketSizeRx local Rx FIFO size
 * @field wPacketSize programmed packet size
 * @field bIsSharedFifo TRUE if FIFO is shared between Tx and Rx
 * @field bAddress programmed bus address
 * @field bEnd programmed remote endpoint address
 * @field bTrafficType programmed traffic type
 * @field bIsClaimed TRUE if claimed
 * @field bIsTx TRUE if current direction is Tx
 * @field bIsReady TRUE if ready (available for new URB)
 */
typedef struct
{
#if MUSB_DEBUG > 0
    uint32_t dwPadFront;
#endif
    spinlock_t Lock;
    struct urb* pUrb;
    struct list_head urb_list;
    unsigned int dwOffset;
    unsigned int dwRequestSize;
    unsigned int dwIsoPacket;
    unsigned int dwWaitFrame;
#if defined(MUSB_DMA)||defined(DMA_HOST_CODE)
    MGC_DmaChannel* pDmaChannel;
#endif
#ifdef DMA_HOST_CODE
	USB_MCD_DmaChStatus	*pChEpStatus;	/* DMA EPstatus	*/
#endif
#ifdef HOST_NULL_FLAG
	USB_MCD_HostInfo	HostInfo;
#endif
#ifdef MUSB_CONFIG_PROC_FS
    unsigned long dwTotalTxBytes;
    unsigned long dwTotalRxBytes;
    unsigned long dwTotalTxPackets;
    unsigned long dwTotalRxPackets;
    unsigned long dwErrorTxPackets;
    unsigned long dwErrorRxPackets;
    unsigned long dwMissedTxPackets;
    unsigned long dwMissedRxPackets;
#endif
    uint16_t wMaxPacketSizeTx;
    uint16_t wMaxPacketSizeRx;
    uint16_t wPacketSize;
    uint8_t bIsSharedFifo;
    uint8_t bAddress;
    uint8_t bEnd;
    uint8_t bTrafficType;
    uint8_t bIsClaimed;
    uint8_t bIsTx;
    uint8_t bIsReady;
    uint8_t bRetries;
    uint8_t bLocalEnd;

    uint8_t bDisableDma;

#if MUSB_DEBUG > 0
    uint32_t dwPadBack;
#endif
} MGC_LinuxLocalEnd;


/* endpoint 0 buffer. When used with the standard ReadUSBHeader
 * routines, struct usb_ctrlrequest.
 */
typedef struct {
    uint8_t aEnd0Data[MUSB_MAX_END0_PACKET];
    uint16_t wEnd0Count; /* bytes to go */
} MGC_End0Buffer;

/** A listener for disconnection */
typedef void (*MGC_pfDisconnectListener)(void*);

/** A handler for the default endpoint interrupt */
typedef void (*MGC_pfDefaultEndHandler)(void*);

/**
 * MGC_LinuxCd.
 * Driver instance data.
 * @field Lock spinlock
 * @field Timer interval timer for various things
 * @field pBus pointer to Linux USBD bus
 * @field RootHub virtual root hub
 * @field PortServices services provided to virtual root hub
 * @field pRootDevice root device pointer, to track connection speed
 * @field pDmaController pointer to DMA controller information
 * @field nIrq IRQ number (needed by free_irq)
 * @field nIsPci TRUE if PCI
 * @field bIsMultipoint TRUE if multi-point core
 * @field bIsHost TRUE if host
 * @field bIsDevice TRUE if peripheral
 * @field nIackAddr IACK address (PCI only)
 * @field nIackSize size of IACK PCI region (needed by release_region)
 * @field nRegsAddr address of registers PCI region (needed by release_region)
 * @field nRegsSize size of registers region (needed by release_region)
 * @field pIack pointer to mapped IACK region (PCI only)
 * @field pRegs pointer to mapped registers
 * @field wFifoHead pointer to next address for dynamic FIFO sizing
 */
typedef struct
{
#if MUSB_DEBUG > 0
    uint32_t dwPadFront;
#endif
    spinlock_t Lock;
    struct timer_list Timer;
    struct usb_bus *pBus;
    char aName[32];
#if !defined(CONFIG_USB_MCD_HOST) 
	struct device *controller;
#endif
    MGC_VirtualHub RootHub;
    MGC_PortServices PortServices;
    struct usb_device* pRootDevice;
#define HCD_BUFFER_POOLS	4	//  Buffer_alloc
	struct dma_pool		*pool [HCD_BUFFER_POOLS];
#if defined(MUSB_DMA)||defined(DMA_HOST_CODE)
    MGC_DmaController* pDmaController;
#endif
#ifdef	DMA_HOST_CODE
	USB_MCD_DmaChStatus	*pChIndStatus;		/*  structure for DMAtransfer info */
#endif
    int nIrq;
    int nIsPci;
    int nBabbleCount;
    unsigned int nIackAddr;
    unsigned int nIackSize;
    unsigned int nRegsAddr;
    unsigned int nRegsSize;
    void* pIack;
    void* pRegs;

    uint16_t wFifoHead;

    MGC_LinuxLocalEnd aLocalEnd[MUSB_C_NUM_EPS];

    uint16_t wEndMask;
    uint8_t bEndCount;
    uint8_t bRootSpeed;
    uint8_t bIsMultipoint;
    uint8_t bIsHost;
    uint8_t bIsDevice;
    
    uint8_t bFailCode; /* one of MUSB_ERR_* failure code */
    
    uint8_t bBulkTxEnd;
    uint8_t bBulkRxEnd;
    uint8_t bBulkSplit;
    uint8_t bBulkCombine;
    uint8_t bEnd0Stage;

#ifdef MUSB_GADGET
    uint8_t bDeviceState;
    uint8_t bSuspended;
    uint8_t bIsSelfPowered;
    uint8_t bSetAddress;
    uint8_t bAddress;
    uint8_t bTestMode;
    uint8_t bTestModeValue;

    /* compatibility, need to be osoleted 
     * used from gstorage */
    uint16_t wEnd0FifoCount;
    uint16_t wEnd0Offset;
    uint8_t aEnd0Data[MUSB_MAX_END0_PACKET];

    struct usb_gadget* pGadget; /* the gadget */
    struct usb_gadget_driver* pGadgetDriver; /* it's driver */

    /* Endpoint 0 buffer and its buffer code; can be custom for 
     * devices that are not usign the default headers.
     *
     * . pfFillBuffer is MGC_HdrcReadUSBControlRequest(); 
     * . pEnd0Buffer is MGC_End0Buffer
     */
    struct usb_request* pEnd0Request; /* current ep0 request */
    int (*pfReadHeader)(void*, uint16_t); /* NULL==MGC_HdrcReadUSBControlRequest*/
    void* pEnd0Buffer; /* this is the buffer, clients can change */
#endif

#ifdef MUSB_OTG
    struct usb_otg_control* otg_control;
#endif

#if MUSB_DEBUG > 0
    uint32_t dwPadBack;
#endif

#ifdef MUSB_CONFIG_PROC_FS
    struct proc_dir_entry* pProcEntry;
    /* A couple of hooks to enable HSET */
    MGC_pfDisconnectListener pfDisconnectListener;
    void* pDisconnectListenerParam;
    MGC_pfDefaultEndHandler pfDefaultEndHandler;
    void* pDefaultEndHandlerParam;    
#endif

} MGC_LinuxCd;


/* Conditionally-compiled to update OTG state machine when necessary */
extern void MGC_HdrcStart(MGC_LinuxCd* pThis);
extern void MGC_HdrcStop(MGC_LinuxCd* pThis);
extern void MGC_OtgUpdate(MGC_LinuxCd* pThis, uint8_t bVbusError, 
			  uint8_t bConnect);
extern void MGC_HdrcServiceUsb(MGC_LinuxCd* pThis, uint8_t reg);
extern void MGC_HdrcLoadFifo(const uint8_t* pBase, uint8_t bEnd, 
	uint16_t wCount, const uint8_t* pSource);
extern void MGC_HdrcUnloadFifo(const uint8_t* pBase, uint8_t bEnd, 
	uint16_t wCount, uint8_t* pDest);


#if defined(MUSB_CONFIG_PROC_FS)

extern struct proc_dir_entry* MGC_LinuxCreateProcFs(char *name, MGC_LinuxCd* data);
extern void MGC_LinuxDeleteProcFs(MGC_LinuxCd* data);

#ifdef MUSB_PROC_TESTMUSB
extern struct proc_dir_entry* MGC_LinuxCreateTestProcFs(char *name, MGC_LinuxCd* data);
extern void MGC_LinuxDeleteTestProcFs(char *name, MGC_LinuxCd* data);
#endif

/**
 * Set a listener for disconnect interrupts.
 * @param pfListener listener, or NULL for none
 * @param pParam parameter to pass listener
 */
extern void MGC_HdrcSetDisconnectListener(MGC_LinuxCd* pCd,
	MGC_pfDisconnectListener pfListener,
	void* pParam);
/**
 * Set a new handler for the default endpoint interrupt.
 * @param pfHandler new handler, or NULL to restore default handler
 * @param pParam parameter to pass handler
 */
extern void MGC_HdrcSetDefaultEndHandler(MGC_LinuxCd* pCd,
	MGC_pfDefaultEndHandler pfHandler,
	void* pParam);
#endif /* defined(MUSB_CONFIG_PROC_FS) */

#ifdef MUSB_GADGET
extern int queue_length(struct list_head *lh);
extern void dump_ep_status(MGC_LinuxCd* pThis);
extern void dump_ep_queue(int index, int verbose);

extern void MGC_GadgetStart(MGC_LinuxCd* pThis);
extern void MGC_GadgetStop(MGC_LinuxCd* pThis);
extern void MGC_GadgetReset(MGC_LinuxCd* pThis);
extern void MGC_GadgetResume(MGC_LinuxCd* pThis);
extern void MGC_GadgetSuspend(MGC_LinuxCd* pThis);
extern void MGC_GadgetDisconnect(MGC_LinuxCd* pThis);
extern void MGC_HdrcServiceDeviceDefaultEnd(MGC_LinuxCd* pThis);
extern void MGC_HdrcServiceDeviceTxAvail(MGC_LinuxCd* pThis, uint8_t bEnd);
extern void MGC_HdrcServiceDeviceRxReady(MGC_LinuxCd* pThis, uint8_t bEnd);

#ifdef MUSB_OBSOLETE_GADGET
struct usb_ep;
extern int MGC_GadgetFindEnd(struct usb_ep* pGadgetEnd);
#endif

#if MUSB_DEBUG > 0
struct usb_endpoint_descriptor;
extern int MGC_GadgetDisableEndPoint(uint8_t bEnd);
extern int MGC_GadgetEnableEndPoint(uint8_t bEnd, const struct usb_endpoint_descriptor *desc);
#endif

#endif

#if defined(MUSB_GADGET) || defined(MUSB_V26)
void* MGC_AllocBufferMemory(MGC_LinuxCd* pThis, size_t bytes, int gfp_flags, dma_addr_t* dma);
void MGC_FreeBufferMemory(MGC_LinuxCd* pThis, size_t bytes, void *address, dma_addr_t dma);
#endif

/*==========================================================*/
/*	Dynamic FIFO configuration for HOST						*/
/*==========================================================*/
/* Total Size */
#define USB_MCD_DFIFO_TOTAL_SZ	4224		/* Maximum size of FIFO */
/* Maximum number of EP */
#define USB_MCD_DFIFO_TOTAL_EP	6			/* except EP0 */

/* EP */
#define USB_MCD_DFIFO_EP0	0
#define USB_MCD_DFIFO_EP1	1
#define USB_MCD_DFIFO_EP2	2
#define USB_MCD_DFIFO_EP3	3
#define USB_MCD_DFIFO_EP4	4
#define USB_MCD_DFIFO_EP5	5
#define USB_MCD_DFIFO_EP6	6

/* FIFO size */
#define USB_MCD_DFIFO_CTRL_SZ	0x40		/* 64Byte 	*/
#define USB_MCD_DFIFO_BULK_SZ	0x200		/* 512Byte 	*/
#define USB_MCD_DFIFO_INT_SZ	0x40		/* 64Byte 	*/
#define USB_MCD_DFIFO_ISO_SZ	0x200		/* 512Byte */
/* Double Buffer used or not */
#define USB_MCD_DFIFO_CTRL_DPB	0			/* single(fix 1) */
#define USB_MCD_DFIFO_BULK_DPB	1			/* double */
#define USB_MCD_DFIFO_INT_DPB	1			/* double */
#define USB_MCD_DFIFO_ISO_DPB	0			/* single */
/* Numbers of EP in each transfer Type */
#define USB_MCD_DFIFO_CTRL_NUM	1			/* x1 fixed number for EP0*/
#define USB_MCD_DFIFO_BULK_NUM	2			/* x2 */
#define USB_MCD_DFIFO_INT_NUM	2			/* x2 */
#define USB_MCD_DFIFO_ISO_NUM	2			/* x2 */
/* check */
#if USB_MCD_DFIFO_TOTAL_EP - (USB_MCD_DFIFO_BULK_NUM + USB_MCD_DFIFO_INT_NUM + USB_MCD_DFIFO_ISO_NUM)
 #error EP TOTAL NUMBERS error!!
#endif
/* check */
#define USB_MCD_DFIFO_RESET_SZ	USB_MCD_DFIFO_TOTAL_SZ - (USB_MCD_DFIFO_CTRL_SZ\
	+ (USB_MCD_DFIFO_BULK_SZ*(USB_MCD_DFIFO_BULK_DPB+1))\
	+ (USB_MCD_DFIFO_INT_SZ*(USB_MCD_DFIFO_INT_DPB+1))\
	+ (USB_MCD_DFIFO_ISO_SZ*(USB_MCD_DFIFO_ISO_DPB+1)))		/* Rest size of FIFO */
#if USB_MCD_DFIFO_RESET_SZ < 0
 #error FIFOSIZE error!!
#endif

typedef struct usb_mcd_dfifo_list{
	uint8_t		ep;			/* epnum | ep_dir */
	uint8_t		type;		/* transfer-type 0:PIPE_ISOCHRONOUS, 1:PIPE_INTERRUPT, 2:PIPE_CONTROL, 3:PIPE_BULK*/
	uint32_t	fifo_sz;	/* fifo-size a single buffer */
	uint8_t		dpb;		/* 0: double buffer , 1: single buffer */
}USB_MCD_DFIFO_LIST;

#endif	/* multiple inclusion protection */
