/*--------------------------------------------------------------------------*
 * drivers/usb/mcd/usb_mcd_fifo.h
 * 
 * USB controller driver
 * 
 * Copyright 2005,2006 Sony Corporation
 *
 * This file is part of the Inventra Controller Driver for Linux.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 *---------------------------------------------------------------------------*/

#ifndef _USB_MCD_FIFO_H_
#define _USB_MCD_FIFO_H_

/**
 * Reset dynamic FIFO size
 *
 * @param pThis controller driver instance pointer
 */
extern void
musb_fifo_reset(
    MGC_LinuxCd* pThis);

/**
 * map dynamic FIFO for specified ep
 *
 * @param pThis controller driver instance pointer
 * @param bEpAdr endpoint address described in endpoint descriptor
 * @param wSize specify FIFO size
 * @param bType specify 2 when double buffer or 1 when single buffer
 * @param bIsHost specify 1 when use as a host, 0 as a gadget.
 * @return FIFO address mapped in FIFO map
 */
extern void*
musb_fifo_map(
    MGC_LinuxCd* pThis,
    uint8_t bEpAdr,
    uint16_t wSize,
    uint8_t bType,
    uint8_t bIsHost);

/**
 * set default dynamic FIFO map
 *
 * @param pThis controller driver instance pointer
 */
extern void
musb_fifo_set_default_map(
    MGC_LinuxCd* pThis);

/**
 * dump FIFO map
 *
 *
 * @param pThis controller driver instance pointer
 */
extern void
musb_fifo_dump(
    MGC_LinuxCd* pThis);

#endif /* _USB_MCD_FIFO_H_ */
