/*--------------------------------------------------------------------------*
 * drivers/usb/mcd/usb_mcd_gadget.h
 * 
 * USB controller driver
 * 
 * Copyright 2005,2006 Sony Corporation
 * 
 * This code is based on musb_gadget.h,
 * Copyright 2005 Mentor Graphics Corporation
 *
 * This file is part of the Inventra Controller Driver for Linux.
 *
 * The Inventra Controller Driver for Linux is free software; you
 * can redistribute it and/or modify it under the terms of the GNU
 * General Public License version 2 as published by the Free Software
 * Foundation.
 *
 * The Inventra Controller Driver for Linux is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with The Inventra Controller Driver for Linux ; if not,
 * write to the Free Software Foundation, Inc., 59 Temple Place,
 * Suite 330, Boston, MA  02111-1307  USA
 *
 * ANY DOWNLOAD, USE, REPRODUCTION, MODIFICATION OR DISTRIBUTION
 * OF THIS DRIVER INDICATES YOUR COMPLETE AND UNCONDITIONAL ACCEPTANCE
 * OF THOSE TERMS.THIS DRIVER IS PROVIDED "AS IS" AND MENTOR GRAPHICS
 * MAKES NO WARRANTIES, EXPRESS OR IMPLIED, RELATED TO THIS DRIVER.
 * MENTOR GRAPHICS SPECIFICALLY DISCLAIMS ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY; FITNESS FOR A PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT.  MENTOR GRAPHICS DOES NOT PROVIDE SUPPORT
 * SERVICES OR UPDATES FOR THIS DRIVER, EVEN IF YOU ARE A MENTOR
 * GRAPHICS SUPPORT CUSTOMER.
 *---------------------------------------------------------------------------*/

#ifndef __MUSB_GADGETDEFS_H
#define __MUSB_GADGETDEFS_H

struct usb_request;

/* compatibility, need to be osoleted 
 * used from gstorage */	
typedef struct {
    struct usb_request* pRequest;
} MGC_GadgetCompletionData;
 
/**
 * MGC_GadgetLocalEnd.
 * A gadget local endpoint (to remember certain things separately
 * from host mode, so we can swap roles in OTG)
 *
 * @field Lock spinlock
 * @field pRequest current request
 * @field wPacketSize programmed packet size
 * @field bTrafficType programmed traffic type
 * @field bIsTx TRUE if current direction is Tx
 */
typedef struct {
    struct usb_ep end_point;
    
    spinlock_t Lock;
    struct list_head req_list;
    unsigned int dwRequestSize;
    uint16_t wPacketSize;
    uint8_t bTrafficType;
    uint8_t bIsTx;   
    uint8_t bDisabled;
	
    /* compatibility, need to be osoleted 
     * used from gstorage */	
    struct usb_request *pRequest;
    unsigned int dwOffset;
    MGC_GadgetCompletionData CompletionData;

} MGC_GadgetLocalEnd;


#define MGC_IsIdle(_pEnd) ( list_empty(&(_pEnd)->req_list) )
extern struct usb_request* MGC_CurrentRequest(MGC_GadgetLocalEnd* pEnd);
extern int MGC_RestartRequest(MGC_LinuxCd* pThis, uint8_t bEnd, struct usb_request *pRequest);

/* dealing with Linix differences */
struct usb_ctrlrequest;
 
/* references */
extern MGC_GadgetLocalEnd MGC_aGadgetLocalEnd[];

//extern struct usb_ep MGC_GadgetEnd0;
//extern struct usb_ep MGC_aGadgetEnd[MUSB_C_NUM_EPS]; 
extern char *decode_usb_ctrlrequest(const struct usb_ctrlrequest *pControlRequest);

/* export from plat_uds.c */
extern MGC_LinuxCd* MGC_pFirstDriver;

extern uint8_t is_tx_request(const struct usb_ctrlrequest *pControlRequest);
extern uint8_t is_rx_request(const struct usb_ctrlrequest *pControlRequest);
extern uint8_t is_zerodata_request(const struct usb_ctrlrequest *pControlRequest);


extern void MGC_FlushRequests(MGC_LinuxCd* pThis, int code);
extern void MGC_FlushEpRequests(const uint8_t bEnd, const int status);


extern void MGC_HdrcServiceDeviceTxAvail(MGC_LinuxCd* pThis, uint8_t bEnd);
extern void MGC_HdrcServiceDeviceRxReady(MGC_LinuxCd* pThis, uint8_t bEnd);
extern void MGC_HdrcLoadFifo(const uint8_t* pBase, uint8_t bEnd, 
	uint16_t wCount, const uint8_t* pSource);
extern void MGC_HdrcUnloadFifo(const uint8_t* pBase, uint8_t bEnd, 
	uint16_t wCount, uint8_t* pDest);


/* export from musb_gadget.c */
extern int MGC_LinuxGadgetQueue(const uint8_t nEnd, struct usb_request *pRequest,
	int gfp_flags);
extern void MGC_GadgetReset(MGC_LinuxCd* pThis);
extern void MGC_GadgetResume(MGC_LinuxCd* pThis);
extern void MGC_GadgetSuspend(MGC_LinuxCd* pThis);
extern void MGC_GadgetDisconnect(MGC_LinuxCd* pThis);


extern void MGC_HdrcHandleGadgetEp0Request(struct usb_request *req);
extern void MGC_CompleteEp0Request(void);

/* gstorage only */
extern void MGC_HdrcServiceDeviceDefaultEnd(MGC_LinuxCd* pThis);

extern int ep0_txstate(void);
extern int ep0_rxstart(void);
extern int ep0_rxstate(void);

extern int MGC_gtest_init(void); 


#endif /* __MUSB_GADGETDEFS_H */
