/*-----------------------------------------------------------------------------
 * drivers/usb/mcd/usb_mcd_otg.h
 * 
 * USB controller driver
 * 
 * Copyright 2005,2006 Sony Corporation
 *
 * This file is part of the Inventra Controller Driver for Linux.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *---------------------------------------------------------------------------*/

#ifndef _MUSB_OTG_H_
#define _MUSB_OTG_H_

/*
 *	ƥ
 */

/* 桼IO */
#define USB_OTG_IO_START	1
#define USB_OTG_IO_STOP		0

/* Vbus */
#define USB_OTG_VBUS_THRU 	2
#define USB_OTG_VBUS_ON 	1
#define USB_OTG_VBUS_OFF	0

/*
 *	¤
 */

struct musb_otgc_intdat {
    spinlock_t lock;
    int state;
    struct timer_list detector;
    struct timer_list timer;
    void* regs;
    struct usb_otg_core* otg_core;
    unsigned int cid_check_count;
    int last_cid;
    int ref_cid;
    unsigned int vbus_check_count;
    enum usb_otg_vbus_stat last_vbus;
    enum usb_otg_vbus_stat ref_vbus;
    struct list_head event_list;
};

struct musb_otgc_event_container {
    union usb_otg_event event;
    struct list_head list;
};

/*
 *	I/Fؿ
 */

/*
 * ؿmusb_otg_control_init
 * MUSB OTGȥɥ饤Фν
 */

extern int
musb_otg_control_init(void);

/*
 * ؿmusb_otg_control_exit
 * MUSB OTGȥɥ饤Фνλ
 */

extern int
musb_otg_control_exit(void);

/*
 * ؿmusb_otg_control_map
 * OTGȥΥޥåԥ󥰡쥸ɥ쥹Ͽ
 */

extern struct usb_otg_control*
musb_otg_control_map(
    const char* name,
    void* regs);

/*
 * ؿmusb_otg_control_unmap
 * OTGȥΥޥåԥ
 */

extern int
musb_otg_control_unmap(
    struct usb_otg_control* otg_control);

/*
 *	桼ؿ
 */

/*
 * ؿmusb_otg_user_init
 * GPIO桼ν桼ؿ
 */

extern int
musb_otg_user_init(void);

/*
 * ؿmusb_otg_user_exit
 * GPIO桼νλ桼ؿ
 */

extern int
musb_otg_user_exit(void);

/*
 *	桼ؿƤӽФؿ
 */

/*
 * ؿmusb_otg_control_create_object
 * usb_otg_control¤ΤΥ󥹥󥹥֥Ȥ
 */

extern struct usb_otg_control*
musb_otg_control_create_object(
    unsigned int user_data,
    unsigned int periodic_delay,
    unsigned int cid_filter_count,
    unsigned int vbus_filter_count,
    int (*check_cid)(struct usb_otg_control* otg_control),
    enum usb_otg_vbus_stat (*check_vbus)(struct usb_otg_control* otg_control),
    int (*check_suspend)(struct usb_otg_control* otg_control),
    int (*control_io)(struct usb_otg_control* otg_control, unsigned int control),
    int (*control_vbus)(struct usb_otg_control* otg_control,
                        unsigned int control),
    int (*control_test)(struct usb_otg_control* otg_control,
                        enum usb_otg_control_test_mode test_mode),
    const char* name);

/*
 * ؿmusb_otg_control_notify_vbuserror
 * Vbus顼Τ롣
 */
extern int
musb_otg_control_notify_vbuserror(
    struct usb_otg_control* otg_control);

/*
 * ؿmusb_otg_control_force_vbus
 * Vbus椹
 */
extern int
musb_otg_control_force_vbus(
    struct usb_otg_control* otg_control,
    unsigned int control);

/*
 * ؿmusb_otg_control_check_suspend
 * ǥХSuspend֤å
 */
extern int
musb_otg_control_check_suspend(
    struct usb_otg_control* otg_control,
    int expected);

/*
 * ؿmusb_otg_control_start_detecotr
 * Vbus/CIDΥޡdetector򳫻Ϥ롣
 */
extern void
musb_otg_control_start_detector(
    struct usb_otg_control* otg_control);

/*
 * ؿmusb_otg_control_stop_detecotr
 * Vbus/CIDΥޡdetectorߤ롣
 */
extern void
musb_otg_control_stop_detector(
    struct usb_otg_control* otg_control);

/*
 * ؿmusb_otg_control_signal
 * OTGơȥޥإʥϤ롣
 */
extern int
musb_otg_control_signal(
    struct usb_otg_control* otg_control,
    union usb_otg_event* otg_event);

/*
 * ؿmusb_otg_control_signal_interrupt
 * ߤOTGơȥޥإʥϤ롣
 */
extern void
musb_otg_control_signal_interrupt(
    struct usb_otg_control* otg_control,
    union usb_otg_event* otg_event);

/*
 * ؿmusb_otg_control_start_rchost
 * SuspendȥꥬȤControllerRCHOST⡼ɤˤ롣
 */

extern int
musb_otg_control_start_rchost(
    struct usb_otg_control* otg_control);

/* ؿmusb_bus_check_vbus
 * Vbusξ֤
 */

extern enum usb_otg_vbus_stat
musb_bus_check_vbus(
    void* regs);

/*
 * ؿmusb_bus_enable_role_change
 * b-deviceHostReq򥻥åȤRoleChangeĤ롣
 */

extern int
musb_bus_enable_role_change(
    void* regs);

/*
 * ؿmusb_bus_disable_role_change
 * b-deviceHostReq򥯥ꥢRoleChangeԵĤȤ롣
 */

extern int
musb_bus_disable_role_change(
    void* regs);

/*
 * ؿmusb_bus_suspend
 * RoleChangeȼSuspendԤ
 */

extern int
musb_bus_suspend(
    void* regs);

/*
 * ؿmusb_bus_start_session
 * a-deviceSessionӥåȤ򥻥åȤVbus򳫻Ϥ롣
 */

extern int
musb_bus_start_session(
    void* regs);

/*
 * ؿmusb_bus_end_session
 * a-deviceSessionӥåȤ򥯥ꥢVbusߤ롣
 */

extern int
musb_bus_end_session(
    void *regs);

/*
 * ؿmusb_bus_notify_vbus_change
 * b-devicevbusѲΤ
 */

extern int
musb_bus_notify_vbus_change(
    void* regs,
    uint8_t old_vbus,
    uint8_t new_vbus);

/*
 * ؿmusb_bus_request_session
 * idleSessionӥåȤ򥻥åȤSRPȯԤ롣
 */

extern int
musb_bus_request_session(
    void* regs);

/*
 * ؿmusb_bus_set_speed
 * HSEnable򥻥åȲ롣
 */

extern int
musb_bus_set_speed(
    void* regs,
    unsigned int speed);

/*
 * ؿmusb_bus_get_speed
 * Υԡɾ֤롣
 */

extern enum usb_otg_control_speed
musb_bus_get_speed(
    void* regs);

/*
 * ؿmusb_bus_set_test_mode
 * ƥȥ⡼ꤹ
 */

extern int
musb_bus_set_test_mode(
    void* regs,
    enum usb_otg_control_test_mode test_mode);

/*
 * ؿmusb_bus_get_test_mode
 * ƥȥ⡼ɼ
 */

extern enum usb_otg_control_test_mode
musb_bus_get_test_mode(
    void *regs);

/*
 * ؿmusb_bus_start_hdrc
 * HDRCDMAC
 */

extern int musb_bus_start_hdrc(void);

/*
 * ؿmusb_bus_stop_hdrc
 * HDRCDMAC
 */

extern int musb_bus_stop_hdrc(void);

/*
 * ؿmusb_bus_start_hdrc_int
 * HDRCߵ
 */
extern int musb_bus_start_hdrc_int(void);

/*
 * ؿmusb_bus_stop_hdrc_int
 * HDRCԵ
 */
extern int musb_bus_stop_hdrc_int(void);

#endif /* _MUSB_OTG_H_ */
