/*--------------------------------------------------------------------------*
 * drivers/usb/mcd/usb_mcd_xgadget.c
 * 
 * USB controller driver
 * 
 * Copyright 2005,2006 Sony Corporation
 * 
 * This code is based on musb_xgadget.c,
 * Copyright 2005 Mentor Graphics Corporation
 *
 * This file is part of the Inventra Controller Driver for Linux.
 *
 * The Inventra Controller Driver for Linux is free software; you
 * can redistribute it and/or modify it under the terms of the GNU
 * General Public License version 2 as published by the Free Software
 * Foundation.
 *
 * The Inventra Controller Driver for Linux is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with The Inventra Controller Driver for Linux ; if not,
 * write to the Free Software Foundation, Inc., 59 Temple Place,
 * Suite 330, Boston, MA  02111-1307  USA
 *
 * ANY DOWNLOAD, USE, REPRODUCTION, MODIFICATION OR DISTRIBUTION
 * OF THIS DRIVER INDICATES YOUR COMPLETE AND UNCONDITIONAL ACCEPTANCE
 * OF THOSE TERMS.THIS DRIVER IS PROVIDED "AS IS" AND MENTOR GRAPHICS
 * MAKES NO WARRANTIES, EXPRESS OR IMPLIED, RELATED TO THIS DRIVER.
 * MENTOR GRAPHICS SPECIFICALLY DISCLAIMS ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY; FITNESS FOR A PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT.  MENTOR GRAPHICS DOES NOT PROVIDE SUPPORT
 * SERVICES OR UPDATES FOR THIS DRIVER, EVEN IF YOU ARE A MENTOR
 * GRAPHICS SUPPORT CUSTOMER.
 *---------------------------------------------------------------------------*/

#include <linux/kernel.h>
#include <linux/list.h>
#include <linux/timer.h>
#include <linux/spinlock.h>
#include <linux/errno.h>
#include <linux/interrupt.h>

#ifdef MUSB_V26
#include <linux/device.h>
#else
struct usb_tt {
        struct usb_device       *hub;   /* upstream highspeed hub */
        int                     multi;  /* true means one TT per port */
};
#endif

#include <linux/init.h>
#include <linux/usb_ch9.h>
#include "usb_mcd_defs.h"
#include "usb_mcd_gadgetdefs.h"


/**********************************************************************
 *
 *
 */

/*
	case USB_REQ_GET_CONFIGURATION:
	case USB_REQ_GET_INTERFACE:
	case USB_REQ_GET_DESCRIPTOR:
	case USB_REQ_GET_STATUS:
	case USB_REQ_SYNC_FRAME: 
*/
uint8_t is_tx_request(const struct usb_ctrlrequest *pControlRequest) {
    return ( pControlRequest->bRequestType & USB_DIR_IN );
}

/*
	case USB_REQ_SET_DESCRIPTOR:
*/
uint8_t is_rx_request(const struct usb_ctrlrequest *pControlRequest) {
    return !( pControlRequest->bRequestType & USB_DIR_IN );
}

/*
    case USB_REQ_SET_INTERFACE: 
    case USB_REQ_SET_CONFIGURATION:
    case USB_REQ_SET_ADDRESS:
    case USB_REQ_CLEAR_FEATURE:
    case USB_REQ_SET_FEATURE:
*/
uint8_t is_zerodata_request(const struct usb_ctrlrequest *pControlRequest) {
    return ( !pControlRequest->wLength ) && !is_tx_request(pControlRequest);
}

/*
 */
int queue_length(struct list_head *lh) {
	int count=0;	
	struct list_head *p=lh;
	
	while ( p && (p->next!=lh) ) {
	    count++;
	    p=p->next;
	}

	return count;
}

/**********************************************************************
 *
 *
 */

/* 
 * @pre pThis!=NULL
 */
void
MGC_GadgetResume(
    MGC_LinuxCd* pThis)
{
    uint8_t suspended;

    DBG(2, "<==\n" );

    suspended = musb_otg_control_check_suspend(pThis->otg_control, FALSE);
    if (suspended) {
        DBG(2, "Resume ignored\n");
        return;
    }

    pThis->bSuspended = FALSE;
    DBG(2, "Resumed\n");

    if( pThis->pGadgetDriver && pThis->pGadgetDriver->resume) {
        pThis->pGadgetDriver->resume( pThis->pGadget );
    }

    DBG(2, "==>\n");
}

/* 
 * @pre pThis!=NULL
 */
void
MGC_GadgetSuspend(
    MGC_LinuxCd* pThis)
{
    uint8_t suspended;
    DBG(2, "<==\n" );

    suspended = musb_otg_control_check_suspend(pThis->otg_control, TRUE);
    if (!suspended) {
        DBG(2, "suspend ignored\n");
        return;
    }

    pThis->bSuspended = TRUE;
    DBG(2, "Suspended\n");

    if( pThis->pGadgetDriver && pThis->pGadgetDriver->suspend) {
        pThis->pGadgetDriver->suspend( pThis->pGadget );
    }

    DBG(2, "==>\n");
}

/* 
 * @pre pThis!=NULL
 */
void
MGC_GadgetDisconnect(
    MGC_LinuxCd* pThis)
{
    uint8_t bEnd;

    DBG(2, "<==\n" );    
    if (pThis->pGadgetDriver) {
        /* EPꥯȤδλ */
        for (bEnd = 0; bEnd < pThis->bEndCount; bEnd++) {
            MGC_FlushEpRequests(&MGC_aGadgetLocalEnd[bEnd], -ESHUTDOWN);
        }
        /* Disconnect */
        if (pThis->pGadgetDriver->disconnect) {
            pThis->pGadgetDriver->disconnect(pThis->pGadget);
        }
    }
    DBG(2, "==>\n" );    
}

/* 
 * @pre pThis!=NULL
 */
void
MGC_GadgetReset(
    MGC_LinuxCd* pThis)
{
    const uint8_t* pBase = (uint8_t*)pThis->pRegs;
    uint8_t devctl = MGC_Read8(pBase, MGC_O_HDRC_DEVCTL);
    
    DBG(2, "<==\n");

#ifdef MUSB_PARANOID 
   ASSERT_SPINLOCK_UNLOCKED(&pThis->Lock); 
   ASSERT_SPINLOCK_UNLOCKED(&MGC_aGadgetLocalEnd[0]); 
#endif

    DBG(1, "peripheral mode\n");
    INFO("mode=%s, addr=%x\n", (devctl&MGC_M_DEVCTL_HM)?"host":"function", \
    	MGC_Read8(pBase, MGC_O_HDRC_FADDR));

    /* HR does NOT clear itself */
    if(devctl & MGC_M_DEVCTL_HR)
    {
	MGC_Write8(pBase, MGC_O_HDRC_DEVCTL, MGC_M_DEVCTL_SESSION);
    }
    
    /* unconfigured */
    pThis->bDeviceState=MGC_STATE_DEFAULT;
    pThis->bSuspended = FALSE;
    pThis->bEnd0Stage=MGC_END0_STAGE_SETUP;
    DBG(2, "Resumed\n");
    
    if ( pThis->pGadgetDriver ) {
        uint8_t power = MGC_Read8(pBase, MGC_O_HDRC_POWER);		
        pThis->pGadget->speed = (power & MGC_M_POWER_HSMODE)
            ? USB_SPEED_HIGH : USB_SPEED_FULL;

        MGC_GadgetDisconnect(pThis);
    }

#ifdef MUSB_MONITOR_DATA
    DBG(-3, "Clearing debug level\n");
    monitor_loglevel=MGC_DebugLevel;
    MGC_DebugLevel=0;
#endif

    DBG(2, "==>\n");
}


/**********************************************************************
 * DEBUG Functions
 *
 */

#if MUSB_DEBUG>0

void dump_ep_status(MGC_LinuxCd *pThis) {
    uint8_t bEnd=0;
    MGC_GadgetLocalEnd *pEnd;
    struct usb_request *req;
	
    for(bEnd = 0; bEnd < pThis->bEndCount; bEnd++) {
	pEnd=&MGC_aGadgetLocalEnd[bEnd];
	req=MGC_CurrentRequest(pEnd);		
	INFO("End-%d (%s): qlenght=%d, %s\n", bEnd, 
	    (pEnd->bIsTx)?"tx":"rx",
	    queue_length(&pEnd->req_list), 
	    (req) ? dump_usb_request(req):"none" );
    }			
	    
}

uint8_t is_ep_corrupted(void *ep) {

    if ( !ep ) {
       ERR("null pointer in is_ep_corrupted()");
       return -EINVAL;
    }
    
    if ( MGC_GadgetFindEnd(ep)<0 ) {
       ERR("ep=%p is not an endpoint()", ep);
       return -EINVAL;
    }
    
    if ( ((MGC_GadgetLocalEnd*)ep)->wPadFront!=MGC_LOCAL_PAD ) {
       ERR("ep=%p pad corrupted!()", ep);
       return -EINVAL;    
    }
    
    return 0; /* not corrupted */
}

int dump_req_list(struct list_head *lh, int verbose) {
    int count=0;	
    struct list_head *p=lh;

    while ( p && p->next!=lh ) {
	count++;
	p=p->next;
	if ( verbose ) {
	    printk( dump_node(p) );			    
	} else {
	    printk("[%p]", list_entry(p, struct usb_request, 
		    list) );
	}
    }
    
    return count;
}

void dump_ep_queue(int index, int verbose) {
#ifdef MUSB_PARANOID
    if ( index<0 ) {
	ERR("index=%d in dump_ep_queue wrong parameter\n", index);
	return;
    }
#endif

    printk(KERN_INFO"ep%d: [len=%d]\n", index, 
    	dump_req_list( &MGC_aGadgetLocalEnd[index].req_list, verbose));
}


#endif
