/* -*- linux-c -*- */

/* fs/fat/procfs.c */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/time.h>
#include <asm/uaccess.h>
#include <linux/init.h>
#include <linux/proc_fs.h>
#include <linux/msdos_fs.h>

static struct proc_dir_entry *proc_info_root = NULL;
static const char proc_info_root_name[] = "fs/fat";
 
struct proc_dir_entry *fat_proc_register_global( char *name, 
						 read_proc_t *func )
{
	return ( proc_info_root ) ? create_proc_read_entry( name, 0, 
							    proc_info_root, 
							    func, NULL ) : NULL;
}

void fat_proc_unregister_global( const char *name )
{
	remove_proc_entry( name, proc_info_root );
}

int fat_proc_info_global_init( void )
{
	if( proc_info_root == NULL ) {
		proc_info_root = proc_mkdir(proc_info_root_name, NULL);
		if( proc_info_root ) {
			proc_info_root -> owner = THIS_MODULE;
		} else {
			printk("fat: cannot create /proc/%s",
					  proc_info_root_name );
			return 1;
		}
	}
	return 0;
}

static int list_num(struct list_head *p_lh_hrad)
{
	int nr = 0;
	struct list_head *p_lh;
	p_lh = p_lh_hrad;
	
	if(list_empty(p_lh)){
		DPRINT("No List\n");
		return 0;
	}
	do{
		nr++;
		p_lh = p_lh->next;
	}while(p_lh != p_lh_hrad);
	return nr;
}
int fat_UDList_in_proc( char *buffer, char **start, off_t offset,
			int count, int *eof, void *data )
{
#if FAT_SOFTUPDATE
	char *out = buffer;
	int nr_udlist = 0;
	int nr_private = 0;
	struct list_head *p;
	DPRINT("[%s]:offset=0x%x, count=0x%x, data=0x%p\n", __FUNCTION__, offset, count, data);
	list_for_each(p, &super_blocks) {
		struct super_block *sb = sb_entry(p);
		if (sb->s_magic == MSDOS_SUPER_MAGIC) {
			DPRINT("[%s]:Get sb\n", __FUNCTION__);
			nr_udlist += list_num(&MSDOS_SB(sb)->d_ud);
			nr_private += list_num(&MSDOS_SB(sb)->d_ud_head);
		}
	}
	
	out += sprintf(out, "Read [%d] UD List\n", nr_udlist);
	out += sprintf(out, "Read [%d] Private data\n", nr_private);
	return (out - buffer);
#else
        return 0;
#endif
}
