/*-----------------------------------------------------------------------------
 * gpio-dev.h
 *
 * a header file for gpio
 *
 * Copyright 2005,2006 Sony Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *---------------------------------------------------------------------------*/
#ifndef __GPIO_DEV_H__
#define __GPIO_DEV_H__

/*-----------------------------------------------------------------------------
 * ioctl cmd base number
 *---------------------------------------------------------------------------*/
#define GPIO_IOCTL_BASE 0x00000000

/*=============================================================================
 *
 * gpio-dev's public method. 
 * 
 * Scripts from "HEAD-START" to "HEAD-END" are created as a specification
 * sheet automatically by generator.
 *
 *===========================================================================*/
/*[HEAD-START] (for "spec sheet generator")*/
/*=============================================================================
 * GPIO dev API.
 *
 *   Applications in user space call these functions  
 * 
 *===========================================================================*/
/*-----------------------------------------------------------------------------
 * open(struct inode *inode, struct file *file)
 *-----------------------------------------------------------------------------
 * Summary	: Open method
 *     IN param	: inode	:
 *              : file	:
 *     OUT param: -		
 *     Return	: Err(0:OK / -1:NG)
 *---------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 * release(struct inode *inode, struct file *file)
 *-----------------------------------------------------------------------------
 * Summary	: Close method
 *     IN param	: inode	:
 *              : file	:
 *     OUT param: -		
 *     Return	: Err(0:OK / -1:NG)
 *---------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 * ioctl(struct inode *inode, struct file *file, 
 *     unsigned int cmd, unsigned long arg)
 *-----------------------------------------------------------------------------
 * Summary	: Ioctl methodGPIO_
 *     IN param	: inode	:
 *              : file	:
 *              : cmd	: ioctl command
 *              : arg	: command parameter(Please set TGpio_ioctl_param*)
 *     OUT param: -		
 *     Return	: Err(0:OK / -1:NG)
 *---------------------------------------------------------------------------*/

/*=============================================================================
 * GPIO Dev API parameter
 *===========================================================================*/

/*-----------------------------------------------------------------------------
 * ioctl cmd number
 *---------------------------------------------------------------------------*/
/* Usually,you should call only these api from user space.
       GPIO_WRITE
       GPIO_READ
   But when you must absolutely call other api, It's OK.
   For example
        1)driver examination by pioneer.
        2)Check H/W in factory line.
*/
enum{
    GPIO_OPEN = GPIO_IOCTL_BASE,
    GPIO_CLOSE,
    GPIO_SET_IO_FUNC,
    GPIO_GET_IO_FUNC,
    GPIO_WRITE,
    GPIO_READ,
    GPIO_SET_CALLBACK_FUNC,
    GPIO_GET_CALLBACK_FUNC,
    GPIO_ENABLE_CALLBACK,
    GPIO_DISABLE_CALLBACK,
    GPIO_IS_ENABLE_CALLBACK,
    GPIO_ENABLE_WAKEUP,
    GPIO_DISBLE_WAKEUP,
    GPIO_IS_ENABLE_WAKEUP,
    GPIO_CMD_NROF,
};

#define GPIO_CMD_MAX	(GPIO_CMD_NROF-GPIO_IOCTL_BASE)
/*-----------------------------------------------------------------------------
 * ioctl cmd parameter
 *---------------------------------------------------------------------------*/
typedef struct{
    unsigned int	nr;
    int			hndl;
    unsigned char	io_func;
    int			val;
    unsigned char	callback_sense;
    unsigned char	callback_timing;
    int			status;
    int			result;
}TGpio_ioctl_param;

/*-----------------------------------------------------------------[HEAD-END]*/

#endif /* __GPIO_DEV_H__ */
