/*
 * include/asm-arm/arch-cxd4105/regs-clk.h
 *
 * CXD4105 address map
 *
 * Copyright 2005,2006 Sony Corporation
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 */
#ifndef __REGS_CLK_H__
#define __REGS_CLK_H__

#define CXD4105_CLKBLK_BASE	(CXD4105_APB2_BASE+SZ_1M*20)
#define VA_CLKBLK_BASE		IO_ADDRESS(CXD4105_CLKBLK_BASE)

/* register offset */
#define CLKBLK_PSCK54		0x00  /* 0:Off, 1:On */
#define CLKBLK_PSCK112		0x04
#define CLKBLK_PSCK24		0x08
#define CLKBLK_PSCK192		0x0c
#define CLKBLK_PSCK38		0x10
#define CLKBLK_PSCK19		0x14
#define CLKBLK_PSCK32		0x18
#define CLKBLK_PSCK16		0x1c
#define CLKBLK_PSCK264		0x20
#define CLKBLK_PSCK66		0x24
#define CLKBLK_PSCK33		0x28
#define CLKBLK_PSCK48		0x2c

#define CLKBLK_PLL1MODE		0x30  /* 0: x8, 1: x45 */
#define CLKBLK_PLL2MODE		0x34  /* PLL2 INPUT FREQ
                                          0: 224.75/1.001MHz
                                          1: 224.75MHz
                                          2: 31.5MHz
                                          3: 12MHz */
#define CLKBLK_PLL2PAT		0x38  /* 3bit divide pattern */
#define CLKBLK_PLL3MODE		0x3c  /* PLL3 OUTPUT FREQ
                                          0: 222.527473MHz
                                          1: 222.750000MHz */
#define CLKBLK_PLL3PAT		0x40  /* 3bit divide pattern */
#define CLKBLK_PLL4MODE		0x44  /* PLL4 OUTPUT FREQ
                                          0: 196.6080MHz
                                          1: 180.6336MHz */
#define CLKBLK_PLL4PAT		0x48  /* 3bit divide pattern */
#define CLKBLK_PLL4OUTDIV	0x4c  /* CK_24 select
                                          0: PLL4 / 8
                                          1: PLL4 / 12 */
#define CLKBLK_SELCKARM		0x50  /* ARM CLOCK     (ARM >= AHB >= APB)
                                         0: 132MHz, 1: 66MHz, 2: 33MHz */
#define CLKBLK_SELCKAHB		0x54  /* AHB CLOCK (ARM >= AHB >= APB)
                                         0: 132MHz, 1: 66MHz, 2: 33MHz */
#define CLKBLK_SELCKAPB		0x58  /* APB CLOCK     (ARM >= AHB >= APB)
                                         1: 66MHz, 2: 33MHz */

#define CLKBLK_PSCKAARM		0x5c  /* 0:Off, 1:On */
#define CLKBLK_AARMSRST		0x60  /* 0:assert 1:negate */

#define CLKBLK_USBPHYCKEN	0x64  /* 0:Off, 1:On */

#define CLKBLK_PSCKEMC		0x68  /* 0:Off, 1:On */
#define CLKBLK_STCKEMC		0x6c  /* 0:Off, 1:On (RO) */
#define CLKBLK_SELCKEMC		0x70  /* 0:132MHz, 1:66MHz */
#define CLKBLK_PSCKEMCREF	0x74  /* 0:Off, 1:On */

#define CLKBLK_PSCKPCI		0x78  /* 0:Off, 1:On */
#define CLKBLK_STCKPCI		0x7c  /* 0:Off, 1:On (RO) */
#define CLKBLK_SELCKPCI		0x80  /* 0:66MHz, 1:33MHz */

#define CLKBLK_PSCKATA		0x84  /* 0:Off, 1:On */
#define CLKBLK_STCKATA		0x88  /* 0:Off, 1:On */
#define CLKBLK_SELCKATA		0x8c  /* 0:66MHz, 1:33MHz */

#define CLKBLK_PSCKCF		0x90  /* 0:Off, 1:On */

#define CLKBLK_PSCKMSI		0x94  /* 0:Off, 1:On */
#define CLKBLK_STCKMSI		0x98  /* 0:Off, 1:On (RO) */
#define CLKBLK_SELCKMSI		0x9c  /* 0:38.4MHz, 1:32MHz,
                                         2:19.2MHz, 3:16MHz */

#define CLKBLK_PSCKMIW		0xa0  /* 0:Off, 1:On */
#define CLKBLK_STCKMIW		0xa4  /* 0:Off, 1:On (RO) */
#define CLKBLK_SELCKMIW		0xa8  /* 0:38.4MHz, 1:32MHz,
                                         2:19.2MHz, 3:16MHz */

#define CLKBLK_PSCKGIR		0xac  /* 0:Off, 1:On */

#define CLKBLK_PSCKI2C		0xb0  /* 0:Off, 1:On */

#define CLKBLK_PSCKUART0	0xb4  /* 0:Off, 1:On */
#define CLKBLK_STCKUART0	0xb8  /* 0:Off, 1:On (RO) */
#define CLKBLK_SELCKUART0	0xbc  /* 0:48MHz, 1:14.769MHz */

#define CLKBLK_PSCKUART1	0xc0  /* 0:Off, 1:On */
#define CLKBLK_STCKUART1	0xc4  /* 0:Off, 1:On (RO) */
#define CLKBLK_SELCKUART1	0xc8  /* 0:48MHz, 1:14.769MHz */

#define CLKBLK_PSCKUART2	0xcc  /* 0:Off, 1:On */
#define CLKBLK_STCKUART2	0xd0  /* 0:Off, 1:On (RO) */
#define CLKBLK_SELCKUART2	0xd4  /* 0:48MHz, 1:14.769MHz */

#define CLKBLK_PSCKUART3	0xd8  /* 0:Off, 1:On */
#define CLKBLK_STCKUART3	0xdc  /* 0:Off, 1:On (RO) */
#define CLKBLK_SELCKUART3	0xe0  /* 0:48MHz, 1:14.769MHz */

#define CLKBLK_UART(x)		(CLKBLK_PSCKUART0 + 12*(x))
#define CLKBLK_UARTSTAT(x)	(CLKBLK_PSCKUART0 + 12*(x) + 4)
#define CLKBLK_UARTSEL(x)	(CLKBLK_PSCKUART0 + 12*(x) + 8)

#define CLKBLK_PSCKADC0		0xe4  /* 0:Off, 1:On */
#define CLKBLK_PSCKADC1		0xe8  /* 0:Off, 1:On */
#define CLKBLK_PSCKADC2		0xec  /* 0:Off, 1:On */
#define CLKBLK_PSCKTIM		0xf0  /* 0:Off, 1:On */
#define CLKBLK_PSCKTPU		0xf4  /* 0:Off, 1:On */
#define CLKBLK_PSCKSIO		0xf8  /* 0:Off, 1:On */

#define CLKBLK_PSCKSSP0		0xfc  /* 0:Off, 1:On */
#define CLKBLK_STCKSSP0		0x100 /* 0:Off, 1:On (RO) */
#define CLKBLK_SELCKSSP0	0x104 /* 0:32MHz, 1:24MHz, 2:16MHz, 3:8MHz */

#define CLKBLK_PSCKSSP1		0x108 /* 0:Off, 1:On */
#define CLKBLK_STCKSSP1		0x10c /* 0:Off, 1:On (RO) */
#define CLKBLK_SELCKSSP1	0x110 /* 0:32MHz, 1:24MHz, 2:16MHz, 3:8MHz */

#define CLKBLK_PSCKJOG1		0x114 /* 0:Off, 1:On */
#define CLKBLK_PSCKICON		0x118 /* 0:Off, 1:On */

#define CLKBLK_PSCKDDRHCLK	0x11c /* 0:Off, 1:On */
#define CLKBLK_STCKDDRHCLK	0x120 /* 0:Off, 1:On (RO) */
#define CLKBLK_SELCKDDRHCLK	0x124 /* 0:132MHz, 1:66MHz */

#define CLKBLK_PSCKDDRPHY	0x128 /* 0:Off, 1:On */
#define CLKBLK_STCKDDRPHY	0x12c /* 0:Off, 1:On (RO) */
#define CLKBLK_SELCKDDRPHY	0x130 /* 0:264MHz, 1:132MHz */

#define CLKBLK_PSCKAUDIO	0x134 /* 0:Off, 1:On */
#define CLKBLK_STCKAUDIO	0x138 /* 0:Off, 1:On (RO) */
#define CLKBLK_SELCKAUDIO	0x13c /* SOURCE SELECT
                                          0:PLL, 1:CLK_AUDIO_IN */

#define CLKBLK_PSCKETB		0x10000 /* 0:Off, 1:On */
#define CLKBLK_PSHCLKATA	0x10004 /* 0:Off, 1:On */
#define CLKBLK_PSHCLKPCI	0x10008 /* 0:Off, 1:On */
#define CLKBLK_PSHCLKWSX	0x1000c /* 0:Off, 1:On */
#define CLKBLK_PSHCLKUSB	0x10010 /* 0:Off, 1:On */
#define CLKBLK_PSHCLKMIV	0x10014 /* 0:Off, 1:On */
#define CLKBLK_PSHCLKMSI	0x10018 /* 0:Off, 1:On */
#define CLKBLK_PSHCLKMSW	0x1001c /* 0:Off, 1:On */
#define CLKBLK_RSTETB		0x10020 /* 0:assert 1:negate */
#define CLKBLK_RSTATA		0x10024 /* 0:assert 1:negate */
#define CLKBLK_RSTPCI		0x10028 /* 0:assert 1:negate */
#define CLKBLK_RSTWSX		0x1002c /* 0:assert 1:negate */
#define CLKBLK_RSTUSB		0x10030 /* 0:assert 1:negate */
#define CLKBLK_RSTMIV		0x10034 /* 0:assert 1:negate */
#define CLKBLK_RSTMSI		0x10038 /* 0:assert 1:negate */
#define CLKBLK_RSTMSW		0x1003c /* 0:assert 1:negate */

#endif /* __REGS_CLK_H__ */
