/*
 * include/asm-arm/mach/warmboot.h
 * 
 * warmboot header
 * 
 * Copyright 2005,2006 Sony Corporation
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 *
 */

#ifndef __WARMBOOT_H__
#define __WARMBOOT_H__

#include <linux/list.h>
#include <linux/suspend.h>

struct wb_device;
struct wb_device {
	struct list_head list;
	
	char *name;
	unsigned long capacity;
	unsigned long phys_sector_size;
	unsigned long sector_size;
	unsigned long flag;
#define WBI_SILENT_MODE 0x01	
	int (*suspend_prepare)(struct wb_device *);
	int (*open)(struct wb_device *);
	int (*write_sector)(struct wb_device *,unsigned char *, u32, u32);
	int (*close)(struct wb_device *);
	int (*suspend_finish)(struct wb_device *);
	void *data;
};

struct wb_header {
	char fVersion[4];
	unsigned long sections;
	unsigned long flag;
#define WBI_COMPRESS 0x01
#define WBI_NOT_KERNEL_RO 0x02
#define WBI_CKSUM 0x04
#define WBI_LZ77_COMPRESS 0x08	
	unsigned long resume_vector;
	unsigned long dVersion;
	unsigned long sector_size;
	unsigned long r_data_size;//real data size
	unsigned long kern_start;
	unsigned long kern_size;
	unsigned long kern_cksum;
	unsigned long o_data_size;//original data size, not compressed
};

struct wb_section {
	unsigned long addr;
	unsigned long rlen;//real length
	unsigned long cksum;
	unsigned long flag;
	unsigned long olen;//original length, not compressed
	unsigned long virt;//just for linux
	unsigned long __pad[1];
	unsigned long meta_cksum;
};

#if defined(CONFIG_WARM_BOOT_IMAGE)
extern int wb_register_device(struct wb_device *,int);
extern int wb_unregister_device(struct wb_device *);

extern int  warm_boot_pm_prepare(suspend_state_t);
extern int warm_boot_pm_finish(suspend_state_t);
#else
#define wb_register_device(device,def) do{}while(0);
#define wb_unregister_device(device) do{}while(0);

static inline int warm_boot_pm_prepare(suspend_state_t state){
	return 0;
}

static inline int warm_boot_pm_finish(suspend_state_t state){
	return 0;
}
#endif

#endif
