/*
 * include/linux/debug_profile.h
 * 
 * header file for boot profiling
 * 
 * Copyright 2005,2006 Sony Corporation
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 */
#ifndef _LINUX_DEBUG_PROFILE_H
#define _LINUX_DEBUG_PROFILE_H

#if defined(CONFIG_DEBUG_PROFILE)
extern void profile_soft_init(void);
extern void profile_printk_init(void);

extern void profile_record(const char *,const char *);
#define profile_printk(callee) \
            profile_record((const char *)__FUNCTION__,(const char *)callee);

extern void profile_record_p(const void *,const void *);
#define profile_print_symbol(addr) \
            profile_record_p(__FUNCTION__, (const void *)addr);

#else
#define profile_soft_init() do{}while(0)
#define profile_printk_init() do{}while(0)
#define profile_printk(callee) do{}while(0)
#define profile_print_symbol(addr) do{}while(0)
#endif

#if defined(CONFIG_DEBUG_PROFILE_HEAP)
extern void profile_heap_record(unsigned long type, unsigned long pages);
#else
#define profile_heap_record(type, pages) do{}while(0)
#endif

#endif
