#ifndef _LINUX_KFI_H
#define _LINUX_KFI_H

#define KFI_MODE_TIMED 		0x01
#define KFI_MODE_AUTO_REPEAT 	0x02
#define KFI_MODE_STOP_ON_FULL 	0x04

#define TIME_NOT_SET	0xffffffff

typedef struct kfi_entry {
	void *va;            /* VA of instrumented function */
	void *call_site;     /* where this func was called */
	unsigned long time;  /* function entry time since trigger start time,
				in usec */
	unsigned long delta; /* delta time from entry to exit, in usec */
	int           pid;
#ifdef CONFIG_KFI_SAVE_ARGS
	unsigned long fp;    /* frame pointer address */
	unsigned long a1;    /* first argument passed */
	unsigned long a2;    /* second argument passed */
	unsigned long a3;    /* third argument passed */
#endif /* CONFIG_KFI_SAVE_ARGS */
} kfi_entry_t;

#define INTR_CONTEXT -1

#define TRIGGER_START_ON_ENTRY	0x01
#define TRIGGER_START_ON_EXIT	0x02
#define TRIGGER_STOP_ON_ENTRY	0x04
#define TRIGGER_STOP_ON_EXIT	0x08

typedef enum kfi_trigger_type {
	TRIGGER_NONE = 0,
	TRIGGER_TIME,
	TRIGGER_FUNC_ENTRY,
	TRIGGER_FUNC_EXIT,
	TRIGGER_PROC,
	TRIGGER_USER,
	TRIGGER_LOG_FULL
} kfi_trigger_type_t;

typedef struct kfi_trigger {
	enum kfi_trigger_type type;
	union {
		unsigned long time; // time since boot, in usec
		void * func_addr;
	};
	unsigned long mark; // time at which this trigger occured
} kfi_trigger_t;

#define DEFAULT_RUN_LOG_ENTRIES 20000 
#define MAX_FUNC_LIST_ENTRIES 512

typedef struct kfi_filters {
	unsigned long min_delta;
	unsigned long max_delta;
	int no_ints;
	int only_ints;
	void** func_list;
	int func_list_size;
	struct {
		int delta;
		int no_ints;
		int only_ints;
		int func_list;
	} cnt;
} kfi_filters_t;

typedef struct kfi_run {
	int primed;	/* is this run ready to start */
	int triggered;	/* has this run started */
	int complete;	/* has this run ended */
	int flags;
	/* int trigger_flag; */
	struct kfi_trigger start_trigger;
	struct kfi_trigger stop_trigger;
	struct kfi_filters filters;
	struct kfi_entry* log;
	int log_is_kmem;
	int num_entries;
	int next_entry;
	int id;
	int notfound;
} kfi_run_t;

#if CONFIG_KFI_CLOCK_SCALE
extern void setup_early_kfi_clock(void);
#else
#define setup_early_kfi_clock()
#endif

#endif // _LINUX_KFI_H
