/*
 *  include/timestamp.h
 *
 *  A cross platform general use high-res timestamp.  This file contains common
 *  routines.
 *
 *  Copyright 2004, 2005 Sony Corp.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#if !defined(_LINUX_TIMESTAMPS_H)
#define _LINUX_TIMESTAMPS_H

#include <linux/config.h>
#include <linux/time.h>
#include <asm/timestamp.h>

#if defined(CONFIG_FAST_TIMESTAMPS)

# if defined(FAST_TIMESTAMPS_DEBUG) /* for debugging */

static inline void
timestamp_print (const struct timestamp* ts)
{
	char* buf = kmalloc(512, GFP_KERNEL);

	timestamp_to_str(ts, buf);
	printk(KERN_INFO "%s\n", buf);
	kfree(buf);
}

static inline void
timestamp_dump (void)
{
	struct timestamp ts;

	timestamp_store(&ts);
	timestamp_print(&ts);
}

# endif /* defined(FAST_TIMESTAMPS_DEBUG) */

#else /* defined(CONFIG_FAST_TIMESTAMPS) */

static inline void
timestamp_store (struct timestamp* ts)
{
	(void)0;
}

static inline void
timestamp_to_timeval (const struct timestamp* ts, struct timeval* tv)
{
	tv->tv_sec = tv->tv_usec = 0;
}

#endif /* defined(CONFIG_FAST_TIMESTAMPS) */
#endif
