/*
 * usb_otgcore.h
 * 
 * Copyright 2005,2006 Sony Corporation
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 */

#ifndef __USB_OTGCORE_H__
#define __USB_OTGCORE_H__

#ifdef __KERNEL__

/******************** Proto Type ********************/
// EXPORT_SYMBOL function
unsigned char usb_otgcore_get_hs_disable(void);
int  usb_otgcore_set_test_mode(unsigned char test_mode);
int  usb_otgcore_gadget_set_feature(__le16 feature_selector);
int usb_otgcore_gadget_suspend(void);
void usb_otgcore_gadget_disconnect(void);

enum {
    USB_OTGCORE_RES_START_RCHOST = 1,
    USB_OTGCORE_RES_SUSPEND,
};

#endif  /* __KERNEL__ */

/******************** ioctl֘A ********************/
struct usb_otgcore_probe_info;

#define USB_IOC_OTGCORE_BASE                0xE3

#define USB_IOC_OTGCORE_PROBE   \
                _IOW(USB_IOC_OTGCORE_BASE, 1, struct usb_otgcore_probe_info)
#define USB_IOC_OTGCORE_REMOVE              _IO (USB_IOC_OTGCORE_BASE, 2)
#define USB_IOC_OTGCORE_STOP                _IO (USB_IOC_OTGCORE_BASE, 3)
#define USB_IOC_OTGCORE_IDLE                _IO (USB_IOC_OTGCORE_BASE, 4)
#define USB_IOC_OTGCORE_START_GADGET        _IO (USB_IOC_OTGCORE_BASE, 5)
#define USB_IOC_OTGCORE_START_HOST          _IO (USB_IOC_OTGCORE_BASE, 6)
#define USB_IOC_OTGCORE_START_E_HOST        _IO (USB_IOC_OTGCORE_BASE, 7)
#define USB_IOC_OTGCORE_ENABLE_RCHOST       _IO (USB_IOC_OTGCORE_BASE, 8)
#define USB_IOC_OTGCORE_DISABLE_RCHOST      _IO (USB_IOC_OTGCORE_BASE, 9)
#define USB_IOC_OTGCORE_START_RCGADGET      _IO (USB_IOC_OTGCORE_BASE, 10)
#define USB_IOC_OTGCORE_GET_SPEED           _IOR(USB_IOC_OTGCORE_BASE, 11, unsigned char)
#define USB_IOC_OTGCORE_SET_SPEED           _IOW(USB_IOC_OTGCORE_BASE, 12, unsigned char)
#define USB_IOC_OTGCORE_SET_TEST_MODE       _IOW(USB_IOC_OTGCORE_BASE, 13, unsigned char)
#define USB_IOC_OTGCORE_SEND_SRP            _IO (USB_IOC_OTGCORE_BASE, 14)

// for USB_IOC_OTGCORE_GET_SPEED
enum {
    USB_OTGCORE_SPEED_LS = 1,
    USB_OTGCORE_SPEED_FS,
    USB_OTGCORE_SPEED_HS,
};

// for USB_IOC_OTGCORE_SET_SPEED
enum {
    USB_OTGCORE_HS_ENABLE = 1,
    USB_OTGCORE_HS_DISABLE,
};

// for USB_IOC_OTGCORE_SET_TEST_MODE
enum {
    USB_OTGCORE_TEST_MODE_NORMAL = 1,
    USB_OTGCORE_TEST_MODE_G_PACKET,
    USB_OTGCORE_TEST_MODE_G_K,
    USB_OTGCORE_TEST_MODE_G_J,
    USB_OTGCORE_TEST_MODE_G_SE0_NAK,
    USB_OTGCORE_TEST_MODE_G_FORCE_PACKET,
    USB_OTGCORE_TEST_MODE_H_PACKET,
    USB_OTGCORE_TEST_MODE_H_LS,
    USB_OTGCORE_TEST_MODE_H_FS,
    USB_OTGCORE_TEST_MODE_H_HS,
};


/******************** Event֘A ********************/
struct otgcore_event {
    void (*cid)(usb_hndl_t, usb_kevent_id_t, unsigned char, void*);
    void (*vbus)(usb_hndl_t, usb_kevent_id_t, unsigned char, void* vp_arg);
    void (*vbus_error)(usb_hndl_t, usb_kevent_id_t, unsigned char, void* vp_arg);
    void (*pullup)(usb_hndl_t, usb_kevent_id_t, unsigned char, void* vp_arg);
    void (*receive_srp)(usb_hndl_t, usb_kevent_id_t, unsigned char, void* vp_arg);
    void (*rchost)(usb_hndl_t, usb_kevent_id_t, unsigned char, void* vp_arg);
    void (*rcgadget)(usb_hndl_t, usb_kevent_id_t, unsigned char, void* vp_arg);
    void (*set_feature)(usb_hndl_t, usb_kevent_id_t, unsigned char, void* vp_arg);
};

struct usb_otgcore_probe_info {
    usb_hndl_t hndl;                        /* nh */
    struct otgcore_event event;             /* CxgR[obN */
};

// Event ID
enum {
    USB_KEVENT_ID_OTGCORE_CID = 0,
    USB_KEVENT_ID_OTGCORE_VBUS,
    USB_KEVENT_ID_OTGCORE_VBUS_ERROR,
    USB_KEVENT_ID_OTGCORE_PULLUP,
    USB_KEVENT_ID_OTGCORE_RECEIVE_SRP,
    USB_KEVENT_ID_OTGCORE_RCHOST,
    USB_KEVENT_ID_OTGCORE_RCGADGET,
    USB_KEVENT_ID_OTGCORE_SET_FEATURE,
    USB_KEVENT_ID_OTGCORE_NBROF,
};

// for USB_KEVENT_ID_OTGCORE_CID
struct usb_kevent_arg_otgcore_cid {
    unsigned char value;        /* CIDl */
                                /* USB_OTGCORE_CID_A / USB_OTGCORE_CID_B */
};

enum { 
    USB_OTGCORE_CID_A = 1,
    USB_OTGCORE_CID_B,
};

// for USB_KEVENT_ID_OTGCORE_VBUS
struct usb_kevent_arg_otgcore_vbus {
    unsigned char value;        /* VBUSl */
                                /* USB_OTGCORE_VBUS_ON / USB_OTGCORE_VBUS_OFF */
};

enum {
    USB_OTGCORE_VBUS_ON = 1,
    USB_OTGCORE_VBUS_OFF,
    USB_OTGCORE_VBUS_UNKNOWN = 99,  /* OTG Core p */
};

// for USB_KEVENT_ID_OTGCORE_PULLUP
struct usb_kevent_arg_otgcore_pullup {
    unsigned char value;        /* Pullup */
                                /* USB_OTGCORE_PULLUP_ON / USB_OTGCORE_PULLUP_OFF */
};

enum {
    USB_OTGCORE_PULLUP_ON = 1,
    USB_OTGCORE_PULLUP_OFF,
};

// for USB_KEVENT_ID_OTGCORE_RCxx
struct usb_kevent_arg_otgcore_rchost {
    unsigned char result;       /*  */
                                /* USB_OTGCORE_SUCCESS / USB_OTGCORE_FAIL */
};

struct usb_kevent_arg_otgcore_rcgadget {
    unsigned char result;       /*  */
                                /* USB_OTGCORE_SUCCESS / USB_OTGCORE_FAIL */
};

enum {
    USB_OTGCORE_SUCCESS = 1,
    USB_OTGCORE_FAIL,
};

// for USB_KEVENT_ID_OTGCORE_SET_FEATURE
struct usb_kevent_arg_otgcore_set_feature {
    unsigned char b_hnp_enable;
    unsigned char a_hnp_support;
    unsigned char a_alt_hnp_support;
};

enum {
    USB_OTGCORE_UNSET = 0,
    USB_OTGCORE_SET,
};

#endif /* __USB_OTGCORE_H__ */
