/*
 * include/linux/lz77_inflate.c
 * 
 * lz77 inflate routine
 * 
 * Copyright 2005,2006 Sony Corporation
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 */
//#define DEBUG
#ifdef __KERNEL__
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/lz77.h>
EXPORT_SYMBOL(lz77_inflate);
#else
#include <stdio.h>
#include <string.h>
#include "lz77.h"
#endif

static inline int lz77_memcpy(unsigned char *src, int src_len, unsigned char *dst, int dst_len)
{
	int ret=src_len>dst_len?dst_len:src_len;
	
	if(ret){
		int len=ret&(~0x03);
		unsigned char *d = dst;
		unsigned char *s = src;

		if(ret>=10){
			if((((unsigned long)(dst-src))&0x03)==0){
				while((unsigned long)d&0x03) *d++=*s++;
				
				while(d<(dst+len)) {
					*(unsigned long *)d = *(unsigned long *)s;
					d += 4;
					s += 4;
				}
			}
			else if((((unsigned long)(dst-src))&0x01)==0){
				while((unsigned long)d&0x01) *d++=*s++;
				
				while(d<(dst+len)) {
					*(unsigned short *)d = *(unsigned short *)s;
					d += 2;
					s += 2;
				}
			}
		}

		/*byte copy*/
		while(d<(dst+ret)) *d++ = *s++;
	}
	return ret;
}

int lz77_inflate(unsigned char *src, int len, unsigned char *dst, int dst_len, unsigned char **sd)
{
	int type=0;
	unsigned char *s=src, *d=dst;
	unsigned char *de = dst+dst_len;
	unsigned char *se = src+len-1;
	int u_i=0;

	if(!src || len<4 || !dst)
		return -1;
	
	switch(*s++){
	case LZ77_COMPRESSED:
		while(s<se){
			u_i = 0;
			type = *s++;
			while(u_i++<8 && s<se){
				if(type & 0x01){//codeword
					int l = (s[0]&0xF0)>>4;
					int bd = (s[0]&0x0F)<<8;
					bd |= s[1];
					bd &= 0xFFF;
					l = index_to_size(l);
					s += 2;
					
					if(bd)
						d += lz77_memcpy(d-bd, l, d, (int)(de-d));
					else{
						lz77_printf("end of stream!\n");
						goto inflate_end;
					}
				}
				else{
					*d++ = *s++;
				}

				type >>= 1;
			}
		}
		
		break;
	case LZ77_RAW:
	{
		int l = s[1] | (s[2]<<8);

		s += 3;
		l = dst_len>(l)?(l):dst_len;
		memcpy(d, s, l);
		d += l;
		s += l;

		lz77_printf("raw block length: %d\n", l);
	}
		
		break;
	default:
		lz77_warning("error compress method!\n");
		
		break;
	};

 inflate_end:	
	lz77_printf("inflate: %d %d %d %d %p %p\n", s-src, d-dst, len, dst_len, s, se);
	if(sd)
		*sd = s;
	return (d-dst);
}

